package de.desy.web2cToolkit.web2c.web2cChatUtilities;

import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cGarbageCollection;

/**
 * @author R.Bacher, DESY
 */
public class Web2cChatHandler implements IWeb2cGarbageCollection {
	
	private ArrayList<Web2cChatChannel> chatChannelList = new ArrayList<Web2cChatChannel>();
	private ArrayList<String> idList = new ArrayList<String>();	
	private ArrayList<Integer> numberChatChannelRequesters = new ArrayList<Integer>();

    /**
     * Constructor
     * Chat channel handler
     */
	public Web2cChatHandler() {
	}
	
    /**
     * Returns the chat channel instance for the channel id specified
     * @param id channel id
     * @return chat channel
     */
	public Web2cChatChannel getChatChannel(String id) {
		
		Web2cChatChannel chatChannel = null;
		int numberRequesters = 0;
		
		if (!idList.contains(id)) {
			synchronized (idList) {
				idList.add(id);
			}
			chatChannel = new Web2cChatChannel();
			synchronized (chatChannelList) {
				chatChannelList.add(chatChannel);
			}
			synchronized (numberChatChannelRequesters) {
				numberChatChannelRequesters.add(1);
			}
		}
		else {
			synchronized (idList) {
				synchronized (chatChannelList) {
					synchronized (numberChatChannelRequesters) {
						Iterator<Integer> it3 = numberChatChannelRequesters.iterator();
						Iterator<Web2cChatChannel> it2 = chatChannelList.iterator();
						for (Iterator<String> it1 = idList.iterator(); it1.hasNext();) {
							String channelId = (String)it1.next();
							Web2cChatChannel channel = (Web2cChatChannel)it2.next();
							int number = (Integer)it3.next();
							if (id.equals(channelId)) {
								chatChannel = channel;
								numberRequesters = number + 1;
							}
						}
					}
				}
			}
			numberChatChannelRequesters.set(idList.indexOf(id), numberRequesters);
		}
		
		return chatChannel;
	}
	
	/**
    * Returns the number of requesters requesting the chat channel specified
    * @param id chat channel id
    * @return number of requesters
    */
	public int getNumberOfRequesters(String id) {
		
		int index = 0;
		int number = 0;
		
		index = idList.indexOf(id);
		if (index >= 0) {
			number = numberChatChannelRequesters.get(idList.indexOf(id));
		}
		else {
			number = -1;
		}
		
		return number;
	}
	
	/**
    * Decreases (-1) the number of requesters requesting the chat channel specified; removes the corresponding chat channel if number <= 0
    * @param id chat channel id
    */
	public void removeRequester(String id) {
		
		int index = 0;
		int number = 0;
		
		index = idList.indexOf(id);
		if (index >= 0) {
			number = getNumberOfRequesters(id);
			numberChatChannelRequesters.set(idList.indexOf(id), number - 1);
//			System.out.println("Chat channel requester for " + id + " gone! Remaining " + getNumberOfRequesters(id) + " requester(s).");
		}
		
		if (number - 1 <= 0) removeResource(id);
	}
	
	/**
    * Removes the chat channel with the id specified and all associated resources
    * @param id chat channel id
    */
	public void removeResource(String id) {
		
		int index = 0;
		
		index = idList.indexOf(id);
		if (index >= 0) {
			chatChannelList.get(index).clearChat();
			chatChannelList.remove(index);
			idList.remove(index);
			numberChatChannelRequesters.remove(index);
//			System.out.println("Chat channel for " + id + " closed!");
		}
	}
}
