package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Button
 * @author R.Bacher, DESY
 */
public class Web2cButton extends Web2cComponent {

	private String protocol = null;
	private String address = null;
	private String parameters = null;
	
	/**
	 * Button
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 * @param access access mode
	 */
	public Web2cButton(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch, String access) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		accessMode = access;
		componentDataSize = 0;	
		registerMonitors(true);
		initSoundAndBlinking(1);
	
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {

		protocol = property.get(0).split("://")[0];
		String str = property.get(0).substring(protocol.length() + 3);
		if (str.indexOf("(") > 0) {
			address = str.substring(0, str.indexOf("("));
			if (str.substring(str.indexOf("("),str.lastIndexOf(")") + 1).length() != 2) {
				parameters = str.substring(str.indexOf("(") + 1, str.lastIndexOf(")"));
				if (address.equals("replace")) {
					if (parameters.indexOf(",") < 0) parameters = parameters + ",-1";
				}
			}
			else {
				parameters = "";
			}
		}
		else {
			address = str;
			parameters = "";
		}
	}
	
	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String buttonHTML = "";
		String html = "";
		
		if (caption.get(0).equals("") || caption.get(0).equals(" ")) {
			buttonHTML = "			<button type=\"button\" id=\"" + id + "\" value=\"" + caption.get(0) + "\" title=\"" + toolTipText + "\"";
			buttonHTML = buttonHTML + " onclick=\"" + id + "_click()\"";			
			buttonHTML = buttonHTML + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; border-style:outset; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "\">";	
			buttonHTML = buttonHTML + "<div><img src=\"" + image + "\" style=\"width:" + width + "; height:" + height + ";\"></div>";
			buttonHTML = buttonHTML + "</button>";
		}
		else {
			buttonHTML = "			<input type=\"button\" id=\"" + id + "\" value=\"" + caption.get(0) + "\" title=\"" + toolTipText + "\"";
			buttonHTML = buttonHTML + " onclick=\"" + id + "_click()\"";
			buttonHTML = buttonHTML + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; border-style:outset; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "\">";	
		}
		
		if (protocol.equals("http")) {		
			html = buttonHTML;
			componentHTML.add(html);
		}	
		else if (protocol.equals("local")) {
			if (address.equals("superLogout")) {
				if (userPrivilege.equals(layman)) {
					html = "";
				}	
				else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
					html = "";
				}
				else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
					html = buttonHTML;
					componentHTML.add(html);
				}
				else {
					html = "";
				}
			}
			else {
				html = buttonHTML;
				componentHTML.add(html);
			}
		}
		else {	
			if (userPrivilege.equals(layman)) {
				html = "";
			}	
			else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
				html = "";
			}
			else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
				html = buttonHTML;
				componentHTML.add(html);
			}
			else {
				html = "";
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (protocol.equals("http")) {		
			componentJavaScript.add("	//Button: " + id);
			componentJavaScript.add("	function " + id + "_click() {");
			componentJavaScript.add("		newWindow = window.open(\"" + property.get(0) + "\");");
			if (keepHistory == false) {
				componentJavaScript.add("		var xhraction;");
				componentJavaScript.add("		try {");                                                  
				componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
				componentJavaScript.add("		} catch (event1) {");                             
				componentJavaScript.add("			try {");
				componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
				componentJavaScript.add("			} catch (event2) {");
				componentJavaScript.add("				xhraction = false;");
				componentJavaScript.add("			}");
				componentJavaScript.add("		}");
				componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
				componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
				componentJavaScript.add("		}"); 	
				componentJavaScript.add("		var now = new Date();"); 
				componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>" + "logout" + "\" + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
				componentJavaScript.add("		xhraction.onreadystatechange = function() {};");  
				componentJavaScript.add("		xhraction.send(null);"); 
				componentJavaScript.add("		document.clear();");  
				componentJavaScript.add("		window.close();");  
			}
			componentJavaScript.add("	}");
		}	
		else if(protocol.equals("local") && address.equals("login")) {
			componentJavaScript.add("	//Button: " + id);
			componentJavaScript.add("	function " + id + "_click() {");
			componentJavaScript.add("		var language = " + language + ";");
			componentJavaScript.add("		var uri = location.href.split(\"=\")[0] + \"=(file)\" + location.href.split(\"]\")[1].split(\",\")[0] + \",\" + Number(language).toString();");
			componentJavaScript.add("		location.replace(uri);");
			componentJavaScript.add("	}");
		}
		else if(protocol.equals("local") && (address.equals("logout") || address.equals("superLogout"))) {
			componentJavaScript.add("	//Button: " + id);
			componentJavaScript.add("	function " + id + "_click() {");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>" + "logout" + "\" + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");  
			componentJavaScript.add("		xhraction.send(null);"); 
			componentJavaScript.add("		document.clear();");  
			componentJavaScript.add("		window.close();");  
			componentJavaScript.add("	}");
		}
		else if(protocol.equals("local") && address.equals("switchLanguage")) {
			componentJavaScript.add("	//Button: " + id);
			componentJavaScript.add("	function " + id + "_click() {");
			componentJavaScript.add("		var nextLanguage = 0;");
			componentJavaScript.add("		var implementedLanguages = " + implementedLanguages + ";");
			componentJavaScript.add("		var language = " + language + ";");
			componentJavaScript.add("		if ((language + 1) < implementedLanguages) nextLanguage = language + 1;");
			componentJavaScript.add("		if ((language + 1) == implementedLanguages) nextLanguage = 0;");
			if (accessMode.equals("open")) {
				componentJavaScript.add("		var uri = location.href.split(\"=\")[0] + \"=(open)\" + location.href.split(\"]\")[1].split(\",\")[0] + \",\" + Number(nextLanguage).toString();");
			}
			else {
				componentJavaScript.add("		var uri = location.href.split(\"=\")[0] + \"=(page)[\" + location.href.split(\"]\")[0].split(\"[\")[1] + \"]\" + location.href.split(\"]\")[1].split(\",\")[0] + \",\" + Number(nextLanguage).toString();");				
			}
			componentJavaScript.add("		location.replace(uri);");
			componentJavaScript.add("	}");
		}
		else if(protocol.equals("local") && address.equals("replace")) {
			componentJavaScript.add("	//Button: " + id);
			componentJavaScript.add("	function " + id + "_click() {");
			componentJavaScript.add("		var nextLanguage = 0;");
			componentJavaScript.add("		var nextLanguageString = \"\";");
			componentJavaScript.add("		var language = " + language + ";");
			componentJavaScript.add("		nextLanguageString = \"" + parameters.split(";")[1] + "\";");
			componentJavaScript.add("		if (nextLanguageString == \"-1\") nextLanguage = language;");
			componentJavaScript.add("		else nextLanguage = parseInt(nextLanguageString);");
			if (accessMode.equals("open")) {
				componentJavaScript.add("		var uri = location.href.split(\"=\")[0] + \"=(open)" + parameters.split(";")[0] + ",\" + Number(nextLanguage).toString();");
			}
			else {
				componentJavaScript.add("		var uri = location.href.split(\"=\")[0] + \"=(page)[\" + location.href.split(\"]\")[0].split(\"[\")[1] + \"]" + parameters.split(";")[0] + ",\" + Number(nextLanguage).toString();");				
			}
			componentJavaScript.add("		location.replace(uri);");
			componentJavaScript.add("	}");
		}
		else {		
			if (userPrivilege.equals(layman)) {
			}
			else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			}
			else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
				componentJavaScript.add("	//Button: " + id);
				componentJavaScript.add("	function " + id + "_click() {");
				for (int i = 0; i < property.size(); i++) {
					componentJavaScript.add("		var xhraction_" + i + ";");
					componentJavaScript.add("		try {");                                                  
					componentJavaScript.add("			xhraction_" + i + " = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
					componentJavaScript.add("		} catch (event1_" + i + ") {");                             
					componentJavaScript.add("			try {");
					componentJavaScript.add("				xhraction_" + i + " = new ActiveXObject(\"Microsoft.XMLHTTP\");");
					componentJavaScript.add("			} catch (event2_" + i + ") {");
					componentJavaScript.add("				xhraction_" + i + " = false;");
					componentJavaScript.add("			}");
					componentJavaScript.add("		}");
					componentJavaScript.add("		if (!xhraction_" + i + " && typeof XMLHttpRequest !=\"undefined\") {");
					componentJavaScript.add("			xhraction_" + i + " = new XMLHttpRequest();");                 
					componentJavaScript.add("		}"); 
					componentJavaScript.add("		var now_" + i + " = new Date();"); 
					componentJavaScript.add("		xhraction_" + i + ".open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>" + property.get(i) + "\" + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now_" + i + ".getTime()) + \"</timeStamp></action>\", true);"); 
					componentJavaScript.add("		xhraction_" + i + ".onreadystatechange = function() {};");  
					componentJavaScript.add("		xhraction_" + i + ".send(null);");  
				}
				componentJavaScript.add("	}");
			}
			else {
			}
		}

		return componentJavaScript;
	}

	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + caption.get(0) + "</value>";

		str = "<update>" + identifier + value + "</update>";
	
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String style = "";
		String blinking = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + caption.get(0) + "</value>";
		style = getStyleUpdate();
		blinking = getBlinking(0);

		str = "<update>" + identifier + value + style + blinking + "</update>";
	
		return str;
	}

	/**
	 * Calls simulator
	 * @param action requested action
	 */	
	public void doSimulatedAction(String action) {	
	}

	/**
	 * Sends client action requests to common device access plug
	 * @param action requested action
	 */	
	public void doRealAction(String action) {
			
		String prop = action.substring(0, action.indexOf("("));
		String param = "";
		if (action.substring(action.indexOf("("), action.indexOf(")") + 1).length() != 2) {
			param = action.substring(action.indexOf("(") + 1, action.indexOf(")"));
		}
		else {
			param = "";
		}

		commonDeviceAccess.execute(prop + "(" + characterEscaper.encodeString(param) + ")", true);
	}
}
