package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Chat
 * @author R.Bacher, DESY
 */
public class Web2cChat extends Web2cComponent {
	
	private String userName = null;
	private int chatLength = 0;
	
	/**
	 * Chat
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 * @param name actual user name
	 */
	public Web2cChat(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch, String name) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);

		userName = name;
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (caption.isEmpty()) caption.add("Chat");
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}

	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String html = null;
		String enterKeyCode = "13";
		
		double headerLeftFraction = 0.05;
		double headerTopFraction = 0.025;
		double headerWidthFraction = 0.9;
		double headerHeightFraction = 0.05;		
		double outLeftFraction = 0.05;
		double outTopFraction = 0.10;
		double outWidthFraction = 0.9;
		double outHeightFraction = 0.7;
		double inLeftFraction = 0.05;
		double inTopFraction = 0.85;
		double inWidthFraction = 0.8;
		double inHeightFraction = 0.1;
		double sendButtonLeftFraction = 0.875;
		double sendButtonTopFraction = 0.85;
		double sendButtonWidthFraction = 0.025;
		double sendButtonHeightFraction = 0.1;
		double clearButtonLeftFraction = 0.925;
		double clearButtonTopFraction = 0.85;
		double clearButtonWidthFraction = 0.025;
		double clearButtonHeightFraction = 0.1;
		
		if (prompt.equals("chat")) {
			headerLeftFraction = 0.05;
			headerTopFraction = 0.025;
			headerWidthFraction = 0.9;
			headerHeightFraction = 0.05;		
			outLeftFraction = 0.05;
			outTopFraction = 0.10;
			outWidthFraction = 0.9;
			outHeightFraction = 0.7;
			inLeftFraction = 0.05;
			inTopFraction = 0.85;
			inWidthFraction = 0.8;
			inHeightFraction = 0.1;
			sendButtonLeftFraction = 0.875;
			sendButtonTopFraction = 0.85;
			sendButtonWidthFraction = 0.025;
			sendButtonHeightFraction = 0.1;
			clearButtonLeftFraction = 0.925;
			clearButtonTopFraction = 0.85;
			clearButtonWidthFraction = 0.025;
			clearButtonHeightFraction = 0.1;
		}
		if (prompt.equals("message")) {
			headerLeftFraction = 0.05;
			headerTopFraction = 0.025;
			headerWidthFraction = 0.9;
			headerHeightFraction = 0.05;		
			outLeftFraction = 0.05;
			outTopFraction = 0.10;
			outWidthFraction = 0.9;
			outHeightFraction = 0.75;
			if (userPrivilege.equals(layman)) {
				headerLeftFraction = 0.05;
				headerTopFraction = 0.025;
				headerWidthFraction = 0.9;
				headerHeightFraction = 0.05;	
				outLeftFraction = 0.05;
				outTopFraction = 0.10;
				outWidthFraction = 0.9;
				outHeightFraction = 0.85;		
			}
			inLeftFraction = 0.05;
			inTopFraction = 0.9;
			inWidthFraction = 0.8;
			inHeightFraction = 0.075;
			sendButtonLeftFraction = 0.875;
			sendButtonTopFraction = 0.9;
			sendButtonWidthFraction = 0.025;
			sendButtonHeightFraction = 0.075;	
			clearButtonLeftFraction = 0.925;
			clearButtonTopFraction = 0.9;
			clearButtonWidthFraction = 0.025;
			clearButtonHeightFraction = 0.075;
			
		}

		if (userPrivilege.equals(expert) || userPrivilege.equals(supervisor)) {
			html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">";		
			componentHTML.add(html);
			
			String headerLeft = (headerLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String headerTop = (headerTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String headerWidth = String.valueOf((headerWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String headerHeight = String.valueOf((headerHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + headerLeft + "; top:" + headerTop + "; width:" + headerWidth + "; height:" + headerHeight + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">" + caption.get(0) + "</div>";		
			componentHTML.add(html);    
			
			String outLeft = (outLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String outTop = (outTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String outWidth = String.valueOf((outWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String outHeight = String.valueOf((outHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<textarea id=\"" + id + "_out" + "\" rows=\"" + maxChatDepth + "\" wrap=\"off\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + outLeft + "; top:" + outTop + "; width:" + outWidth + "; height:" + outHeight + "; color:" + color + "; background-color:" + white + "; border-style:" + solid + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">" + "" + "</textarea>";		
			componentHTML.add(html);
			
			String inLeft = (inLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String inTop = (inTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String inWidth = String.valueOf((inWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String inHeight = String.valueOf((inHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<input type=\"text\" id=\"" + id + "_in" + "\" value=\"Enter Text\" title=\"" + toolTipText + "\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {" + id + "_button_setValue()" + "; return false;}\"";
			html = html + " style=\"position:absolute; left:" + inLeft + "; top:" + inTop + "; width:" + inWidth + "; height:" + inHeight + "; color:" + color + "; background-color:" + white + "; border-style:" + solid + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">";		
			componentHTML.add(html);
			
			String sendButtonLeft = (sendButtonLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String sendButtonTop = (sendButtonTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String sendButtonWidth = String.valueOf((sendButtonWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String sendButtonHeight = String.valueOf((sendButtonHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<input type=\"button\" id=\"" + id + "_sendButton" + "\" value=\"&#8658;\" title=\"" + toolTipText + "\"";
			html = html + " onclick=\"" + id + "_button_setValue()" + "\"";
			html = html + " style=\"position:absolute; left:" + sendButtonLeft + "; top:" + sendButtonTop + "; width:" + sendButtonWidth + "; height:" + sendButtonHeight + "; color:" + black + "; background-color:" + white + "; border-style:outset; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">";		
			componentHTML.add(html);
			
			String clearButtonLeft = (clearButtonLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String clearButtonTop = (clearButtonTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String clearButtonWidth = String.valueOf((clearButtonWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String clearButtonHeight = String.valueOf((clearButtonHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<input type=\"button\" id=\"" + id + "_clearButton" + "\" value=\"clr\" title=\"" + toolTipText + "\"";
			html = html + " onclick=\"" + id + "_button_clear()" + "\"";
			html = html + " style=\"position:absolute; left:" + clearButtonLeft + "; top:" + clearButtonTop + "; width:" + clearButtonWidth + "; height:" + clearButtonHeight + "; color:" + black + "; background-color:" + white + "; border-style:outset; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">";		
			componentHTML.add(html);
			
			html = "			</div>";		
			componentHTML.add(html);
		}

		if (userPrivilege.equals(layman) && expertComponent.equals(no)&& prompt.equals("message")) {
			html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">";		
			componentHTML.add(html);
			
			String headerLeft = (headerLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String headerTop = (headerTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String headerWidth = String.valueOf((headerWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String headerHeight = String.valueOf((headerHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + headerLeft + "; top:" + headerTop + "; width:" + headerWidth + "; height:" + headerHeight + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">" + caption.get(0) + "</div>";		
			componentHTML.add(html);    
				
			String outLeft = (outLeftFraction * (Integer.valueOf(width.split("p")[0]))) + "px";
			String outTop = (outTopFraction * (Integer.valueOf(height.split("p")[0]))) + "px";
			String outWidth = String.valueOf((outWidthFraction * (Integer.valueOf(width.split("p")[0])))) + "px";
			String outHeight = String.valueOf((outHeightFraction * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<textarea id=\"" + id + "_out" + "\" rows=\"" + maxChatDepth + "\" wrap=\"off\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + outLeft + "; top:" + outTop + "; width:" + outWidth + "; height:" + outHeight + "; color:" + color + "; background-color:" + white + "; border-style:" + solid + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">" + "" + "</textarea>";		
			componentHTML.add(html);
			
			html = "			</div>";		
			componentHTML.add(html);
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	

		if (userPrivilege.equals(expert) || userPrivilege.equals(supervisor)) {
			componentJavaScript.add("	//Chat: " + id);
			componentJavaScript.add("	function " + id + "_button_setValue()" + "{");
			componentJavaScript.add("		var ref;");
			componentJavaScript.add("		var inText;");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		ref = document.getElementById(\"" +  id + "_in" + "\");");
			componentJavaScript.add("		inText = escape(\"<param1>add</param1><param2>\" + ref.value + \"</param2>\");");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>\" + inText + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");   
			componentJavaScript.add("		xhraction.send(null);");  
			componentJavaScript.add("		ref.value = \"\";");
			componentJavaScript.add("	}");

			componentJavaScript.add("	function " + id + "_button_clear()" + "{");
			componentJavaScript.add("		var clr;");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		clr = escape(\"<param1>clr</param1><param2></param2>\");");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>\" + clr + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");   
			componentJavaScript.add("		xhraction.send(null);");  
			componentJavaScript.add("	}");
		}

		return componentJavaScript;
	}

	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String soundAndBlinking = "";
		ArrayList<String> chat = null;
		
		chat = chatChannel.getChat();
		for (Iterator<String> it = chat.iterator(); it.hasNext();) {
			String entry = (String)it.next();
			str = str + entry + "\n";  
		}
		str = str.replace("\n", "%a");
		
		identifier = "<id>" + id + "_out" + "</id>";
		if (chat.toArray().length == 0) {
			value = "<value> </value>";
		}
		else {
			value = "<value>" + str + "</value>";
			if (chat.toArray().length != chatLength) {
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
			}
			else {
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
			}
			chatLength = chat.toArray().length;
		}
		
		str = "<update>" + identifier + value + soundAndBlinking + "</update>";
		
		return str;
	}

	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String soundAndBlinking = "";
		String style = "";
		ArrayList<String> chat = null;
		
		chat = chatChannel.getChat();
		for (Iterator<String> it = chat.iterator(); it.hasNext();) {
			String entry = (String)it.next();
			str = str + entry + "\n";  
		}
		str = str.replace("\n", "%a");
		
		identifier = "<id>" + id + "_out" + "</id>";
		if (chat.toArray().length == 0) {
			value = "<value> </value>";
		}
		else {
			value = "<value>" + str + "</value>";
				if (chat.toArray().length != chatLength) {
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
				}
				else {
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
				}
				chatLength = chat.toArray().length;
		}
		
		style = style + getStyleUpdate();
		
		str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		
		return str;
	}
	
	/**
	 * Calls simulator
	 * @param action requested action
	 */
	public void doSimulatedAction(String action) {
		
		doRealAction(action);
	}

	/**
	 * Sends client action requests to common device access plug
	 * @param action requested action
	 */	
	public void doRealAction(String action) {
		
		String param1 = "";
		String param2 = "";
		
		param1 = action.split("</param1><param2>")[0].replace("<param1>", "");
		param2 = action.split("</param1><param2>")[1].replace("</param2>", "");
		
		if (param1.equals("add")) {
			chatChannel.appendChat(userName, param2, prompt);
		}
		if (param1.equals("clr")) {
			chatChannel.clearChat();
		}		
	}
}
