package de.desy.web2cToolkit.web2c.web2cComponents;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cExtension.Web2cExtensionExecutor;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cExtensionManifestReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cComponent;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * Component
 * @author R.Bacher, DESY
 */
public class Web2cComponent implements IWeb2cConstants, IWeb2cComponent {
	
	public String referenceId = null;
	public String homeDir = null;
	public String id = null;
	public int language = 0;
	public double xScale = 0.0;
	public double yScale = 0.0;
	public String width = null;
	public String height = null;
	public String marginLeft = null;
	public String marginTop  = null;
	public String toolTipText = null;
	public String textAlignment = null;
	public String color = null;
	public String fontSize = null;
	public String fontThickness= null;
	public String borderStyle = null;
	public String borderWidth = null;
	public String borderColor = null;
	public String headerCaption  = null;
	public String headerColor  = null;
	public String headerBackgroundColor  = null;
	public ArrayList<String> property = null;
	public ArrayList<String> condition = null;
	public String backgroundColor = null;
	public boolean scrollBarEnabled = false;
	public Web2cCommonDeviceAccess commonDeviceAccess = null;
	public Web2cCommonVideoConnector commonVideoConnector = null;
	public Web2cChatChannel chatChannel = null;
	public String[] soundFile = null;
	public boolean[] soundEnabled = null;
	public String[] blinkingColor = null;
	public boolean[] blinkingEnabled = null;
	public String alarmThreshold = null;
	public String lowIcon = null;
	public String highIcon = null;
	public ArrayList<String> caption = null;
	public String image = null;
	public boolean keepHistory = true;
	public String userPrivilege = null;
	public String prompt = null;
	public ArrayList<String> legend = null;
	public ArrayList<Double> yMin = null;
	public ArrayList<Double> yMax = null;
	public String xmin = "";
	public String xmax = "";
	public String xLabel = null;
	public String functionType = null;
	public String functionParameter = null;
	public String plotAxis = null;
	public String plotType = null;
	public String plotOrder = null;
	public Color plotColor[] = null;
	public String timeIntervalInSeconds = null;
	public String file = null;
	public String[] trueValue = null;
	public String falseIcon = null;
	public String[] trueIcon = null;
	public String superComponent = null;
	public String expertComponent = null;
	public int maxInputTextLength = 0;	
	public String minimumValue = null;
	public String maximumValue = null;
	public String offset = null;
	public String lowColor = null;
	public String highColor = null;
	public String format = null;
	public String min = null;
	public String extensionManifest = "";

	public boolean simulation = true;
	public String error = "";
	public boolean propertyLinksEstablished = false;
	public boolean conditionLinksEstablished = false;
	public boolean extensionPropertyLinksEstablished = false;
	private int linkFlag = 0;
	public String[] text = null;
	private String[] blinkingColor1 = null;
	private String[] blinkingColor2 = null;
	private String[] blinkingColorUpdate = null;
	public boolean[] stateChangedToAlarmTrue = null;
	public boolean[] stateChangedToAlarmFalse = null;
	public int[] stateChangedToAlarmCounterTrue = null;
	public int[] stateChangedToAlarmCounterFalse = null;
	private ArrayList<String> monitorList = new ArrayList<String>();
	private ArrayList<String> receiverList = new ArrayList<String>();
	public String propertyList = null;
	public String legendList = null;
	private ArrayList<String> componentAttributeNames = null;
	private ArrayList<String> componentAttributeValues = null;
	public int implementedLanguages = 0;
	public String accessMode = "";
	public int componentDataSize = 0;
	public String extensionClassName = "";
	public ArrayList<String> monitorURI = new ArrayList<String>();
	public String[] monitorData = null;
	public ArrayList<String> receiverURI = new ArrayList<String>();
	public ByteArrayOutputStream[] receiverData = null;
	public String soundString = "";
	private String blinkingString = "";
	public String soundEnabledString = "";
	private String blinkingEnabledString = "";
	public Web2cExtensionExecutor extensionExecutor = null;
	public Web2cASCIICharacterEscaper characterEscaper = null;

	/**
	 * Web2cComponent
	*/
	public Web2cComponent() {		
	}
	
	/**
	 * Web2cComponent
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cComponent(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		referenceId = refId;
		homeDir = home;
		language = lang;
		xScale = x;
		yScale = y;
		componentAttributeNames = names;
		componentAttributeValues = values;
		simulation = simulatedData;
		error = "";
		if (privilege == null) {
			userPrivilege = layman;
		}
		else {
			userPrivilege = privilege;
		}
		commonDeviceAccess = cda;	
		commonVideoConnector = cvc;
		chatChannel = ch;
		componentDataSize = 0;

		characterEscaper = new Web2cASCIICharacterEscaper();
		
		decodeAttributes(componentAttributeNames, componentAttributeValues);
	}

	/**
	 * Returns the component id
	 */
	public String getID() {
		
		return id;
	}

	/**
	 * Decodes the component's attributes and sets default values. Initializes the component.
	 */
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {	
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;
		
		property = new ArrayList<String>();
		condition = new ArrayList<String>();
		caption = new ArrayList<String>();
		legend = new ArrayList<String>();	
		yMin = new ArrayList<Double>();	
		yMax = new ArrayList<Double>();
		plotColor = new Color[maxTraces];

		toolTipText = "";
		textAlignment = left;		
		fontSize = medium;
		fontThickness = normal;
		textAlignment = left;
		headerColor = black;
		headerBackgroundColor = gray;
		color = black;
		backgroundColor = transparent;
		borderStyle = none;
		borderWidth = thin;
		borderColor = black;
		format = expformat;
		alarmThreshold= "0.0";
		lowColor = black;
		highColor = black;
		lowIcon = web2cdocs + directorySeparator + "redvalveicon.jpg";
		highIcon = web2cdocs + directorySeparator + "greenvalveicon.jpg";		
		image = web2cdocs + directorySeparator + "button.jpg";
		superComponent = no;
		expertComponent = no;
		keepHistory = true;	
		prompt = "chat";
		falseIcon = web2cdocs + directorySeparator + redIcon;
		minimumValue = "0.00";
		maximumValue = "10.00";		
		offset = " ";
		plotAxis = multi;
		plotType = linePlot;
		timeIntervalInSeconds = timeInterval3600;
		for (int i = 0; i < maxTraces; i++) {
			plotColor[i] = traceColor[i];
		}
		functionType = " ";
		functionParameter = " ";
		min = "0.0";
		plotOrder = normal;
		xmin = "";
		xmax = "";
		xLabel = " ";
		scrollBarEnabled = false;
		file = "";	
		extensionManifest = "";
		
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);		
			
		// id
			if (name.equals("id")) {
				id = referenceId + "_" + value;
				if (value.equals("enter Id") || value.equals("") || value.equals(" ")) {
					error = error + "[Component Decoder] Missing or wrong component ID;";
				}
			}
			
		// bounds, scale		
			if (name.equals("bounds")) {
				xLeft = (int)(Integer.valueOf(value.split(",")[0]) * xScale);
				yTop = (int)(Integer.valueOf(value.split(",")[1]) * yScale);
				xRight = (int)(Integer.valueOf(value.split(",")[2]) * xScale);
				yBottom = (int)(Integer.valueOf(value.split(",")[3]) * yScale);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}
			if (name.equals("scale")) {
				if (!value.equals("") && !value.equals(" ")) {
					xScale = Double.valueOf(value.split(",")[0]);
					yScale = Double.valueOf(value.split(",")[1]);
				}
			}
			
		// border, borderColor
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			
		//color, backgroundColor, headerColor, headerBackgroundColor
			if (name.equals("color")) {
				color = value;
			}		
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}			
			if (name.equals("headerColor")) {
				headerColor = value;
			}
			if (name.equals("headerBackgroundColor")) {
				headerBackgroundColor = value;
			}
			
		//fontSize, fontThickness			
			if (name.equals("fontSize")) {
				if (value.equals("tiny")) {
					fontSize = tiny;
				}
				else if (value.equals("small")) {
					fontSize = small;
				}
				else if (value.equals("medium")) {
					fontSize = medium;
				}
				else if (value.equals("large")) {
					fontSize = large;
				}
				else if (value.equals("extraLarge")) {
					fontSize = extraLarge;
				}
				else if (value.equals("huge")) {
					fontSize = huge;
				}
				else if (value.equals("extraHuge")) {
					fontSize = extraHuge;
				}
				else if (value.equals("gigantic")) {
					fontSize = gigantic;
				}
				else {
					fontSize = large;
				}
			}			
			if (name.equals("fontThickness")) {
				if (value.equals("normal")) {
					fontThickness = normal;
				}
				else {
					fontThickness = bold;
				}
			}
			
		// caption, headerCaption, alignment, scrollbar, title
			if (name.equals("caption")) {
				if (value.split(",").length > 1) {
					caption.add(getStringWithSpecialCharacters(value.split(",")[language]));
					implementedLanguages = value.split(",").length;
				}
				else {
					caption.add(getStringWithSpecialCharacters(value));
					implementedLanguages = 1;
				}
			}			
			if (name.equals("headerCaption")) {
				if (value.split(",").length > 1) {
					headerCaption = getStringWithSpecialCharacters(value.split(",")[language]);
				}
				else {
					headerCaption = getStringWithSpecialCharacters(value);
				}
			}	
			if (name.equals("alignment")) {
				textAlignment = value;
			}						
			if (name.equals("scrollBar")) {
				if (value.equals("yes")) {
					scrollBarEnabled = true;
				}
				else {
					scrollBarEnabled = false;
				}	
			}
			
		// sound, soundEnabled, blinking, blinkingEnabled
			if (name.equals("sound")) {
				soundString = value;
			}
			if (name.equals("soundEnabled")) {
				soundEnabledString = value;
			}
			if (name.equals("blinking")) {
				blinkingString = value;
			}
			if (name.equals("blinkingEnabled")) {
				blinkingEnabledString = value;
			}
			
		// plotAxis, plotType, plotOrder, plotColor, yMin, yMax, xMin, xMax, xLabel

			if (name.equals("plotAxis")) {
				plotAxis = value;
			}		
			if (name.equals("plotType")) {
				plotType = value;
			}		
			if (name.equals("plotOrder")) {
				plotOrder = value;
			}	
			if (name.equals("plotColor")) {
				for (int j = 0; j < value.split(",").length; j++) {
					if (value.split(",")[j].equals("red")) {plotColor[j] = red;}
					if (value.split(",")[j].equals("blue")) {plotColor[j] = blue;}
					if (value.split(",")[j].equals("magenta")) {plotColor[j] = magenta;}
					if (value.split(",")[j].equals("green")) {plotColor[j] = green;}
					if (value.split(",")[j].equals("darkRed")) {plotColor[j] = darkRed;}
					if (value.split(",")[j].equals("yellow")) {plotColor[j] = yellow;}
					if (value.split(",")[j].equals("orange")) {plotColor[j] = orange;}
					if (value.split(",")[j].equals("cyan")) {plotColor[j] = cyan;}
					if (value.split(",")[j].equals("darkGray")) {plotColor[j] = darkGray;}
					if (value.split(",")[j].equals("black")) {plotColor[j] = darkBlack;}
					if (value.split(",")[j].contains("_")) {
						plotColor[j] = new Color(Integer.valueOf(value.split(",")[j].split("_")[0]),Integer.valueOf(value.split(",")[j].split("_")[1]),Integer.valueOf(value.split(",")[j].split("_")[2]));
					}
				}
			}
			if (name.equals("yMin")) {
				yMin.add(Double.valueOf(value));
			}	
			if (name.equals("yMax")) {
				yMax.add(Double.valueOf(value));
			}	
			if (name.equals("xMin")) {
				if (value.equals("") || value.equals(" ")) {
					xmin = "";
				}
				else {
					xmin = value;
				}
			}	
			if (name.equals("xMax")) {
				if (value.equals("") || value.equals(" ")) {
					xmax = "";
				}
				else {
					xmax = value;
				}
			}		
			if (name.equals("xLabel")) {
				xLabel = getStringWithSpecialCharacters(value);
			}
			
		// trueValue, falseIcon, trueIcon
			if (name.equals("trueValue")) {
				trueValue = new String[value.split(",").length];
				trueValue = value.split(",");
			}	
			if (name.equals("falseIcon")) {
				falseIcon = userdocs + directorySeparator + value;
			}
			if (name.equals("trueIcon")) {
				trueIcon = new String[value.split(",").length];
				for (int j = 0; j < value.split(",").length; j++) {
					trueIcon[j] = userdocs + directorySeparator + value.split(",")[j];
				}
			}
			
		// alarmThreshold, lowIcon, highIcon, lowColor, highColor, format, minLimit, format, function
			if (name.equals("alarmThreshold")) {
				alarmThreshold = value;
			}
			if (name.equals("lowIcon")) {
				lowIcon = userdocs + directorySeparator + value;
			}	
			if (name.equals("highIcon")) {
				highIcon = userdocs + directorySeparator + value;
			}	
			if (name.equals("lowColor")) {
				lowColor = value;
			}	
			if (name.equals("highColor")) {
				highColor = value;
			}		
			if (name.equals("format")) {
				format = value;
			}
			if (name.equals("minLimit")) {
				min = value;
			}
			if (name.equals("format")) {
				format = value;
			}
			if (name.equals("function")) {
				if (!value.equals("") && !value.equals(" ")) {
					try {
						functionType = value.substring(0, value.indexOf("("));
						if (value.substring(value.indexOf("("), value.indexOf(")") + 1).length() != 2) {
							functionParameter = value.substring(value.indexOf("(") + 1, value.indexOf(")"));
						}
						else {
							functionParameter = "";
						}
					}
					catch (Exception e) {}
				}
			}
			
		//condition, uri, legend, extensionManifest
			if (name.equals("condition")) {
				if (!value.equals("") && !value.equals(" ")) condition.add(value);
			}
			if (name.equals("uri")) {
				if (value.contains("file://")) file = userdocs + directorySeparator + value.substring(7);
				else property.add(value);
			}	
			if (name.equals("legend")) {
				if (value.split(",").length > 1) {
					legend.add(getStringWithSpecialCharacters(value.split(",")[language]));
				}
				else {
					legend.add(getStringWithSpecialCharacters(value));
				}
			}
			if (name.equals("extensionManifest")) {
				if (!value.equals("") && !value.equals(" ")) {
					extensionManifest = value;
					Web2cExtensionManifestReader reader = new Web2cExtensionManifestReader(new File(homeDir + File.separator + extensionManifest));
					componentDataSize = reader.getComponentSize();
					extensionClassName = reader.getClassName();
					monitorURI = reader.getMonitors();
					receiverURI = reader.getReceivers();
				}
				else extensionManifest = "";
			}
			
		//minimum, maximum, offset, timeInterval, fieldLength
			if (name.equals("minimum")) {
				minimumValue = value;
			}	
			if (name.equals("maximum")) {
				maximumValue = value;
			}
			if (name.equals("offset")) {
				offset = value;
			}
			if (name.equals("timeInterval")) {
				timeIntervalInSeconds = value;
			}		
			if (name.equals("fieldLength")) {
				if (value.equals("") || value.equals(" ")) maxInputTextLength = 0;
				else maxInputTextLength = Integer.valueOf(value);
			}
			
		//keepHistory, superComponent, image, tooltipText, prompt
			if (name.equals("keepHistory")) {
				if (value.equals("true")) {
					keepHistory = true;
				}
				else {
					keepHistory = false;
				}
			}					
			if (name.equals("superComponent")) {
				superComponent = value;
			}					
			if (name.equals("expertComponent")) {
				expertComponent = value;
			}
			if (name.equals("image")) {
				image = userdocs + directorySeparator + value;
			}
			if (name.equals("tooltipText")) {
				if (value.split(",").length > 1) {
					toolTipText = getStringWithSpecialCharacters(value.split(",")[language]);
				}
				else {
					toolTipText = getStringWithSpecialCharacters(value);
				}
			}
			if (name.equals("prompt")) {
				prompt = value;
			}		
		}
		
		if (xLabel.equals("") || xLabel.equals(" ")) xLabel = "Index";
		
		if (yMin.isEmpty()) {
			yMin.add(0.0);
		}		
		if (yMax.isEmpty()) {
			yMax.add(0.0);
		}
		
		if(!extensionManifest.equals("")) {
			if (!monitorURI.isEmpty()) {
				registerExtensionMonitors();
				monitorData = new String[monitorURI.size()];
			}
			if (!receiverURI.isEmpty()) {
				registerExtensionReceivers();
				receiverData = new ByteArrayOutputStream[monitorURI.size()];
			}
			extensionExecutor = new Web2cExtensionExecutor(new File(homeDir + File.separator + extensionManifest), commonDeviceAccess, commonVideoConnector);
		}
	}
	
	/**
	 * Initializes acoustic and visual animation
	 * @param size component data size
	 */
	public void initSoundAndBlinking(int size) {

		soundFile = new String[size];
		soundEnabled = new boolean[size];
		blinkingColor = new String[size];
		blinkingEnabled = new boolean[size];
		blinkingColor1 = new String[size];
		blinkingColor2 = new String[size];
		blinkingColorUpdate = new String[size];
		text = new String[size];
		stateChangedToAlarmTrue = new boolean[size];
		stateChangedToAlarmFalse = new boolean[size];
		stateChangedToAlarmCounterTrue = new int[size];
		stateChangedToAlarmCounterFalse = new int[size];
		
		for (int j = 0; j < text.length; j++) {
			text[j] = "";
		}
		
		for (int j = 0; j < soundFile.length; j++) {
			if (soundString.equals("") || soundString.equals(" ")) {
				soundFile[j] = "none";
			}
			else {
				soundFile[j] = soundString;
			}
		}
		for (int j = 0; j < soundEnabled.length; j++) {
			if (soundEnabledString.equals("yes")) {
				soundEnabled[j] = true;
			}
			else {
				soundEnabled[j] = false;
			}
		}
		
		for (int j = 0; j < blinkingColor.length; j++) {
			if (blinkingString.equals("") || blinkingString.equals(" ")) {
				blinkingColor[j] = backgroundColor + "-" + backgroundColor;
			}
			else {
				blinkingColor[j] = blinkingString;
			}
			blinkingColor1[j] = backgroundColor;
			blinkingColor2[j] = backgroundColor;
			blinkingColorUpdate[j] = backgroundColor;
		}
		for (int j = 0; j < blinkingEnabled.length; j++) {
			if (blinkingEnabledString.equals("yes")) {
				blinkingEnabled[j] = true;
			}
			else {
				blinkingEnabled[j] = false;
			}
		}

		for (int i = 0; i < stateChangedToAlarmTrue.length; i++) {
			stateChangedToAlarmTrue[i] = false;
		}
		for (int i = 0; i < stateChangedToAlarmFalse.length; i++) {
			stateChangedToAlarmFalse[i] = true;
		}
		for (int i = 0; i < stateChangedToAlarmCounterTrue.length; i++) {
			stateChangedToAlarmCounterTrue[i] = 0;
		}
		for (int i = 0; i < stateChangedToAlarmCounterFalse.length; i++) {
			stateChangedToAlarmCounterFalse[i] = alarmPersistenceInterval;
		}
	}
	
	/**
	 * Initializes the property and legend list for the WebHistory component
	 */
	public void initLegendList() {
		
		if (extensionManifest.equals("")) {
			propertyList = "";
			for (Iterator<String> it = property.iterator(); it.hasNext();) {
				propertyList = propertyList + it.next() + ",";
			}
			if (propertyList.length() > 0) propertyList = propertyList.substring(0, propertyList.length() - 1);
			
			legendList = "";
			if (legend.isEmpty()) {
				legend = property;
				legendList = propertyList;
			}
			else {
				int i = -1;
				Iterator<String> it1 = legend.iterator();
				for (Iterator<String> it2 = property.iterator(); it2.hasNext();) {
					String nextLegend = (String)it1.next();
					String nextProperty = (String)it2.next();
					i = i + 1;
					if (nextLegend.equals("") || nextLegend.equals(" ")) legend.set(i, nextProperty);
					legendList = legendList + legend.get(i) + ",";
				}
				if (legendList.length() > 0) legendList = legendList.substring(0,legendList.length() - 1);
			}
		}
		else {
			propertyList = "";
			
			legendList = "";
			if (legend.isEmpty()) {
				for(int i = 0; i < componentDataSize; i++) {
					legend.add("Trace" + (i + 1));
				}
			}
			for(int i = 0; i < componentDataSize; i++) {
				legendList = legendList + legend.get(i) + ",";
			}
			if (legendList.length() > 0) legendList = legendList.substring(0,legendList.length() - 1);
		}
	}
	
	/**
	 * Registers all monitors
	 * @param flag condition links only
	 */
	public void registerMonitors(boolean conditionLinksOnly) {

		if (simulation == false) {
			String str = "";
			int i = 0;
			linkFlag = 0;
			for (Iterator<String> it = property.iterator();it.hasNext();) {
				String prop = (String)(it.next());
				i = i + 1;
				if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
					if (conditionLinksOnly == false) {
						int ret = 0;
						ret = commonDeviceAccess.registerMonitor(prop);
						error = error + commonDeviceAccess.getError();
						if (ret != 0) {
							linkFlag = linkFlag + 1;
						}
						else {
							monitorList.add(prop);
						}
					}
					String propertyDescription = commonDeviceAccess.getPropertyDescription(prop);
					if (!propertyDescription.equals("") && (toolTipText.equals("") || toolTipText.equals(" "))) str = str + "(" + i + ")" + propertyDescription + "; ";
				}
			}
			if (!str.equals("")) toolTipText = str.substring(0, str.length() - 2);
			if (conditionLinksOnly == false) {
				if (linkFlag == 0) {				
					propertyLinksEstablished = true;
				}
				else {			
					propertyLinksEstablished = false;
				}
			}
			
			linkFlag = 0;
			for (Iterator<String> it = condition.iterator();it.hasNext();) {
				String prop = ((String)(it.next())).split(" ")[0].split("=")[0];
				if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
					int ret = 0;
					ret = commonDeviceAccess.registerMonitor(prop);
					error = error + commonDeviceAccess.getError();
					if (ret != 0) {
						linkFlag = linkFlag + 1;
					}
					else {
						monitorList.add(prop);
					}
				}
			}
			if (linkFlag == 0) {				
				conditionLinksEstablished = true;
			}
			else {			
				conditionLinksEstablished = false;
			}
		}
	}
	
	/**
	 * Registers all monitors required to execute the extension code
	 */
	public void registerExtensionMonitors() {

		if (simulation == false) {
			int i = 0;
			linkFlag = 0;
			for (Iterator<String> it = monitorURI.iterator();it.hasNext();) {
				String prop = (String)(it.next());
				i = i + 1;
				if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
					int ret = 0;
					ret = commonDeviceAccess.registerMonitor(prop);
					error = error + commonDeviceAccess.getError();
					if (ret != 0) {
						linkFlag = linkFlag + 1;
					}
					else {
						monitorList.add(prop);
					}
				}
			}
			if (linkFlag == 0) {				
				extensionPropertyLinksEstablished = true;
			}
			else {			
				extensionPropertyLinksEstablished = false;
			}
		}
	}
	
	/**
	 * Registers all receivers
	 */
	public void registerReceivers() {
		
		if (simulation == false) {
			linkFlag = 0;
			commonVideoConnector.connectToVideoSource(property.get(0));
			receiverList.add(property.get(0));
			String description = commonVideoConnector.getDescription(property.get(0));
			if (!description.equals("") && (toolTipText.equals("") || toolTipText.equals(" "))) toolTipText = description;
			
			for (Iterator<String> it = condition.iterator();it.hasNext();) {
				String prop = ((String)(it.next())).split(" ")[0].split("=")[0];
				if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
					int ret = 0;
					ret = commonDeviceAccess.registerMonitor(prop);
					error = error + commonDeviceAccess.getError();
					if (ret != 0) {
						linkFlag = linkFlag + 1;
					}
					else {
						monitorList.add(prop);
					}
				}
			}
			if (linkFlag == 0) {				
				conditionLinksEstablished = true;
			}
			else {			
				conditionLinksEstablished = false;
			}
		}
	}
	
	/**
	 * Registers all receivers required to execute the extension code
	 */
	public void registerExtensionReceivers() {
		
		if (simulation == false) {
			for (Iterator<String> it = receiverURI.iterator();it.hasNext();) {
				String prop = (String)(it.next());
				if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
					commonVideoConnector.connectToVideoSource(prop);
					receiverList.add(prop);
				}
			}
		}
	}
	
	/**
	 * Returns the list of monitors attached to this component
	 * @return list of monitors
	 */
	public ArrayList<String> getMonitors() {
		
		return monitorList;
	}
	
	/**
	 * Returns the list of video receivers attached to this component
	 * @return list of receivers
	 */
	public ArrayList<String> getReceivers() {
		
		return receiverList;
	}

	/**
	 * Returns component's HTML tags 
	 * (Override, if applicable)
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		componentHTML.add("");
		
		return componentHTML;	
	}
	
	/**
	 * Returns component's JavaScript code snippets
	 * (Override, if applicable)
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("");

		return componentJavaScript;
	}
	
	/**
	 * Returns the error code of the class
	 * @return error
	 */
	public String getError() {
		
		return error;
	}
		
	/**
	 * Handles and returns data to update client page
	 * @return data (xml-type)
	 */
	public String getUpdate() {
		
		String update = "";
		
		if (simulation == true) {
			update = getSimulatedData();
		}
		else {
			update = getRealData();
		}
	
		return update;
	}
	
	/**
	 * Returns real data from common device access plug
	 * (Override, if applicable)
	 * @return data (xml-type)
	 */
	public String getRealData() {
	
		return "";
	}
	
	/**
	 * Returns simulated data
	 * (Override, if applicable)
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		return "";
	}
	
	/**
	 * Handles client action requests
	 * (Override, if applicable)
	 * @param elementAction action requested
	 * @param elementId ID of requesting HTML element
	 */
	public void doAction(String elementAction, String elementId) {
		
		if (elementId.equals(id)) {
			if (simulation == true) {
				doSimulatedAction(elementAction);
			}
			else {
				doRealAction(elementAction);
			}
		}
	}	
	
	/**
	 * Sends client action requests to common device access plug
	 * (Override, if applicable)
	 * @param action requested action
	 */	
	public void doRealAction(String action) {
		
	}
	
	/**
	 * Calls simulator
	 * (Override, if applicable)
	 * @param action requested action
	 */	
	public void doSimulatedAction(String action) {
		
	}
	
	/**
	 * Returns the global style string for the component
	 * @return style string (xml-type)
	 */
	public String getStyleUpdate() {
		
		String style = "";
		if (conditionLinksEstablished == true) {
			for (Iterator<String> it = condition.iterator(); it.hasNext();) {
				String nextCondition = (String)it.next();
				if (!nextCondition.equals("") && !nextCondition.equals(" ")) {
					String conditionProperty = nextCondition.split(" ")[0].split("=")[0];
					String conditionValue = nextCondition.split(" ")[0].split("=")[1];
					if (conditionValue.equals(characterEscaper.decodeString((commonDeviceAccess.getMonitorData(conditionProperty)).split(">")[1].split("<")[0]))) {
						for (int j = 1; j < nextCondition.split(" ").length; j++) {
							String tag = nextCondition.split(" ")[j].split("=")[0];
							String value = nextCondition.split(" ")[j].split("=")[1];
							if (tag.equals("color")) {style = style + "<style>" + "color," + value + "</style>";}
							if (tag.equals("backgroundColor")) {style = style + "<style>" + "background-color," + value + "</style>";}
							if (tag.equals("tooltipText")) {style = style + "<style>" + "title," + getStringWithSpecialCharacters(value) + "</style>";}
							if (tag.equals("fontSize")) {
								if (value.equals("tiny")) {
									fontSize = tiny;
								}
								else if (value.equals("small")) {
									fontSize = small;
								}
								else if (value.equals("medium")) {
									fontSize = medium;
								}
								else if (value.equals("large")) {
									fontSize = large;
								}
								else if (value.equals("extraLarge")) {
									fontSize = extraLarge;
								}
								else if (value.equals("huge")) {
									fontSize = huge;
								}
								else if (value.equals("extraHuge")) {
									fontSize = extraHuge;
								}
								else if (value.equals("gigantic")) {
									fontSize = gigantic;
								}
								else {
									fontSize = large;
								}
								style = style + "<style>" + "font-size," + value + "</style>";
							}
							if (tag.equals("fontThickness")) {
								if (value.equals("normal")) {
									fontThickness = normal;
								}
								else {
									fontThickness = bold;
								}
								style = style + "<style>" + "font-weight," + value + "</style>";
							}
							if (tag.equals("plotColor")) {
								for (int i = 0; i < value.split(",").length; i++) {
									if (value.split(",")[i].equals("red")) {plotColor[i] = red;}
									if (value.split(",")[i].equals("blue")) {plotColor[i] = blue;}
									if (value.split(",")[i].equals("magenta")) {plotColor[i] = magenta;}
									if (value.split(",")[i].equals("green")) {plotColor[i] = green;}
									if (value.split(",")[i].equals("darkRed")) {plotColor[i] = darkRed;}
									if (value.split(",")[i].equals("yellow")) {plotColor[i] = yellow;}
									if (value.split(",")[i].equals("orange")) {plotColor[i] = orange;}
									if (value.split(",")[i].equals("cyan")) {plotColor[i] = cyan;}
									if (value.split(",")[i].equals("darkGray")) {plotColor[i] = darkGray;}
									if (value.split(",")[i].equals("black")) {plotColor[i] = darkBlack;}
									if (value.split(",")[i].contains("_")) {
										plotColor[i] = new Color(Integer.valueOf(value.split(",")[i].split("_")[0]),Integer.valueOf(value.split(",")[i].split("_")[1]),Integer.valueOf(value.split(",")[i].split("_")[2]));
									}
								}						
							}
						}	
					}
				}
			}
		}
		
		return style;		
	}
	
	/**
	 * Returns the blinking style string for the sub-component associated with uri[index]
	 * @param index index
	 * @return blinking style string (xml-type)
	 */
	public String getBlinking(int index) {

		String blinking = "";
		String blinkingString = "";
		String blinkingEnabledString = "";
		int i = index;
		
		blinkingString = this.blinkingString;
		blinkingEnabledString = this.blinkingEnabledString;
		
		if (conditionLinksEstablished == true) {
			for (Iterator<String> it = condition.iterator(); it.hasNext();) {
				String nextCondition = (String)it.next();
				if (!nextCondition.equals("") && !nextCondition.equals(" ")) {
					String conditionProperty = nextCondition.split(" ")[0].split("=")[0];
					String conditionValue = nextCondition.split(" ")[0].split("=")[1];
					if (conditionValue.equals(characterEscaper.decodeString((commonDeviceAccess.getMonitorData(conditionProperty)).split(">")[1].split("<")[0]))) {
						for (int j = 1; j < nextCondition.split(" ").length; j++) {
							String tag = nextCondition.split(" ")[j].split("=")[0];
							String value = nextCondition.split(" ")[j].split("=")[1];
							if (tag.equals("blinking")) {
								blinkingString = value;
							}
							if (tag.equals("blinkingEnabled")) {
								blinkingEnabledString = value;
							}
						}
					}
				}
			}
		}
		
		for (int j = 0; j < blinkingColor.length; j++) {
			if (blinkingString.equals("") || blinkingString.equals(" ")) {
				blinkingColor[j] = backgroundColor + "-" + backgroundColor;
			}
			else {
				blinkingColor[j] = blinkingString;
			}
		}
		for (int j = 0; j < blinkingEnabled.length; j++) {
			if (blinkingEnabledString.equals("yes")) {
				blinkingEnabled[j] = true;
			}
			else {
				blinkingEnabled[j] = false;
			}
		}
	
		blinking = "<style>" + "background-color," + backgroundColor + "</style>";
		if (blinkingEnabled[i] == true) {
			blinkingColor1[i] = blinkingColor[i].split("-")[0];
			blinkingColor2[i] = blinkingColor[i].split("-")[1];
			if (blinkingColorUpdate[i].equals(blinkingColor1[i])) {
				blinkingColorUpdate[i] = blinkingColor2[i];
			}
			else {
				blinkingColorUpdate[i] = blinkingColor1[i];
			}			
			blinking = "<style>" + "background-color," + blinkingColorUpdate[i] + "</style>";
		}
		
		return blinking;
	}
	
	/**
	 * Returns the sound style string for the sub-component associated with uri[index]
	 * @param index index
	 * @return sound style string (xml-type)
	 */
	public String getSound(int index) {
		
		String sound = "";
		String soundString = "";
		String soundEnabledString = "";
		int i = index;
		
		soundString = this.soundString;
		soundEnabledString = this.soundEnabledString;
		
		if (conditionLinksEstablished == true) {
			for (Iterator<String> it = condition.iterator(); it.hasNext();) {
				String nextCondition = (String)it.next();
				if (!nextCondition.equals("") && !nextCondition.equals(" ")) {
					String conditionProperty = nextCondition.split(" ")[0].split("=")[0];
					String conditionValue = nextCondition.split(" ")[0].split("=")[1];
					if (conditionValue.equals(characterEscaper.decodeString((commonDeviceAccess.getMonitorData(conditionProperty)).split(">")[1].split("<")[0]))) {
						for (int j = 1; j < nextCondition.split(" ").length; j++) {
							String tag = nextCondition.split(" ")[j].split("=")[0];
							String value = nextCondition.split(" ")[j].split("=")[1];
							if (tag.equals("sound")) {
								soundString = value;
							}
							if (tag.equals("soundEnabled")) {
								soundEnabledString = value;
							}
						}
					}
				}
			}
		}
		
		for (int j = 0; j < soundFile.length; j++) {
			if (soundString.equals("") || soundString.equals(" ")) {
				soundFile[j] = "none";
			}
			else {
				soundFile[j] = soundString;
			}
		}
		for (int j = 0; j < soundEnabled.length; j++) {
			if (soundEnabledString.equals("yes")) {
				soundEnabled[j] = true;
			}
			else {
				soundEnabled[j] = false;
			}
		}
	
		sound = "<style>" + "sound,none" + "</style>";
		if (soundEnabled[i] == true) {
			sound = "<style>" + "sound," + soundFile[i] + "</style>";
		}
		
		return sound;
	}
	
	/**
	 * Returns the actual sound and blinking settings; latches an alarm condition (used for sound- or blinking-based notification) for a specific time interval ( == soundGenerationInterval)
	 * @param flag condition flag
	 * @param index index
	 * @return soundAndBlinking string
	 */
	public String getConditionedSoundAndBlinkingUpdate(boolean flag, int index) {
		
		String sound = "";
		String blinking = "";
		String soundAndBlinking = "";

		sound = getSound(index);
		blinking = getBlinking(index);
		
		if (flag == true) {
			stateChangedToAlarmFalse[index] = false;
			stateChangedToAlarmCounterFalse[index] = 0;
			if (stateChangedToAlarmTrue[index] == false) {
				if (stateChangedToAlarmCounterTrue[index] == alarmPersistenceInterval) stateChangedToAlarmTrue[index] = true;
				stateChangedToAlarmCounterTrue[index] = stateChangedToAlarmCounterTrue[index] + 1;
			}
			else {
				sound = "<style>" + "sound,none" + "</style>";
			}
		}
		else {
			stateChangedToAlarmTrue[index] = false;
			stateChangedToAlarmCounterTrue[index] = 0;
			if (stateChangedToAlarmFalse[index] == false) {
				if (stateChangedToAlarmCounterFalse[index] == alarmPersistenceInterval) stateChangedToAlarmFalse[index] = true;
				stateChangedToAlarmCounterFalse[index] = stateChangedToAlarmCounterFalse[index] + 1;
				sound = "<style>" + "sound,none" + "</style>";
			}
			else {
				blinking = "<style>" + "background-color," + backgroundColor + "</style>";
				sound = "<style>" + "sound,none" + "</style>";
			}
		}
		
		soundAndBlinking = sound + blinking;
		
		return soundAndBlinking;
	}
	
	/**
	 * Handles strings with special characters
	 * @param value string
	 * @return converted string
	 */
	private String getStringWithSpecialCharacters(String value) {

		String str = null;
		if (value.contains("ASCIIC")) {
			if ((value.substring(0,7)).contains("ASCIIC(")) {
				str = "";
			}
			else {
				str = value.split("ASCIIC")[0];
			}
			for (int j = 1; j < (value.split("ASCIIC")).length; j++) {
				String string = value.split("ASCIIC")[j];
				str = str + String.valueOf((char)((int)(Integer.valueOf(string.substring(string.indexOf("(") + 1, string.indexOf(")")))))) + string.substring(string.indexOf(")") + 1);	
			}
		}
		else {
			str = value;
		}
		
		return str;
	}
}
