package de.desy.web2cToolkit.web2c.web2cComponents;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cExtension.Web2cExtensionExecutor;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * History
 * @author R.Bacher, DESY
 */
public class Web2cHistory implements IWeb2cConstants{
	
	private String width = null;
	private String height = null;
	private String[] property = null;
	private String[] legend = null;
	private String timeIntervalInSeconds = null;
	private String fromInSeconds = null;
	private String toInSeconds= null;
	private float[][] dataValues = null;
	private long[][] dataTimeStamps = null;
	private int numberOfDataTraces = 0;
	private String[] dataAddressName = null;
	private String title = null;
	private int[] dataArrayCompletionLength = null;	
	private Web2cCommonDeviceAccess cda = null;
	private Web2cCommonVideoConnector cvc = null;
	private String web2cSession = null;
	private String extensionManifest = "";
	
	private boolean simulation = true;
	private Random randomGenerator = null;
	private String error = "";
	private String homeDir = "";
	private Web2cASCIICharacterEscaper characterEscaper = null;
	
	/**
	 * History Plot
	 * @param session Web2c session id
	 * @param home directory which contains user-specific files
	 * @param interval time interval of history plot [s]
	 * @param histWidth width of history plot window [px]
	 * @param histHeight height of history plot window [px]
	 * @param simulatedData flag for simulated or real data
	 * @param commonDeviceAccess actual common device access instance
	 * @param commonVideoConnector actual common video connector instance
	 */
	public Web2cHistory(String session, String home, String interval, String histWidth, String histHeight, boolean simulatedData, Web2cCommonDeviceAccess commonDeviceAccess, Web2cCommonVideoConnector commonVideoConnector) {
		
		web2cSession = session;
		homeDir = home;
		timeIntervalInSeconds = interval;
		width = histWidth;
		height = histHeight;
		simulation = simulatedData;
		randomGenerator = new Random();
		cda = commonDeviceAccess;
		cvc = commonVideoConnector;

		characterEscaper = new Web2cASCIICharacterEscaper();
	}
	
	/**
	 * Returns the page
	 * @param manifest extension manifest
	 * @param propertyList list of control system addresses
	 * @param legendList list of descriptions of control system addresses, used for plot legend
	 * @return list
	 */
	public ArrayList<String> getPage(String manifest, String propertyList, String legendList) {
		
		Calendar calendar = null;
		int now = 0;
		int backButtonWidth = 30;
		int backButtonHeight = 30;
		int backButtonXPos = getChartWidth() + 20;
		int backButtonYPos = 0;
		
		ArrayList<String> page = new ArrayList<String>();
		
		calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		toInSeconds = String.valueOf(now);	

		page.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		page.add("<html>");
		page.add("	<head>");
		page.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		page.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		page.add("      <meta http-equiv=\"content-script-type\" content=\"text/javascript\">");
		page.add("		<title>History Plot</title>");
		page.add("	    <script type=\"text/javascript\">");
		page.add("			function backButton_click() {");
		page.add("				document.clear();");  
		page.add("				window.close();");  
		page.add("			};");  
		page.add("	    </script>");
		page.add("	</head>");
		page.add("	<body>");
		page.add("		<img id=\"history\" width=\"" + width + "\" height=\"" + height + "\" alt=\"Image temporarily unavailable\" title=\"" + "Dies ist ein History Chart" + "\" src=\"Web2c?param=<hplt><sessionId>" + web2cSession + "</sessionId><value>" + manifest + "</value><value>" + propertyList + "</value><value>" + legendList + "</value><timeInterval>" + toInSeconds  + "</timeInterval></hplt>\">");
		page.add("      <input type=\"button\" id=\"backButton\" value=\"X\" title=\"\" onclick=\"backButton_click()\" style=\"position:absolute; left:" + String.valueOf(backButtonXPos) + "px" + "; top:" + String.valueOf(backButtonYPos) + "px" + "; width:" + String.valueOf(backButtonWidth) + "px" + "; height:" + String.valueOf(backButtonHeight) + "px" + "; border-style:outset; color:black; background-color:white; font-size:18px; font-weight:bold\">");
		page.add("	</body>");
		page.add("</html>");
	
		return page;
	}
	
	/**
	 * Generate simulated data
	 */
	public void getSimulatedData() {

		Calendar calendar = null;
		int now = 0;
		int from = 0;
		int i = 0;
		int j = 0;
		int maxLength = 0;

		calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		fromInSeconds = String.valueOf(now - Long.valueOf(timeIntervalInSeconds));
		toInSeconds = String.valueOf(now);		
		from = Integer.valueOf(fromInSeconds);
	
		numberOfDataTraces = property.length;
		dataArrayCompletionLength = new int[numberOfDataTraces];
		for (i = 0; i < numberOfDataTraces; i++) {
			dataArrayCompletionLength[i] = 100;
			if (dataArrayCompletionLength[i] > maxLength) {
				maxLength = dataArrayCompletionLength[i];
			}
		}
		dataAddressName = new String[numberOfDataTraces];
		dataTimeStamps = new long[numberOfDataTraces][maxLength];
		dataValues = new float[numberOfDataTraces][maxLength];

		for (j = 0; j < numberOfDataTraces; j++) {
			for (i = 0; i < dataArrayCompletionLength[j]; i++) {
				dataTimeStamps[j][i] = (i + from) * 1000;
			}
			for (i = 0; i < dataArrayCompletionLength[j]; i++) {
				dataValues[j][i] = randomGenerator.nextInt((j + 1) * 100);
			}
			dataAddressName[j] = "Simulated Data " + String.valueOf(j);
		}
		title = "Simulation";
	}	
	
	/**
	 * Read real data
	 */
	public void getRealData() {

		Calendar calendar = null;
		int now = 0;
		int i = 0;
		int j = 0;
		String[] ret = null;
		int maxLength = 0;
		
		calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		fromInSeconds = String.valueOf(now - Long.valueOf(timeIntervalInSeconds));
		toInSeconds = String.valueOf(now);	

		if (extensionManifest.equals("")) {
			ret = new String[property.length];
			for (i = 0; i < property.length; i++) {
				ret[i] = cda.getHistory(property[i]);
			}
		}
		else {
			Web2cExtensionExecutor extensionExecutor = new Web2cExtensionExecutor(new File(homeDir + File.separator + extensionManifest), cda, cvc);
			extensionExecutor.executeExtensionCode("history");
			ret = (String[])extensionExecutor.getData();
		}

		numberOfDataTraces = ret.length;
		dataArrayCompletionLength = new int[numberOfDataTraces];
		for (i = 0; i < numberOfDataTraces; i++) {
			dataArrayCompletionLength[i] = (ret[i].split("</timeStamp>")).length;
			if (dataArrayCompletionLength[i] > maxLength) {
				maxLength = dataArrayCompletionLength[i];
			}
		}
		dataAddressName = new String[numberOfDataTraces];
		dataTimeStamps = new long[numberOfDataTraces][maxLength];
		dataValues = new float[numberOfDataTraces][maxLength];
		
		for (j = 0; j < numberOfDataTraces; j++) {
			for (i = 0; i < dataArrayCompletionLength[j]; i++) {
				dataTimeStamps[j][i] = Long.valueOf(((ret[j].split("</timeStamp>")[i]).split(">")[3]));
			}
			for (i = 0; i < dataArrayCompletionLength[j]; i++) {
				dataValues[j][i] = Float.valueOf(characterEscaper.decodeString(((ret[j].split("</timeStamp>")[i]).split(">")[1]).split("<")[0]));
			}
			dataAddressName[j] = legend[j];
		}
		title = "History";
	}
	
	/**
	 * Returns chart width [px]
	 * @return width of chart
	 */
	public int getChartWidth() {
		
		return Integer.valueOf(width.substring(0,width.length() - 2));		
	}
	
	/**
	 * Returns chart height [px]
	 * @return height of chart
	 */
	public int getChartHeight() {
		
		return Integer.valueOf(height.substring(0,height.length() - 2));		
	}
	
	/**
	 * Returns chart
	 * @param manifest extension manifest
	 * @param propertyList list of control system addresses
	 * @param legendList list of descriptions of control system addresses, used for plot legend
	 * @return chart
	 */
	public JFreeChart getChart(String manifest, String propertyList, String legendList) {
		
		int i = 0;
		JFreeChart chart = null;
		
		extensionManifest = manifest;
		property = propertyList.split(",");	
		legend = legendList.split(",");
		
		if (simulation == true) {
			getSimulatedData();
		}
		else {
			getRealData();
		}
		
		if (propertyList != null) {
			try {		
				chart = ChartFactory.createTimeSeriesChart(title , "Date", "", null, true, true, false);
				XYPlot plot = chart.getXYPlot();	
				plot.setForegroundAlpha(1.0f);
				ValueAxis domainAxis = plot.getDomainAxis();
				domainAxis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(small.substring(0, small.length() - 2))));
				domainAxis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(small.substring(0, small.length() - 2))));
				plot.setDomainAxis(domainAxis);
				NumberAxis axis = new NumberAxis("");
				axis.setLabelPaint(Color.BLACK);
				axis.setTickLabelPaint(Color.BLACK);	
				axis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(small.substring(0, small.length() - 2))));	
				axis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(small.substring(0, small.length() - 2))));
				plot.setRangeAxis(0,axis);
				for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
					plot.setRangeAxisLocation(i, AxisLocation.BOTTOM_OR_LEFT);
					XYDataset dataset = createDataset(i);
					plot.setDataset(i, dataset);
					StandardXYItemRenderer renderer = new StandardXYItemRenderer();
					renderer.setSeriesPaint(0, traceColor[i]);
					plot.setRenderer(i, renderer);
				}
			}
			catch (Exception e) {
//				System.out.println(e.toString());
				System.out.println("Error generating chart!");
			}
		}
		
		return chart;
	}
	
	/**
	 * Returns the error code of the class
	 * @return error
	 */
	public String getError() {
		
		return error;
	}
	
	/**
	 * Returns the data set to plot (values as function of time)
	 * @param i data set index
	 * @return data
	 */
	private XYDataset createDataset(int i) {
		
		int j = 0;
		Calendar calendar = null;
		
		TimeSeries series = new TimeSeries(legend[i], Millisecond.class);	
		calendar = Calendar.getInstance();	

		for (j = 0; j < dataArrayCompletionLength[i]; j++) {
			if (dataTimeStamps[i][j] != 0) {
				calendar.setTimeInMillis((long)(dataTimeStamps[i][j]));
				series.addOrUpdate(new Millisecond(calendar.getTime()),(double)(dataValues[i][j]));
			}
		}
		
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		dataset.addSeries(series);
		
		return dataset;
	}
}
