package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Static Image
 * @author R.Bacher, DESY
 */
public class Web2cImage extends Web2cComponent {
	
	/**
	 * Static Image
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cImage(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			componentHTML.add("			<img id=\"" + id + "\" src=\"" + file + "\" title=\"" + toolTipText + "\" alt=\"Image temporarily unavailable\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			componentJavaScript.add("	//Image: " + id);
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + file + "</value>";
			
		str = "<update>" + identifier + value + "</update>";
		
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String style = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + file + "</value>";
		style = getStyleUpdate();
			
		str = "<update>" + identifier + value + style + "</update>";
		
		return str;
	}
}
