package de.desy.web2cToolkit.web2c.web2cComponents;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import org.jfree.chart.JFreeChart;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatHandler;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cHisto;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cHistory;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cSection;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cTrend;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cConfigReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * Page
 * @author R.Bacher, DESY
 */
public class Web2cPage implements IWeb2cConstants, Observer {
	
	protected ArrayList<ArrayList<String>> sectionsHTML = new ArrayList<ArrayList<String>>();
	protected ArrayList<ArrayList<String>> sectionsJavaScript = new ArrayList<ArrayList<String>>();	
	protected ArrayList<Web2cSection> sections = new ArrayList<Web2cSection>();
	protected ArrayList<Web2cEmbeddedPage> embeddedPages = new ArrayList<Web2cEmbeddedPage>();
	protected String hostURL = null;
	protected String homeDir = null;
	protected String webDir = null;
	protected int language = 0;
	protected Web2cPage rootPage = null;
	private String title = null;
	private String updateTime = null;
	private String message = null;
	protected double xScale = 0.0;
	protected double yScale = 0.0;
	protected String referencePoint = null;
	protected String referenceId = null;
	private Web2cHistory history = null;
	protected String userName = null;
	private String userAddress = null;
	private boolean reconnect = false;
	private String accessMode = null;
	private ArrayList<String> expertName = null;
	private ArrayList<String> supervisorName = null;
	private ArrayList<String> expertAddress = null;
	private ArrayList<String> exclusiveName = null;
	protected String userPrivilege = null;
	private String networkPrivilege = null;
	protected Web2cCommonDeviceAccess commonDeviceAccess = null;
	protected Web2cCommonVideoConnector commonVideoConnector = null;
	protected Web2cChatChannel chatChannel = null;
	private String chatChannelId = null;
	private Web2cChatHandler chatHandler = null;
	protected Web2cFileLogger debugLogger = null;
	protected Web2cKeyBoardListener debugListener = null;
	protected String web2cSession = null;
	protected String pageConfigFile = null;
	protected boolean soundEnabled = false;
	
	protected boolean simulation = true;
	protected String error = "";
	protected boolean debug = false;
	protected boolean exclusive = false;
	private PageUpdater pageUpdater = null;
	private String pageUpdate = "";
	private volatile Double[] constructionProgress = new Double[1];
	private double scale = 1.0;
	private double scalePowerFactor = 10.0;
	private Web2cASCIICharacterEscaper characterEscaper = null;

	/**
	 * Page
	 * @param url URL of host servicing the request
	 * @param session Web2c session id
	 * @param mode access mode
	 * @param home directory which contains user-specific files
	 * @param web directory which contains Web2cToolkit-specific files 
	 * @param configFile actual configuration file
	 * @param lang language
	 * @param name actual user name
	 * @param ipAddress actual IP address of user
	 * @param simulatedData flag for simulated or real data
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat handler instance
	 * @param logger debug logger
	 * @param listener debug flag listener
	 * @param progress construction progress
	 */
	public Web2cPage(String url, String session, String mode, String home, String web, String configFile, int lang, String name, String ipAddress, boolean simulatedData, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatHandler ch, Web2cFileLogger logger, Web2cKeyBoardListener listener, Double[] progress) {
		
		hostURL = url;
		web2cSession = session;	
		accessMode = mode;
		homeDir = home;
		webDir = web;
		language = lang;
		rootPage = this;
		referenceId = "mainPage";
		referencePoint = "0,0";
		userPrivilege = layman;
		networkPrivilege = layman;
		userName = "guest";
		userAddress = "";
		if (name == null) {
			userName = "guest";
		}
		else {
			userName = name;
		}
		if (ipAddress == null) {
			userAddress = "";
		}
		else {
			userAddress = ipAddress;
		}
		simulation = simulatedData;		
		debugLogger = logger;
		debugListener = listener;
		commonDeviceAccess = cda;
		commonVideoConnector = cvc;
		chatHandler = ch;
		constructionProgress = progress;
		pageConfigFile = configFile;
		chatChannelId = configFile;
		chatChannel = chatHandler.getChatChannel(chatChannelId);
		constructPage();
		getComponents();
		history = new Web2cHistory(web2cSession, homeDir, historyInterval, historyWidth, historyHeight, simulation, cda, cvc);
		characterEscaper = new Web2cASCIICharacterEscaper();
		pageUpdater = new PageUpdater(this, getUpdateTime());
		pageUpdater.addObserver(this);
	}
	
	/**
	 * Returns the session id of the page
	 * @return session id
	 */
	public String getSessionId() {
		
		return web2cSession;
	}
	
	/**
	 * Returns data to update client page
	 * @return data (xml-type)
	 */
	public String getUpdate() {
		
		return pageUpdate;
	}
	
	/**
	 * Returns page update time [ms]
	 * @return update time
	 */
	public int getUpdateTime() {
		
		return Integer.valueOf(updateTime);
	}
	
	/**
	 * Terminates page updating
	 */
	public void stopUpdate() {
		
		pageUpdater.deleteObserver(this);
		pageUpdater.stopUpdate();
	}
	
	/**
	 * Returns the updater status
	 * @return updater status
	 */
	public boolean getUpdaterStatus() {
		
		return pageUpdater.getUpdaterStatus();
	}
	
	/**
	 * Gets the update from the associated observables (pageUpdater)
	 * @param observable associated observable
	 * @param object update
	 */
	public void update(Observable observable, Object object) {
		
		if (observable.equals(pageUpdater)) pageUpdate = (String)object;
	}

	/**
	 * Returns the control system monitors attached to this page
	 * @return list of monitors
	 */
	public ArrayList<String> getMonitors() {
		
		ArrayList<String> monitorList = new ArrayList<String>();

		for (Iterator<Web2cComponent> it = getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			ArrayList<String> monitors = web2cComponent.getMonitors();
			if (!monitors.isEmpty()) monitorList.addAll(monitors);
		}
		
		return monitorList;
	}
	
	/**
	 * Returns the video receivers attached to this page
	 * @return list of receivers
	 */
	public ArrayList<String> getVideoReceivers() {
		
		ArrayList<String> receiverList = new ArrayList<String>();

		for (Iterator<Web2cComponent> it = getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			ArrayList<String> receivers = web2cComponent.getReceivers();
			if (!receivers.isEmpty()) receiverList.addAll(receivers);
		}
		
		return receiverList;
	}
	
	/**
	 * Returns common device access instance attached to this page 
	 * @return common device access
	 */
	public Web2cCommonDeviceAccess getCommonDeviceAccess() {
		
		return commonDeviceAccess;
	}
	
	/**
	 * Returns video connector instance attached to this page 
	 * @return video connector
	 */
	public Web2cCommonVideoConnector getCommonVideoConnector() {
		
		return commonVideoConnector;
	}
	
	/**
	 * Returns the id of the chat channel instance attached to this page 
	 * @return id
	 */
	public String getChatChannelId() {
		
		return chatChannelId;
	}
	
	/**
	 * Returns chat handler instance attached to this page 
	 * @return chat handler
	 */
	public Web2cChatHandler getChatHandler() {
		
		return chatHandler;
	}
	
	/**
	 * Returns chart 
	 * @param id component id
	 * @return chart
	 */
	public JFreeChart getChart(String id) {
		
		JFreeChart chart = null;

		for (Iterator<Web2cComponent> it = this.getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cTrend")) {
				if (id.equals(web2cComponent.getID())) {
					chart = ((Web2cTrend)web2cComponent).getChart();
				}
			}
			else if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cHisto")) {
				if (id.equals(web2cComponent.getID())) {
					chart = ((Web2cHisto)web2cComponent).getChart();
				}
			}
		}
		
		return chart;
	}
	
	/**
	 * Returns chart width [px]
	 * @param id component id
	 * @return width of chart
	 */
	public int getChartWidth(String id) {
		
		int chartWidth = 0;
		
		for (Iterator<Web2cComponent> it = this.getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cTrend")) {
				if (id.equals(web2cComponent.getID())) {
					chartWidth = ((Web2cTrend)web2cComponent).getChartWidth();
				}
			}
			else if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cHisto")) {
				if (id.equals(web2cComponent.getID())) {
					chartWidth = ((Web2cHisto)web2cComponent).getChartWidth();
				}
			}
		}
		
		return chartWidth;
	}
	
	/**
	 * Returns chart height [px]
	 * @param id component id
	 * @return height of chart
	 */
	public int getChartHeight(String id) {
		
		int chartHeight = 0;
		
		for (Iterator<Web2cComponent> it = this.getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cTrend")) {
				if (id.equals(web2cComponent.getID())) {
					chartHeight = ((Web2cTrend)web2cComponent).getChartHeight();
				}
			}
			else if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cHisto")) {
				if (id.equals(web2cComponent.getID())) {
					chartHeight = ((Web2cHisto)web2cComponent).getChartHeight();
				}
			}
		}
		
		return chartHeight;
	}
	
	/**
 	 * Assigns the video frame to the output stream
 	 * @param frame output stream
	 */
	public void getVideoImage(String id, ByteArrayOutputStream frame) {

		for (Iterator<Web2cComponent> it = this.getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cVideo")) {
				if (id.equals(web2cComponent.getID())) {
					((Web2cVideo)web2cComponent).getVideoFrame(frame);
				}
			}
		}
	}
	
	/**
	 * Returns list of components on page 
	 * @return list
	 */
	public ArrayList<Web2cComponent> getComponents() {
		
		ArrayList<Web2cComponent> list = new ArrayList<Web2cComponent>();
		for (Iterator<Web2cSection> it1 = sections.iterator(); it1.hasNext();) {
			Web2cSection section = (Web2cSection)it1.next();
			list.add((Web2cComponent)section);
			for (Iterator<Web2cComponent> it2 = (section.getComponents()).iterator(); it2.hasNext();) {
				list.add(it2.next());
			}
		}
		for (Iterator<Web2cEmbeddedPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			for (Iterator<Web2cComponent> it2 = (((Web2cEmbeddedPage)it1.next()).getComponents()).iterator(); it2.hasNext();) {
				list.add(it2.next());
			}
		}
		
		return list;
	}
	
	/**
	 * Returns page's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> pageHTML = new ArrayList<String>();	
		
		for(Iterator<ArrayList<String>> it1 = sectionsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				String html = (String)it2.next();
				pageHTML.add(html);
			}
		}
		
		for(Iterator<Web2cEmbeddedPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			ArrayList<String> embeddedPageHTML = ((Web2cEmbeddedPage)it1.next()).getHTML();
			for(Iterator<String> it2 = embeddedPageHTML.iterator(); it2.hasNext();) {
				String html = (String)it2.next();
				pageHTML.add(html);
			}
		}

		return pageHTML;
	}


	/**
	 * Returns page's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {	
		
		ArrayList<String> pageJavaScript = new ArrayList<String>();	

		pageJavaScript.add("	//Page: " + referenceId);
		
		pageJavaScript.add("		var xhr;"); 
		pageJavaScript.add("		var updateTimer;");
		pageJavaScript.add("		var searchForServerTimer;");
		pageJavaScript.add("		var sessionId = " + web2cSession + ";");
		pageJavaScript.add("		var platform = navigator.platform;");
		pageJavaScript.add("		var fontScale = " + scale + ";");
		
		pageJavaScript.add("		function initPage() {");
		pageJavaScript.add("			handleBrowserSpecifica();");
		pageJavaScript.add("			setTimeout(startUpdate, 2000);");
		pageJavaScript.add("		}");
		
		pageJavaScript.add("		function handleBrowserSpecifica() {");
		pageJavaScript.add("			var browser = navigator.userAgent;");
		pageJavaScript.add("			if (browser.indexOf(\"MSIE\") >= 0) {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"DIV\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (browser.indexOf(\"Firefox\") >= 0) {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"IMG\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) - (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) - (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (platform.indexOf(\"Linux\") >= 0) {");
		pageJavaScript.add("				var fontScaleLinux = 0.75;");
		pageJavaScript.add("				fontScale = fontScale * fontScaleLinux;");
		pageJavaScript.add("			}");
		pageJavaScript.add("			var element = document.getElementsByTagName(\"DIV\");");
		pageJavaScript.add("			for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("				if (element[i].style.fontSize.indexOf(\"p\") >= 0) {");
		pageJavaScript.add("					var fontsizeandunits = element[i].style.fontSize.split(\"p\");");
		pageJavaScript.add("					var fontsize = fontsizeandunits[0];");
		pageJavaScript.add("					element[i].style.fontSize = String(parseInt(fontsize) * fontScale) + \"px\";");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			var element = document.getElementsByTagName(\"TEXTAREA\");");
		pageJavaScript.add("			for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("				if (element[i].style.fontSize.indexOf(\"p\") >= 0) {");
		pageJavaScript.add("					var fontsizeandunits = element[i].style.fontSize.split(\"p\");");
		pageJavaScript.add("					var fontsize = fontsizeandunits[0];");
		pageJavaScript.add("					element[i].style.fontSize = String(parseInt(fontsize) * fontScale) + \"px\";");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			var element = document.getElementsByTagName(\"BUTTON\");");
		pageJavaScript.add("			for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("				if (element[i].style.fontSize.indexOf(\"p\") >= 0) {");
		pageJavaScript.add("					var fontsizeandunits = element[i].style.fontSize.split(\"p\");");
		pageJavaScript.add("					var fontsize = fontsizeandunits[0];");
		pageJavaScript.add("					element[i].style.fontSize = String(parseInt(fontsize) * fontScale) + \"px\";");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			var element = document.getElementsByTagName(\"INPUT\");");
		pageJavaScript.add("			for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("				if (element[i].style.fontSize.indexOf(\"p\") >= 0) {");
		pageJavaScript.add("					var fontsizeandunits = element[i].style.fontSize.split(\"p\");");
		pageJavaScript.add("					var fontsize = fontsizeandunits[0];");
		pageJavaScript.add("					element[i].style.fontSize = String(parseInt(fontsize) * fontScale) + \"px\";");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("		}");
		
		pageJavaScript.add("		function startUpdate() {");
		pageJavaScript.add("			initXhr();");
		if (Integer.valueOf(updateTime) > 0 ) {
			pageJavaScript.add("			var last = new Date();"); 
			pageJavaScript.add("			lastUpdate = last.getTime();"); 
//			pageJavaScript.add("			getServerData();");	
			pageJavaScript.add("			updateTimer = setInterval(\"getServerData()\"," + updateTime + ");");
		}
		pageJavaScript.add("		}");
		
		pageJavaScript.add("		function initXhr() {"); 
		pageJavaScript.add("			try {");                                                  
		pageJavaScript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		pageJavaScript.add("			} catch (event1) {");                             
		pageJavaScript.add("				try {");
		pageJavaScript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		pageJavaScript.add("				} catch (event2) {");
		pageJavaScript.add("					xhr = false;");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		pageJavaScript.add("				xhr = new XMLHttpRequest();");                 
		pageJavaScript.add("			}"); 																	
		pageJavaScript.add("		}");  
		
		pageJavaScript.add("		function getServerData() {"); 
		pageJavaScript.add("			var now = new Date();");
		pageJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2c?param=<update><sessionId>\" + sessionId + \"</sessionId><timeStamp>\" + String(now.getTime()) + \"</timeStamp></update>\", true);"); 
		pageJavaScript.add("			xhr.onreadystatechange = updatePage;");   
		pageJavaScript.add("			xhr.send(null);");          
		pageJavaScript.add("		}");  
		
		pageJavaScript.add("		function playSound(soundUrl) {");	 
		if (soundEnabled == true) {    
			pageJavaScript.add("			var parentElement = document.getElementById(\"" + referenceId + "_sound\");");    
			pageJavaScript.add("			try {");       
			pageJavaScript.add("				var oldElement = document.getElementById(\"" + referenceId + "_sound_player\");");  
			pageJavaScript.add("				parentElement.removeChild(oldElement);");  
			pageJavaScript.add("			} catch (event) {}");
			pageJavaScript.add("			var newElement = document.createElement(\"embed\");");  
			pageJavaScript.add("			newElement.id = \"" + referenceId + "_sound_player\";");          
			pageJavaScript.add("			newElement.style.width = \"1px\";");          
			pageJavaScript.add("			newElement.style.height = \"1px\";");         
			pageJavaScript.add("			newElement.hidden = \"true\";");                   
			pageJavaScript.add("			newElement.autostart = \"true\";");                   
			pageJavaScript.add("			newElement.loop = \"false\";");                   
			pageJavaScript.add("			newElement.src = \"" + userdocs + directorySeparator + "\" + soundUrl;");                   
			pageJavaScript.add("			parentElement.appendChild(newElement);"); 
		}
		pageJavaScript.add("		}");  

		if ((reconnect == true) && (accessMode.equals("open")) && (Integer.valueOf(updateTime) > 0)) {
			pageJavaScript.add("		function searchForServer() {"); 
			pageJavaScript.add("			var xhrsearch;");
			pageJavaScript.add("			try {");                                                  
			pageJavaScript.add("				xhrsearch = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			pageJavaScript.add("			} catch (event1) {");                             
			pageJavaScript.add("				try {");
			pageJavaScript.add("					xhrsearch = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			pageJavaScript.add("				} catch (event2) {");
			pageJavaScript.add("					xhrsearch = false;");
			pageJavaScript.add("				}");
			pageJavaScript.add("			}");
			pageJavaScript.add("			if (!xhrsearch && typeof XMLHttpRequest !=\"undefined\") {");
			pageJavaScript.add("				xhrsearch = new XMLHttpRequest();");                 
			pageJavaScript.add("			}"); 	
			pageJavaScript.add("			xhrsearch.open(\"post\", \"/web2cToolkit/Web2c?param=<search></search>\", true);"); 
			pageJavaScript.add("			xhrsearch.onreadystatechange = function() {");       
			pageJavaScript.add("				if (xhrsearch.readyState == 4) {"); 
			pageJavaScript.add("					if (xhrsearch.responseText == \"alive\") {");  
			pageJavaScript.add("						window.location.href = location.href.split(\"<load>\")[0] + \"(open)\" + location.href.split(\"]\")[1];");
			pageJavaScript.add("					}");      
			pageJavaScript.add("				}");          
			pageJavaScript.add("			};");   
			pageJavaScript.add("			xhrsearch.send(null);");  
			pageJavaScript.add("		}");  
		}
		
		pageJavaScript.add("		function updatePage() {"); 
		pageJavaScript.add("			var response;"); 
		pageJavaScript.add("			var ret;"); 
		pageJavaScript.add("			var element"); 
		pageJavaScript.add("			var value;"); 
		pageJavaScript.add("			var update;"); 
		pageJavaScript.add("			var styleMod;"); 
		pageJavaScript.add("			var ref;"); 
		pageJavaScript.add("			var elementType;"); 
		pageJavaScript.add("			if (xhr.readyState == 4) {"); 
		pageJavaScript.add("				if (xhr.responseText == \"\") {");
		if ((reconnect == false) && (!message.equals("")) && (!message.equals(" "))) {
			pageJavaScript.add("					window.alert(\"" + message + "\");");
		}
		if ((reconnect == true) && (accessMode.equals("open")) && (Integer.valueOf(updateTime) > 0)) {
			pageJavaScript.add("					clearInterval(updateTimer);");
			pageJavaScript.add("					searchForServerTimer = setInterval(\"searchForServer()\", 30000);");
		}
		pageJavaScript.add("				}");  
		pageJavaScript.add("				else {");
		pageJavaScript.add("					response = unescape(xhr.responseText);");
		pageJavaScript.add("					response = response.substring(8, response.length - 9);");
		pageJavaScript.add("					ret = response.split(\"</update>\");");
		pageJavaScript.add("					for (var i = 0; i < ret.length - 1; i++) {");  
		pageJavaScript.add("						try {");     
		pageJavaScript.add("							update = ret[i].replace(\"<update>\", \"\");");
		pageJavaScript.add("							element = update.split(\"</value>\")[0].split(\"</id><value>\")[0].replace(\"<id>\", \"\");");
		pageJavaScript.add("							value = update.split(\"</value>\")[0].split(\"</id><value>\")[1];");
		pageJavaScript.add("							value = unescape(value);");
		pageJavaScript.add("							styleMod = update.split(\"</value>\")[1].split(\"</style>\");");	
		pageJavaScript.add("							for (var j = 0; j < styleMod.length - 1; j++) {");
		pageJavaScript.add("								styleMod[j] = styleMod[j].replace(\"<style>\", \"\");");
		pageJavaScript.add("							}"); 
		pageJavaScript.add("							ref = document.getElementById(element);");
		pageJavaScript.add("							if (ref != null) {;");
		pageJavaScript.add("								elementType = ref.tagName;");
		pageJavaScript.add("								if (String(elementType) == \"IMG\") {");
		pageJavaScript.add("									if (value.indexOf(\"<plot>\") != -1) {");           //Web2cHisto, Web2cTrend
		pageJavaScript.add("										if (ref.src != \"Web2c?param=\" + value) {");
		pageJavaScript.add("											ref.src = \"Web2c?param=\" + value;");
		pageJavaScript.add("										}");  
		pageJavaScript.add("									}");
		pageJavaScript.add("									else {");
		pageJavaScript.add("										if (value.indexOf(\"<vimg>\") != -1) {");       //Web2cVideo
		pageJavaScript.add("											if (ref.src != \"Web2c?param=\" + value) {");
		pageJavaScript.add("												ref.src = \"Web2c?param=\" + value;");
		pageJavaScript.add("											}");  
		pageJavaScript.add("										}");
		pageJavaScript.add("										else {");                                       //AnimatedImage, Indicator, IndicatorList
		pageJavaScript.add("											if (ref.src != value) {");
		pageJavaScript.add("												ref.src = value;");
		pageJavaScript.add("											}"); 
		pageJavaScript.add("										}");  
		pageJavaScript.add("									}");
		pageJavaScript.add("								}"); 
		pageJavaScript.add("								if (String(elementType) == \"DIV\" && value != \"\") {");
		pageJavaScript.add("									var data = value.replace(/%a/g,\"<br>\");");
		pageJavaScript.add("									if (ref.innerHTML != data) {");
		pageJavaScript.add("										ref.innerHTML = data;");   
		pageJavaScript.add("									}");                                         
		pageJavaScript.add("								}"); 
		pageJavaScript.add("								if (String(elementType) == \"TEXTAREA\") {");	
		pageJavaScript.add("									var data = value.replace(/%a/g,\"\\n\");");
		pageJavaScript.add("									if (ref.value != data) {");
		pageJavaScript.add("										ref.value = data;"); 
		pageJavaScript.add("									}");                                                             
		pageJavaScript.add("								}"); 
		pageJavaScript.add("								if (styleMod.length > 0) {");		
		pageJavaScript.add("									for (var j = 0; j < styleMod.length - 1; j++) {");
		pageJavaScript.add("										var data = styleMod[j].split(\",\");");
		pageJavaScript.add("										if (data[0] == \"color\") {");
		pageJavaScript.add("											ref.style.color = data[1];"); 
		pageJavaScript.add("										}"); 
		pageJavaScript.add("										if (data[0] == \"background-color\") {");
		pageJavaScript.add("											ref.style.backgroundColor = data[1];"); 
		pageJavaScript.add("										}"); 
		pageJavaScript.add("										if (data[0] == \"title\") {");
		pageJavaScript.add("											ref.title = data[1];"); 
		pageJavaScript.add("										}"); 
		pageJavaScript.add("										if (data[0] == \"sound\") {");
		pageJavaScript.add("											if (data[1] != \"none\" && data[1] != \"undefined\") {"); 
		pageJavaScript.add("												playSound(data[1]);");      
		pageJavaScript.add("											}");
		pageJavaScript.add("										}"); 
		pageJavaScript.add("										if (data[0] == \"font-size\") {");
		pageJavaScript.add("											ref.style.fontSize = data[1];"); 
		pageJavaScript.add("											if ((String(elementType) == \"BUTTON\" || String(elementType) == \"DIV\" ||String(elementType) == \"TEXTAREA\")&& platform.indexOf(\"Linux\") >= 0) {");
		pageJavaScript.add("												if (data[1].style.fontSize.indexOf(\"p\") >= 0) {");
		pageJavaScript.add("													var fontsizeandunits = data[1].style.fontSize.split(\"p\");");
		pageJavaScript.add("													var fontsize = fontsizeandunits[0];");
		pageJavaScript.add("													ref.style.fontSize = String(parseInt(fontsize) * fontScale) + \"px\";");
		pageJavaScript.add("												}");
		pageJavaScript.add("											}");
		pageJavaScript.add("										}");
		pageJavaScript.add("										if (data[0] == \"font-weight\") {");
		pageJavaScript.add("											ref.style.fontWeight = data[1];"); 
		pageJavaScript.add("										}"); 
		pageJavaScript.add("									}"); 
		pageJavaScript.add("								}"); 
		pageJavaScript.add("							}"); 
		pageJavaScript.add("						} catch (event) {}");
		pageJavaScript.add("					}"); 
		pageJavaScript.add("				}");
		pageJavaScript.add("			}"); 
		pageJavaScript.add("		}"); 
		
		pageJavaScript.add("		function localHistory(manifest, list,legend) {");
		pageJavaScript.add("			var now = new Date();");
		pageJavaScript.add("			var extension = manifest;");
		pageJavaScript.add("			var histlist = list;");
		pageJavaScript.add("			var legendlist = legend;");
		pageJavaScript.add("			newWindow = window.open(\"" + hostURL + "?param=<htxt><sessionId>\" + sessionId + \"</sessionId><value>\" + extension + \"</value><value>\" + histlist + \"</value><value>\" + legendlist + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></htxt>\");");
		pageJavaScript.add("		}");
		
		for(Iterator<ArrayList<String>> it1 = sectionsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				pageJavaScript.add((String)it2.next());
			}
		}
		
		for(Iterator<Web2cEmbeddedPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			ArrayList<String> embeddedPageJavaScript = ((Web2cEmbeddedPage)it1.next()).getJavaScript();
			for(Iterator<String> it2 = embeddedPageJavaScript.iterator(); it2.hasNext();) {
				pageJavaScript.add((String)it2.next());
			}
		}

		return pageJavaScript;
	}
	
	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<String> getPage() {
			
		int i = 0;
		ArrayList<String> page = new ArrayList<String>();
		Calendar calendar = Calendar.getInstance();
		File jScriptDir = null;
		File jScriptFile = null;
		String[] fileList = null;
		
		jScriptDir = new File(webDir);
		fileList = jScriptDir.list();
		for (i = 0; i < fileList.length; i++) {
			if (fileList[i].contains("Web2c_")) {
				jScriptFile = new File(webDir + File.separator + fileList[i]);
				if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
			}
		}
		
		Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2c_" + calendar.getTimeInMillis() + ".js"), debugLogger, debugListener);
		javascriptWriter.setJavascript(getJavaScript());
	
		page.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		page.add("<html>");
		page.add("	<head>");
		page.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		page.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		page.add("		<title>" + title + "</title>");
		page.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		page.add("		<link rel=\"shortcut icon\"  type=\"image/png\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		page.add("		<script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2c_" + calendar.getTimeInMillis() + ".js\"></script>");
		page.add("	</head>");
		page.add("	<body onload=\"initPage()\">"); 

		if (soundEnabled == true) {
			page.add("		<div id=\"" + referenceId + "_sound" + "\">");
			
//			page.add("			<object id=\"" + referenceId + "_sound_player" + "\" data=\"" + userdocs + directorySeparator + soundFile + "\" type=\"audio/wav\" width=\"1px\" height=\"1px\"></object>");

			page.add("			<embed id=\"" + referenceId + "_sound_player" + "\" width=\"1px\" height=\"1px\" hidden=\"true\" autostart=\"false\" loop=\"false\" src=\"" + web2cdocs + directorySeparator + pageSound + "\"></embed>");
			page.add("		</div>");
		}
		for(Iterator<String> it = getHTML().iterator(); it.hasNext();) {
			page.add((String)it.next());
		}
		page.add("	</body>");
		page.add("</html>");

		return page;
	}

	/**
	 * Returns the history component instance which belongs to the actual page
	 * @return history component
	 */
	public Web2cHistory getHistory() {
		
		return history;
	}
	
	public void doAction(String elementAction, String elementId) {

		for (Iterator<Web2cComponent> it = getComponents().iterator(); it.hasNext();) {
			Web2cComponent web2cComponent = it.next();
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cButton")) {				
				if (elementId.equals(web2cComponent.getID())) {
					web2cComponent.doAction(elementAction, elementId);
				}
			}
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cSlider")) {
				if (elementId.equals(web2cComponent.getID())) {
					web2cComponent.doAction(elementAction, elementId);
				}
			}
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cWheelSwitch")) {				
				if (elementId.equals(web2cComponent.getID())) {
					web2cComponent.doAction(elementAction, elementId);
				}
			}
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cInput")) {
				if (elementId.equals(web2cComponent.getID())) {
					web2cComponent.doAction(elementAction, elementId);
				}
			}
			if (web2cComponent.getClass().getName().equals("de.desy.web2cToolkit.web2c.web2cComponents.Web2cChat")) {
				if (elementId.equals(web2cComponent.getID())) {
					web2cComponent.doAction(elementAction, elementId);
				}
			}
		}
	}
	
	/**
	 * Constructs the page
	 */
	private void constructPage() {

		Web2cConfigReader configReader = null;
		
		try {
			configReader = new Web2cConfigReader(new File(homeDir + File.separator + pageConfigFile), debugLogger, debugListener);
			decodeAttributes(configReader.getPageAttributeNames(), configReader.getPageAttributeValues());
			if (exclusive == false) {
				constructSections(configReader.getSectionsAttributeNames(), configReader.getSectionsAttributeValues(), configReader.getSectionsElements(), configReader.getSectionsElementsAttributeNames(), configReader.getSectionsElementsAttributeValues());
				constructEmbeddedPages(configReader.getEmbeddedPagesAttributeNames(), configReader.getEmbeddedPagesAttributeValues());
			}
			else {
				error = error + "Exclusive access only!";
			}
		}
		catch (Exception e) {
			error = error + configReader.getError();
			e.printStackTrace();
		}
	}
	
	/**
	 * Constructs the sections of the page
	 * @param sectionsAttributeNames list containing lists of attribute names of all sections on the page
	 * @param sectionsAttributeValues list containing lists of attribute values of all sections on the page
	 * @param sectionsElements list containing lists of element names of all sections on the page
	 * @param sectionsElementsAttributeNames list containing lists containing lists of attribute names of all elements of all sections on the page
	 * @param sectionsElementsAttributeValues list containing lists containing lists of attribute values of all elements of all sections on the page
	 */
	private void constructSections(ArrayList<ArrayList<String>> sectionsAttributeNames, ArrayList<ArrayList<String>> sectionsAttributeValues, ArrayList<ArrayList<String>> sectionsElements, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeNames, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeValues) {

		Iterator<ArrayList<String>> it1 = sectionsAttributeNames.iterator();
		Iterator<ArrayList<String>> it2 = sectionsAttributeValues.iterator();
		Iterator<ArrayList<ArrayList<String>>> it4 = sectionsElementsAttributeNames.iterator();
		Iterator<ArrayList<ArrayList<String>>> it5 = sectionsElementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it3 = sectionsElements.iterator(); it3.hasNext();) {
			ArrayList<String> attributeNames = (ArrayList<String>)it1.next();
			ArrayList<String> attributeValues = (ArrayList<String>)it2.next();
			ArrayList<String> elementsNames = (ArrayList<String>)it3.next();
			ArrayList<ArrayList<String>> elementsAttributeNames = (ArrayList<ArrayList<String>>)it4.next();
			ArrayList<ArrayList<String>> elementsAttributeValues = (ArrayList<ArrayList<String>>)it5.next();
			Web2cSection section = new Web2cSection(referenceId, homeDir, language, xScale, yScale, attributeNames, attributeValues, elementsNames, elementsAttributeNames, elementsAttributeValues, simulation, userPrivilege, commonDeviceAccess, commonVideoConnector, chatChannel, web2cSession, referencePoint, userName, accessMode, constructionProgress);
			sections.add(section);
			sectionsHTML.add(section.getHTML());
			sectionsJavaScript.add(section.getJavaScript());
			error = error + section.getError();
		}
	}
	
	/**
	 * Constructs the embedded pages of the page
	 * @param embeddedPagesAttributeNames list containing lists of attribute names of all embedded pages on the page
	 * @param embeddedPagesAttributeValues list containing lists of attribute values of all embedded pages on the page
	 */
	private void constructEmbeddedPages(ArrayList<ArrayList<String>> embeddedPagesAttributeNames, ArrayList<ArrayList<String>> embeddedPagesAttributeValues) {

		ArrayList<String> attributeNames = new ArrayList<String>();
		ArrayList<String> attributeValues = new ArrayList<String>();
		Iterator<ArrayList<String>> it2 = embeddedPagesAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it1 = embeddedPagesAttributeNames.iterator(); it1.hasNext();) {
			attributeNames = (ArrayList<String>)it1.next();
			attributeValues = (ArrayList<String>)it2.next();
			int xLeft = 0;
			int yTop = 0;
			String refFile = null;
			String refId = null;
			String refPoint = null;
			Iterator<String> it3 = attributeValues.iterator();
			for (Iterator<String> it4 = attributeNames.iterator(); it4.hasNext();) {
				String name = (String)it4.next();
				String value = (String)it3.next();
				if (name.equals("uri")) {
					refFile = value.substring(7).replace("/", File.separator);
				}
				if (name.equals("id")) {
					if (value.equals("enter Id") || value.equals("") || value.equals(" ")) {
						error = error + "[Component Decoder] Missing or wrong component ID;";
					}
					refId = referenceId + "_" + value;
				}
				if (name.equals("bounds")) {
					xLeft = Integer.valueOf(value.split(",")[0]);
					yTop = Integer.valueOf(value.split(",")[1]);
					refPoint = ((Integer)((int)((Integer.valueOf(referencePoint.split(",")[0]) + xLeft) * xScale))).toString() + "," + ((Integer)((int)((Integer.valueOf(referencePoint.split(",")[1]) + yTop) * yScale))).toString();
				}
			}
			Web2cEmbeddedPage embeddedPage = new Web2cEmbeddedPage(web2cSession, homeDir, refFile, userName, language, refId, refPoint, xScale, yScale, userPrivilege, simulation, commonDeviceAccess, commonVideoConnector, chatChannel, debugLogger, debugListener, accessMode, constructionProgress);
			embeddedPages.add(embeddedPage); 
			error = error + embeddedPage.getError();
		}
	}

	/**
	 * Decodes the page's attributes and sets default values.
	 */
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {

		updateTime = "5000";	
		message = " ";
		reconnect = false;		
		xScale = 1.0;
		yScale = 1.0;
		scale = 1.0;
		debug = false;
		soundEnabled = false;
		
		expertName = new ArrayList<String>();
		supervisorName = new ArrayList<String>();
		expertAddress = new ArrayList<String>();
		exclusiveName = new ArrayList<String>();
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("caption")) {
				if (value.split(",").length > 1) {
					title = getStringWithSpecialCharacters(value.split(",")[language]);
				}
				else {
					title = getStringWithSpecialCharacters(value);
				}
			}
			if (name.equals("timeInterval")) {
				updateTime = String.valueOf(Integer.valueOf(value) * 1000);
			}	
			if (name.equals("exclusive")) {
				if (!value.equals("") && !value.equals(" ")) {
					exclusiveName.add(value);
				}
			}	
			if (name.equals("expert")) {
				if (!value.equals("") && !value.equals(" ")) {
					expertName.add(value);
				}
			}	
			if (name.equals("supervisor")) {
				if (!value.equals("") && !value.equals(" ")) {
					supervisorName.add(value); 
				}
			}	
			if (name.equals("ipAddress")) {
				if (!value.equals("") && !value.equals(" ")) {
					expertAddress.add(value);
				}
			}	
			if (name.equals("message")) {
				if (!value.equals("") && !value.equals(" ")) {
					if (value.split(",").length > 1) {
						message = message + getStringWithSpecialCharacters(value.split(",")[language]);
					}
					else {
						message = message + getStringWithSpecialCharacters(value);
					}
				}
			}	
			if (name.equals("reconnect")) {
				if (value.equals("true")) {
					reconnect = true;
				}
				else {
					reconnect = false;
				}
			}
			if (name.equals("scale")) {
				if (!value.equals("") && !value.equals(" ")) {
					xScale = Double.valueOf(value.split(",")[0]);
					yScale = Double.valueOf(value.split(",")[1]);
					scale = (Math.pow(Math.pow(xScale,scalePowerFactor) * Math.pow(yScale,scalePowerFactor) / (Math.pow(xScale,scalePowerFactor) + Math.pow(yScale,scalePowerFactor)), 1.0 / scalePowerFactor));
				}
			}
			if (name.equals("debug")) {
				if (!value.equals("") && !value.equals(" ")) {
					if (value.equals("no")) debug = false;
					else if (value.equals("yes")) debug = true;
					else debug = false;
				}
				else {
					debug = false;
				}
			}			
			if (name.equals("soundEnabled")) {
				if (!value.equals("yes")) {
					soundEnabled = false;
				}
				else {
					soundEnabled = true;
				}
			}
		}
		
		exclusive = false;
		if (exclusiveName.isEmpty()) {
			exclusive = false;
		}
		else {
			exclusive = true;
			for (Iterator<String> it = exclusiveName.iterator(); it.hasNext();) {
				String item = it.next().toString();
				if (item.equals(userName)) {
					exclusive = false;
				}
			}
		}

		networkPrivilege = layman;
		if (expertAddress.isEmpty()) {
			networkPrivilege = layman;
		}
		else {
			for (Iterator<String> it = expertAddress.iterator(); it.hasNext();) {
				String nextAddress = it.next().toString();
				if (nextAddress.replace(".", ">").split(">")[3].equals("*") || nextAddress.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
					if (nextAddress.replace(".", ">").split(">")[2].equals("*") || nextAddress.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
						if (nextAddress.replace(".", ">").split(">")[1].equals("*") || nextAddress.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
							if (nextAddress.replace(".", ">").split(">")[0].equals("*") || nextAddress.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
								networkPrivilege = expert;
							}
						}
					}
				}
			}
		}
		
		if (userAddress.equals("*.*.*.*")) {
			userPrivilege = supervisor;
		}
		else {
			userPrivilege = layman;
			if (expertName.isEmpty() && supervisorName.isEmpty()) {
				userPrivilege = layman;
			}
			if (!expertName.isEmpty()) {
				for (Iterator<String> it = expertName.iterator(); it.hasNext();) {
					String item = it.next().toString();
					if (item.indexOf("(") < 0) {
						if (item.equals(userName)) {
							userPrivilege = expert;
						}
					}
					else {
						String name = item.substring(0, item.indexOf("("));
						String address = item.substring(item.indexOf("(") + 1, item.indexOf(")"));
						if (name.equals(userName)) {
							if (address.replace(".", ">").split(">")[3].equals("*") || address.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
								if (address.replace(".", ">").split(">")[2].equals("*") || address.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
									if (address.replace(".", ">").split(">")[1].equals("*") || address.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
										if (address.replace(".", ">").split(">")[0].equals("*") || address.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
											networkPrivilege = expert;
											userPrivilege = expert;
										}
										else {
											networkPrivilege = layman;
											userPrivilege = layman;
										}
									}
								}
							}
						}
					}
				}		
			}
			if (!supervisorName.isEmpty()) {
				for (Iterator<String> it = supervisorName.iterator(); it.hasNext();) {
					String item = it.next().toString();
					if (item.indexOf("(") < 0) {
						if (item.equals(userName)) {
							userPrivilege = supervisor;
						}
					}
					else {
						String name = item.substring(0, item.indexOf("("));
						String address = item.substring(item.indexOf("(") + 1, item.indexOf(")"));
						if (name.equals(userName)) {
							if (address.replace(".", ">").split(">")[3].equals("*") || address.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
								if (address.replace(".", ">").split(">")[2].equals("*") || address.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
									if (address.replace(".", ">").split(">")[1].equals("*") || address.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
										if (address.replace(".", ">").split(">")[0].equals("*") || address.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
											networkPrivilege = expert;
											userPrivilege = supervisor;
										}
										else {
											networkPrivilege = layman;
											userPrivilege = layman;
										}
									}
								}
							}
						}
					}
				}
			}
			
			if (userPrivilege == expert && networkPrivilege == expert) {
				userPrivilege = expert;
			}
			else if (userPrivilege == supervisor && networkPrivilege == expert) {
				userPrivilege = supervisor;
			}
			else {
				userPrivilege = layman;
			}
		}
		
		if (userName.equals("Web2cUpdateUser")) {
			userPrivilege = supervisor;
			exclusive = false;
		}
		
		title = title + " (User: " + userName;
		if (simulation == simulated) {
			title = title + ", Data: simulated";
		}
		if (userPrivilege == expert) {
			title = title + ", Privilege: expert";
		}
		else if (userPrivilege == supervisor) {
			title = title + ", Privilege: supervisor";
		}
		else {
			title = title + ", Privilege: layman";
		}
		title = title + ", IP Address: " + userAddress;
		title = title + ")";
	}
	
	/**
	 * Returns page construction error
	 * @return error string
	 */
	public String getError() {
		
		String str = "";
		
		if (debug == true) {
			str = error;
		}
		else {
			str = "";
		}
		
		return str;
	}
	
	/**
	 * Handles strings with special characters
	 * @param value string
	 * @return converted string
	 */
	private String getStringWithSpecialCharacters(String value) {

		String str = null;
		if (value.contains("ASCIIC")) {
			if ((value.substring(0,7)).contains("ASCIIC(")) {
				str = "";
			}
			else {
				str = value.split("ASCIIC")[0];
			}
			for (int j = 1; j < (value.split("ASCIIC")).length; j++) {
				String string = value.split("ASCIIC")[j];
				str = str + String.valueOf((char)((int)(Integer.valueOf(string.substring(string.indexOf("(") + 1, string.indexOf(")")))))) + string.substring(string.indexOf(")") + 1);	
			}
		}
		else {
			str = value;
		}
		
		return str;
	}
	
	/**
	 * @author R.Bacher, DESY
	 */
	public class PageUpdater extends Observable implements Runnable {

		private Thread updater = null;
		boolean updaterStarted = false;
		private int updateInterval = 0;
		private Web2cPage updatePage = null;
		private String update = "";
		
		/**	
		 * @param page update page
		 * @param interval update time interval
		 */
		public PageUpdater (Web2cPage page, int interval) {
			
			super();
			
			updatePage = page;
			updateInterval = interval;
		
			if (updater == null) {
				updater = new Thread(this);
				updaterStarted = true;
				updater.start();
			}
		}
		
		/**
		 * Updates periodically the page
		 */
		public void run() {

			while (updaterStarted == true) {
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<Web2cComponent> it = updatePage.getComponents().iterator(); it.hasNext();) {
					Web2cComponent web2cComponent = it.next();
					try {
						strBfr = strBfr.append(web2cComponent.getUpdate());
					}
					catch(Exception e) {
						System.out.println("Error updating " + web2cComponent.getID() + " at " + String.valueOf(Calendar.getInstance().getTime().toString()));
						System.err.println(e.toString());
						e.printStackTrace();
					}
				}
				update = characterEscaper.encodeString(strBfr.toString());
				
				setChanged();
				notifyObservers(update);

				try {Thread.sleep(updateInterval);}
				catch (InterruptedException e) {}
			}
			
			updater.interrupt();
		}
		
		/**
		 * Terminates updating cache data
		 */
		public void stopUpdate() {
			
			updaterStarted = false;
		}
		
		/**
		 * Returns the updater status
		 * @return updater status
		 */
		public boolean getUpdaterStatus() {
			
			return updaterStarted;
		}
	}
}
