package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Text Label
 * @author R.Bacher, DESY
 */
public class Web2cText extends Web2cComponent{
	
	private Random randomGenerator = null;

	/**
	 * Text Label
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cText(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(false);
		initSoundAndBlinking(1);

		randomGenerator = new Random();		
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String html = null;

		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				if (property.get(0).split(":")[0].equalsIgnoreCase("tine") || property.get(0).split(":")[0].equalsIgnoreCase("tango") || property.get(0).split(":")[0].equalsIgnoreCase("epics") || property.get(0).split(":")[0].equalsIgnoreCase("doocs")) {	
					if (userPrivilege.equals(layman) && superComponent.equals(yes)) {
						html = "";
						componentHTML.add(html);
					}
					else {
						if (scrollBarEnabled == true) {
							html = "			<textarea id=\"" + id + "\" rows=\"1\" cols=\"1\" title=\"" + toolTipText + "\"";
							html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "" + "</textarea>";	
						}
						else {
							html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
							html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "xxx" + "</div>";					
						}
						componentHTML.add(html);
					}
				}
				else if (property.get(0).split(":")[0].equals("local") && property.get(0).split(":")[1].equals("//activityCounter")) {	
					html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
					html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "xxx" + "</div>";	
					componentHTML.add(html);
				}
				else {
					componentHTML.add("");
				}
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//Text: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {

		String  str = "";
		String value = "";
		String identifier = "";
		double number = 0.0;
		String soundAndBlinking = "";
		String style = "";
	
		number = (double)randomGenerator.nextInt(10);

		identifier = "<id>" + id + "</id>";
		value = "<value>" + String.valueOf(number) + "</value>";
		if (!String.valueOf(number).equals(text[0])) soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
		else soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
		style = getStyleUpdate();			
		str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		text[0] = String.valueOf(number);

		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String ret[] = new String[componentDataSize];
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;

		if (!extensionManifest.equals("") || property.get(0).split(":")[0].equalsIgnoreCase("tine") || property.get(0).split(":")[0].equalsIgnoreCase("tango") || property.get(0).split(":")[0].equalsIgnoreCase("epics") || property.get(0).split(":")[0].equalsIgnoreCase("doocs")) {	
			if (!extensionManifest.equals("")) {
				if (extensionPropertyLinksEstablished == true) {
					extensionExecutor.executeExtensionCode("live");
					ret[0] = (String)(extensionExecutor.getData()[0]);
					if (ret[0].contains("<value></value>")) {
						ret[0] = ret[0].replace("<value></value>", "<value> </value>");
					}
					ret[0] = ret[0].split(">")[1].split("<")[0];
				}
				if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
					act = extensionExecutor.getActions();
					for (int i = 0; i < act.length; i++) {
						commonDeviceAccess.execute(act[i], true);
					}
				}
			}
			else {
				if (propertyLinksEstablished == true) {
					ret[0] = commonDeviceAccess.getMonitorData(property.get(0));
					if (ret[0].contains("<value></value>")) {
						ret[0] = ret[0].replace("<value></value>", "<value> </value>");
					}
					ret[0] = ret[0].split(">")[1].split("<")[0];
				}
			}
				
			if ((propertyLinksEstablished == true) || (extensionPropertyLinksEstablished == true)) {
				identifier = "<id>" + id + "</id>";
				value = "<value>" + ret[0] + "</value>";
				if (!ret[0].equals(text[0])) soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
				else soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
				style = getStyleUpdate();			
				str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
				text[0] = ret[0];
			}
		}
		else if (property.get(0).split(":")[0].equals("local") && property.get(0).split(":")[1].equals("//activityCounter")) {				
			ret[0] = commonDeviceAccess.getActivityCounter();			
			identifier = "<id>" + id + "</id>";
			value = "<value>" + ret[0] + "</value>";
			soundAndBlinking = "<style>" + "sound,none" + "</style><style>" + "background-color," + backgroundColor + "</style>";			
			style = getStyleUpdate();			
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		else {		
			identifier = "<id>" + id + "</id>";
			value = "<value>" + " " + "</value>";
			soundAndBlinking = "<style>" + "sound,none" + "</style><style>" + "background-color," + backgroundColor + "</style>";			
			style = getStyleUpdate();			
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		
		return str;
	}
}
