package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Live Ticker Label
 * @author R.Bacher, DESY
 */
public class Web2cTicker extends Web2cComponent {

	private int letterSpacing = 0;
	private char[] messageArray = null;
	private int messageArrayLength = 0;
	private int tickerArrayLength = 0;
	private char[] tickerArray = null;
	private String ticker = null;
	private int tickerStartIndex = 0;

	/**
	 * Live Ticker Label
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cTicker(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(false);
		initSoundAndBlinking(1);

		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		String space = "*";
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
		
		ticker = "";
		letterSpacing = Integer.valueOf(fontSize.split("p")[0]);
		tickerArrayLength = (Integer.valueOf(width.replace("px", "")) - 1) / (2 * letterSpacing);
		tickerStartIndex = tickerArrayLength - 1;
		tickerArray = new char[tickerArrayLength];
		for (int i = 0; i < tickerArrayLength; i++) tickerArray[i] = space.charAt(0);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<div id=\"" + id + "\" title=\"" + toolTipText + "\" align=\"" + textAlignment + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + ";  border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; letter-spacing:" + String.valueOf(letterSpacing) + "px\">" + ticker + "</div>");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//Ticker: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		int i = 0;
		String str = "";
		String identifier = "";
		String value = "";
		String soundAndBlinking = "";
		String space = "*";
		String style = "";
		String tickerMessage = "Dies ist ein Live-Ticker";
		
		if (!ticker.equals(tickerMessage)) {
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);	
			ticker = tickerMessage.replace(" ", "*");
			for (int j = 0; j < tickerArrayLength; j++) tickerArray[j] = space.charAt(0);
			tickerStartIndex = 0;
			messageArray = ticker.toCharArray();
			messageArrayLength = messageArray.length;
		}
		else {	
			ticker = tickerMessage.replace(" ", "*");
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
		}
		
		if (tickerStartIndex >= 0) {
			for (i = 0; i < tickerArrayLength; i++) {
				if (i < tickerStartIndex) {
					tickerArray[i] = space.charAt(0);
				}
				else {
					if (tickerArrayLength <= messageArrayLength) {
						tickerArray[i] = messageArray[i - tickerStartIndex];
					}
					else {
						if (i - tickerStartIndex < messageArrayLength) {
							tickerArray[i] = messageArray[i - tickerStartIndex];
						}
						else {
							tickerArray[i] = space.charAt(0);
						}
					}
				}
			}
		}
		else {
			for (i = 0; i < tickerArrayLength - 1; i++) {
				tickerArray[i] = tickerArray[i + 1];
			}
			if (tickerArrayLength <= messageArrayLength) {
				if ((messageArrayLength + tickerStartIndex) < tickerArrayLength ) {
					tickerArray[tickerArrayLength - 1] = space.charAt(0);
				}
				else {
					tickerArray[tickerArrayLength - 1] = messageArray[tickerArrayLength - tickerStartIndex - 1];
				}
			}
			else {
				if ((tickerArrayLength + tickerStartIndex) >  messageArrayLength) {
					tickerArray[tickerArrayLength + tickerStartIndex] = space.charAt(0);
				}
			}
		}
		
		tickerStartIndex = tickerStartIndex - 1;
		if (tickerStartIndex < - messageArrayLength) {
			tickerStartIndex = tickerArrayLength - 1;
		}
		
		String tickerString = "";
		for (i = 0; i < tickerArrayLength; i++) {
			tickerString = tickerString + Character.toString(tickerArray[i]);
		}
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + characterEscaper.encodeString(tickerString) + "</value>";
		style = getStyleUpdate();			
		str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";

		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		int i = 0;
		String str = "";
		String identifier = "";
		String value = "";
		String space = "*";
		String[] data = new String[componentDataSize];
		String[] act = null;
		String soundAndBlinking = "";
		String style = "";
		
		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				data[0] = ((String)(extensionExecutor.getData()[0])).split(">")[1].split("<")[0];
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				data[0] = (commonDeviceAccess.getMonitorData(property.get(0))).split(">")[1].split("<")[0];
			}
		}
		data[0] = characterEscaper.decodeString(data[0]);
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			if (!((data[0].replace(" ", "*")).equals(ticker))) {
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);	
				ticker = data[0].replace(" ", "*");
				for (int j = 0; j < tickerArrayLength; j++) tickerArray[j] = space.charAt(0);
				tickerStartIndex = 0;
				messageArray = ticker.toCharArray();
				messageArrayLength = messageArray.length;	
			}
			else {	
				ticker = data[0].replace(" ", "*");
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
			}
			
			if (tickerStartIndex >= 0) {
				for (i = 0; i < tickerArrayLength; i++) {
					if (i < tickerStartIndex) {
						tickerArray[i] = space.charAt(0);
					}
					else {
						if (tickerArrayLength <= messageArrayLength) {
							tickerArray[i] = messageArray[i - tickerStartIndex];
						}
						else {
							if (i - tickerStartIndex < messageArrayLength) {
								tickerArray[i] = messageArray[i - tickerStartIndex];
							}
							else {
								tickerArray[i] = space.charAt(0);
							}
						}
					}
				}
			}
			else {
				for (i = 0; i < tickerArrayLength - 1; i++) {
					tickerArray[i] = tickerArray[i + 1];
				}
				if (tickerArrayLength <= messageArrayLength) {
					if ((messageArrayLength + tickerStartIndex) < tickerArrayLength ) {
						tickerArray[tickerArrayLength - 1] = space.charAt(0);
					}
					else {
						tickerArray[tickerArrayLength - 1] = messageArray[tickerArrayLength - tickerStartIndex - 1];
					}
				}
				else {
					if ((tickerArrayLength + tickerStartIndex) >  messageArrayLength) {
						tickerArray[tickerArrayLength + tickerStartIndex] = space.charAt(0);
					}
				}
			}
			
			tickerStartIndex = tickerStartIndex - 1;
			if (tickerStartIndex < - messageArrayLength) {
				tickerStartIndex = tickerArrayLength - 1;
			}
			
			String tickerString = "";
			for (i = 0; i < tickerArrayLength; i++) {
				tickerString = tickerString + Character.toString(tickerArray[i]);
			}
			
			identifier = "<id>" + id + "</id>";
			value = "<value>" + characterEscaper.encodeString(tickerString) + "</value>";
			style = getStyleUpdate();			
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		
		return str;
	}
}
