package de.desy.web2cToolkit.web2c.web2cComponents;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Value Label
 * @author R.Bacher, DESY
 */
public class Web2cValue extends Web2cComponent {

	private Random randomGenerator = null;

	/**
	 * Value Label
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cValue(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(false);
		initSoundAndBlinking(1);
		initLegendList();

		randomGenerator = new Random();		
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String html = null;
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
				html = html + " onclick=\"localHistory('" + extensionManifest + "','" +  propertyList + "','" + legendList + "')\"";
				html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:black; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "xxx" + "</div>";	
				componentHTML.add(html);
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//Value: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		double number = 0.0;
		String soundAndBlinking = "";
		String style = "";
		
		DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));

		alarmThreshold = "5.0";	
		number = (double)randomGenerator.nextInt(10);
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + characterEscaper.encodeString(formatter.format(number)) + "</value>";
		style = style + "<style>" + "background-color," + backgroundColor + "</style>";
		if (number > Double.valueOf(alarmThreshold)) {
			style = "<style>" + "color," + highColor + "</style>";
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
		}
		else {
			style = "<style>" + "color," + lowColor + "</style>";
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);		
		}
		if (number < Double.valueOf(min)) {
			style = "<style>" + "color," + highColor + "</style>";
			value = "<value>" + "n.a." + "</value>";
		}
		String[] styleArray = getStyleUpdate().split("</style>");
		for (int i = 0; i < styleArray.length; i++) {
			if (!styleArray[i].contains("color")) style = style + styleArray[i] + "</style>";
		}
		str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		
		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String[] ret = new String[componentDataSize];
		String[] retData = null;
		String[] act = null;
		double retVal = 0.0;
		String soundAndBlinking = "";
		String style = "";

		DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));
		
		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				ret[0] = ((String)(extensionExecutor.getData()[0])).split(">")[1].split("<")[0];
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				ret[0] = (commonDeviceAccess.getMonitorData(property.get(0))).split(">")[1].split("<")[0];
			}
		}
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			if (ret[0].contains(",")) {
				retData = ret[0].split(",");
				for (int i = 0; i < retData.length; i++) {
					retData[i] = characterEscaper.decodeString(retData[i]);
				}
				if (functionType.contains("rms")) {
					retVal = 0.0;
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					for (int i = 0; i < retData.length; i++) {
						retVal = retVal + Math.pow(Double.valueOf(retData[i]), 2);
					}
					retVal = Math.sqrt(retVal / retData.length) * Double.valueOf(functionParameter);
				}
				if (functionType.contains("ave")) {
					retVal = 0.0;
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					for (int i = 0; i < retData.length; i++) {
						retVal = retVal + Double.valueOf(retData[i]);
					}
					retVal = (retVal / retData.length)* Double.valueOf(functionParameter);
				}
				if (functionType.contains("std")) {
					retVal = 0.0;
					double mean = 0.0;
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					for (int i = 0; i < retData.length; i++) {
						mean = mean + Double.valueOf(retData[i]);
					}
					mean = mean / retData.length;
					for (int i = 0; i < retData.length; i++) {
						retVal = retVal + Math.pow(Double.valueOf(retData[i]) - mean, 2);
					}
					retVal = Math.sqrt(retVal / retData.length) * Double.valueOf(functionParameter);
				}
				if (functionType.contains("min")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					retVal = Double.valueOf(retData[0]);
					for (int i = 0; i < retData.length; i++) {
						if (Double.valueOf(retData[i]) < retVal) retVal = Double.valueOf(retData[i]);
					}
					retVal = retVal * Double.valueOf(functionParameter);
				}
				if (functionType.contains("max")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					retVal = Double.valueOf(retData[0]);
					for (int i = 0; i < retData.length; i++) {
						if (Double.valueOf(retData[i]) > retVal) retVal = Double.valueOf(retData[i]);
					}
					retVal = retVal * Double.valueOf(functionParameter);
				}
			}
			else {
				retVal = Double.valueOf(characterEscaper.decodeString(ret[0]));
			}
	
			if (functionType.contains("factor")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1.0";
				}
				retVal = retVal * Double.valueOf(functionParameter);
			}
			if (functionType.contains("offset")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "0.0";
				}
				retVal =  retVal + Double.valueOf(functionParameter);
			}
			if (functionType.contains("power")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1.0";
				}
				retVal = Math.pow(retVal, Double.valueOf(functionParameter));
			}
			if (functionType.contains("log")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1.0";
				}
				if (retVal == 0.0) {
					retVal = 1.0;
				}
				retVal = Math.log10(retVal * Double.valueOf(functionParameter));
			}
	
			identifier = "<id>" + id + "</id>";
			value = "<value>" + characterEscaper.encodeString(formatter.format(retVal)) + "</value>";
			style = style + "<style>" + "background-color," + backgroundColor + "</style>";
			if (retVal > Double.valueOf(alarmThreshold)) {
				style = "<style>" + "color," + highColor + "</style>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
			}
			else {
				style = "<style>" + "color," + lowColor + "</style>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);		
			}
			if (retVal < Double.valueOf(min)) {
				style = "<style>" + "color," + highColor + "</style>";
				value = "<value>" + "n.a." + "</value>";
			}
			String[] styleArray = getStyleUpdate().split("</style>");
			for (int i = 0; i < styleArray.length; i++) {
				if (!styleArray[i].contains("color")) style = style + styleArray[i] + "</style>";
			}
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		
		return str;
	}
}
