package de.desy.web2cToolkit.web2c.web2cComponents;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * List (Table) of Value Labels
 * @author R.Bacher, DESY
 */
public class Web2cValueList extends Web2cComponent {
	
	private Random randomGenerator = null;

	/**
	 * List (Table) of Value Labels
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cValueList(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);

		if (extensionManifest.equals("")) componentDataSize = property.size();
		registerMonitors(false);
		initSoundAndBlinking(componentDataSize);
		initLegendList();

		randomGenerator = new Random();	
	}

	public ArrayList<String> getHTML() {
		
		int i = 0;
		int xPos = 0;
		int yPos = 0;
		int cellHeight = 0;
		int cellWidth = 0;
		int headerCellHeight = 0;
		int headerCellWidth = 0;
		int yStep = 0;
		int rows = 0;
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		width = String.valueOf(Integer.valueOf(width.split("p")[0]) - 1) + "px";
		height = String.valueOf(Integer.valueOf(height.split("p")[0]) - 1) + "px";
		
		rows = property.size();

		if (borderStyle.equals(solid)) {
			cellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				headerCellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				headerCellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) ;
			}
			else {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		else {	
			cellWidth = (Integer.valueOf(width.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1));
			}
			else {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		xPos = 0;
		yPos = 0;
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");	
				if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
					componentHTML.add("			<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)(yPos)).toString() + "px" + "; width:" + ((Integer)(headerCellWidth)).toString() + "px" + "; height:" + ((Integer)headerCellHeight).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + "\">" + headerCaption + "</div>");
					yPos = yPos + yStep;
				}
				for (i = 0; i < rows; i++) {
					componentHTML.add("			<div id=\"" + id + "_cell" + ((Integer)i).toString() + "\" onclick=\"localHistory('" + extensionManifest + "','" +  propertyList + "','" + legendList + "')\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidth)).toString() + "px" + "; height:" + ((Integer)cellHeight).toString() + "px" + "; color:" + highColor + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "xxx" + "</div>");
					yPos = yPos + yStep;
				}		
				componentHTML.add("			</div>");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//ValueList: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		double number = 0.0;
		int i = -1;
		String soundAndBlinking = "";
		String style = "";
		
		DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));
		
		alarmThreshold = "5.0";	
		for (Iterator<String> it = property.iterator(); it.hasNext(); it.next()) {
			number = (double)randomGenerator.nextInt(10);
			
			i = i + 1;		
			identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";	
			value = "<value>" + characterEscaper.encodeString(formatter.format(number)) + "</value>";
			style = style + "<style>" + "background-color," + backgroundColor + "</style>";
			if (number > Double.valueOf(alarmThreshold)) {
				style = "<style>" + "color," + highColor + "</style>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
			}
			else {
				style = "<style>" + "color," + lowColor + "</style>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, i);		
			}
			if (number < Double.valueOf(min)) {
				style = "<style>" + "color," + highColor + "</style>";
				value = "<value>" + "n.a." + "</value>";
			}				
			String[] styleArray = getStyleUpdate().split("</style>");
			for (int j = 0; j < styleArray.length; j++) {
				if (!styleArray[j].contains("color")) style = style + styleArray[j] + "</style>";
			}			
			str = str + "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}

		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String ret[] = new String[componentDataSize];
		double retVal = 0.0;
		String[] retData = null;
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;

		DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));

		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				for (int i = 0; i < ret.length; i++) {
					ret[i] = (String)(extensionExecutor.getData()[i]);
					ret[i] = ret[i].split(">")[1].split("<")[0];
				}
			}	
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				for (int i = 0; i < ret.length; i++) {
					String address = (property.get(i));
					ret[i] = (commonDeviceAccess.getMonitorData(address)).split(">")[1].split("<")[0];	
				}
			}
		}	
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			for (int i = 0; i < ret.length; i++) {
				if (ret[i].contains(",")) {
					retData = ret[i].split(",");
					for (int j = 0; j < retData.length; j++) {
						retData[j] = characterEscaper.decodeString(retData[j]);
					}
					if (functionType.contains("rms")) {
						retVal = 0.0;
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						for (int j = 0; j < retData.length; j++) {
							retVal = retVal + Math.pow(Double.valueOf(retData[j]), 2);
						}
						retVal = Math.sqrt(retVal / retData.length) * Double.valueOf(functionParameter);
					}
					if (functionType.contains("ave")) {
						retVal = 0.0;
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						for (int j = 0; j < retData.length; j++) {
							retVal = retVal + Double.valueOf(retData[j]);
						}
						retVal = (retVal / retData.length)* Double.valueOf(functionParameter);
					}
					if (functionType.contains("std")) {
						retVal = 0.0;
						double mean = 0.0;
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						for (int j = 0; j < retData.length; j++) {
							mean = mean + Double.valueOf(retData[j]);
						}
						mean = mean / retData.length;
						for (int j = 0; j < retData.length; j++) {
							retVal = retVal + Math.pow(Double.valueOf(retData[j]) - mean, 2);
						}
						retVal = Math.sqrt(retVal / retData.length) * Double.valueOf(functionParameter);
					}
					if (functionType.contains("min")) {
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						retVal = Double.valueOf(retData[0]);
						for (int j = 0; j < retData.length; j++) {
							if (Double.valueOf(retData[j]) < retVal) retVal = Double.valueOf(retData[j]);
						}
						retVal = retVal * Double.valueOf(functionParameter);
					}
					if (functionType.contains("max")) {
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						retVal = Double.valueOf(retData[0]);
						for (int j = 0; j < retData.length; j++) {
							if (Double.valueOf(retData[j]) > retVal) retVal = Double.valueOf(retData[j]);
						}
						retVal = retVal * Double.valueOf(functionParameter);
					}
				}
				else {
					retVal = Double.valueOf(characterEscaper.decodeString(ret[i]));
				}
	
				if (functionType.contains("factor")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					retVal = retVal * Double.valueOf(functionParameter);
				}
				if (functionType.contains("offset")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "0.0";
					}
					retVal =  retVal + Double.valueOf(functionParameter);
				}
				if (functionType.contains("power")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					retVal = Math.pow(retVal, Double.valueOf(functionParameter));
				}
				if (functionType.contains("log")) {
					if (functionParameter.equals("") || functionParameter.equals(" ")) {
						functionParameter = "1.0";
					}
					if (retVal == 0.0) {
						retVal = 1.0;
					}
					retVal = Math.log10(retVal * Double.valueOf(functionParameter));
				}

				identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";	
				value = "<value>" + characterEscaper.encodeString(formatter.format(retVal)) + "</value>";
				style = style + "<style>" + "background-color," + backgroundColor + "</style>";
				if (retVal > Double.valueOf(alarmThreshold)) {
					style = "<style>" + "color," + highColor + "</style>";
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
				}
				else {
					style = "<style>" + "color," + lowColor + "</style>";
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, i);		
				}
				if (retVal < Double.valueOf(min)) {
					style = "<style>" + "color," + highColor + "</style>";
					value = "<value>" + "n.a." + "</value>";
				}				
				String[] styleArray = getStyleUpdate().split("</style>");
				for (int j = 0; j < styleArray.length; j++) {
					if (!styleArray[j].contains("color")) style = style + styleArray[j] + "</style>";
				}			
				str = str + "<update>" + identifier + value + style + soundAndBlinking + "</update>";
			}
		}

		return str;
	}
}
