package de.desy.web2cToolkit.web2c.web2cDeviceAccess;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Vector;

import de.desy.tine.client.TCallback;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TMode;
import de.desy.tine.queryUtils.TPropertyQuery;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.NAME16;
import de.desy.tine.types.NAME16I;
import de.desy.tine.types.NAME32;
import de.desy.tine.types.NAME32I;
import de.desy.tine.types.NAME48;
import de.desy.tine.types.NAME48I;
import de.desy.tine.types.NAME64;
import de.desy.tine.types.NAME64I;
import de.desy.tine.types.NAME8;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cDeviceAccess;

/**
 * Web2c TINE device access plug
 * @author R.Bacher, DESY
 */
public class Web2cTINEDeviceAccess implements IWeb2cDeviceAccess, IWeb2cConstants {
	
	private ArrayList<String> monitorPropertyList = null;
	private ArrayList<String> monitorPropertyDescriptionList = null;
	private ArrayList<TLink> linkList = null;
	private ArrayList<Monitor> monitorList = null;
	private ArrayList<long[][]> historyTimeStampList = null;
	private ArrayList<Object[][]> historyDataList = null;
	private ArrayList<TPropertyQuery[]> propertyInformationCache = null;
	private ArrayList<String> propertyCache = null;
	
	private String error = null;
	private String activityStatus = "";
	private int pollInterval = 1000;
	private int maxCacheLength = 1000;
	private Web2cASCIICharacterEscaper characterEscaper = null;

	/**
	 * Web2c TINE device access plug: constructor
	 * @param protocol protocol id
	 */
	public Web2cTINEDeviceAccess(String protocol) {
		
		monitorPropertyList = new ArrayList<String>();
		monitorPropertyDescriptionList = new ArrayList<String>();
		linkList = new ArrayList<TLink>();
		monitorList = new ArrayList<Monitor>();
		historyTimeStampList = new ArrayList<long[][]>();
		historyDataList = new ArrayList<Object[][]>();
		propertyInformationCache = new ArrayList<TPropertyQuery[]>();
		propertyCache = new ArrayList<String>();
		error = "";

		characterEscaper = new Web2cASCIICharacterEscaper();
	}
	
	/**
	 * Returns error string
	 * @return error
	 */
	public String getError() {
		
		String lastError = error;
		error = "";
		
		return lastError;
	}
	
	/**
	 * Closes the monitor link specified
	 * @param address monitor link address
	 */
	public void closeMonitorLink(String address) {
		
		int index = 0;
		TLink link = null;

		synchronized(monitorPropertyList) {
			synchronized(monitorPropertyDescriptionList) {
				synchronized(linkList) {
					synchronized(monitorList) {
						synchronized(historyTimeStampList) {
							synchronized(historyDataList) {
								index = monitorPropertyList.indexOf(address);
								link = linkList.get(index);
								int ret = link.close();
								if (ret == 0) {
//									System.out.println("Asynchronous TINE Link canceled: " + address);
								}
								else {
//									System.out.println("Could not cancel asynchronous TINE Link: " + address);
								}
								monitorPropertyList.remove(index);
								monitorPropertyDescriptionList.remove(index);
								linkList.remove(index);
								monitorList.remove(index);
								historyTimeStampList.remove(index);
								historyDataList.remove(index);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Executes a synchronous write call
	 * @param address address
	 * @param messageEnabled action message enabled flag
	 */
	public void execute(String address, boolean messageEnabled) {

		int err = 0;
		TLink tLink = null;
		TPropertyQuery[] propertyInformation = null;
		TDataType tData = null;
		String parameter = null;
		String property = null;
		String context = null;
		String server = null;
		String device = null;
		String proper = null;
		short format = -1;
		int size = -1;
		int timeOut = 1000;
		boolean isCDIServer = false;
		int status = 0;
		
		property = getLinkAddress(address);
		parameter = getLinkParameters(address);
		context = property.substring(1).split("/")[0];
		server = property.substring(1).split("/")[1];
		device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		proper = property.substring(property.lastIndexOf("/") + 1);
		String devName = "/" + context + "/" + server + "/" + device;
		String devProperty = proper;
		String linkName = devName + "/" + devProperty;

		try {
			status = 0;
			try {
				char[] data = new char[32];
				tLink = null;
				tLink = new TLink(devName + "/" + "CDIVERSION", new TDataType(data), null, TAccess.CA_READ);
			}
			catch (Exception e) {
				status = status + 1;
//				System.out.println("Error creating new TLink!");
			}
			if (status == 0) {
				err = tLink.execute(timeOut, true);
				if ((err == 36) || (err == 113)) {    // illegal property or link blacklisted -> is no CDI Server
					isCDIServer = false;
				}
				else if (err == 45) {                 // link time out    
					status = status + 1;
				}			
				else {
					isCDIServer = true;				  // -> is no CDI Server  
				}
				tLink.cancel();
			
				if (err != 45) {		
					if (isCDIServer == false) {
						try {
							propertyInformation = getPropertyInfos(property);
							if (propertyInformation == null) {
								format = -1;
								size = -1;
							}
							else {
								format = propertyInformation[0].prpFormatIn;
								size = 0;
							}
						}
						catch (Exception e) {
							format = -1;
							size = -1;
							status = status + 1;
						}
					}
					else {
						int[] sizeData = new int[1];
						try {
							tLink = null;
							tLink = new TLink(devName + "/" + "DISPLAYSIZE", new TDataType(sizeData), null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
						if (tLink != null) {
							err = tLink.execute(timeOut, true);	
							size = sizeData[0];
							if (size == 0) {
								try {
									propertyInformation = getPropertyInfos(property);
									if (propertyInformation == null) {
										size = -1;
									}
									else {
										size = propertyInformation[0].prpSizeIn;
									}
								}
								catch (Exception e) {
									size = -1;
								}
							}
							if (size > maxInputLength) size = maxInputLength;
							tLink.cancel();
						}
						
						int[] formatData = new int[1];
						try {
							tLink = null;
							tLink = new TLink(devName + "/" + "DISPLAYFORMAT", new TDataType(formatData), null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
							format = -1;
//							System.out.println("Error creating new TLink!");
						}	
						if (status == 0) {
							err = tLink.execute(timeOut, true);
							format = ((Integer)formatData[0]).shortValue();
							if (TFormat.toString(format).equalsIgnoreCase("null")) {
								try {
									propertyInformation = getPropertyInfos(property);
									if (propertyInformation == null) {
										format = -1;
									}
									else {
										format = propertyInformation[0].prpFormatIn;
									}
								}
								catch (Exception e) {
									format = -1;
								}
							}
							tLink.cancel();
						}
//						System.out.println("Check synchronous TINE Link: " + linkName + " is a CDI property! Format: " + TFormat.toString(format) + " Size: " + size);
					} 
					
					if (status == 0 && format != -1 && size != -1) {
						tData = null;
						if (parameter == null) {	
							try {
								tLink = null;
								tLink = new TLink(property, null, null, TAccess.CA_READ);
							}
							catch (Exception e) {
//								System.out.println("Error creating new TLink!");
							}
							if (tLink != null) {
								err = tLink.execute(timeOut, true);
								if (err != 0) {
//									System.out.println("Link error: " + tLink.getError(err));
								}
								tLink.cancel();
							}
						}
						else { 
							parameter = characterEscaper.decodeString(parameter);
							if ((TFormat.toString(format)).equalsIgnoreCase("char")) {
								char[] inData = null;
								if (isCDIServer == false) {
									inData = new char[parameter.length()];
									for (int i = 0; i < inData.length; i++) {
										inData[i] = parameter.charAt(i); 
									}
								}
								else {
									int textLength = 0;
									inData = new char[size];
									if (size <= parameter.length()) textLength = size;
									else textLength = parameter.length();
									for (int i = 0; i < textLength; i++) {
										inData[i] = parameter.charAt(i); 
									}
								}
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("byte")) {
								byte[] inData = new byte[1];
								inData[0] = Byte.valueOf(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("NAME8")) {
								NAME8[] inData = new NAME8[1];
								inData[0] = new NAME8(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("NAME16")) {
								NAME16[] inData = new NAME16[1];
								inData[0] = new NAME16(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("NAME32")) {
								NAME32[] inData = new NAME32[1];
								inData[0] = new NAME32(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("NAME48")) {
								NAME48[] inData = new NAME48[1];
								inData[0] = new NAME48(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("NAME64")) {
								NAME64[] inData = new NAME64[1];
								inData[0] = new NAME64(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("int16")) {
								short[] inData = new short[1];
								inData[0] = Short.valueOf(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("int32")) {
								int[] inData = new int[1];
								inData[0] = Integer.valueOf(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("float")) {
								float[] inData = new float[1];
								inData[0] = Float.valueOf(parameter);
								tData = new TDataType(inData);
							}
							else if ((TFormat.toString(format)).equalsIgnoreCase("double")) {
								double[] inData = new double[1];
								inData[0] = Double.valueOf(parameter);
								tData = new TDataType(inData);
							}
							else {
//								System.out.println("Could not execute synchronous TINE Link: " + linkName + ", Data Type: " + format + " = " + TFormat.toString(format) + " not implemented!");
							}
							if (tData != null) {
								try {
									tLink = null;
									tLink = new TLink(property, null, tData, TAccess.CA_WRITE);
								}
								catch (Exception e) {
//									System.out.println("Error creating new TLink!");
								}
								if (tLink != null) {
									err = tLink.execute(timeOut, true);
									if (err != 0) {
//										System.out.println("Link error: " + tLink.getError(err));
									}
									tLink.close();
								}
							}
						}
					}
//					System.out.println("Could not execute synchronous TINE Link: " + linkName);
				}
			}
//			System.out.println("Could not execute synchronous TINE Link: " + linkName);
		}
		catch (Exception e) {
//			System.out.println("Could not execute synchronous TINE Link: " + linkName);
		}
	}
	
	/**
	 * Returns local history data
	 * @param address address
	 * @return data
	 */
	public synchronized String getHistory(String address) {
		
		String out = "";
		StringBuffer outBuf = null;
		String property = "";
		int[] index = null;
		
		index = new int[2];

		property = getLinkAddress(address);
		if (getLinkParameters(address).equals("")) {
			index[0] = 0;
			index[1] = index[0];
		}
		else {
			if ((getLinkParameters(address).split(";")).length == 1) {
				index[0] = Integer.valueOf(getLinkParameters(address).split(";")[0]);
				index[1] = index[0];
			}
			else {
				index[0] = Integer.valueOf(getLinkParameters(address).split(";")[0]);
				index[1] = Integer.valueOf(getLinkParameters(address).split(";")[1]);
			}
		}
		
		synchronized(monitorPropertyList) {
			synchronized(monitorPropertyDescriptionList) {
				synchronized(linkList) {
					synchronized(monitorList) {
						synchronized(historyTimeStampList) {
							synchronized(historyDataList) {
								if (monitorPropertyList.contains(property)) {
									outBuf = new StringBuffer("");
									int ind = monitorPropertyList.indexOf(property);
									long[][] hTimeStamp = historyTimeStampList.get(ind);
									Object[][] hData= historyDataList.get(ind);
									int pointer = monitorList.get(ind).getArchiveDataPointer();
									for (int i = pointer + 1; i < Integer.valueOf(historyInterval); i++) {
										if (hData[index[0]][i] == null) {hData[index[0]][i] = characterEscaper.encodeString("0.0");}
										outBuf.append("<value>" + hData[index[0]][i] + "</value><timeStamp>" + hTimeStamp[index[0]][i] + "</timeStamp>");
									}
									for (int i = 0; i <= pointer; i++) {
										if (hData[index[0]][i] == null) {hData[index[0]][i] = characterEscaper.encodeString("0.0");}
										outBuf.append("<value>" + hData[index[0]][i] + "</value><timeStamp>" + hTimeStamp[index[0]][i] + "</timeStamp>");
									}
								}
								out = outBuf.toString();
							}
						}
					}
				}
			}
		}
		
		return out;
	}

	/**
	 * Returns monitor data
	 * @param address address
	 * @return data
	 */
	public synchronized String getMonitorData(String address) {

		String out = "";
		String property = "";
		int[] index = null;
		int i = 0;
		
		index = new int[2];
		
		property = getLinkAddress(address);
		if (getLinkParameters(address).equals("")) {
			index[0] = 0;
			index[1] = index[0];
		}
		else {
			if ((getLinkParameters(address).split(";")).length == 1) {
				index[0] = Integer.valueOf(getLinkParameters(address).split(";")[0]);
				index[1] = index[0];
			}
			else {
				index[0] = Integer.valueOf(getLinkParameters(address).split(";")[0]);
				index[1] = Integer.valueOf(getLinkParameters(address).split(";")[1]);
			}
		}
		
		synchronized(monitorPropertyList) {
			synchronized(monitorPropertyDescriptionList) {
				synchronized(linkList) {
					synchronized(monitorList) {
						synchronized(historyTimeStampList) {
							synchronized(historyDataList) {
								if (monitorPropertyList.contains(property)) {
									String timeStamp = "";
									StringBuffer dataBuf = new StringBuffer("");
									int ind = monitorPropertyList.indexOf(property);
									Monitor monitor = monitorList.get(ind);
									dataBuf.append("<value>");
									try {	  
										String[] retData = ((monitor.getData()).split(";")[0]).split("<");
										for (i = index[0]; i <= index[1]; i++) {
											dataBuf.append(retData[i] + ",");
										}
										timeStamp = (monitor.getData()).split(";")[1];
									}
									catch (Exception e) {
//										System.out.println("Error getting monitor data: " + prop);
									}
									String data = dataBuf.toString();
									out = data.substring(0, data.length() - 1) + "</value><timeStamp>" + timeStamp + "</timeStamp>";
								}
							}
						}
					}
				}
			}
		}
		
		return out;
	}

	/**
	 * Registers monitor link
	 * @param address address
	 * @return status
	 */
	public int registerMonitor(String address) {

		Monitor monitor = null;
		TLink tLink = null;
		TDataType tData = null;
		TPropertyQuery[] propertyInformation = null;
		short format = -1;
		int size = -1;
		String description = "";
		String property = null;
		String context = null;
		String server = null;
		String device = null;
		String proper = null;
		long[][] hTimeStamp = null;
		Object[][] hData = null;
		int status = 0;
		boolean isCDIServer = false;		
		int err = 0;
		int timeOut = 1000;
		
		property = getLinkAddress(address);	
		context = property.substring(1).split("/")[0];
		server = property.substring(1).split("/")[1];
		device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		proper = property.substring(property.lastIndexOf("/") + 1);
		String devName = "/" + context + "/" + server + "/" + device;
		String devProperty = proper;
		String linkName = devName + "/" + devProperty;

//		System.out.println(String.valueOf(Calendar.getInstance().getTime().toString()) + " Asynchronous TINE Link being opend: " + linkName);
		
		try {
			status = 0;
			error = "";
			try {
				char[] data = new char[32];
				tLink = null;
				tLink = new TLink(devName + "/" + "CDIVERSION", new TDataType(data), null, TAccess.CA_READ);
			}
			catch (Exception e) {
				status = status + 1;
//				System.out.println("Error creating new TLink!");
			}
			if (status == 0 && tLink != null) {
				err = tLink.execute(timeOut, true);
				if ((err == 36) || (err == 113)) {    // illegal property or link blacklisted -> is no CDI Server
					isCDIServer = false;
				}
				else if (err == 45) {                 // link time out -> server off-line  
					status = status + 1;
				}		
				else {
					isCDIServer = true;				  // -> is CDI Server
				}
				tLink.cancel();
			
				if (err != 45) {
					if (isCDIServer == false) {	
						propertyInformation = getPropertyInfos(property);
						if (propertyInformation == null) {
							format = -1;
							size = -1;
							description = "";
						}
						else {
							format = propertyInformation[0].prpFormat;
							size = propertyInformation[0].prpSize;
							description = propertyInformation[0].prpDescription;
						}
						if (size > maxLinkDataSize) size = maxLinkDataSize;
					}
					else {
						int[] sizeData = new int[1];
						try {
							tLink = null;
							tLink = new TLink(devName + "/" + "DISPLAYSIZE", new TDataType(sizeData), null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
						if (status == 0 && tLink != null) {
							err = tLink.execute(timeOut, true);	
							size = sizeData[0];
							if (size == 0) {
								try {
									propertyInformation = getPropertyInfos(property);
									if (propertyInformation == null) {
										size = -1;
									}
									else {
										size = propertyInformation[0].prpSize;
									}
								}
								catch (Exception e) {
									size = -1;
								}
							}
							if (size > maxLinkDataSize) size = maxLinkDataSize;
							tLink.cancel();
						}
						
						int[] formatData = new int[1];
						try {
							tLink = null;
							tLink = new TLink(devName + "/" + "DISPLAYFORMAT", new TDataType(formatData), null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
						if (status == 0 && tLink != null) {
							err = tLink.execute(timeOut, true);
							format = ((Integer)formatData[0]).shortValue();
							if (TFormat.toString(format).equalsIgnoreCase("null")) {
								try {
									propertyInformation = getPropertyInfos(property);
									if (propertyInformation == null) {
										format = -1;
									}
									else {
										format = propertyInformation[0].prpFormat;
									}
								}
								catch (Exception e) {
									format = -1;
								}
							}
							tLink.cancel();
						}
						
						if (format != -1 && size != -1) {
//							System.out.println("Check asynchronous TINE Link: " + linkName + " is a CDI property! Format: " + TFormat.toString(format) + " Size: " + size);
						}
					}
					
					String[] devices = TQuery.getDeviceNames(context, server);
					boolean serverHasDevice = false;
					if (devices != null) {
						for (int i = 0; i < devices.length; i++) {
							if (devices[i].equals(device)) serverHasDevice = true;
						}	
						if (serverHasDevice == false) {
							status = status + 1;
						}
						else {			
							String[] properties = TQuery.getDeviceProperties(context, server, device, 1000);
							if (properties != null) {
								boolean deviceHasProperty = false;
								for (int i = 0; i < properties.length; i++) {
									if (properties[i].equals(proper)) deviceHasProperty = true;
								}
								if (deviceHasProperty == false) {
									status = status + 1;
								}
							}
						}
					}
				}

				if (format != -1 && size != -1 && status == 0) {
					if ((TFormat.toString(format)).equalsIgnoreCase("char")) {
						if (isCDIServer == true) size = maxInputLength; 
						try {
							tLink = null;
							tData = new TDataType(new char[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("byte")) {
						try {
							tLink = null;
							tData = new TDataType(new byte[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME8")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME64[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME16")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME64[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME32")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME64[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME48")) {
						try {
							tLink = null;
							tData =  new TDataType(new NAME64[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME64")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME64[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME16I")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME16I[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME32I")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME32I[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME48I")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME48I[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("NAME64I")) {
						try {
							tLink = null;
							tData = new TDataType(new NAME64I[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("int16")) {
						try {
							tLink = null;
							tData =  new TDataType(new short[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("int32")) {
						try {
							tLink = null;
							tData = new TDataType(new int[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("float")) {
						try {
							tLink = null;
							tData = new TDataType(new float[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else if ((TFormat.toString(format)).equalsIgnoreCase("double")) {
						try {
							tLink = null;
							tData = new TDataType(new double[size]);
							tLink = new TLink(devName, devProperty, tData, null, TAccess.CA_READ);
						}
						catch (Exception e) {
							status = status + 1;
//							System.out.println("Error creating new TLink!");
						}
					}
					else {
						status = status + 1;
//						System.out.println("Could not establish asynchronous TINE Link: " + linkName + ", Data Type: " + format + " = " + TFormat.toString(format) + " not implemented!");
						error = error + "[Control system] Error establishing monitor link " + linkName + ";";
					}
					if (status == 0 && tLink != null) {	
						monitorPropertyList.add(property);
						monitorPropertyDescriptionList.add(description);
						hTimeStamp = new long[size][Integer.valueOf(historyInterval)];
						historyTimeStampList.add(hTimeStamp);
						hData = new Object[size][Integer.valueOf(historyInterval)];
						historyDataList.add(hData);
						linkList.add(tLink);
						monitor = new Monitor(tLink, property, format, size, hTimeStamp, hData);
						monitorList.add(monitor);	
						try {
							int ret = -1;
							ret = tLink.attach(TMode.CM_POLL | TMode.CM_WAIT, monitor, pollInterval);
							if (ret < 0) {
								tLink.cancel();
								status = status + 1;			
								monitorPropertyList.remove(property);
								monitorPropertyDescriptionList.remove(description);
								historyTimeStampList.remove(hTimeStamp);
								historyDataList.remove(hData);
								linkList.remove(tLink);
								monitorList.remove(monitor);
//								System.out.println("Asynchronous TINE Link canceled: " + linkName + ", err: " + tLink.linkStatus);
							}
							else {
								if (tLink.getLinkStatus() != 0) {
									tLink.cancel();
									status = status + 1;		
									monitorPropertyList.remove(property);
									monitorPropertyDescriptionList.remove(description);
									historyTimeStampList.remove(hTimeStamp);
									historyDataList.remove(hData);
									linkList.remove(tLink);
									monitorList.remove(monitor);
//									System.out.println("Asynchronous TINE Link canceled: " + linkName + ", err: " + tLink.linkStatus);
								}
								else {
//									System.out.println(String.valueOf(Calendar.getInstance().getTime().toString()) + " Asynchronous TINE Link opened: " + linkName + ", Data Type: " + TFormat.toString(format) + "(" + format + "), Data Size: " + size);
								}
							}
						}
						catch (Exception e) {
							tLink.cancel();
							status = status + 1;			
							monitorPropertyList.remove(property);
							monitorPropertyDescriptionList.remove(description);
							historyTimeStampList.remove(hTimeStamp);
							historyDataList.remove(hData);
							linkList.remove(tLink);
							monitorList.remove(monitor);
//							System.out.println("Asynchronous TINE Link canceled: " + linkName + ", err: " + tLink.linkStatus);
						}
					}
				}
				if (status != 0) error = error + "[Control system] Error establishing monitor link " + linkName + ";";
			}
			else {
				status = status + 1;
				error = error + "[Control system] Error establishing monitor link " + linkName + ";";
			}
		}
		catch (Exception e) {
			status = status + 1;
			error = error + "[Control system] Error establishing monitor link " + linkName + ";";
		}
		
		return status;
	}
	
	/**
	 * TINE Monitor
	 */
	public class Monitor implements TCallback {
		
		TLink tLink = null;
		String monitorProperty = null;
		short linkFormat = 0;
		int linkSize = 0;
		long[][] hTimeStamp = null;
		Object[][] hData = null;
		String linkData = "";
		int linkStatus = 0;
		long timeStamp = 0;			
		int[] index = null;
		String errorData = "";
		StringBuffer errorDataBuf = new StringBuffer("");
		int archiveDataPointer = 0;	
		Vector<Object> out = new Vector<Object>();
		
		/**
		 * Constructor
		 * @param link link object
		 * @param address link address
		 * @param format link format
		 * @param size link size
		 * @param timeStamp time stamp array for local history
		 * @param data data array for local history
		 */
		public Monitor (TLink link, String address, short format, int size, long[][] timeStamp, Object[][] data) {
			
			tLink= link;
			monitorProperty = address;
			linkFormat = format;
			linkSize = size;
			hTimeStamp = timeStamp;
			hData = data;
			if ((TFormat.toString(linkFormat)).equalsIgnoreCase("char")) {  				
				for (int i = 0; i < linkSize; i++) {	
					errorDataBuf.append(characterEscaper.encodeString("?") + "<");
				}
			}
			else if ((TFormat.toString(linkFormat)).contains("NAME")) {
				for (int i = 0; i < linkSize; i++) {	
					errorDataBuf.append(characterEscaper.encodeString("?") + "<");
				}
			}
			else {				
				for (int i = 0; i < linkSize; i++) {	
					errorDataBuf.append(characterEscaper.encodeString("0") + "<");
				}
			}
			errorData = errorDataBuf.toString();
			errorData = errorData.substring(0, errorData.length() - 1);
			linkData = errorData + ";" + String.valueOf(Calendar.getInstance().getTimeInMillis() / 1000);		
			
			long startTime = System.currentTimeMillis();
			for (int i = 0; i < linkSize; i++) {
				for (int j = 0; j < Integer.valueOf(historyInterval); j++) {
					hTimeStamp[i][j] = startTime - ((Integer.valueOf(historyInterval) - j) * pollInterval);
					hData[i][j] = characterEscaper.encodeString("0.0");
				}
			}
		}
		
		/**
		 * Executed if called back by TINE lib
		 * @param tId link id
		 * @param tStatus link status
		 */
		public void callback(int tId, int tStatus) {

			String outData = "";
			StringBuffer outDataBuf = new StringBuffer("");

			linkStatus = tStatus;
			if (tLink.linkId == tId) {
				activityStatus = " [TINE] " + "Error: " + linkStatus + " (" + Calendar.getInstance().getTimeInMillis() + ")";
				
				if (linkStatus != 0) {
//					System.out.println("Link Error : " + monitorProperty + " " + TErrorList.getErrorString(linkStatus));
					if ((TFormat.toString(linkFormat)).equalsIgnoreCase("char")) {  				
						for (int i = 0; i < linkSize; i++) {	
							outDataBuf.append(characterEscaper.encodeString("?") + "<");
						}
					}
					else if ((TFormat.toString(linkFormat)).contains("NAME")) {
						for (int i = 0; i < linkSize; i++) {	
							outDataBuf.append(characterEscaper.encodeString("?") + "<");
						}
					}
					else {				
						for (int i = 0; i < linkSize; i++) {	
							outDataBuf.append(characterEscaper.encodeString("?") + "0");
						}
					}
					outData = outDataBuf.toString();
					outData = outData.substring(0, outData.length() - 1);
					linkData = outData + ";" + String.valueOf(timeStamp);
				}
				else {
				    timeStamp = tLink.getLastTimeStamp();
				    out = tLink.getOutputDataObject().getVector();
					if (out.equals(null)) {
						if ((TFormat.toString(linkFormat)).equalsIgnoreCase("char")) {  				
							for (int i = 0; i < linkSize; i++) {	
								outDataBuf.append(characterEscaper.encodeString("?") + "<");
							}
						}
						else if ((TFormat.toString(linkFormat)).contains("NAME")) {
							for (int i = 0; i < linkSize; i++) {	
								outDataBuf.append(characterEscaper.encodeString("?") + "<");
							}
						}
						else {				
							for (int i = 0; i < linkSize; i++) {	
								outDataBuf.append(characterEscaper.encodeString("0") + "<");
							}
						}
						outData = outDataBuf.toString();
						outData = outData.substring(0, outData.length() - 1);
						linkData = outData + ";" + String.valueOf(timeStamp);
					}
					else {
						if ((TFormat.toString(linkFormat)).equalsIgnoreCase("char")) { 
//							for (Iterator<Object> it = out.iterator(); it.hasNext();) {
//								outDataBuf.append(characterEscaper.encodeString(String.valueOf((Object)it.next())));
//							}
//							outData = outDataBuf.toString();
//							linkData = outData + ";" + String.valueOf(timeStamp);
							linkData = characterEscaper.encodeString((tLink.getOutputDataObject()).toString()) + ";" + String.valueOf(timeStamp);
						}
						else if ((TFormat.toString(linkFormat)).equalsIgnoreCase("NAME16I") || (TFormat.toString(linkFormat)).equalsIgnoreCase("NAME32I") || 
								 (TFormat.toString(linkFormat)).equalsIgnoreCase("NAME48I") || (TFormat.toString(linkFormat)).equalsIgnoreCase("NAME64I")) { 	
								for (Iterator<Object> it = out.iterator(); it.hasNext();) {
									String name = "";
									if ((TFormat.toString(linkFormat)).equalsIgnoreCase("NAME16I")) {
										NAME16I value = (NAME16I)it.next();
										name = value.name;
									}
									if ((TFormat.toString(linkFormat)).equalsIgnoreCase("NAME32I")) {
										NAME32I value = (NAME32I)it.next();
										name = value.name;
									}
									if ((TFormat.toString(linkFormat)).equalsIgnoreCase("NAME48I")) {
										NAME48I value = (NAME48I)it.next();
										name = value.name;
									}
									if ((TFormat.toString(linkFormat)).equalsIgnoreCase("NAME64I")) {
										NAME64I value = (NAME64I)it.next();
										name = value.name;
									}
									outDataBuf.append(characterEscaper.encodeString(name) + "<") ;
								}
								outData = outDataBuf.toString();
								linkData = outData.substring(0, outData.length() - 1) + ";" + String.valueOf(timeStamp);
						}
						else {
							for (Iterator<Object> it = out.iterator(); it.hasNext();) {
								outDataBuf.append(characterEscaper.encodeString(String.valueOf((Object)it.next())) + "<") ;
							}
							outData = outDataBuf.toString();
							linkData = outData.substring(0, outData.length() - 1) + ";" + String.valueOf(timeStamp);
						}
					}
					
					if ((TFormat.toString(linkFormat)).equalsIgnoreCase("byte") || (TFormat.toString(linkFormat)).equalsIgnoreCase("int16") || 
					(TFormat.toString(linkFormat)).equalsIgnoreCase("int32") || (TFormat.toString(linkFormat)).equalsIgnoreCase("float") || 
					(TFormat.toString(linkFormat)).equalsIgnoreCase("double")) {
						String[] data = linkData.split(";")[0].split("<");
						for (int j = 0; j < data.length; j++) {
							hTimeStamp[j][archiveDataPointer] = timeStamp;
							hData[j][archiveDataPointer] = (Object)(data[j]);
						}
						archiveDataPointer = archiveDataPointer + 1;
						if (archiveDataPointer == Integer.valueOf(historyInterval)) {
							archiveDataPointer = 0;
						}
					}
				}
			}
	    }
		
		/**
		 * Returns actual link data
		 * @return data 
		 */
		public String getData() {
			
			return linkData; 
		}
		
		/**
		 * Returns actual link status
		 * @return link status 
		 */
		public int getLinkStatus() {
			
			return linkStatus; 
		}
		
		/**
		 * Returns actual archive data pointer
		 * @return pointer 
		 */
		public int getArchiveDataPointer() {
			
			return archiveDataPointer; 
		}
	}
	
	private String getLinkAddress(String address) {
		
		String property = null;
		
		property = address.substring(0,address.indexOf("("));
		
		return property;
	}
	
	private String getLinkParameters(String address) {

		String parameters = "";

		if (address.substring(address.indexOf("("),address.indexOf(")") + 1).length() != 2) {
			parameters = address.substring(address.indexOf("(") + 1, address.indexOf(")"));
		}
		else {
			parameters = "";
		}

		return parameters;
	}

	/**
	 * Returns activity status
	 * @return activity status
	 */
	public String getActivityCounter() {
		return activityStatus;   // "[TINE] errorStatus (timeStamp)"
	}
	
	/**
	 * Returns property description
	 * @param address address
	 * @return description
	 */
	public String getPropertyDescription(String address) {		
		
		int index = 0;
		String description = "";

		synchronized(monitorPropertyList) {
			synchronized(monitorPropertyDescriptionList) {
				synchronized(linkList) {
					synchronized(monitorList) {
						synchronized(historyTimeStampList) {
							synchronized(historyDataList) {
								index = monitorPropertyDescriptionList.indexOf(address);
								if (index >= 0) {
									description = monitorPropertyDescriptionList.get(index);
								}
								else {		
									String property = getLinkAddress(address);
									TPropertyQuery[] propertyInformations = getPropertyInfos(property);
									if (propertyInformations == null) {
										description = "";
									}
									else {
										description = propertyInformations[0].prpDescription;
									}
								}
							}
						}
					}
				}
			}
		}
		
		return description;
	}
	
	/**
	 * Returns property informations
	 * @param property property
	 * @return property informations
	 */
	private TPropertyQuery[] getPropertyInfos(String property) {
		
		TPropertyQuery[] propertyInformations = null;

		String context = property.substring(1).split("/")[0];
		String server = property.substring(1).split("/")[1];
		String device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		String proper = property.substring(property.lastIndexOf("/") + 1);
		String linkName = "/" + context + "/" + server + "/" + device + "/" + proper;
		
		if (propertyCache.toArray().length > maxCacheLength) {
			propertyInformationCache.clear();
			propertyCache.clear();
		}

		if (propertyCache.contains(linkName)) {
			propertyInformations  = (TPropertyQuery[])(propertyInformationCache.toArray()[propertyCache.indexOf(linkName)]);
//			System.out.println("Property [" + property + "] informations existing in cache.");
		}
		else {
			propertyInformations  = TQuery.getPropertyInformation(context, server, device, proper, 1000);
			if (propertyInformations != null) {
				propertyInformationCache.add(propertyInformations);
				propertyCache.add(linkName);
			}
//			System.out.println("Property [" + property + "] informations added to cache.");
		}
		
		return propertyInformations;	
	}
}
