package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cVideoClient;

/**
 * @author R.Bacher, DESY
 */
public class Web2cRTPVideoConnector implements IWeb2cVideoClient {
	
	private ArrayList<Web2cRTPVideoReceiver> clientList = new ArrayList<Web2cRTPVideoReceiver>();
	private ArrayList<String> idList = new ArrayList<String>();
	
    /**
     * Constructor
     * RTP video connector
     */
	public Web2cRTPVideoConnector() {
	}
	
	/**
    * Closes the receiver for the address requested
    * @param address address
    */
    public void closeReceiver(String address) {
		
		int index = 0;
		Web2cRTPVideoReceiver client = null;
		
		index = idList.indexOf(address);
		client = clientList.get(index);
		
		client.closeReceiver(address);		
		clientList.remove(index);
		idList.remove(index);
    }
	
    /**
     * Connects to the video source of the address specified
     * @param address address
     */
	public void connectToVideoSource(String address) {

		idList.add(address);
		Web2cRTPVideoReceiver client = new Web2cRTPVideoReceiver();
		client.connectToVideoSource(address);
		clientList.add(client);
	}
	
	/**
 	 * Assigns the frame of the address requested to the output stream
 	 * @param frame output stream
	 * @param address address
	 */
    public void getVideo(ByteArrayOutputStream frame, String address) {
		
		int index = 0;
		Web2cRTPVideoReceiver client = null;
		
		index = idList.indexOf(address);
		client = clientList.get(index);
		
		client.getVideo(frame, address);	
    }
	
	/**
	 * Returns error string of the address requested
	 * @param address address
	 * @return error string
	 */
    public String getError(String address) {
		
		int index = 0;
		Web2cRTPVideoReceiver client = null;
		String errorString = "";
		
		index = idList.indexOf(address);
		client = clientList.get(index);
		
		errorString = client.getError(address);	
    	
    	return errorString;
    }
    
	/**
	 * Returns description for the address specified
	 * @param address address
	 * @return description
	 */
    public String getDescription(String address) {
		
		int index = 0;
		Web2cRTPVideoReceiver client = null;
		String description = "";
		
		index = idList.indexOf(address);
		client = clientList.get(index);
		
		description = client.getDescription(address);	
    	
    	return description;
    }
}
