package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.media.Buffer;
import javax.media.Codec;
import javax.media.Format;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/*
 * Class has been adapted from example software FrameAccess.java (Sun Microsystems, Inc.). 
 * See copyright notice below.
 * 
 * @(#)FrameAccess.java	1.5 01/03/13
 *
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * @author R.Bacher, DESY
 */
public class Web2cRTPVideoFrameGrabber implements Codec, IWeb2cConstants {

	private Format[] supportedVideoFormatIn = {defaultVideoFormat};
	private Format[] supportedVideoFormatOut = {defaultVideoFormat};
	private Format videoFormatIn = null;
	private Format videoFormatOut = null;
	private Buffer videoBuffer = null;

	
	/**
 	 * Constructor
 	 * Grants access to individual frames of an RTP video stream
 	 * */
	public Web2cRTPVideoFrameGrabber() {
		
		videoBuffer = new Buffer();

	}
	
	/**
 	 * Assigns the frame to the output stream
 	 * @param frame output stream
	 */
	public void assignFrame(ByteArrayOutputStream frame) {
		
		ObjectOutputStream object = null;
		
		try {
			ByteArrayOutputStream stream = new ByteArrayOutputStream();
			object = new ObjectOutputStream(stream);
			object.writeObject(videoBuffer.getData());
			object.flush(); 
			object.close(); 
			stream.close();
		    byte [] data = stream.toByteArray();
			int startMarkerIndex = 0;
			int stopMarkerIndex = 0;
			for (int i = 0; i < data.length; i++) {
				if (data[i] == (byte)0xff) {
					if (data[i + 1] == (byte)0xd8) startMarkerIndex = i; //jpeg frame start marker
			    	if (data[i + 1] == (byte)0xd9) stopMarkerIndex = i;  //jpeg frame stop marker
			    }
			}
			synchronized (frame) {
				frame.write(data, startMarkerIndex, stopMarkerIndex + 2 - startMarkerIndex); 
			}
		} 
		catch (IOException e) {
			System.out.println("Error converting video frame buffer to byte stream !");
		} 
	}

	/**
 	 * Returns the supported input video formats
 	 * @return supported input video formats
	 */
	public Format[] getSupportedInputFormats() {
	    return supportedVideoFormatIn;
	}

	/**
 	 * Returns the supported output video formats
 	 * @param format video format
 	 * @return supported output video formats
	 */
	public Format[] getSupportedOutputFormats(Format format) {
		
		Format[] supportedFormat = null;

	    if (format == null) {
	    	supportedFormat = supportedVideoFormatOut;
	    }
	    else {
	    	supportedFormat = new Format[1];
	    	supportedFormat[0] = format;
	    }
		
		return supportedFormat;
	}

	/**
 	 * Sets the input video format
 	 * @param format video format
 	 * @return input video format
	 */
	public Format setInputFormat(Format format) {	
		
		videoFormatIn = format;
		
		return videoFormatIn;
	}

	/**
 	 * Sets the output video format
 	 * @param format video format
 	 * @return output video format
	 */
	public Format setOutputFormat(Format format) {	
		
		videoFormatOut = format;
	
		return videoFormatOut;
	}

	/**
 	 * Does the data processing and assigns the data output to the captured frame
 	 * @param in input frame
 	 * @param out output frame
 	 * @return status
	 */
	public int process(Buffer in, Buffer out) {
		
	    Object data = null;
		
	    data = in.getData();
	    out.setData(data);

	    out.setFormat(in.getFormat());
	    out.setLength(in.getLength());
	    out.setOffset(in.getOffset());
	    
	    synchronized(videoBuffer) {
	    	videoBuffer = out;
	    }

	    return BUFFER_PROCESSED_OK;
	}
	
	/**
 	 * Not implemented
	 */
	public String getName() {
		
	    return "Web2cRTPClientFrameGrabber";
	}
	
	/**
 	 * Not implemented
	 */
    public void open() {
	}
    
	/**
 	 * Not implemented
	 */
	public void close() {
	}

	public void reset() {
	}

	/**
 	 * Not implemented
	 */
	public Object[] getControls() {
		
	    return new Object[0];
	}

	/**
 	 * Not implemented
	 */
	public Object getControl(String type) {
		
	    return null;
	}
}
