package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.io.ByteArrayOutputStream;
import javax.media.ConfigureCompleteEvent;
import javax.media.Controller;
import javax.media.ControllerEvent;
import javax.media.ControllerListener;
import javax.media.EndOfMediaEvent;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.PrefetchCompleteEvent;
import javax.media.Processor;
import javax.media.RealizeCompleteEvent;
import javax.media.ResourceUnavailableEvent;
import javax.media.control.TrackControl;
import javax.media.format.VideoFormat;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cVideoClient;

/*
 * Class has been adapted from example software FrameAccess.java (Sun Microsystems, Inc.). 
 * See copyright notice below.
 * 
 * @(#)FrameAccess.java	1.5 01/03/13
 *
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * @author R.Bacher, DESY
 */
public class Web2cRTPVideoReceiver implements ControllerListener, IWeb2cVideoClient {
	
    private Processor processor = null;
    private Web2cRTPVideoFrameGrabber[] frameGrabber = null;
    private Object waitSync = new Object();
    private boolean stateTransitionOK = true;
    private String errorString = "";
    
    /**
     * Constructor
     * Receives video stream via rtp and grants access to individual frames
     */
    public Web2cRTPVideoReceiver() {
    }
    
    /**
     * Closes the RTP receiver
     * @param address address
     */
    public void closeReceiver(String address) {
    	
    	processor.close();
    }
    
    /**
     * Connects to video source to the address specified
     * @param address address 
     */
    public void connectToVideoSource(String address) {
    	
    	int error = 0;
        MediaLocator mediaLocator = null;
        TrackControl trackControl[] = null;
    	TrackControl videoTrackControl = null;
        boolean status = false;

		String ip = address.split("/")[0];
		String port = address.split("/")[1];        
    	address = "rtp://" + ip + ":" + port +  "/video";
    		
    	try {    
    		mediaLocator = new MediaLocator(address); 	
			processor = Manager.createProcessor(Manager.createDataSource(mediaLocator));
        	processor.addControllerListener(this); 	
        	processor.configure(); 
        	status = waitForState(Processor.Configured);
        	if (status == false) error = error + 1; 	
    		processor.setContentDescriptor(null); 	
    		trackControl = processor.getTrackControls(); 
    		if (trackControl == null) error = error + 1;  
    		for (int i = 0; i < trackControl.length; i++) {
        	    if (trackControl[i].getFormat() instanceof VideoFormat) {
            		videoTrackControl = trackControl[i];
            	}
            } 	
            if (videoTrackControl == null) error = error + 1; 
            frameGrabber = new Web2cRTPVideoFrameGrabber[1];  
    		frameGrabber[0] = new Web2cRTPVideoFrameGrabber();
    		videoTrackControl.setCodecChain(frameGrabber);
        	processor.prefetch();   
        	status = waitForState(Controller.Prefetched);  
        	if (status == false) error = error + 1;    	
        	processor.start();   
    	} 
    	catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
    		error = error + 1;
    	}
    	
    	if (error == 0) {
    	   	System.out.println("Connected to video source " + address + " (" + videoTrackControl.getFormat() + ").");
    	}
    	else {
    	   	System.out.println("Failed to connect to video source " + address + "!");
			errorString = errorString + "[Video] Error establishing video connection " + address + ";";
    	}
    }
    
	/**
 	 * Assigns the frame to the output stream
 	 * @param frame output stream
 	 * @param address address
	 */
    public void getVideo(ByteArrayOutputStream frame, String address) {
    	
    	frameGrabber[0].assignFrame(frame);
    }
    
	/**
	 * Returns error string
	 * @param address address
	 * @return error string
	 */
    public String getError(String address) {
    	
    	return errorString;
    }
    
	/**
	 * Returns description for the address specified
	 * @param address address
	 * @return description
	 */
    public String getDescription(String address) {
    	
    	return "";
    }
    
    /**
     * Blocks until the processor has transitioned to the given state. 
     * Returns false if the transition failed.
     * @param state state waited for
     * @return status
     */
    boolean waitForState(int state) {
    	
    	int timeOut = 10000;
    	long startTime = 0;
    	long actualTime = 0;
    	
		startTime = System.currentTimeMillis();
		actualTime = startTime;
    	
    	synchronized (waitSync) {
    		try {
    			while (processor.getState() != state && stateTransitionOK) {
    				actualTime = System.currentTimeMillis();
    				if (actualTime > startTime + timeOut) {
    					System.out.println("RTPClient: Wait-for-State-Transition Timeout!");
    					break;
    				}
    				waitSync.wait(1000);
    			}
    				
    		} catch (Exception e) {}
    	}
    	
    	return stateTransitionOK;
    }

    /**
     * Controller Listener.
     * @param evt controller event
     */
	public void controllerUpdate(ControllerEvent evt) {

		if (evt instanceof ConfigureCompleteEvent || evt instanceof RealizeCompleteEvent || evt instanceof PrefetchCompleteEvent) {
		    synchronized (waitSync) {
//		    	System.out.println("Controller Event: Processor configure/realize/prefetch complete!");
		    	stateTransitionOK = true;
		    	waitSync.notifyAll();
		    }
		} 
		else if (evt instanceof ResourceUnavailableEvent) {
		    synchronized (waitSync) {
//		    	System.out.println("Controller Event: Resource unavailable!");
		    	stateTransitionOK = false;
		    	waitSync.notifyAll();
		    }
		} 
		else if (evt instanceof EndOfMediaEvent) {
//	    	System.out.println("Controller Event: End of media!");
		    processor.close();
		}
	}
}
