package de.desy.web2cToolkit.web2cArchive.web2cArchiveAccess;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cArchiveAccess;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import ttf.doocs.clnt.EqAdr;
import ttf.doocs.clnt.EqCall;
import ttf.doocs.clnt.EqData;
import ttf.doocs.clnt.time_t;

/**
 * @author R.Bacher, DESY
 */
public class Web2cDOOCSArchiveAcess implements IWeb2cArchiveAccess, IWeb2cConstants {

	/**
	 * DOOCS archive access plug
	 */
	public Web2cDOOCSArchiveAcess() {
	}

	public String getArchiveData(String address) {
		
		int i = 0;
		int err = 0;
		int[] inData = null;
		String parameter = null;
		String property = null;
		int numberOfDataPoints = 3600;
		int suppressionFactor = 1;
		int completionLength = 0;
		int totalLength = 0;
		int step = 0;
		String str = null;		
		StringBuffer strBuf = null;
		String[] strAsArray = null;
		String redStr = null;
		String context = "";
		String server = "";
		String device = "";
		String proper = "";
		int dataBeforeRequestedInterval = 0;
		int dataWithinRequestedInterval = 0;
		int timeOutSteps = 10; // 10 time intervals
		
		property = getLinkAddress(address);
		parameter = getLinkParameters(address);
		
		context = property.substring(1).split("/")[0];
		server = property.substring(1).split("/")[1];
		device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		proper = property.substring(property.lastIndexOf("/") + 1);
		String devName = "/" + context + "/" + server + "/" + device;
		String devProperty = proper;
		String linkName = devName + "/" + devProperty;
		
		inData = new int[(parameter.split(";")).length];
		for (i = 0; i < inData.length; i++) {
			inData[i] = (int)(Float.valueOf(parameter.split(";")[i]) / 1000.);
		}
		
		numberOfDataPoints = (numberOfDataPoints + 1) / (1 * suppressionFactor);
		if ((inData[1] - inData[0] + 1) < 3283200) {  //1month + 1week
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (912 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 691200) {   //1week + 1d
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (192 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 90000) {    //1d + 1h
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (25 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 3660) {    //1h + 1min
			numberOfDataPoints = inData[1] - inData[0] + 1 / (1 * suppressionFactor);
		}
		
		err = 0;
		completionLength = 0; 	
		totalLength = 0;
		str = "";	
		strBuf = new StringBuffer("");
		redStr = "";
		
		try {
			time_t from = new time_t();
		    time_t to = new time_t();
		    EqCall eqCall = new EqCall();
		    EqData eqOutData = new EqData();
		    EqData eqInData = new EqData();
		    EqAdr eqAddress = new EqAdr(linkName + ".HIST");
		    from.value =  inData[0];
		    to.value = inData[1]; 
		    time_t fromTime = new time_t();
		    time_t diffTime = new time_t();
		    time_t minTime = new time_t();
		    fromTime.value = from.value;
		    diffTime.value = to.value - from.value;
		    minTime.value = fromTime.value - (timeOutSteps * diffTime.value);  
		    
//		    System.out.println("inData[0]: " + inData[0] + " indata[1]: " + inData[1]);
		    
		    while (completionLength <=3 && fromTime.value >minTime.value) {  // if necessary, expand time interval to the past to get at least 3 data points
		    	from.value = fromTime.value;
		    	eqInData.set(fromTime, to, 0, 0);		
			    eqCall.get(eqAddress, eqInData, eqOutData);
			    completionLength = eqOutData.array_length();
			    fromTime.value = fromTime.value - diffTime.value;
		    }

//		    System.out.println("completionLength: " + completionLength);
//		    System.out.println("from:  " + from.value + " to: " + to.value + " interval: " + (to.value - from.value));
		    
		    completionLength = 2000;
		    while ((completionLength == 2000) && (from.value < to.value) && (err == 0)) {  // get data points within from.value<t<to.value
			    eqInData.set(from, to, 0, 0);		
			    eqCall.get(eqAddress, eqInData, eqOutData);
			    completionLength = eqOutData.array_length();
			    err = eqOutData.error();
			    if (err == 0) {
					for (i = 0; i < completionLength; i++) {
						if (eqOutData.get_time(i) != 0) {
							if ((eqOutData.get_time(i) / 1000) < inData[0]) {
									dataBeforeRequestedInterval = dataBeforeRequestedInterval + 1;
									strBuf.append("<value>" + eqOutData.get_float(i) + "</value><timeStamp>" + inData[0] + "</timeStamp>");
									if ((completionLength == 2000) && (i == (completionLength - 1))) {
										from.value = ((Long)(eqOutData.get_time(i) / 1000)).intValue() + 1;
									}
							}
							else {
								dataWithinRequestedInterval = dataWithinRequestedInterval + 1;
								if (i == (completionLength - 1)) {
									if (completionLength < 2000) {
										if (((Long)(eqOutData.get_time(i) / 1000)).intValue() > to.value) {
											strBuf.append("<value>" + eqOutData.get_float(i - 1) + "</value><timeStamp>" + to.value + "</timeStamp>");
										}
										if (((Long)(eqOutData.get_time(i) / 1000)).intValue() <= to.value) {
											strBuf.append("<value>" + eqOutData.get_float(i) + "</value><timeStamp>" + to.value + "</timeStamp>");
										}
										from.value = to.value + 1;
									}
									else {
										strBuf.append("<value>" + eqOutData.get_float(i) + "</value><timeStamp>" + eqOutData.get_time(i) / 1000 + "</timeStamp>");
										from.value = ((Long)(eqOutData.get_time(i) / 1000)).intValue() + 1;
									}
								}
								else {
									strBuf.append("<value>" + eqOutData.get_float(i) + "</value><timeStamp>" + eqOutData.get_time(i) / 1000 + "</timeStamp>");
								}
							}
						}
					}
				}
				else {
					System.out.println("DOOCSHistory link: Failed to get archived data from " + linkName + " [Error: " + eqOutData.get_string() + "]");
				}
		    }
		    str = strBuf.toString();
		    
//		    System.out.println("Original string: " + str);
//		    System.out.println("Number of data before requested interval: " + dataBeforeRequestedInterval + " Number of data within requested interval: " + dataWithinRequestedInterval);
		    
		    strAsArray = str.split("</timeStamp>");
		    for (i = 0; i < strAsArray.length - 1; i++) {
		    	strAsArray[i] = strAsArray[i] + "</timeStamp>";
		    }
		    if (dataWithinRequestedInterval == 0) {  //no data point within inData[0]<t<inData[1]
		    	String string = strAsArray[strAsArray.length - 1];
		    	str = string.split("<timeStamp>")[0] + "<timeStamp>" + inData[0] + "</timeStamp>" + string.split("<timeStamp>")[0] + "<timeStamp>" + inData[1] + "</timeStamp>";
		    	totalLength = 2;
		    }
		    else if (dataWithinRequestedInterval == 1) { //1 data point within inData[0]<t<inData[1]
		    	String string1 = strAsArray[strAsArray.length - 2];
		    	String string2 = strAsArray[strAsArray.length - 1];
		    	str = string1.split("<timeStamp>")[0] + "<timeStamp>" + inData[0] + "</timeStamp>" + string2.split("<timeStamp>")[0] + "<timeStamp>" + inData[1] + "</timeStamp>";
		    	totalLength = 2;
		    }
		    else if (dataWithinRequestedInterval == 2) {  //2 data points within inData[0]<t<inData[1]
		    	String string1 = strAsArray[strAsArray.length - 3];		    	
		    	String string2 = strAsArray[strAsArray.length - 2];
		    	String string3 = strAsArray[strAsArray.length - 1];
		    	str = string1.split("<timeStamp>")[0] + "<timeStamp>" + inData[0] + "</timeStamp>" + string2 + string3.split("<timeStamp>")[0] + "<timeStamp>" + inData[1] + "</timeStamp>";
		    	totalLength = 3;
		    }
		    else {  //more than 2 data points within inData[0]<t<inData[1]
		    	str = "";
		    	strBuf = new StringBuffer("");
		    	int j = 0;
		    	for (i = 0; i < strAsArray.length; i++) {
		    		int timeStamp = Integer.valueOf(strAsArray[i].split("<timeStamp>")[1].replace("</timeStamp>", ""));
		    		if (timeStamp > inData[0]) {
		    			if (j == 0) {
		    				String string = "";
		    				if (i > 0) {
		    					string = strAsArray[i - 1];
		    				}
		    				else {
		    					string = strAsArray[i];
		    				}
		    				strBuf.append(string.split("<timeStamp>")[0] + "<timeStamp>" + inData[0] + "</timeStamp>");
		    			}
		    			else {
		    				strBuf.append(strAsArray[i]);
		    			}
		    			j = j + 1;
		    		}
		    	}
		    	totalLength = j;
		    	str = strBuf.toString();
		    }
		    
//		    System.out.println("Total lenght: " + totalLength);
//		    System.out.println("Final string: " + str);
		    
		    strBuf = new StringBuffer("");
		    if (totalLength > numberOfDataPoints) {  // reduce number of data points
		    	step = ((Long)(((Double)Math.floor((double)totalLength / (double)numberOfDataPoints)).longValue() + 1)).intValue();
		    	for (i = 0; i < totalLength; i = i + step) {
		    		strBuf.append(str.split("</timeStamp>")[i] + "</timeStamp>");
		    	} 
		    	redStr = strBuf.toString();
		    }
		    else {
		    	redStr = str;
		    }
		    
//		    System.out.println("Number of data points (maximal): " + numberOfDataPoints);
//		    System.out.println("Reduced string: " + redStr);
		}
		catch (Exception e) {
			System.out.println("DOOCSHistory link: Failed to get archived data from " + property);
		}

		return redStr;
	}
	
	private String getLinkAddress(String address) {
		
		String property = null;
		
		property = address.substring(0,address.indexOf("("));
		
		return property;
	}
	
	private String getLinkParameters(String address) {

		String parameters = null;

		if (address.substring(address.indexOf("("),address.indexOf(")") + 1).length() != 2) {
			parameters = address.substring(address.indexOf("(") + 1, address.indexOf(")"));
		}
		else {
			parameters = null;
		}
		
		return parameters;
	}

}
