package de.desy.web2cToolkit.web2cArchive.web2cArchiveAccess;

import java.util.ArrayList;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.histUtils.THistory;
import de.desy.tine.queryUtils.TPropertyQuery;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.FLTINT;
import de.desy.tine.types.NAME16I;
import de.desy.tine.types.NAME32I;
import de.desy.tine.types.NAME64I;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cArchiveAccess;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cTINEArchiveAccess implements IWeb2cArchiveAccess, IWeb2cConstants {

	private ArrayList<TPropertyQuery[]> propertyInformationCache = null;
	private ArrayList<String> propertyCache = null;

	private int maxCacheLength = 1000;

	/**
	 * TINE archive access plug
	 */
	public Web2cTINEArchiveAccess() {
		
		propertyInformationCache = new ArrayList<TPropertyQuery[]>();
		propertyCache = new ArrayList<String>();
	}

	public String getArchiveData(String address) {
		
		int i = 0;
		int err = 0;
		int[] inData = null;
		NAME16I[] outDataNAME16 = null;
		NAME32I[] outDataNAME32 = null;
		NAME64I[] outDataNAME64 = null;
		FLTINT[] outDataFLT = null;
		FLTINT[] outDataINT = null;
		FLTINT[] outDataByte = null;
		String parameter = null;
		String property = null;
		int numberOfDataPoints = 3600;
		int suppressionFactor = 1;
		int completionLength = 0;
		String str = null;		
		StringBuffer strBuf = null;	
		String context = "";
		String server = "";
		String device = "";
		String proper = "";
		int linkTimeOut = 15000;  //15 s
		short format = -1;
		
		property = getLinkAddress(address);
		parameter = getLinkParameters(address);
		
		context = property.substring(1).split("/")[0];
		server = property.substring(1).split("/")[1];
		device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		proper = property.substring(property.lastIndexOf("/") + 1);
		String devName = "/" + context + "/" + server + "/" + device;
		String devProperty = proper;
		String linkName = devName + "/" + devProperty;
		
		inData = new int[(parameter.split(";")).length];
		for (i = 0; i < inData.length; i++) {
			inData[i] = (int)(Float.valueOf(parameter.split(";")[i]) / 1000.);
		}
		
		numberOfDataPoints = (numberOfDataPoints + 1) / (1 * suppressionFactor);
		if ((inData[1] - inData[0] + 1) < 3283200) {  //1month + 1week
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (912 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 691200) {   //1week + 1d
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (192 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 90000) {    //1d + 1h
			numberOfDataPoints = (inData[1] - inData[0] + 1) / (25 * suppressionFactor);
		}
		if ((inData[1] - inData[0] + 1) < 3660) {    //1h + 1min
			numberOfDataPoints = inData[1] - inData[0] + 1 / (1 * suppressionFactor);
		}
		
		err = 0;
		completionLength = 0; 	
		str = "";	
		strBuf = new StringBuffer("");

		try {
			TPropertyQuery[] propertyInformation = getPropertyInfos(property);
			if (propertyInformation != null) {
				format = propertyInformation[0].prpFormat;
			}
			
			if (TFormat.toString(format).equalsIgnoreCase("char")) {
				outDataNAME64 = new NAME64I[numberOfDataPoints];
				err = THistory.getArchivedData(context, null, proper, device, (double)inData[0], (double)inData[1], outDataNAME64, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataNAME64[i].ival != 0) {
						strBuf.append("<value>" + outDataNAME64[i].name + "</value><timeStamp>" + outDataNAME64[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("NAME16")){
				outDataNAME16 = new NAME16I[numberOfDataPoints];	
				err = THistory.getArchivedData(context, null, proper, device, (double)inData[0], (double)inData[1], outDataNAME16, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataNAME16[i].ival != 0) {
						strBuf.append("<value>" + outDataNAME16[i].name + "</value><timeStamp>" + outDataNAME16[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("NAME32")){
				outDataNAME32 = new NAME32I[numberOfDataPoints];	
				err = THistory.getArchivedData(context, null, proper, device, (double)inData[0], (double)inData[1], outDataNAME32, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataNAME32[i].ival != 0) {
						strBuf.append("<value>" + outDataNAME32[i].name + "</value><timeStamp>" + outDataNAME32[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("NAME64")){
				outDataNAME64 = new NAME64I[numberOfDataPoints];	
				err = THistory.getArchivedData(context, null, proper, device, (double)inData[0], (double)inData[1], outDataNAME64, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataNAME64[i].ival != 0) {
						strBuf.append("<value>" + outDataNAME64[i].name + "</value><timeStamp>" + outDataNAME64[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("float") || TFormat.toString(format).equalsIgnoreCase("double")){
				outDataFLT = new FLTINT[numberOfDataPoints];	
				err = THistory.getArchivedData(context, null, proper, device, 0, (double)inData[0], (double)inData[1], outDataFLT, linkTimeOut);		
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataFLT[i].ival != 0) {
							strBuf.append("<value>" + outDataFLT[i].fval + "</value><timeStamp>" + outDataFLT[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("int32") || TFormat.toString(format).equalsIgnoreCase("int16")){
				outDataINT = new FLTINT[numberOfDataPoints];
				err = THistory.getArchivedData(context, null, proper, device, 0, (double)inData[0], (double)inData[1], outDataINT, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataINT[i].ival != 0) {
						strBuf.append("<value>" + outDataINT[i].fval + "</value><timeStamp>" + outDataINT[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else if (TFormat.toString(format).equalsIgnoreCase("byte")){
				outDataByte = new FLTINT[numberOfDataPoints];	
				err = THistory.getArchivedData(context, null, proper, device, 0, (double)inData[0], (double)inData[1], outDataByte, linkTimeOut);
				if (err > 0) {
					completionLength = err;   
					for (i = 0; i < completionLength; i++) {
//						if (outDataByte[i].fval != 0) {
						strBuf.append("<value>" + outDataByte[i].fval + "</value><timeStamp>" + outDataByte[i].ival + "</timeStamp>");
//						}
					}
				}
				else {
//					System.out.println("THistory link: Failed to get archived data from " + linkName + " [Error: " + TErrorList.getErrorString(err) + "]");
				}
			}
			else {
//				System.out.println(" THistory Link: data format " +  format + " = " + TFormat.toString(format) + " not implemented!");
			}
		}
		catch (Exception e) {
//			System.out.println("THistory link: Failed to get archived data from " + property);
		}
		
		str = strBuf.toString();
		
		return str;
	}
	
	private String getLinkAddress(String address) {
		
		String property = null;
		
		property = address.substring(0,address.indexOf("("));
		
		return property;
	}
	
	private String getLinkParameters(String address) {

		String parameters = null;

		if (address.substring(address.indexOf("("),address.indexOf(")") + 1).length() != 2) {
			parameters = address.substring(address.indexOf("(") + 1, address.indexOf(")"));
		}
		else {
			parameters = null;
		}
		
		return parameters;
	}
	
	/**
	 * Returns property informations
	 * @param property property
	 * @return property informations
	 */
	private TPropertyQuery[] getPropertyInfos(String property) {
		
		TPropertyQuery[] propertyInformations = null;

		String context = property.substring(1).split("/")[0];
		String server = property.substring(1).split("/")[1];
		String device = property.substring(context.length() + server.length() + 3, property.lastIndexOf("/"));
		String proper = property.substring(property.lastIndexOf("/") + 1);
		String linkName = "/" + context + "/" + server + "/" + device + "/" + proper;
		
		if (propertyCache.toArray().length > maxCacheLength) {
			propertyInformationCache.clear();
			propertyCache.clear();
		}

		if (propertyCache.contains(linkName)) {
			propertyInformations  = (TPropertyQuery[])(propertyInformationCache.toArray()[propertyCache.indexOf(linkName)]);
//			System.out.println("Property [" + property + "] informations existing in cache.");
		}
		else {
			propertyInformations  = TQuery.getPropertyInformation(context, server, device, proper, 1000);
			if (propertyInformations != null) {
				propertyInformationCache.add(propertyInformations);
				propertyCache.add(linkName);
			}
//			System.out.println("Property [" + property + "] informations added to cache.");
		}
		
		return propertyInformations;	
	}

}
