package de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Calendar;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cArchiveViewerDataSelector implements IWeb2cConstants {

	private String hostURL = null;
	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop  = null;
	private String backgroundColor = null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private String toolTipText = null;
	private String fromYear = null;
	private String fromMonth = null;
	private String fromDay = null;
	private String fromHour = null;
	private String fromMinute = null;
	private String fromSecond = null;
	private String toYear = null;
	private String toMonth = null;
	private String toDay = null;
	private String toHour = null;
	private String toMinute = null;
	private String toSecond = null;
	private String fromTime = null;
	private String toTime = null;
	private ArrayList<String> userDefinedDataEntryName = null;
	private ArrayList<Integer> userDefinedDataEntryAddressIndex = null;
	private ArrayList<String> userDefinedDataPlotType = null;
	private ArrayList<String> userDefinedDataPlotMin = null;
	private ArrayList<String> userDefinedDataPlotMax = null;
	private ArrayList<String> preDefinedDataEntryName = null;
	private ArrayList<Integer> preDefinedDataNumberOfSubEntries = null;
	private ArrayList<ArrayList<String>> preDefinedDataSubEntryName = null;
	private ArrayList<ArrayList<Integer>> preDefinedDataSubEntryAddressIndex = null;
	private ArrayList<ArrayList<String>> preDefinedDataSubPlotType = null;
	private ArrayList<ArrayList<String>> preDefinedDataSubPlotMin = null;
	private ArrayList<ArrayList<String>> preDefinedDataSubPlotMax = null;
	private ArrayList<String> dataAddressString = null;
	private ArrayList<Integer> dataAddressID = null;
	private String defaultSingleEntry = "";
	private String defaultCompositeEntry = "";
	private String defaultEntry = "";
	private String singleChecked = "";
	private String compositeChecked = "checked";
	private String selectionType = "predefined";
	private String componentVisible = visible;
	
	/**
	 * Data Selector
	 * @param url URL of host servicing the request
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param from start time of archive data interval
	 * @param to end time of archive data interval
	 */
	public Web2cArchiveViewerDataSelector(String url, String refId, ArrayList<String> names, ArrayList<String> values, String from, String to) {
		
		hostURL = url;
		referenceId = refId;
		fromTime = from;
		toTime = to;
		decodeAttributes(names, values);
	}
	
	/**
	 * @return component id
	 */
	public String getId() {
		
		return id;
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;
		
		userDefinedDataEntryName = new ArrayList<String>();
		userDefinedDataEntryAddressIndex = new ArrayList<Integer>();
		userDefinedDataPlotType = new ArrayList<String>();
		userDefinedDataPlotMin = new ArrayList<String>();
		userDefinedDataPlotMax = new ArrayList<String>();
		preDefinedDataEntryName = new ArrayList<String>();
		preDefinedDataNumberOfSubEntries = new ArrayList<Integer>();
		preDefinedDataSubEntryName = new ArrayList<ArrayList<String>>();
		preDefinedDataSubEntryAddressIndex = new ArrayList<ArrayList<Integer>>();
		preDefinedDataSubPlotType = new ArrayList<ArrayList<String>>();
		preDefinedDataSubPlotMin = new ArrayList<ArrayList<String>>();
		preDefinedDataSubPlotMax = new ArrayList<ArrayList<String>>();
		dataAddressString = new ArrayList<String>();
		dataAddressID = new ArrayList<Integer>();

		backgroundColor = white;
		borderStyle = none;
		borderWidth = thin;
		borderColor = black;
		toolTipText = "";
		defaultSingleEntry = "";
		defaultCompositeEntry = "";

		xLeft = 0;
		yTop = 0;
		xRight = 1260;
		yBottom = 470;
		marginLeft = ((Integer)(xLeft)).toString() + "px";
		marginTop = ((Integer)(yTop)).toString() + "px";
		width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
		height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}
			if (name.equals("bounds") && !value.equals("")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}		
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}					
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}		
			if (name.equals("uri")) {
				dataAddressID.add(Integer.valueOf(value.split(",")[0]));
				dataAddressString.add(value.split(",")[1]);
			}	
			if (name.equals("singleEntry")) {
				userDefinedDataEntryName.add(value.split(",")[0]);
				userDefinedDataEntryAddressIndex.add(Integer.valueOf(value.split(",")[1]));
				userDefinedDataPlotType.add(value.split(",")[2]);
				userDefinedDataPlotMin.add(value.split(",")[3]);
				userDefinedDataPlotMax.add(value.split(",")[4]);
			}		
			if (name.equals("compositeEntry")) {
				preDefinedDataEntryName.add(value.split(",")[0]);
				preDefinedDataNumberOfSubEntries.add((value.split(",").length - 1) / 5);
				ArrayList<String> entryName = new ArrayList<String>();
				ArrayList<Integer> entryAddressIndex = new ArrayList<Integer>();
				ArrayList<String> plotType = new ArrayList<String>();
				ArrayList<String> plotMin = new ArrayList<String>();
				ArrayList<String> plotMax = new ArrayList<String>();
				for (int j = 1; j < value.split(",").length; j = j + 5) {
					entryName.add(value.split(",")[j]);
					entryAddressIndex.add(Integer.valueOf(value.split(",")[j+1]));
					plotType.add(value.split(",")[j+2]);
					plotMin.add(value.split(",")[j+3]);
					plotMax.add(value.split(",")[j+4]);
				}
				preDefinedDataSubEntryName.add(entryName);
				preDefinedDataSubEntryAddressIndex.add(entryAddressIndex);
				preDefinedDataSubPlotType.add(plotType);
				preDefinedDataSubPlotMin.add(plotMin);
				preDefinedDataSubPlotMax.add(plotMax);
			}				
			if (name.equals("defaultSingleEntry")) {
				defaultSingleEntry = value;
			}			
			if (name.equals("defaultCompositeEntry")) {
				defaultCompositeEntry = value;
			}			
			if (name.equals("visible")) {
				componentVisible = value;
			}	
		}		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
		
		singleChecked = "";
		compositeChecked = "checked";
		if (!defaultSingleEntry.equals("")) {
			singleChecked = "checked";
			compositeChecked = "";		
			if (componentVisible.equals(visible)) {
				defaultSingleEntry = defaultSingleEntry + "\n";
			}
			else {
				defaultSingleEntry = defaultSingleEntry + ",";
			}
			defaultEntry = defaultSingleEntry;
			selectionType = "userdefined";
		}
		if (!defaultCompositeEntry.equals("")) {
			singleChecked = "";
			compositeChecked = "checked";
			defaultEntry = defaultCompositeEntry;
			selectionType = "predefined";
		}

		if (!fromTime.equals("")) {
			String date = fromTime.split(" ")[0];
			fromYear = date.split("-")[0];
			fromMonth = String.valueOf(Integer.valueOf(date.split("-")[1]) - 1);
			fromDay = date.split("-")[2];
			String time = fromTime.split(" ")[1];
			fromHour = time.split(":")[0];
			fromMinute = time.split(":")[1];
			fromSecond = time.split(":")[2];
		}
		else {
			fromTime = "";
		}
		if (!toTime.equals("")) {
			String date = toTime.split(" ")[0];
			toYear = date.split("-")[0];
			toMonth = String.valueOf(Integer.valueOf(date.split("-")[1]) - 1);
			toDay = date.split("-")[2];
			String time = toTime.split(" ")[1];
			toHour = time.split(":")[0];
			toMinute = time.split(":")[1];
			toSecond = time.split(":")[2];
		}
		else {
			toTime = "";
		}
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		int i = 0;
		int actualYear = 0;
		int actualMonth = 0;
		int actualDay = 0;
		String selectedOption = "";
		ArrayList<String> componentHTML = new ArrayList<String>();
		DecimalFormat formatter;
		
		actualYear = Calendar.getInstance().get(Calendar.YEAR);
		actualMonth = Calendar.getInstance().get(Calendar.MONTH) + 1;
		actualDay = Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
//		actualYear = Integer.valueOf(Calendar.getInstance().getTime().toString().split(" ")[5]);
//		actualMonth = Integer.valueOf(Calendar.getInstance().getTime().toString().split(" ")[5]);
//		actualDay = Integer.valueOf(Calendar.getInstance().getTime().toString().split(" ")[2]);
		
		formatter = new DecimalFormat("00", new DecimalFormatSymbols(localSettings));

		if (componentVisible.equals(visible)) {
			componentHTML.add("                <div id=\"" + id + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");
			componentHTML.add("                    <div id=\"" + id + "_dataSelectionHeader\" style=\"position:absolute; left:10px; top:10px; width:200px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">" + "Data Selection" + "</div>");
			componentHTML.add("                    <input type=\"radio\" id=\"" + id + "_userDefinedDataSelectionOption\" name=\"dataSelectionOption\" value=\"user\" " +  singleChecked + " style=\"position:absolute; left:0px; top:62px; width:50px; height:30px; color:" + black + "; background-color:" + white + "; font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">");
			componentHTML.add("                    <div id=\"" + id + "_userDefinedDataSelection\" style=\"position:absolute; left:50px; top:70px; width:200px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "User-defined selection:" + "</div>");
			componentHTML.add("                    <select id=\"" + id + "_userDefinedDataSelectionField\" size=\"3\" style=\"position:absolute; left:250px; top:50px; width:350px; height:80px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">");
			for (i = 0; i < userDefinedDataEntryName.toArray().length; i++) {
				if (i == 0) {
			        componentHTML.add("                        <option id=\"" + id + "_userDefinedDataSelectionOption_" + i + "\" selected=\"selected\" value=\"" + userDefinedDataEntryName.toArray()[i] + "\">" + userDefinedDataEntryName.toArray()[i] + "</option>");
				}
				else {
					componentHTML.add("                        <option id=\"" + id + "_userDefinedDataSelectionOption_" + i + "\" value=\"" + userDefinedDataEntryName.toArray()[i] + "\">" + userDefinedDataEntryName.toArray()[i] + "</option>");
				}
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_userDefinedDataSelectButton\" value=\"&gt;&gt;\" onclick=\"" + id + "_selectUserDefinedData(" + id + "_userDefinedDataSelectionField," + id + "_userDefinedSelectedDataField)\" style=\"position:absolute; left:630px; top:70px; width:40px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <textarea id=\"" + id + "_userDefinedSelectedDataField\" rows=\"3\" cols=\"30\" style=\"position:absolute; left:710px; top:50px; width:350px; height:80px; color:" + black + "; background-color:" + white + "; font-family: Helvetica, Arial, sans-serif; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + defaultSingleEntry + "</textarea>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_userDefinedDataResetButton\" value=\"clear selection\" onclick=\"" + id + "_clearSelectedData(" + id + "_userDefinedSelectedDataField)\" style=\"position:absolute; left:1090px; top:70px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey"+ "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"radio\" id=\"" + id + "_preDefinedDataSelectionOption\" name=\"dataSelectionOption\" value=\"predefined\" " +  compositeChecked + " style=\"position:absolute; left:0px; top:162px; width:50px; height:30px; color:" + black + "; background-color:" + white + "; font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">");
			componentHTML.add("                    <div id=\"" + id + "_preDefinedDataSelection\" style=\"position:absolute; left:50px; top:170px; width:200px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Predefined selection:" + "</div>");
			componentHTML.add("                    <select id=\"" + id + "_preDefinedDataSelectionField\" size=\"3\" style=\"position:absolute; left:250px; top:150px; width:350px; height:80px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + ";\">");
			for (i = 0; i < preDefinedDataEntryName.toArray().length; i++) {
				if (i == 0) {
			        componentHTML.add("                        <option id=\"" + id + "_preDefinedDataSelectionOption_" + i + "\" selected=\"selected\" value=\"" + preDefinedDataEntryName.toArray()[i] + "\">" + preDefinedDataEntryName.toArray()[i] + "</option>");
				}
				else {
					componentHTML.add("                        <option id=\"" + id + "_preDefinedDataSelectionOption_" + i + "\" value=\"" + preDefinedDataEntryName.toArray()[i] + "\">" + preDefinedDataEntryName.toArray()[i] + "</option>");
				}
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_preDefinedDataSelectButton\" value=\"&gt;&gt;\" onclick=\"" + id + "_selectPreDefinedData(" + id + "_preDefinedDataSelectionField," + id + "_preDefinedSelectedDataField)\" style=\"position:absolute; left:630px; top:170px; width:40px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <textarea id=\"" + id + "_preDefinedSelectedDataField\" rows=\"3\" cols=\"30\" style=\"position:absolute; left:710px; top:150px; width:350px; height:80px; color:" + black + "; background-color:" + white + "; font-family: Helvetica, Arial, sans-serif; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + defaultCompositeEntry + "</textarea>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_preDefinedDataResetButton\" value=\"clear selection\" onclick=\"" + id + "_clearSelectedData(" + id + "_preDefinedSelectedDataField)\" style=\"position:absolute; left:1090px; top:170px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <div id=\"" + id + "_timePeriodSelectionHeader\" style=\"position:absolute; left:10px; top:250px; width:400px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">" + "Time Period Selection" + "</div>");
			componentHTML.add("                    <input type=\"radio\" id=\"" + id + "_archiveSelectionOption\" name=\"periodSelectionOption\" value=\"archive\" style=\"position:absolute; left:0px; top:297px; width:50px; height:30px; color:" + black + "; background-color:" + white + "; font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">");
			componentHTML.add("                    <div id=\"" + id + "_archive\" style=\"position:absolute; left:50px; top:305px; width:100px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Archive:" + "</div>");
			componentHTML.add("                    <div id=\"" + id + "_Year\" style=\"position:absolute; left:130px; top:305px; width:100px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Year" + "</div>");
			componentHTML.add("                    <div id=\"" + id + "_Month\" style=\"position:absolute; left:210px; top:305px; width:80px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Month" + "</div>");
			componentHTML.add("                    <div id=\"" + id + "_Day\" style=\"position:absolute; left:290px; top:305px; width:80px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Day" + "</div>");
			componentHTML.add("                    <div id=\"" + id + "_Hour\" style=\"position:absolute; left:370px; top:305px; width:80px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Hour" + "</div>");
			componentHTML.add("                    <div id=\"" + id + "_Minute\" style=\"position:absolute; left:450px; top:305px; width:80px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Minute" + "</div>");
			componentHTML.add("                    <select id=\"" + id + "_yearSelectionField\" size=\"1\" style=\"position:absolute; left:130px; top:335px; width:65px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			for (i = actualYear; i >= actualYear - 10; i--) {
				if (i == actualYear) selectedOption = " selected";
				else selectedOption = "";
			    componentHTML.add("                        <option id=\"" + id + "_yearSelectionOption_" + i + "\" " + selectedOption + " value=\"" + i + "\">" + i + "</option>");	
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <select id=\"" + id + "_monthSelectionField\" size=\"1\" style=\"position:absolute; left:210px; top:335px; width:65px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			for (i = 1; i <= 12; i++) {
				if (i == actualMonth) selectedOption = " selected";
				else selectedOption = "";
			    componentHTML.add("                        <option id=\"" + id + "_monthSelectionOption_" + i + "\" " + selectedOption + " value=\"" + formatter.format(i)+ "\">" + formatter.format(i) + "</option>");	
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <select id=\"" + id + "_daySelectionField\" size=\"1\" style=\"position:absolute; left:290px; top:335px; width:65px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			for (i = 1; i <= 31; i++) {
				if (i == actualDay) selectedOption = " selected";
				else selectedOption = "";
			    componentHTML.add("                        <option id=\"daySelectionOption_" + i + "\" " + selectedOption + " value=\"" + formatter.format(i)+ "\">" + formatter.format(i) + "</option>");	
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <select id=\"" + id + "_hourSelectionField\" size=\"1\" style=\"position:absolute; left:370px; top:335px; width:65px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			for (i = 0; i <= 23; i++) {
			    componentHTML.add("                        <option id=\"" + id + "_hourSelectionOption_" + i + "\" value=\"" + formatter.format(i)+ "\">" + formatter.format(i) + "</option>");	
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <select id=\"" + id + "_minuteSelectionField\" size=\"1\" style=\"position:absolute; left:450px; top:335px; width:65px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			for (i = 0; i <= 59; i++) {
			    componentHTML.add("                        <option id=\"" + id + "_minuteSelectionOption_" + i + "\" value=\"" + formatter.format(i)+ "\">" + formatter.format(i) + "</option>");	
			}
			componentHTML.add("                    </select>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_selectFromButton\" value=\"from\" onclick=\"" + id + "_selectArchivePeriod(" + id + "_yearSelectionField," + id + "_monthSelectionField," + id + "_daySelectionField," + id + "_hourSelectionField," + id + "_minuteSelectionField," + id + "_fromField)\" style=\"position:absolute; left:540px; top:335px; width:50px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"text\" id=\"" + id + "_fromField\" size=\"20\" style=\"position:absolute; left:600px; top:335px; width:200px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_selectToButton\" value=\"to\" onclick=\"" + id + "_selectArchivePeriod(" + id + "_yearSelectionField," + id + "_monthSelectionField," + id + "_daySelectionField," + id + "_hourSelectionField," + id + "_minuteSelectionField," + id + "_toField)\" style=\"position:absolute; left:815px; top:335px; width:50px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"text\" id=\"" + id + "_toField\" size=\"20\" style=\"position:absolute; left:875px; top:335px; width:200px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_archiveResetButton\" value=\"clear selection\" onclick=\"" + id + "_clearArchivePeriod(" + id + "_fromField," + id + "_toField)\" style=\"position:absolute; left:1090px; top:335px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"radio\" id=\"" + id + "_liveSelectionOption\" name=\"periodSelectionOption\" value=\"live\" checked style=\"position:absolute; left:0px; top:382px; width:50px; height:30px; color:" + black + "; background-color:" + white + "; font-size:" + large + "; font-weight:" + bold + "; text-align:" + left + "\">");
			componentHTML.add("                    <div id=\"" + id + "_Live\" style=\"position:absolute; left:50px; top:390px; width:100px; height:30px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + left + "\">" + "Live:" + "</div>");
			componentHTML.add("                    <select id=\"" + id + "_liveSelectionField\" size=\"1\" style=\"position:absolute; left:130px; top:390px; width:90px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			componentHTML.add("                        <option id=\"" + id + "_liveSelectionOption_0\" value=\"last 12h\">" + "last 12h" + "</option>");	
			componentHTML.add("                        <option id=\"" + id + "_liveSelectionOption_1\" value=\"last 24h\">" + "last 24h" + "</option>");
			componentHTML.add("                        <option id=\"" + id + "_liveSelectionOption_2\" value=\"last 48h\">" + "last 48h" + "</option>");		
			componentHTML.add("                    </select>");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_liveSelectButton\" value=\"&gt;&gt;\" onclick=\"" + id + "_selectLivePeriod(" + id + "_liveSelectionField," + id + "_liveField)\" style=\"position:absolute; left:540px; top:390px; width:50px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"text\" id=\"" + id + "_liveField\" size=\"10\" value=\"" + "last 24h" + "\" style=\"position:absolute; left:600px; top:390px; width:200px; height:25px; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + right + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_liveResetButton\" value=\"clear selection\" onclick=\"" + id + "_clearLivePeriod(" + id + "_liveField)\" style=\"position:absolute; left:1090px; top:390px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_getPlotButton\" value=\"get data (plot)\" onclick=\"" + referenceId + "_getDataAsPlot()\" style=\"position:absolute; left:440px; top:435px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_getTableButton\" value=\"get data (table)\" onclick=\"" + referenceId + "_getDataAsTable()\" style=\"position:absolute; left:640px; top:435px; width:140px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                </div>");
		}
		else {
			componentHTML.add("");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("            //ArchiveViewerDataSelector: " + id);
		
		componentJavaScript.add("                var " + referenceId + "_dataSelectorId = \"" + id + "\";");
		if (componentVisible.equals(visible)) {
			if (!defaultSingleEntry.equals("") || !defaultCompositeEntry.equals("")) {
				componentJavaScript.add("                var " + referenceId + "_defaultSelection = true;");	
			}
			else {
				componentJavaScript.add("                var " + referenceId + "_defaultSelection = false;");
			}
		}
		else {
			componentJavaScript.add("                var " + referenceId + "_defaultSelection = true;");
		}
		
		if (componentVisible.equals(visible)) {
			componentJavaScript.add("                function " + id + "_selectUserDefinedData(selectedData,dataText) {");
			componentJavaScript.add("                	var data = selectedData.options[selectedData.options.selectedIndex].value;");
			componentJavaScript.add("                	dataText.value += data + \"\\n\";");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_selectPreDefinedData(selectedData,dataText) {");
			componentJavaScript.add("                	var data = selectedData.options[selectedData.options.selectedIndex].value;");
			componentJavaScript.add("                	dataText.value = data;");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_selectArchivePeriod(selectedYear,selectedMonth,selectedDay,selectedHour,selectedMinute,dataText) {");
			componentJavaScript.add("                	var year = selectedYear.options[selectedYear.options.selectedIndex].value;");
			componentJavaScript.add("                	var month = selectedMonth.options[selectedMonth.options.selectedIndex].value;");
			componentJavaScript.add("                	var day = selectedDay.options[selectedDay.options.selectedIndex].value;");
			componentJavaScript.add("                	var hour = selectedHour.options[selectedHour.options.selectedIndex].value;");
			componentJavaScript.add("                	var minute = selectedMinute.options[selectedMinute.options.selectedIndex].value;");
			componentJavaScript.add("                	dataText.value = year + \"-\" + month + \"-\" + day + \" \" + hour + \":\" + minute + \":00\";");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_selectLivePeriod(selectedPeriod,dataText) {");
			componentJavaScript.add("                	var period = selectedPeriod.options[selectedPeriod.options.selectedIndex].value;");
			componentJavaScript.add("                	dataText.value = period;");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_clearSelectedData(dataText) {");
			componentJavaScript.add("                	dataText.value = \"\";");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_clearArchivePeriod(from,to) {");
			componentJavaScript.add("                	from.value = \"\";");
			componentJavaScript.add("                	to.value = \"\";");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_clearLivePeriod(period) {");
			componentJavaScript.add("                	period.value = \"\";");
			componentJavaScript.add("                }");
		}
		
		componentJavaScript.add("                function " + referenceId + "_getSelectedData() {");		
		if (componentVisible.equals(visible)) {
			componentJavaScript.add("                	if (document.getElementById(\"" + id + "_userDefinedDataSelectionOption\").checked) {");
			componentJavaScript.add("                		" + referenceId + "_selectionType = \"userdefined\";");
			componentJavaScript.add("                		" + referenceId + "_selectedData = document.getElementById(\"" + id + "_userDefinedSelectedDataField\").value;");
			componentJavaScript.add("                	}");
			componentJavaScript.add("                	if (document.getElementById(\"" + id + "_preDefinedDataSelectionOption\").checked) {");
			componentJavaScript.add("                		" + referenceId + "_selectionType = \"predefined\";");
			componentJavaScript.add("                		" + referenceId + "_selectedData = document.getElementById(\"" + id + "_preDefinedSelectedDataField\").value;");
			componentJavaScript.add("                	}");
		}
		else {
			componentJavaScript.add("                	" + referenceId + "_selectedData = \"" + defaultEntry + "\";");
			if (selectionType.equals("userdefined")) {
				componentJavaScript.add("                	" + referenceId + "_selectionType = \"userdefined\";");
			}
			else {
				componentJavaScript.add("                	" + referenceId + "_selectionType = \"predefined\";");
			}
		}
		componentJavaScript.add("                }");		
		
		componentJavaScript.add("                function " + referenceId + "_getSelectedPeriod() {");
		if (componentVisible.equals(visible)) {
			componentJavaScript.add("                	if (document.getElementById(\"" + id + "_archiveSelectionOption\").checked) {");
			componentJavaScript.add("                		var fromField = document.getElementById(\"" + id + "_fromField\").value;");
			componentJavaScript.add("                		var toField = document.getElementById(\"" + id + "_toField\").value;");
			componentJavaScript.add("                		var str = fromField.split(\" \");");
			componentJavaScript.add("                		var year = str[0].split(\"-\")[0];");
			componentJavaScript.add("                		var month = String(Number(str[0].split(\"-\")[1]) - 1);");
			componentJavaScript.add("                		var day = str[0].split(\"-\")[2];");
			componentJavaScript.add("                		var hour = str[1].split(\":\")[0];");
			componentJavaScript.add("                		var minute = str[1].split(\":\")[1];");
			componentJavaScript.add("                		var second = str[1].split(\":\")[2];");
			componentJavaScript.add("                	    var fromDate = new Date(year, month, day, hour, minute, second);");
			componentJavaScript.add("                		str = toField.split(\" \");");
			componentJavaScript.add("                		year = str[0].split(\"-\")[0];");
			componentJavaScript.add("                		month = String(Number(str[0].split(\"-\")[1]) - 1);");
			componentJavaScript.add("                		day = str[0].split(\"-\")[2];");
			componentJavaScript.add("                		hour = str[1].split(\":\")[0];");
			componentJavaScript.add("                		minute = str[1].split(\":\")[1];");
			componentJavaScript.add("                		second = str[1].split(\":\")[2];");
			componentJavaScript.add("                	    var toDate = new Date(year, month, day, hour, minute, second);");
			componentJavaScript.add("                	}");
			componentJavaScript.add("                	if (document.getElementById(\"" + id + "_liveSelectionOption\").checked) {");
			componentJavaScript.add("                		var fromDate = new Date();");
			componentJavaScript.add("                		var toDate = new Date();");
			componentJavaScript.add("                		var liveField = document.getElementById(\"" + id + "_liveField\").value;");
			componentJavaScript.add("                		if (liveField == \"last 12h\") {");
			componentJavaScript.add("                			fromDate.setTime(toDate.getTime() - 43200000);");
			componentJavaScript.add("                		}");
			componentJavaScript.add("                		if (liveField == \"last 24h\") {");
			componentJavaScript.add("                			fromDate.setTime(toDate.getTime() - 86400000);");
			componentJavaScript.add("                		}");
			componentJavaScript.add("                		if (liveField == \"last 48h\") {");
			componentJavaScript.add("                			fromDate.setTime(toDate.getTime() - 172800000);");
			componentJavaScript.add("                		}");
			componentJavaScript.add("                	}");
			componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = fromDate.getTime();");
			componentJavaScript.add("                	" + referenceId + "_selectedToInMillis = toDate.getTime();");
			componentJavaScript.add("                	" + referenceId + "_selectedFromInMillisOld = " + referenceId + "_selectedFromInMillis;");
			componentJavaScript.add("                	" + referenceId + "_selectedToInMillisOld = " + referenceId + "_selectedToInMillis;");
		}
		else {
			if (fromTime.equals("") || toTime.equals("")) {
				componentJavaScript.add("                	var fromDate = new Date();");
				componentJavaScript.add("                	var toDate = new Date();");
				componentJavaScript.add("                	fromDate.setTime(toDate.getTime() - 86400000);");
			}
			else {
				componentJavaScript.add("                	    var fromDate = new Date(" + fromYear + ", " + fromMonth + ", " + fromDay + ", " + fromHour + ", " + fromMinute + ", " + fromSecond + ");");
				componentJavaScript.add("                	    var toDate = new Date(" + toYear + ", " + toMonth + ", " + toDay + ", " + toHour + ", " + toMinute + ", " + toSecond + ");");
			}
			componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = fromDate.getTime();");
			componentJavaScript.add("                	" + referenceId + "_selectedToInMillis = toDate.getTime();");
			componentJavaScript.add("                	" + referenceId + "_selectedFromInMillisOld = " + referenceId + "_selectedFromInMillis;");
			componentJavaScript.add("                	" + referenceId + "_selectedToInMillisOld = " + referenceId + "_selectedToInMillis;");
		}
		componentJavaScript.add("                }");
		
		componentJavaScript.add("        			function " + referenceId + "_getDataChannelNames(parameter) {");
		componentJavaScript.add("            			var reg = /\\n/g;");
		componentJavaScript.add("		    			var now = new Date();");		
		componentJavaScript.add("            			if ((" + referenceId + "_selectedData != \"\") && (" + referenceId + "_selectedFromInMillis > 0) && (" + referenceId + "_selectedToInMillis > " + referenceId + "_selectedFromInMillis)) {");
		componentJavaScript.add("            				try {");
		componentJavaScript.add("                				if(" + referenceId + "_selectionType == \"userdefined\") {");
		componentJavaScript.add("                    				" + referenceId + "_xhr.open(\"post\", \"/web2cToolkit/Web2cArchiveViewer?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><value>channelNames</value><value>\" + " + referenceId + "_dataSelectorId + \"</value><value>\" + " + referenceId + "_selectionType + \"</value><value>\" + " + referenceId + "_selectedData.replace(reg,\",\").substring(0," + referenceId + "_selectedData.replace(reg,\",\").length - 1) + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		componentJavaScript.add("                				}");
		componentJavaScript.add("                				if(" + referenceId + "_selectionType == \"predefined\") {");
		componentJavaScript.add("                    				" + referenceId + "_xhr.open(\"post\", \"/web2cToolkit/Web2cArchiveViewer?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><value>channelNames</value><value>\" + " + referenceId + "_dataSelectorId + \"</value><value>\" + " + referenceId + "_selectionType + \"</value><value>\" + " + referenceId + "_selectedData + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
		componentJavaScript.add("                				}");
		componentJavaScript.add("                				" + referenceId + "_xhr.onreadystatechange = function() {"); 
		componentJavaScript.add("                   				if (" + referenceId + "_xhr.readyState == 4) {"); 
		componentJavaScript.add("										" + referenceId + "_dataChannelNames = " + referenceId + "_xhr.responseText.substring(0, " + referenceId + "_xhr.responseText.length);");
		componentJavaScript.add("										if (parameter == \"plot\") {");
		componentJavaScript.add("        									" + referenceId + "_plotReloadCounter = 0;");
		componentJavaScript.add("											" + referenceId + "_getPlotData();");
		componentJavaScript.add("                						}");
		componentJavaScript.add("										if (parameter == \"table\") " + referenceId + "_getTableData();");
		componentJavaScript.add("                					}");
		componentJavaScript.add("                				}");
		componentJavaScript.add("                				" + referenceId + "_xhr.send(null);");  
		componentJavaScript.add("            				} catch (e) {");
		componentJavaScript.add("            					window.alert(\"Error getting data channel names from server!\");");
		componentJavaScript.add("	        				}");
		componentJavaScript.add("	        			} else {");
		componentJavaScript.add("	        				window.alert(\"Please select a valid data set and/or time period! Check radio buttons!\");");
		componentJavaScript.add("        				}");	
		componentJavaScript.add("        			}");
		
		componentJavaScript.add("                function " + referenceId + "_getDataAsPlot(){");
		componentJavaScript.add("                	" + referenceId + "_getSelectedData();");
		componentJavaScript.add("                	" + referenceId + "_getSelectedPeriod();");
		componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
		componentJavaScript.add("                	" + referenceId + "_yMax = 100;");		
		componentJavaScript.add("                	" + referenceId + "_getDataChannelNames(\"plot\");");
		componentJavaScript.add("                }");
		
		componentJavaScript.add("                function " + referenceId + "_getDataAsTable(){");
		componentJavaScript.add("                	" + referenceId + "_getSelectedData();");
		componentJavaScript.add("                	" + referenceId + "_getSelectedPeriod();");
		componentJavaScript.add("                	" + referenceId + "_getDataChannelNames(\"table\");");
		componentJavaScript.add("                }");
		
		componentJavaScript.add("                function " + referenceId + "_getPlotData() {");		
		componentJavaScript.add("		            var now = new Date();");
		componentJavaScript.add("					" + referenceId + "_progress(\"0\");");
		componentJavaScript.add("					" + referenceId + "_plot.src = \"/web2cToolkit/Web2cArchiveViewer?param=<plot><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><id>\" + " + referenceId + "_dataPlotId + \"</id><value>\" + " + referenceId + "_dataChannelNames + \"</value><value>\" + " + referenceId + "_selectedFromInMillis + \"</value><value>\" + " + referenceId + "_selectedToInMillis + \"</value><value>\" + " + referenceId + "_yMin + \"</value><value>\" + " + referenceId + "_yMax + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></plot>\";");
		componentJavaScript.add("                }");
		
		componentJavaScript.add("                function " + referenceId + "_getTableData() {");		
		componentJavaScript.add("		            var now = new Date();");
		componentJavaScript.add("					" + referenceId + "_progress(\"0\");");
		componentJavaScript.add("					newWindow = window.open(\"" + hostURL + "?param=<text><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><id>\" + " + referenceId + "_dataPlotId + \"</id><value>\" + " + referenceId + "_dataChannelNames + \"</value><value>\" + " + referenceId + "_selectedFromInMillis + \"</value><value>\" + " + referenceId + "_selectedToInMillis + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></text>\");");
		componentJavaScript.add("                }");

		return componentJavaScript;
	}
	
	/**
	 * Handles client action request
	 * @param parameter1 first action parameter
	 * @param parameter2 second action parameter
	 * @return returns channel parameters for plot
	 */
	public String doAction (String parameter1, String parameter2) {
		
		String str = null;	

		str = "";	
		
		String selectionTyp = null;
		int i = 0;
		int j = 0;
		int k = 0;
		
		selectionTyp = parameter1;

		if (selectionTyp.equals("userdefined")) {
			String[] selectedData = parameter2.split(",");
			for (i = 0; i < userDefinedDataEntryName.toArray().length; i++) {
				for (j = 0; j < selectedData.length; j++) {
					if (userDefinedDataEntryName.toArray()[i].equals(selectedData[j])) {
						for (k = 0; k < dataAddressID.toArray().length; k++) {
							if (userDefinedDataEntryAddressIndex.toArray()[i].equals(dataAddressID.toArray()[k])) {
								str = str + "<trace><uri>" + dataAddressString.toArray()[k] + "</uri><name>" + userDefinedDataEntryName.toArray()[i] + "</name><plotType>" + userDefinedDataPlotType.toArray()[i] + "</plotType><plotMin>" + userDefinedDataPlotMin.toArray()[i] + "</plotMin><plotMax>" + userDefinedDataPlotMax.toArray()[i] + "</plotMax></trace>";
							}
						}
					}
				}
			}
		}
		if (selectionTyp.equals("predefined")) {
			String selectedData = parameter2;
			for (i = 0; i < preDefinedDataEntryName.toArray().length; i++) {
				if (preDefinedDataEntryName.toArray()[i].equals(selectedData)) {
					for (j = 0; j < preDefinedDataSubEntryName.get(i).toArray().length; j++) {
						for (k = 0; k < dataAddressID.toArray().length; k++) {
							if (preDefinedDataSubEntryAddressIndex.get(i).toArray()[j].equals(dataAddressID.toArray()[k])) {
								str = str + "<trace><uri>" + dataAddressString.toArray()[k] + "</uri><name>" + preDefinedDataSubEntryName.get(i).toArray()[j] + "</name><plotType>" + preDefinedDataSubPlotType.get(i).toArray()[j] + "</plotType><plotMin>" + preDefinedDataSubPlotMin.get(i).toArray()[j] + "</plotMin><plotMax>" + preDefinedDataSubPlotMax.get(i).toArray()[j] + "</plotMax></trace>";
							}
						}
					}
				}
			}
			
		}

		str = str.replaceAll("#", "%23");
		
		return str;
	}
}
