package de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveAccess.Web2cCommonArchiveAccess;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataPlot;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataSelector;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerPage;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerSection;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cConfigReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cArchiveViewerPage implements IWeb2cConstants {

	private ArrayList<ArrayList<String>> sectionsHTML = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> sectionsJavaScript = new ArrayList<ArrayList<String>>();	
	private ArrayList<Web2cArchiveViewerSection> sections = new ArrayList<Web2cArchiveViewerSection>();
	private ArrayList<Web2cArchiveViewerPage> embeddedPages = new ArrayList<Web2cArchiveViewerPage>();
	private ArrayList<String> embeddedPagesId = new ArrayList<String>();
	private String hostURL = null;
	private String homeDir = null;
	private String webDir = null;
	private Web2cArchiveViewerPage rootPage = null;
	private String title = null;
	private String referenceId = "";
	private String referencePoint = "0,0";
	private String userName = null;
	private String userAddress = null;
	private Web2cCommonArchiveAccess caa = null;
	private ArrayList<String> expertName = null;
	private ArrayList<String> expertAddress = null;
	private String userPrivilege = null;
	private String networkPrivilege = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private String id = null;
	private String web2cSession = null;
	private String fromTime = "";
	private String toTime = "";

	private String error = null;

	/**
	 * Archive Viewer Page
	 * @param url URL of host servicing the request
	 * @param session Web2c session id
	 * @param home directory which contains user-specific files
	 * @param web directory which contains Web2cToolkit-specific files 
	 * @param configFile actual configuration file
	 * @param from start time of archive data interval
	 * @param to end time of archive data interval
	 * @param name actual user name
	 * @param ipAddress actual IP address of user
	 * @param logger debug logger
	 * @param listener debug flag listener
	 */
	public Web2cArchiveViewerPage(String url, String session, String home, String web, String configFile, String from, String to, String name, String ipAddress, Web2cFileLogger logger, Web2cKeyBoardListener listener) {
		
		hostURL = url;
		web2cSession = session;		
		homeDir = home;
		webDir = web;
		fromTime = from;
		toTime = to;
		rootPage = this;
		id = "root";
		referenceId = "archiveViewerPage";
		referencePoint = "0,0";
		userPrivilege = layman;
		networkPrivilege = layman;
		userName = "guest";
		userAddress = "";
		if (name == null) {
			userName = "guest";
		}
		else {
			userName = name;
		}
		if (ipAddress == null) {
			userAddress = "";
		}
		else {
			userAddress = ipAddress;
		}
		debugLogger = logger;
		debugListener = listener;
		caa = new Web2cCommonArchiveAccess(debugListener, debugLogger);
		pageExecute(new File(homeDir + File.separator + configFile));
	}	
	
	/**
	 * Embedded Page
	 * @param url URL of host servicing the request
	 * @param session Web2c session id
	 * @param home directory which contains user-specific files
	 * @param web directory which contains Web2cToolkit-specific files 
	 * @param configFile configuration file
	 * @param from start time of archive data interval
	 * @param to end time of archive data interval
	 * @param root container page
	 * @param refId ID of embedded page
	 * @param refPoint coordinates (left upper corner) of embedded page
	 * @param privilege actual user privilege
	 * @param commonDeviceAccess actual common archive access instance
	 * @param logger debug logger
	 * @param listener debug flag listener
	 */
	public Web2cArchiveViewerPage(String url, String session, String home, String web, String configFile, String from, String to, Web2cArchiveViewerPage root, String refId, String refPoint, String privilege, Web2cCommonArchiveAccess commonDeviceAccess, Web2cFileLogger logger, Web2cKeyBoardListener listener) {
		
		hostURL = url;
		web2cSession = session;
		homeDir = home;
		webDir = web;
		fromTime = from;
		toTime = to;
		id = refId;
		if (root == null) {
			rootPage = this;
		}
		else {
			rootPage = root;
		}
		if (refId == null) {
			referenceId = "archiveViewerPage";
		}
		else {
			referenceId = refId;
		}
		if (refPoint == null) {
			referencePoint = "0,0";
		}
		else {
			referencePoint = refPoint;
		}
		userPrivilege = layman;
		if (privilege == null) {
			userPrivilege = layman;
		}
		else {
			userPrivilege = privilege;
		}
		caa = commonDeviceAccess;
		debugLogger = logger;
		debugListener = listener;
		pageExecute(new File(configFile));
	}
	
	/**
	 * @return component id
	 */
	public String getId() {
		
		return id;
	}

	/**
	 * Returns full contents of page (HTML tags, JavaScript snippets) 
	 * @return list
	 */
	public ArrayList<Object> getComponents() {
		
		ArrayList<Object> list = new ArrayList<Object>();
		for (Iterator<Web2cArchiveViewerSection> it1 = sections.iterator(); it1.hasNext();) {
			Web2cArchiveViewerSection section = (Web2cArchiveViewerSection)it1.next();
			list.add((Object)section);
			for (Iterator<Object> it2 = (section.getComponents()).iterator(); it2.hasNext();) {
				list.add(it2.next());
			}
		}
		for (Iterator<Web2cArchiveViewerPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			for (Iterator<Object> it2 = (((Web2cArchiveViewerPage)it1.next()).getComponents()).iterator(); it2.hasNext();) {
				list.add(it2.next());
			}
		}
		
		return list;
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> pageHTML = new ArrayList<String>();	
		
		for(Iterator<ArrayList<String>> it1 = sectionsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				pageHTML.add((String)it2.next());
			}
		}
		
		for(Iterator<Web2cArchiveViewerPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			ArrayList<String> embeddedPageHTML = ((Web2cArchiveViewerPage)it1.next()).getHTML();
			for(Iterator<String> it2 = embeddedPageHTML.iterator(); it2.hasNext();) {
				pageHTML.add((String)it2.next());
			}
		}

		
		return pageHTML;
	}
	
	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> pageJavaScript = new ArrayList<String>();	
		
		pageJavaScript.add("	    //Archive Viewer: " + referenceId);		
		pageJavaScript.add("		var " + referenceId + "_sessionId = " + web2cSession + ";");
		pageJavaScript.add("        var " + referenceId + "_pageId = \"" + referenceId + "\";");	
		pageJavaScript.add("		var " + referenceId + "_xhr;");
		pageJavaScript.add("        var " + referenceId + "_selectionType;");
		pageJavaScript.add("        var " + referenceId + "_selectedData;");
		pageJavaScript.add("	    var " + referenceId + "_dataChannelNames;");
		pageJavaScript.add("        var " + referenceId + "_selectedFromInMillis = 0;");
		pageJavaScript.add("        var " + referenceId + "_selectedToInMillis = 0;");
		pageJavaScript.add("        var " + referenceId + "_selectedFromInMillisOld = 0;");
		pageJavaScript.add("        var " + referenceId + "_selectedToInMillisOld = 0;"); 
		pageJavaScript.add("        var " + referenceId + "_yMin = 0;"); 
		pageJavaScript.add("        var " + referenceId + "_yMax = 100;"); 
		pageJavaScript.add("	    var " + referenceId + "_tStart;");
		pageJavaScript.add("	    var " + referenceId + "_tWidth;");
		pageJavaScript.add("	    var " + referenceId + "_yStart;");
		pageJavaScript.add("	    var " + referenceId + "_yWidth;");
		pageJavaScript.add("		var " + referenceId + "_plot;");
		pageJavaScript.add("		var " + referenceId + "_status;"); 
		pageJavaScript.add("		var " + referenceId + "_browser;"); 
		pageJavaScript.add("		var " + referenceId + "_plotReloadCounter = 0;"); 
		pageJavaScript.add("		var " + referenceId + "_plotMaxReloadCounter = 30;"); 
		
		pageJavaScript.add("		function " + referenceId + "_initPage() {");
		pageJavaScript.add("			" + referenceId + "_handleBrowserSpecifica();");
		pageJavaScript.add("			" + referenceId + "_initXhr();"); 
		pageJavaScript.add("			" + referenceId + "_plot = document.getElementById(" + referenceId + "_dataPlotId + \"_plot\");");
		pageJavaScript.add("			" + referenceId + "_zoomFrame = document.getElementById(" + referenceId + "_zoomFrameId);");
		pageJavaScript.add("			" + referenceId + "_status = document.getElementById(" + referenceId + "_dataStatusId);");
		pageJavaScript.add("		    " + referenceId + "_plotZoomClick = false;");
		pageJavaScript.add("			if (" + referenceId + "_browser == \"MSIE\") {");
//		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onclick\", " + referenceId + "_plotMouseClick);");
//		pageJavaScript.add("				" + referenceId + "_zoomFrame.attachEvent(\"onclick\", " + referenceId + "_plotMouseClick);");
		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onerror\", " + referenceId + "_plotDataErrorOnLoad);");
		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onload\", " + referenceId + "_plotDataLoaded);");
		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onmousemove\", " + referenceId + "_plotMouseMove);");
		pageJavaScript.add("				" + referenceId + "_zoomFrame.attachEvent(\"onmousemove\", " + referenceId + "_plotMouseMove);");
		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onmousedown\", " + referenceId + "_plotMouseDown);");
		pageJavaScript.add("				" + referenceId + "_plot.attachEvent(\"onmouseup\", " + referenceId + "_plotMouseUp);");
		pageJavaScript.add("				" + referenceId + "_zoomFrame.attachEvent(\"onmouseup\", " + referenceId + "_plotMouseUp);");
		pageJavaScript.add("			}");
		pageJavaScript.add("			else {");
//		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"click\", " + referenceId + "_plotMouseClick, false);");
//		pageJavaScript.add("				" + referenceId + "_zoomFrame.addEventListener(\"click\", " + referenceId + "_plotMouseClick, false);");
		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"error\", " + referenceId + "_plotDataErrorOnLoad, false);");
		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"load\", " + referenceId + "_plotDataLoaded, false);");
		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"mousemove\", " + referenceId + "_plotMouseMove, false);");
		pageJavaScript.add("				" + referenceId + "_zoomFrame.addEventListener(\"mousemove\", " + referenceId + "_plotMouseMove, false);");
		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"mousedown\", " + referenceId + "_plotMouseDown, false);");
		pageJavaScript.add("				" + referenceId + "_plot.addEventListener(\"mouseup\", " + referenceId + "_plotMouseUp, false);");
		pageJavaScript.add("				" + referenceId + "_zoomFrame.addEventListener(\"mouseup\", " + referenceId + "_plotMouseUp, false);");
		pageJavaScript.add("			}"); 
		pageJavaScript.add("			if (" + referenceId + "_defaultSelection == true) " + referenceId + "_getDataAsPlot();");  
		for (Iterator<String> it = embeddedPagesId.iterator(); it.hasNext();) {
			String id = (String)it.next();
			pageJavaScript.add("			" + id + "_initPage();");
		} 
		pageJavaScript.add("		}");
		
		pageJavaScript.add("		function " + referenceId + "_handleBrowserSpecifica() {");
		pageJavaScript.add("			var browserType = navigator.appName;");  
		pageJavaScript.add("			if (browserType == \"Microsoft Internet Explorer\") {");
		pageJavaScript.add("				" + referenceId + "_browser = \"MSIE\";");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (" + referenceId + "_browser == \"MSIE\") {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"DIV\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"TEXTAREA\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			else {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"TEXTAREA\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle != \"none none none none\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var topandunits = element[i].style.top.split(\"p\");");
		pageJavaScript.add("						var top = topandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.top = String(parseInt(top) - (1 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("		}");
		
		pageJavaScript.add("        function " + referenceId + "_initXhr() {"); 
		pageJavaScript.add("			try {");                                                  
		pageJavaScript.add("				" + referenceId + "_xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		pageJavaScript.add("			} catch (event1) {");                             
		pageJavaScript.add("				try {");
		pageJavaScript.add("					" + referenceId + "_xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		pageJavaScript.add("				} catch (event2) {");
		pageJavaScript.add("					" + referenceId + "_xhr = false;");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (!" + referenceId + "_xhr && typeof XMLHttpRequest !=\"undefined\") {");
		pageJavaScript.add("				" + referenceId + "_xhr = new XMLHttpRequest();");                 
		pageJavaScript.add("			}"); 																	
		pageJavaScript.add("		}");   
		
		pageJavaScript.add("        function " + referenceId + "_getZoomBounds() {");	
		pageJavaScript.add("		         var now = new Date();");
		pageJavaScript.add("	    		 var zoomBounds;");		
		pageJavaScript.add("	           	 try {");
		pageJavaScript.add("					var param1 = \"\";");
		pageJavaScript.add("					var param2 = \"\";");
		pageJavaScript.add("                    " + referenceId + "_xhr.open(\"post\", \"/web2cToolkit/Web2cArchiveViewer?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><value>zoomBounds</value><value>\" + " + referenceId + "_dataPlotId + \"</value><value>\" + param1 + \"</value><value>\" + param2 + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		pageJavaScript.add("                	" + referenceId + "_xhr.onreadystatechange = function() {"); 
		pageJavaScript.add("                   		if (" + referenceId + "_xhr.readyState == 4) {"); 
		pageJavaScript.add("							zoomBounds = " + referenceId + "_xhr.responseText.substring(0, " + referenceId + "_xhr.responseText.length);");
		pageJavaScript.add("                			zoomBounds = zoomBounds.replace(\"<zoom>\", \"\");");
		pageJavaScript.add("                			zoomBounds = zoomBounds.replace(\"<tstart>\", \"\").replace(\"<twidth>\", \"\");");
		pageJavaScript.add("                			zoomBounds = zoomBounds.replace(\"<ystart>\", \"\").replace(\"<ywidth>\", \"\");");
		pageJavaScript.add("                			zoomBounds = zoomBounds.replace(\"/tstart>\", \"\").replace(\"/twidth>\", \"\");");
		pageJavaScript.add("                			zoomBounds = zoomBounds.replace(\"/ystart>\", \"\").replace(\"/ywidth>\", \"\");");
		pageJavaScript.add("                			" + referenceId + "_tStart = zoomBounds.split(\"<\")[0];");
		pageJavaScript.add("                			" + referenceId + "_tWidth = zoomBounds.split(\"<\")[1];");
		pageJavaScript.add("                			" + referenceId + "_yStart = zoomBounds.split(\"<\")[2];");
		pageJavaScript.add("                			" + referenceId + "_yWidth = zoomBounds.split(\"<\")[3];");
		pageJavaScript.add("                		}");
		pageJavaScript.add("                	}");
		pageJavaScript.add("                	" + referenceId + "_xhr.send(null);"); 
		pageJavaScript.add("            	} catch (e) {");
		pageJavaScript.add("           		window.alert(\"Error getting zoom bounds from server!\");");
		pageJavaScript.add("	        	}");
		pageJavaScript.add("        }");
		
		pageJavaScript.add("		function " + referenceId + "_progress(fraction) {");
		pageJavaScript.add("	        if (fraction == \"0\") " + referenceId + "_status.innerHTML = \"data loading ...\";");
		pageJavaScript.add("	        if (fraction == \"100\") {");
		pageJavaScript.add("        		" + referenceId + "_getZoomBounds();");
		pageJavaScript.add("        		" + referenceId + "_status.innerHTML = \"data loaded\";");
		pageJavaScript.add("        	}");
		pageJavaScript.add("        }");
		
		pageJavaScript.add("		function " + referenceId + "_plotDataLoaded(event) {");
		pageJavaScript.add("        	" + referenceId + "_plotReloadCounter = 0;");
		pageJavaScript.add("        	" + referenceId + "_progress(\"100\");");
		pageJavaScript.add("        	document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").alt=\"No data to plot\";");
		pageJavaScript.add("        }");
		
		pageJavaScript.add("		function " + referenceId + "_plotDataErrorOnLoad(event) {");
		pageJavaScript.add("        	if (" + referenceId + "_plotReloadCounter < " + referenceId + "_plotMaxReloadCounter) {");
		pageJavaScript.add("        		" + referenceId + "_plotReloadCounter = " + referenceId + "_plotReloadCounter + 1;");
		pageJavaScript.add("        		document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").alt=\"Plot loading\";");
		pageJavaScript.add("        		setTimeout(\"" + referenceId + "_getPlotData()\", 1000);");
		pageJavaScript.add("        	}");
		pageJavaScript.add("			else {");
		pageJavaScript.add("        		document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").alt=\"Timeout! Error generating plot.\";");
		pageJavaScript.add("        	}");
		pageJavaScript.add("        }");
		
		for(Iterator<ArrayList<String>> it1 = sectionsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				pageJavaScript.add((String)it2.next());
			}
		}
		
		for(Iterator<Web2cArchiveViewerPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
			ArrayList<String> embeddedPageJavaScript = ((Web2cArchiveViewerPage)it1.next()).getJavaScript();
			for(Iterator<String> it2 = embeddedPageJavaScript.iterator(); it2.hasNext();) {
				pageJavaScript.add((String)it2.next());
			}
		}		
		
		if (rootPage == this) {
			pageJavaScript.add("	    function handleAll(fromInMillis, toInMillis, callerId) {");
			pageJavaScript.add("	    	var from = fromInMillis;");
			pageJavaScript.add("	    	var to = toInMillis;");
			pageJavaScript.add("		    var now = new Date();");
			pageJavaScript.add("			if (callerId != \"" + referenceId + "\") " + referenceId + "_getPlotData();");
			for(Iterator<Web2cArchiveViewerPage> it1 = embeddedPages.iterator(); it1.hasNext();) {
				String embeddedPageId = ((Web2cArchiveViewerPage)it1.next()).getId();
				pageJavaScript.add("			if (callerId != \"" + embeddedPageId + "\") {");
				pageJavaScript.add("        		" + embeddedPageId + "_plotReloadCounter = 0;");
				pageJavaScript.add("				" + embeddedPageId + "_progress(\"0\");");
				pageJavaScript.add("				" + embeddedPageId + "_plot.src = \"/web2cToolkit/Web2cArchiveViewer?param=<plot><sessionId>\" + " + embeddedPageId + "_sessionId + \"</sessionId><id>\" + " + embeddedPageId + "_dataPlotId + \"</id><value>\" + " + embeddedPageId + "_dataChannelNames + \"</value><value>\" + fromInMillis + \"</value><value>\" + toInMillis + \"</value><value>\" + " + embeddedPageId + "_yMin + \"</value><value>\" + " + embeddedPageId + "_yMax + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></plot>\";");
				pageJavaScript.add("	    	}");
			}
			pageJavaScript.add("	    }");
		}
		
		return pageJavaScript;
	}
	
	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<String> getPage() {
		
		int i = 0;
		ArrayList<String> page = new ArrayList<String>();
		Calendar calendar = Calendar.getInstance();
		File jScriptDir = null;
		File jScriptFile = null;
		String[] fileList = null;
		
		jScriptDir = new File(webDir);
		fileList = jScriptDir.list();
		for (i = 0; i < fileList.length; i++) {
			if (fileList[i].contains("Web2cArchiveViewer_")) {
				jScriptFile = new File(webDir + File.separator + fileList[i]);
				if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
			}
		}

		Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cArchiveViewer_" + calendar.getTimeInMillis() + ".js"), debugLogger, debugListener);
		javascriptWriter.setJavascript(getJavaScript());

		page.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		page.add("<html>");
		page.add("	<head>");
		page.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		page.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		page.add("		<title>" + title + "</title>");
		page.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		page.add("		<link rel=\"shortcut icon\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		page.add("		<script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2cArchiveViewer_" + calendar.getTimeInMillis() + ".js\"></script>");
		page.add("	</head>");
		page.add("	<body onload=\"" + referenceId + "_initPage()\">");
		for(Iterator<String> it = getHTML().iterator(); it.hasNext();) {
			page.add((String)it.next());
		}
		page.add("	</body>");
		page.add("</html>");
		
		return page;
	}
	
	/**
	 * Returns data plot
	 * @param parameter1 id of requesting component
	 * @param parameter2 parameter list for plot requested
	 * @return plot
	 */
	public ByteArrayOutputStream getPlot(String parameter1, String parameter2) {

		ByteArrayOutputStream plotOut = null;

		for (Iterator<Object> it = getComponents().iterator(); it.hasNext();) {
			Object object = (Object)it.next();
			if (object.getClass().getName().equals("de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataPlot")) {
				if (((Web2cArchiveViewerDataPlot)object).getId().equals(parameter1)) {
					plotOut = (ByteArrayOutputStream)(((Web2cArchiveViewerDataPlot)object).getPlot(parameter2));
				}
			}
		}
		
		return plotOut;
	}
	
	/**
	 * Returns data table
	 * @param parameter1 id of requesting component
	 * @param parameter2 parameter list for table requested
	 * @return table
	 */
	public String[] getTable(String parameter1, String parameter2) {
		
		String[] out = null;
		
		for (Iterator<Object> it = getComponents().iterator(); it.hasNext();) {
			Object object = (Object)it.next();
			if (object.getClass().getName().equals("de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataPlot")) {
				if (((Web2cArchiveViewerDataPlot)object).getId().equals(parameter1)) {
					out = ((Web2cArchiveViewerDataPlot)object).getTable(parameter2);
				}
			}
		}
		
		return out;
	}
	
	/**
	 * Handles client action request
	 * @param parameter1 first action parameter
	 * @param parameter2 second action parameter
	 * @param parameter3 third action parameter
	 * @param parameter4 third action parameter
	 * @return parameters in response to request 
	 */
	public String doAction(String parameter1, String parameter2, String parameter3, String parameter4) {
		
		String str = null;	

		str = "";	

		if (parameter1.equals("channelNames")) {
			for (Iterator<Object> it = getComponents().iterator(); it.hasNext();) {
				Object object = (Object)it.next();
				if (object.getClass().getName().equals("de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataSelector")) {
					if (((Web2cArchiveViewerDataSelector)object).getId().equals(parameter2)) {
						str = ((Web2cArchiveViewerDataSelector)object).doAction(parameter3, parameter4);
					}
				}
			}
		}
		if (parameter1.equals("zoomBounds")) {
			for (Iterator<Object> it = getComponents().iterator(); it.hasNext();) {
				Object object = (Object)it.next();
				if (object.getClass().getName().equals("de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataPlot")) {
					if (((Web2cArchiveViewerDataPlot)object).getId().equals(parameter2)) {
						str = ((Web2cArchiveViewerDataPlot)object).doAction(parameter3, parameter4);
					}
				}
			}
		}

		return str;
	}
	
	public String getError() {
		
		return error;
	}
	
	private void pageExecute(File configFile) {
		
		Web2cConfigReader configReader = null;
		try {
			configReader = new Web2cConfigReader(configFile, debugLogger, debugListener);
			decodeAttributes(configReader.getPageAttributeNames(), configReader.getPageAttributeValues());
			setPageElements(configReader.getSectionsAttributeNames(), configReader.getSectionsAttributeValues(), configReader.getSectionsElements(), configReader.getSectionsElementsAttributeNames(), configReader.getSectionsElementsAttributeValues());
			if (configReader.getEmbeddedPagesAttributeNames() != null) {
				ArrayList<String> attributeNames = new ArrayList<String>();
				ArrayList<String> attributeValues = new ArrayList<String>();
				Iterator<ArrayList<String>> it2 = configReader.getEmbeddedPagesAttributeValues().iterator();
				for (Iterator<ArrayList<String>> it1 = configReader.getEmbeddedPagesAttributeNames().iterator(); it1.hasNext();) {
					attributeNames = (ArrayList<String>)it1.next();
					attributeValues = (ArrayList<String>)it2.next();
					setEmbeddedPage(attributeNames, attributeValues);
				}
			}
		}
		catch (Exception e) {
			error = error + configReader.getError();
		}
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		expertName = new ArrayList<String>();
		expertAddress = new ArrayList<String>();
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("caption")) {
				title = value;
			}
			if (name.equals("expert")) {
				expertName.add(value);
			}	
			if (name.equals("ipAddress")) {
				expertAddress.add(value);
			}	
		}	
		
		
		for (Iterator<String> it = expertName.iterator(); it.hasNext();) {
			if ((it.next()).equals(userName)) {
				userPrivilege = expert;
			}
		}		
		for (Iterator<String> it = expertAddress.iterator(); it.hasNext();) {
			String nextAddress = it.next().toString();
			if (nextAddress.replace(".", ">").split(">")[3].equals("*") || nextAddress.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
				if (nextAddress.replace(".", ">").split(">")[2].equals("*") || nextAddress.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
					if (nextAddress.replace(".", ">").split(">")[1].equals("*") || nextAddress.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
						if (nextAddress.replace(".", ">").split(">")[0].equals("*") || nextAddress.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
							networkPrivilege = expert;
						}
					}
				}
			}
		}
		
		if (userPrivilege == expert && networkPrivilege == expert) {
			userPrivilege = expert;
		}
		else if (userPrivilege == expert && expertAddress.isEmpty()) {
			userPrivilege = expert;
		}
		else {
			userPrivilege = layman;
		}
		
		title = title + " (User: " + userName;
		if (userPrivilege == expert) {
			title = title + ", Privilege: expert";
		}
		else {
			title = title + ", Privilege: layman";
		}
		title = title + ", IP Address: " + userAddress;
		title = title + ")";
	}
	
	private void setPageElements(ArrayList<ArrayList<String>> sectionsAttributeNames, ArrayList<ArrayList<String>> sectionsAttributeValues, ArrayList<ArrayList<String>> sectionsElements, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeNames, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeValues) {
		
		Iterator<ArrayList<String>> it1 = sectionsAttributeNames.iterator();
		Iterator<ArrayList<String>> it2 = sectionsAttributeValues.iterator();
		Iterator<ArrayList<ArrayList<String>>> it4 = sectionsElementsAttributeNames.iterator();
		Iterator<ArrayList<ArrayList<String>>> it5 = sectionsElementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it3 = sectionsElements.iterator(); it3.hasNext();) {
			ArrayList<String> attributeNames = (ArrayList<String>)it1.next();
			ArrayList<String> attributeValues = (ArrayList<String>)it2.next();
			ArrayList<String> elementsNames = (ArrayList<String>)it3.next();
			ArrayList<ArrayList<String>> elementsAttributeNames = (ArrayList<ArrayList<String>>)it4.next();
			ArrayList<ArrayList<String>> elementsAttributeValues = (ArrayList<ArrayList<String>>)it5.next();
			Web2cArchiveViewerSection section = new Web2cArchiveViewerSection(hostURL, referenceId, referencePoint, attributeNames, attributeValues, elementsNames, elementsAttributeNames, elementsAttributeValues, caa, fromTime, toTime);
			sections.add(section);
			sectionsHTML.add(section.getHTML());
			sectionsJavaScript.add(section.getJavaScript());
			error = error + section.getError();
		}
	}
	
	private void setEmbeddedPage(ArrayList<String> names, ArrayList<String> values) {
		
		int xLeft = 0;
		int yTop = 0;
		String refFile = null;
		String refId = null;
		String refPoint = null;
		Iterator<String> it2 = values.iterator();
		for (Iterator<String> it1 = names.iterator(); it1.hasNext();) {
			String name = (String)it1.next();
			String value = (String)it2.next();
			if (name.equals("uri")) {
				refFile = value.substring(7).replace("/", File.separator);
			}
			if (name.equals("id")) {
				if (value.equals("enter Id") || value.equals("") || value.equals(" ")) {
					error = error + "[Component Decoder] Missing or wrong component ID;";
				}
				refId = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				refPoint = ((Integer)(Integer.valueOf(referencePoint.split(",")[0]) + xLeft)).toString() + "," + ((Integer)(Integer.valueOf(referencePoint.split(",")[1]) + yTop)).toString();
			}
		}
		Web2cArchiveViewerPage embeddedPage = new Web2cArchiveViewerPage(hostURL, web2cSession, homeDir, webDir, homeDir + File.separator + refFile, fromTime, toTime, rootPage, refId, refPoint, userPrivilege, caa, debugLogger, debugListener);
		embeddedPages.add(embeddedPage);
		embeddedPagesId.add(refId);
		error = error + embeddedPage.getError();
	}
}