package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorAttributeList implements IWeb2cConstants {

	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop  = null;
	private String toolTipText = null;
	private String color = null;
	private String backgroundColor = null;
	private String textAlignment = null;
	private String fontSize = null;
	private String fontThickness= null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private String headerColor  = null;
	private String headerBackgroundColor  = null;
	private int rows = 20;

	/**
	 * Attribute List
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cEditorAttributeList(String refId, ArrayList<String> names, ArrayList<String> values) {

		referenceId = refId;
		decodeAttributes(names, values);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		int i = 0;
		int xPos = 0;
		int yPos = 0;
		int cellHeightAttribute = 0;
		int cellWidthAttribute = 0;
		int cellHeightValue = 0;
		int cellWidthValue = 0;
		int xStep = 0;
		int yStep = 0;
		String nullPos = "0";
		
		ArrayList<String> componentHTML = new ArrayList<String>();	

		if (borderStyle.equals(solid)) {
			cellWidthAttribute = (2 * Integer.valueOf(width.split("p")[0]) / 5) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			cellHeightAttribute = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 3)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
			cellWidthValue = (3 * Integer.valueOf(width.split("p")[0]) / 5) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			cellHeightValue = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 3)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
			xStep = (Integer.valueOf(width.split("p")[0]) / 5) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 3)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));		
			xPos = (2 * xStep);
			yPos = 0;
		}
		else {	
			cellWidthAttribute = (2 * Integer.valueOf(width.split("p")[0]) / 5);
			cellHeightAttribute = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 3));	
			cellWidthValue = (3 * Integer.valueOf(width.split("p")[0]) / 5);
			cellHeightValue = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 3));	
			xStep = (Integer.valueOf(width.split("p")[0]) / 5) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 3)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));		
			xPos = (2 * xStep);
			yPos = 0;
		}

		componentHTML.add("			<div id=\"" + id + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");
		
		componentHTML.add("				<div id=\"" + id + "_headerAttribute" + "\" style=\"position:absolute; left:" + nullPos + "; top:" + nullPos + "; width:" + ((Integer)(cellWidthAttribute)).toString() + "px" + "; height:" + ((Integer)cellHeightAttribute).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + ";\">" + "Attribute" + "</div>");
		componentHTML.add("				<div id=\"" + id + "_headerValue" + "\" style=\"position:absolute; left:" + ((Integer)xPos).toString() + "px" + "; top:" + marginTop + "; width:" + ((Integer)(cellWidthValue)).toString() + "px" + "; height:" + ((Integer)cellHeightValue).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + ";\">" + "Value" + "</div>");
	
		yPos = yPos + yStep;
		componentHTML.add("				<div id=\"" + id + "_typeAttribute" + "\" style=\"position:absolute; left:" + nullPos + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthAttribute)).toString() + "px" + "; height:" + ((Integer)cellHeightAttribute).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + left + ";\">" + "type" + "</div>");
		componentHTML.add("				<div id=\"" + id + "_typeValue" + "\" style=\"position:absolute; left:" + ((Integer)xPos).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthValue)).toString() + "px" + "; height:" + ((Integer)cellHeightValue).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + left + ";\">" + " " + "</div>");
		
		yPos = yPos + yStep;
		componentHTML.add("				<div id=\"" + id + "_sectionIdAttribute" + "\" style=\"position:absolute; left:" + nullPos + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthAttribute)).toString() + "px" + "; height:" + ((Integer)cellHeightAttribute).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + ";\">" + "sectionId" + "</div>");
		componentHTML.add("				<textarea id=\"" + id + "_sectionIdValue" + "\" style=\"position:absolute; left:" + ((Integer)xPos).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthValue)).toString() + "px" + "; height:" + ((Integer)cellHeightValue).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + ";\">" + " " + "</textarea>");
	
		for (i = 0; i < rows; i++) {
			yPos = yPos + yStep;
			componentHTML.add("				<div id=\"" + id + "_cellAttribute" + ((Integer)i).toString() + "\" style=\"position:absolute; left:" + nullPos + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthAttribute)).toString() + "px" + "; height:" + ((Integer)cellHeightAttribute).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + ";\">" + " " + "</div>");
			componentHTML.add("				<textarea id=\"" + id + "_cellValue" + ((Integer)i).toString() + "\" rows=\"1\" style=\"position:absolute; left:" + ((Integer)xPos).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidthValue)).toString() + "px" + "; height:" + ((Integer)cellHeightValue).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + ";\">" + " " + "</textarea>");
		}
		
		componentHTML.add("			</div>");
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("	//EditorAttributeList: " + id);
		componentJavaScript.add("	var attributeListId = \"" + id + "\";");
		componentJavaScript.add("	var configFileName;");    
		componentJavaScript.add("	var attributeListRows = " + ((Integer)(rows)).toString() + ";");
		componentJavaScript.add("	var attributeValueElement;");
		
		componentJavaScript.add("	function setConfigFileName(file) {");
		componentJavaScript.add("		configFileName = file;");
		componentJavaScript.add("	}");       
		
		componentJavaScript.add("	function updateIcons(element) {");
		componentJavaScript.add("		var element1;"); 
		componentJavaScript.add("		var element2;"); 
		componentJavaScript.add("		var i = 0;");
		componentJavaScript.add("		var elementType;");
		componentJavaScript.add("		var elementType = document.getElementById(attributeListId + \"_typeValue\").innerHTML;"); 
		componentJavaScript.add("		var web2cdir = \"" + web2cdocs + "\";");
		componentJavaScript.add("		var userdir = \"" + userdocs + "\";");
		componentJavaScript.add("		if (elementType == \"Histo\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"histo.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Trend\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"trend.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Slider\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"slider.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"WheelSwitch\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"wheelswitch.jpg\";"); 
		componentJavaScript.add("		}");   
		componentJavaScript.add("		if (elementType == \"EmbeddedPage\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"embeddedpage.jpg\";"); 
		componentJavaScript.add("		}");    
		componentJavaScript.add("		if (elementType == \"Section\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"section.jpg\";"); 
		componentJavaScript.add("		}");   
		componentJavaScript.add("		if (elementType == \"Label\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"label.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"LabelList\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"labellist.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Text\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"text.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"TextList\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"textlist.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Value\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"value.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"ValueList\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"valuelist.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Button\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"button.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Ticker\") {"); 
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"ticker.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Time\") {");
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"time.jpg\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("		if (elementType == \"Video\") {");
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"videoimage.jpg\";"); 
		componentJavaScript.add("		}"); 
		componentJavaScript.add("		if (elementType == \"Input\") {");
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"input.jpg\";"); 
		componentJavaScript.add("		}"); 
		componentJavaScript.add("		if (elementType == \"Chat\") {");
		componentJavaScript.add("			element.src = web2cdir + \"" + directorySeparator + "\" + \"chat.jpg\";"); 
		componentJavaScript.add("		}"); 
		componentJavaScript.add("		for (i = 0; i < attributeListRows; i++) {");
		componentJavaScript.add("			element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(i));");
		componentJavaScript.add("			element2 = document.getElementById(attributeListId + \"_cellValue\" + String(i));");
		componentJavaScript.add("			if ((elementType == \"Image\") && (element1.innerHTML == \"uri\")) {"); 
		componentJavaScript.add("				element.src = userdir + \"" + directorySeparator + "\" + element2.value.substring(7);"); 
		componentJavaScript.add("			}"); 
		componentJavaScript.add("			if ((elementType == \"AnimatedImage\") && (element1.innerHTML == \"lowIcon\")) {"); 
		componentJavaScript.add("				element.src = userdir + \"" + directorySeparator + "\" + element2.value;"); 
		componentJavaScript.add("			}"); 
		componentJavaScript.add("			if ((elementType == \"Indicator\") && (element1.innerHTML == \"falseIcon\")) {"); 
		componentJavaScript.add("				element.src = userdir + \"" + directorySeparator + "\" + element2.value;"); 
		componentJavaScript.add("			}"); 
		componentJavaScript.add("			if ((elementType == \"IndicatorList\") && (element1.innerHTML == \"falseIcon\")) {"); 
		componentJavaScript.add("				element.src = userdir + \"" + directorySeparator + "\" + element2.value;"); 
		componentJavaScript.add("			}"); 
		componentJavaScript.add("		}"); 
		componentJavaScript.add("	}");  
		
		componentJavaScript.add("	function postAttributeList() {");
		componentJavaScript.add("		var element1;"); 
		componentJavaScript.add("		var element2;"); 
		componentJavaScript.add("		var i = 0;"); 
		componentJavaScript.add("		var str = \"\";"); 
		componentJavaScript.add("		element1 = document.getElementById(attributeListId + \"_typeAttribute\");");
		componentJavaScript.add("		str = str + \"[\" + element1.innerHTML + \"]\";");
		componentJavaScript.add("		element2= document.getElementById(attributeListId + \"_typeValue\");");
		componentJavaScript.add("		str = str + element2.innerHTML + \"&#044;\";");
		componentJavaScript.add("		element1 = document.getElementById(attributeListId + \"_sectionIdAttribute\");");
		componentJavaScript.add("		str = str + \"[\" + element1.innerHTML + \"]\";");
		componentJavaScript.add("		element2= document.getElementById(attributeListId + \"_sectionIdValue\");");
		componentJavaScript.add("		str = str + element2.value + \"&#044;\";");
		componentJavaScript.add("		for (i = 0; i < attributeListRows; i++) {");
		componentJavaScript.add("			element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(i));");
		componentJavaScript.add("			if (element1.innerHTML != \" \") {");
		componentJavaScript.add("				str = str + \"[\" + element1.innerHTML + \"]\";");
		componentJavaScript.add("				element2 = document.getElementById(attributeListId + \"_cellValue\" + String(i));");
		componentJavaScript.add("				str = str + element2.value + \"&#044;\";");
		componentJavaScript.add("			}");
		componentJavaScript.add("		}");
		componentJavaScript.add("		str = escape(str.substring(0,str.length - 6));");
		componentJavaScript.add("		return str;");
		componentJavaScript.add("	}");        
		
		componentJavaScript.add("	function updateAttributeList() {"); 
		componentJavaScript.add("		try {");
		componentJavaScript.add("			var response;"); 
		componentJavaScript.add("			var element1;"); 
		componentJavaScript.add("			var element2;"); 
		componentJavaScript.add("			var i = 0;");
		componentJavaScript.add("			if (xhr.readyState == 4) {"); 
		componentJavaScript.add("				element1 = document.getElementById(attributeListId + \"_typeAttribute\");");
		componentJavaScript.add("				element1.innerHTML = \"type\";");
		componentJavaScript.add("				element2= document.getElementById(attributeListId + \"_typeValue\");");
		componentJavaScript.add("				element2.innerHTML = \" \";");
		componentJavaScript.add("				element1 = document.getElementById(attributeListId + \"_sectionIdAttribute\");");
		componentJavaScript.add("				element1.innerHTML = \"sectionId\";");
		componentJavaScript.add("				element2= document.getElementById(attributeListId + \"_sectionIdValue\");");
		componentJavaScript.add("				element2.value = \" \";");
		componentJavaScript.add("		        attributeValueElement = element2;");
		componentJavaScript.add("				for (i = 0; i < " + ((Integer)(rows)).toString() + "; i++) {");
		componentJavaScript.add("					element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(i));");
		componentJavaScript.add("					element1.innerHTML = \" \";");
		componentJavaScript.add("					element2 = document.getElementById(attributeListId + \"_cellValue\" + String(i));");
		componentJavaScript.add("					element2.value = \" \";");
		componentJavaScript.add("		            attributeValueElement = element2;");
		componentJavaScript.add("				}"); 
		componentJavaScript.add("				if (xhr.responseText.length > 2) {");
		componentJavaScript.add("					response = xhr.responseText.substring(7, xhr.responseText.length - 8).split(\"&#044;\");");	
		componentJavaScript.add("					var attribute = response[1].substring(response[1].indexOf(\"[\") + 1,response[1].lastIndexOf(\"]\"));");
		componentJavaScript.add("					var serverData = response[1].split(\"]\");");
		componentJavaScript.add("					var value = serverData[1];");
		componentJavaScript.add("					element1 = document.getElementById(attributeListId + \"_typeAttribute\");");
		componentJavaScript.add("					element1.innerHTML = attribute;");
		componentJavaScript.add("					element2= document.getElementById(attributeListId + \"_typeValue\");");
		componentJavaScript.add("					element2.innerHTML = value;");
		componentJavaScript.add("					attribute = response[2].substring(response[2].indexOf(\"[\") + 1,response[2].lastIndexOf(\"]\"));");
		componentJavaScript.add("					serverData = response[2].split(\"]\");");
		componentJavaScript.add("					value = serverData[1];");
		componentJavaScript.add("					element1 = document.getElementById(attributeListId + \"_sectionIdAttribute\");");
		componentJavaScript.add("					element1.innerHTML = attribute;");
		componentJavaScript.add("					element2 = document.getElementById(attributeListId + \"_sectionIdValue\");");
		componentJavaScript.add("					element2.value = value;");
		componentJavaScript.add("					for (i = 3; i < response.length; i++) {");
		componentJavaScript.add("						attribute = response[i].substring(response[i].indexOf(\"[\") + 1,response[i].lastIndexOf(\"]\"));");
		componentJavaScript.add("						serverData = response[i].split(\"]\");");
		componentJavaScript.add("						value = serverData[1];");
		componentJavaScript.add("						element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(i-3));");
		componentJavaScript.add("						element1.innerHTML = attribute;");
		componentJavaScript.add("						element2 = document.getElementById(attributeListId + \"_cellValue\" + String(i-3));");
		componentJavaScript.add("						element2.value = value;");
		componentJavaScript.add("					}");  
		componentJavaScript.add("				}");   	
		componentJavaScript.add("				updateIcons(selectedElement);");         
		componentJavaScript.add("			}");  
		componentJavaScript.add("		} catch (e) {");
		componentJavaScript.add("			window.alert(\"Error updating attribute list\");");
		componentJavaScript.add("		}");                    
		componentJavaScript.add("	}");
		
		return componentJavaScript;
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;

		toolTipText = "";
		color = black;
		backgroundColor = white;
		textAlignment = left;	
		fontSize = medium;
		fontThickness = normal;	
		borderStyle = solid;
		borderWidth = thin;
		borderColor = black;
		headerColor = black;
		headerBackgroundColor = gray;
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}		
			if (name.equals("color")) {
				color = value;
			}	
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}			
			if (name.equals("alignment")) {
				textAlignment = value;
			}						
			if (name.equals("fontSize")) {
				if (value.equals("small")) {
					fontSize = small;
				}
				else if (value.equals("medium")) {
					fontSize = medium;
				}
				else {
					fontSize = large;
				}
			}			
			if (name.equals("fontThickness")) {
				if (value.equals("normal")) {
					fontThickness = normal;
				}
				else {
					fontThickness = bold;
				}
			}						
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}
		}
	}
}
