package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import java.util.Iterator;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorComponentList implements IWeb2cConstants{

	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop  = null;
	private String textAlignment = null;
	private String fontSize = null;
	private String fontThickness= null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private ArrayList<String> caption = null;
	private String color = null;
	private String backgroundColor = null;
	private String toolTipText = null;
	private String rows = null;

	/**
	 * Component List
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cEditorComponentList(String refId, ArrayList<String> names, ArrayList<String> values) {

		referenceId = refId;
		decodeAttributes(names, values);
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	

		int i = 0;
		String str = null;
		componentHTML.add("			<select id=\"" + id +  "\" title=\"" + toolTipText + "\" size=\"" + rows + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + ";  font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + ";\">");
		for (Iterator<String> it = caption.iterator(); it.hasNext();) {	
			str = (String)it.next();
			if (i == 0) {
				componentHTML.add("				<option selected id=\"" + id + "_option" + i + "\" value=\"" + str + "\">" + str + "</option>");
			}
			else {

				componentHTML.add("				<option id=\"" + id + "_option" + i + "\" value=\"" + str + "\">" + str + "</option>");
			}
			i = i + 1;
		}
		componentHTML.add("			</select>");
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("	//EditorComponentList: " + id);
		componentJavaScript.add("	var componentListId = \"" + id + "\";");

		return componentJavaScript;
	}

	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;

		caption = new ArrayList<String>();

		fontSize = medium;
		fontThickness = normal;
		textAlignment = left;
		color = black;
		backgroundColor = white;	
		borderStyle = thin;
		borderWidth = thin;
		borderColor = black;
		rows = "4";
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}
			if (name.equals("caption")) {
				caption.add(value);
			}	
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}	
			if (name.equals("rows")) {
				rows = value;
			}	
		}	
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}

}
