package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import java.util.Iterator;
import web2cEditorComponents.Web2cEditorActionButton;
import web2cEditorComponents.Web2cEditorAttributeList;
import web2cEditorComponents.Web2cEditorComponentList;
import web2cEditorComponents.Web2cEditorDrawingFrame;
import web2cEditorComponents.Web2cEditorTextInput;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorSection implements IWeb2cConstants {

	private ArrayList<Object> sectionsElements = new ArrayList<Object>();
	private ArrayList<ArrayList<String>> sectionsElementsHTML = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> sectionsElementsJavaScript = new ArrayList<ArrayList<String>>();
	private String hostURL = null;
	private String referenceId = null;
	private String referencePoint = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop = null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private String backgroundColor = null;
	private String userPrivilege = null;

	/**
	 * Section 
	 * @param url URL of host servicing the request
	 * @param refId ID of reference component which component belongs to
	 * @param refPoint coordinates (left upper corner) of reference component which component belongs to
	 * @param attributeNames list of section's valid attribute names
	 * @param attributeValues list of actual section's attribute values
	 * @param elementsNames list of names of components which belong to the actual section
	 * @param elementsAttributeNames list of valid attribute names of components which belong to the actual section
	 * @param elementsAttributeValues list of actual attribute values of components which belong to the actual section
	 * @param privilege actual user privilege
	 */
	public Web2cEditorSection(String url, String refId, String refPoint, ArrayList<String> attributeNames, ArrayList<String> attributeValues, ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues, String privilege) {

		hostURL = url;
		referenceId = refId;
		referencePoint = refPoint;
		userPrivilege = layman;
		if (userPrivilege == null) {
			userPrivilege = layman;
		}
		else {
			userPrivilege = privilege;
		}
		decodeAttributes(attributeNames, attributeValues);
		setSectionElements(elementsNames, elementsAttributeNames, elementsAttributeValues);
	}
	
	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<Object> getComponents() {
		
		
		ArrayList<Object> list = new ArrayList<Object>();
		for (Iterator<Object> it = sectionsElements.iterator(); it.hasNext();) {
			list.add(it.next());
		}
		
		return list;
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> sectionHTML = new ArrayList<String>();	
		
		sectionHTML.add("		<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + ";\">");
		for(Iterator<ArrayList<String>> it1 = sectionsElementsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionHTML.add((String)it2.next());
			}
		}
		sectionHTML.add("		</div>");
		
		return sectionHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> sectionJavaScript = new ArrayList<String>();	
		
		sectionJavaScript.add("	//EditorSection: " + id);
		sectionJavaScript.add("	var sectionId = \"" + id + "\";");
		for(Iterator<ArrayList<String>> it1 = sectionsElementsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionJavaScript.add((String)it2.next());
			}
		}
		
		return sectionJavaScript;
	}

	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;
		int xRef = 0;
		int yRef = 0;

		borderStyle = none;
		borderWidth = thin;
		borderColor = black;
		backgroundColor = white;
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xRef = Integer.valueOf((referencePoint).split(",")[0]);
				yRef = Integer.valueOf((referencePoint).split(",")[1]);
				xLeft = Integer.valueOf(value.split(",")[0]) + xRef;
				yTop = Integer.valueOf(value.split(",")[1]) + yRef;
				xRight = Integer.valueOf(value.split(",")[2]) + xRef;
				yBottom = Integer.valueOf(value.split(",")[3]) + yRef;
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}			
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}
		}		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	private void setSectionElements(ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues) {
		
		Iterator<String> it3 = elementsNames.iterator();
		Iterator<ArrayList<String>> it2 = elementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it1 = elementsAttributeNames.iterator(); it1.hasNext();) {
			String element = (String)it3.next();
			ArrayList<String> names = (ArrayList<String>)it1.next();
			ArrayList<String> values = (ArrayList<String>)it2.next();
			if (element == "EditorComponentList") {
				Web2cEditorComponentList value = new Web2cEditorComponentList(referenceId, names, values);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
			if (element == "EditorTextInput") {
				Web2cEditorTextInput value = new Web2cEditorTextInput(referenceId, names, values);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
			if (element == "EditorActionButton") {
				Web2cEditorActionButton value = new Web2cEditorActionButton(hostURL, referenceId, names, values, userPrivilege);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
			if (element == "EditorAttributeList") {
				Web2cEditorAttributeList value = new Web2cEditorAttributeList(referenceId, names, values);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
			if (element == "EditorDrawingFrame") {
				Web2cEditorDrawingFrame value = new Web2cEditorDrawingFrame(referenceId, names, values);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
		}
	}
}
