package de.desy.web2cToolkit.web2cMessenger;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import twitter4j.AsyncTwitter;
import twitter4j.AsyncTwitterFactory;
import twitter4j.ResponseList;
import twitter4j.Status;
import twitter4j.TwitterAdapter;
import twitter4j.TwitterException;
import twitter4j.TwitterMethod;
import twitter4j.http.RequestToken;

/**
 * @author R.Bacher, DESY
 */
public class Web2cTwitter implements IWeb2cConstants, Observer  {

	private TwitterAdapter adapter = null;
    private AsyncTwitterFactory factory = null;
    private AsyncTwitter twitter = null;
    private RequestToken requestToken = null;
    private String tweet = "";
    private long statusId = 0;
    private long oldStatusId = 0;
    private ArrayList<Long> statusIdList = new ArrayList<Long>();
    private int nextTweetIndex = 0;
    
    private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private String twitterDir = "";
	private Web2cFileLogger twitterLogger = null;
    
	private ArrayList<String> groupCaption = null;
	private ArrayList<ArrayList<String>> lineText = null;
	private ArrayList<ArrayList<String>> lineUri = null;
	private ArrayList<ArrayList<String>> lineFormat = null;
	ArrayList<String> monitorList = new ArrayList<String>();
	
	private int error = 0;
	private int resetCounter = 0;
	private int resetCounterMax = 20;
	private boolean connectToTwitterEnabled = false;
	private boolean loggingEnabled = false;
	private TwitterUpdater twitterUpdater = null;
	private TwitterConfigFileParser twitterParser = null;
	private Web2cASCIICharacterEscaper characterEscaper = null;
	
    /**
     * Twitter interface
     * Constructor
	 * @param cda actual common device access instance
	 * @param dir directory of file messenger.xml
	 * @param logger logger
     */
	public Web2cTwitter(Web2cCommonDeviceAccess cda, String dir, Web2cFileLogger logger) {
		
		commonDeviceAccess = cda;
		twitterDir = dir;
		twitterLogger = logger;
		characterEscaper = new Web2cASCIICharacterEscaper();
		initialize();
		if (error == 0 && connectToTwitterEnabled == true) {
			twitterParser = new TwitterConfigFileParser(error);
			twitterParser.addObserver(this);
			twitterParser.startParser();
		}
	}	
	
	/**
	 * Returns the control system monitors attached to this service
	 * @return list of monitors
	 */
	public ArrayList<String> getMonitors() {
		
		return monitorList;
	}
	
	/**
    * Closes the connection to twitter.
    */
	public void closeMessenger() {

		twitterUpdater.stopUpdater();
	}
	
	/**
    * Opens the connection to twitter.
    */
	public void openMessenger() {

		twitterUpdater.startUpdater();
	}
	
	/**
	* Returns the error status.
	* @return error
	*/
	public int getError() {
			
		return error;
	}
	
	/**
	* Returns the last tweet.
	* @return tweet
	*/
	public String getMessage() {
			
		return tweet;
	}
	
	/**
	* Returns the connection status.
	* @return status
	*/
	public boolean serviceEnabled() {
			
		return connectToTwitterEnabled;
	}	
	
	/**
	* Returns the publishing status.
	* @return status
	*/
	public boolean serviceStarted() {
			
		return twitterUpdater.getUpdaterStatus();
	}
	
	/**
	* Returns the authorization URL.
	*/
	public String getAuthorizationURL() {
			
		return requestToken.getAuthorizationURL();
	}
	
    /**
     * Initializes the twitter factory
     */
	@SuppressWarnings("unchecked")
	private void initialize() {

		String id = null;
		String password = null;
		String enabled = null;
		String logging = null;
		
	   	SAXBuilder builder = null;
    	Document twitterProfile = null;
    	Element profile = null;
    	List<Element> messengerList = null;
    	Element messenger = null;
    	List<Element> messengerAttributes = null;
    	Element messengerAttribute = null;
    	
    	int index = 0;

		try {
			id = "";
			password = "";
			enabled = "";
			logging = "";
			builder = new SAXBuilder();
			twitterProfile = builder.build(new File(registryDir + File.separator + messengerProfileFile));
			profile = twitterProfile.getRootElement();
			messengerList = (List<Element>)profile.getChildren();    
			for (Iterator<Element> it1 = messengerList.iterator(); it1.hasNext();) {
				messenger = (Element)it1.next();  
				messengerAttributes = messenger.getChildren();
				for (Iterator<Element> it2 = messengerAttributes.iterator(); it2.hasNext();) {
					messengerAttribute = (Element)it2.next();  
					if ((messengerAttribute.getName()).equals("interface") && messengerAttribute.getValue().equals("twitter")) index = messengerList.indexOf(messenger);
				}
			}		
			messengerAttributes = ((Element)(messengerList.toArray()[index])).getChildren();		
			for (Iterator<Element> it3 = messengerAttributes.iterator(); it3.hasNext();) {
				messengerAttribute = (Element)it3.next();  
				if ((messengerAttribute.getName()).equals("id")) {
					id = messengerAttribute.getValue();
				} 
				else if ((messengerAttribute.getName()).equals("password")) {
					password = messengerAttribute.getValue();
				}
				else if ((messengerAttribute.getName()).equals("enabled")) {
					enabled = messengerAttribute.getValue();
				}
				else if ((messengerAttribute.getName()).equals("logging")) {
					logging = messengerAttribute.getValue();
				}
			}
			
			if (logging.equals("yes")) {
				loggingEnabled = true;
			}
			else {
				loggingEnabled = false;
			}

			if (enabled.equals("yes")) {
				connectToTwitterEnabled = true;
				adapter = new TwitterAdapter() {
					public void gotUserTimeline(ResponseList<Status> statuses) {
						if (statuses.isEmpty() == false) {
							for (int i = 0; i < statuses.toArray().length; i++) {
								Status status = (Status)(statuses.toArray()[i]);
								if (i > 0) {
									if (error == 0) twitter.destroyStatus(status.getId());
								}
								if (i == 0 && twitterUpdater.getUpdaterStatus() == false) {
									if (error == 0) twitter.destroyStatus(status.getId());
								}
							}
						}
					}
		            public void updatedStatus(Status status) {
		            	if (twitterUpdater.getUpdaterStatus() == true) {
			            	statusId = status.getId();  
			            	if (loggingEnabled == true) twitterLogger.appendLog("Twitter status (" + statusId + ") updated: " + status.getText());
		            		if (!statusIdList.contains(statusId)) {
		            			statusIdList.add(0, statusId);
		            		}
	            			if (statusIdList.size() > groupCaption.size()) {
	            				for (int i = groupCaption.size(); i < statusIdList.size(); i++) {
		            				twitter.destroyStatus(statusIdList.get(i));
	            				}
	            			}
		            	}
		            }
		            public void destroyedStatus(Status status) {
		            	if (twitterUpdater.getUpdaterStatus() == true) {
            				oldStatusId = status.getId();  
            				statusIdList.remove(oldStatusId);
			            	if (loggingEnabled == true) twitterLogger.appendLog("Twitter status (" + oldStatusId + ") destroyed: " + status.getText());
		            	}
		            }
		            public void onException(TwitterException e, TwitterMethod method) {
		            	if (twitterUpdater.getUpdaterStatus() == true) {
			                if (method == TwitterMethod.UPDATE_STATUS) {	
			                	if (loggingEnabled == true) twitterLogger.appendLog("Error updating twitter status: " + e.getMessage());
			                }
			                if (method == TwitterMethod.DESTROY_STATUS) {
			                	if (loggingEnabled == true) twitterLogger.appendLog("Error destroying twitter status: " + e.getMessage());
			                }
		            	}
		                if (method == TwitterMethod.USER_TIMELINE) {
		                	if (loggingEnabled == true) twitterLogger.appendLog("Error destroying user timeline: " + e.getMessage());
		                }
		            }
				};
		        factory = new AsyncTwitterFactory(adapter);
				twitter = factory.getOAuthAuthorizedInstance(id, password);
			    try {
					requestToken = twitter.getOAuthRequestToken();
					System.out.println("Please grant access to your twitter account: " + requestToken.getAuthorizationURL());
			    }
			    catch (TwitterException e) {
					error = error + 1;
					System.out.println("Error getting request token from twitter!");
				}
			}
			else {
				connectToTwitterEnabled = false;
				System.out.println("Twitter disabled!");
			}
		} 
		catch (JDOMException e) {
			error = error + 1;
			System.out.println("Error parsing twitter profile file!");
		} 
		catch (IOException e) {
			error = error + 1;
			System.out.println("Error reading twitter profile file!");
		}
	}
	
	/**
	 * Gets the completion notification from the twitter configuration file parser and starts the twitter update service
	 * @param parser twitter file parser
	 * @param parserStatus error status
	 */
	public void update(Observable parser, Object parserStatus) {
		
		error = (Integer)parserStatus;
		if (error == 0 && connectToTwitterEnabled == true) twitterUpdater = new TwitterUpdater();
	}
	
	/**
	 * @author R.Bacher, DESY
	 */
	public class TwitterConfigFileParser extends Observable implements Runnable {
		
		private Thread parser = null;
		private int parserError = 0;
		
		/**
		 * Constructor
		 * Parses the twitter configuration file
		 * @param error error status after initialization
		 */
		public TwitterConfigFileParser(int error) {
			
			parserError = error;
			
			if (parser == null) {
				parser = new Thread(this);
			}
		}
		
		/**
		 * Starts the parser
		 */
		public void startParser() {  
			
			parser.start();
		}
		
		/**
		 * Parses the configuration file containing the content to be sent to twitter and establishes the corresponding monitor links
		 */
		@SuppressWarnings("unchecked")
		public void run() {  
			
		   	SAXBuilder builder = null;
	    	Document twitterContent = null;
	    	Element content = null;
	    	List<Element> groupList = null;
	    	Element group = null;
	    	List<Element> groupAttributes = null;
	    	Element attribute = null;

			try {
				groupCaption = new ArrayList<String>();
				lineText = new ArrayList<ArrayList<String>>();
				lineUri = new ArrayList<ArrayList<String>>();
				lineFormat = new ArrayList<ArrayList<String>>();
				builder = new SAXBuilder();
				twitterContent = builder.build(new File(twitterDir + File.separator + messengerFile));
				content = twitterContent.getRootElement();
				groupList = (List<Element>)content.getChildren();   
				for (Iterator<Element> it1 = groupList.iterator(); it1.hasNext();) {
					group = (Element)it1.next();  
					groupAttributes = (List<Element>)group.getChildren();
					ArrayList<String> text = new ArrayList<String>();
					ArrayList<String> uri = new ArrayList<String>();
					ArrayList<String> format = new ArrayList<String>();
					for (Iterator<Element> it2 = groupAttributes.iterator(); it2.hasNext();) {
						attribute = (Element)it2.next();  
						if ((attribute.getName()).equals("caption")) {
							groupCaption.add(attribute.getValue());
//							System.out.println(attribute.getValue());
						} 
						else if ((attribute.getName()).equals("line")) {
							int ret = 0;
							ret = commonDeviceAccess.registerMonitor(attribute.getValue().split(",")[2]);
							if (ret == 0) {
								text.add(attribute.getValue().split(",")[0]);
								format.add(attribute.getValue().split(",")[1]);
								uri.add(attribute.getValue().split(",")[2]);
								monitorList.add(attribute.getValue().split(",")[2]);
							}
//							System.out.println(attribute.getValue().split(",")[0]);
//							System.out.println(attribute.getValue().split(",")[1]);
//							System.out.println(attribute.getValue().split(",")[2]);
						}
					}
					lineText.add(text);
					lineFormat.add(format);
					lineUri.add(uri);
				}
			} 
			catch (JDOMException e) {
				parserError = parserError + 1;
				System.out.println("Error parsing twitter file!");
			} 
			catch (IOException e) {
				parserError = parserError + 1;
				System.out.println("Error reading twitter file!");
			}
			
			setChanged();
			notifyObservers(parserError);
			
			parser.interrupt();   
		}
	}
	
	/**
	 * @author R.Bacher, DESY
	 */
	public class TwitterUpdater implements Runnable {
		
		private Thread updater = null;
		private boolean updaterStarted = false;
		
		/**
		 * Constructor
		 * Update timer
		 */
		public TwitterUpdater() {
			
			if (updater == null) {
				updater = new Thread(this);
			}
		}
		
		/**
		 * Starts updating the twitter status
		 */
	    public void startUpdater() {
	    	
	    	if (updaterStarted == false) {
		    	try {
		    		twitter.getOAuthAccessToken();
			    	updater.start();
		    	} 
		    	catch (TwitterException e) {
		    		error = error + 1;
					System.out.println("Error getting access token from twitter!");
		    	}
	    	}
	    }
		
		/**
		 * Stops updating the twitter status
		 */
	    public void stopUpdater() {
	    	
	    	updaterStarted = false;
	    }
		
		/**
		 * Returns the updater status
		 * @return updater status
		 */
		public boolean getUpdaterStatus() {
			
			return updaterStarted;
		}

		/**
		 * Updates the twitter status and clears the time line
		 */
		public void run() {    
			
	    	updaterStarted = true;
			twitter.getUserTimeline();
			System.out.println("Updating twitter status started!");
			while (updaterStarted == true) {
				try {
					if (updaterStarted == true) {
						for (int i = 0; i < (twitterUpdateInterval / 2); i++) {
							if (updaterStarted == true) Thread.sleep(1000);
						}
					}
					if (updaterStarted == true) generateAndUpdateStatus();
					if (updaterStarted == true) {
						for (int i = 0; i < (twitterUpdateInterval / 2); i++) {
							if (updaterStarted == true) Thread.sleep(1000);
						}
					}
					if (updaterStarted == true) {
						if (resetCounter == resetCounterMax) {
							resetCounter = 0;
							if (error == 0) twitter.getUserTimeline();
						}
						else if (resetCounter < resetCounterMax) {
							resetCounter = resetCounter + 1;
						}
						else {
							resetCounter = 0;
						}
					}
				} 
				catch (InterruptedException e) {}
			}
			
			updater.interrupt();   
			System.out.println("Updating twitter status stopped!");
		}
		
		/**
	    * Generates the status and sends it to twitter
	    */
		private void generateAndUpdateStatus() {
			
			String status = "";
			String ret = null;

			try {
				Iterator<String> it2 = groupCaption.iterator();
				Iterator<ArrayList<String>> it3 = lineText.iterator();
				Iterator<ArrayList<String>> it4 = lineFormat.iterator();
				for (Iterator<ArrayList<String>> it1 = lineUri.iterator(); it1.hasNext();) {
					String caption = (String)it2.next(); 
					ArrayList<String> uriList = (ArrayList<String>)it1.next(); 
					ArrayList<String> textList = (ArrayList<String>)it3.next(); 
					ArrayList<String> formatList = (ArrayList<String>)it4.next(); 
					if (nextTweetIndex == groupCaption.indexOf(caption)) {
						status = caption + ": ";
						Iterator<String> it7 = textList.iterator();
						Iterator<String> it8 = formatList.iterator();
						for (Iterator<String> it6 = uriList.iterator(); it6.hasNext();) {
							String text = (String)it7.next(); 
							String format = (String)it8.next(); 
							ret = characterEscaper.decodeString((commonDeviceAccess.getMonitorData((String)(it6.next()))).split(">")[1].split("<")[0]);
							if (format.equals("") || format.equals(" ")) {  //process ret as string as it is
								status = status + text + ret + " ";
							}
							else if (format.contains("x")) {                //fixed-length string, process ret as string
								if (format.length() <= ret.length()) {
									status = status + text + ret.substring(0, format.length() - 1) + " ";
								}
								else {
									status = status + text + ret + " ";
								}
							}
							else {                                          //number format, process ret as number
								DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));
								status = status + text + formatter.format(Double.valueOf(ret)) + " ";
							}
						}
						status = status.substring(0, status.length() - 1);
					}
				}
			}
			catch (Exception e) {
				status = "Sorry! No status available.";
			}
					
			if (error == 0) {
				twitter.updateStatus(status);
//				System.out.println("Last tweet: " + status);
				tweet = status;
			}
			
			if (nextTweetIndex < groupCaption.toArray().length - 1) {
				nextTweetIndex = nextTweetIndex + 1;
			}
			else {
				nextTweetIndex = 0;
			}
		}
	}
}