package de.desy.web2cToolkit.web2cServlets;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jfree.chart.ChartUtilities;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatHandler;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cProbe;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cInitReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cImageCache;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;
import de.desy.web2cToolkit.web2cUtil.web2cSecurity.Web2cMD5HashGenerator;

/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2c extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants, Observer {
	 
	private int sessionInactivityTimeout = sessionTimeoutInterval;   // [s]
	
	private String ipAddress = null;
	private HttpSession httpSession = null;
	private String hostURL = null;	
	
	private String homeDir = null;
	private String webDir = null;
	private Web2cRegistryReader registryReader = null;
	private Web2cInitReader initReader = null;
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cImageCache imageCache = null;
	private Web2cCommonVideoConnector commonVideoConnector = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cChatHandler chatHandler = null;
	private Web2cGarbageCollector garbageCollector = null;
	private Web2cProbe probe = null;
	
	private Web2cContextHandler web2cContextHandler = null;
	private Calendar calendar = null;	
	private boolean earlyRegistering = true;
	private volatile Double[] constructionProgress = new Double[1];
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;
	private boolean cacheIsEnabled = cacheEnabled;
	private boolean sessionIsWatcherEnabled = sessionWatcherEnabled;
	
	/**
	 * Web2c Synoptic Display Servlet: handles client login, requests and updates
	 */
	public Web2c() {

		super();
		calendar = Calendar.getInstance();
	}   
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (web2cContextHandler == null) {
			web2cContextHandler = new Web2cContextHandler(config.getServletContext(), "synopticDisplay");
		}			
		getContextRelevantFields();	
		doEarlyLinkRegistering();
		
		if (garbageCollectionIsEnabled == true) System.out.println("Java garbage collection enabled!");	
		if (garbageCollectionIsEnabled == false) System.out.println("Java garbage collection disabled!");
		
		if (cacheIsEnabled == true) System.out.println("Web2c cache enabled!");	
		if (cacheIsEnabled == false) System.out.println("Web2c cache disabled!");
		
		if (sessionIsWatcherEnabled == true) System.out.println("Web2c garbage collection enabled!");	
		if (sessionIsWatcherEnabled == false) System.out.println("Web2c garbage collection disabled!");
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {	
			initializeServletMethod(request);			
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);		    	
//				long start = System.currentTimeMillis();		    	
				if (servletInputParameter.compareToIgnoreCase("<plot>") == 0){
					doGetPlot(request, response);
				}				
				else if (servletInputParameter.compareToIgnoreCase("<vimg>") == 0){
					doGetVideoImage(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("<hplt>") == 0){
					doGetHplt(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("<htxt>") == 0){
					doGetHtxt(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("<load>") == 0) {
					doGetLoad(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					doGetPage(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(open)") == 0) {
					doGetOpen(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {
					doGetFile(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(simu)") == 0) {
					doGetSimu(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(diag)") == 0) {
					doGetDiag(request, response);
				}
				else {
					System.out.println("Web2c-GET: Unknown servlet input parameter specified!");
				}
//				long stop = System.currentTimeMillis();
//				System.out.println("Web2c-GET server response [ms] for " + servletInputParameter + ": " + (stop - start));
		    }	
		    else {
		    	System.out.println("Web2c-GET: No servlet input parameter specified!");
		    }
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2c-GET: Error processing servlet!");
		}
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {	
			initializeServletMethod(request);
			if (request.getParameter("param") != null) {
				String servletInputParameter = (request.getParameter("param")).substring(0,8);
//				long start = System.currentTimeMillis();
			    if (servletInputParameter.compareToIgnoreCase("<update>") == 0) {
					doPostUpdate(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<action>") == 0) {
					doPostAction(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<search>") == 0) {
					doPostSearch(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<elogin>") == 0) {
					doPostElogin(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<ldchck>") == 0) {
					doPostCheckPageLoadingStatus(request, response);
			    }
			    else {
			    	System.out.println("Web2c-POST: Unknown servlet input parameter specified!");
			    }
//				long stop = System.currentTimeMillis();
//				System.out.println("Web2c-POST server response [ms] for " + servletInputParameter + ": " + (stop - start));	
			}
			else {
		    	System.out.println("Web2c-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2c-POST: Error processing servlet!");
		}
	} 
	
	/**
	 * Performs early monitor/receiver link registering
	 */
	private void doEarlyLinkRegistering() {

		earlyRegistering = true;
		ArrayList<String> monitorToRegisterList = initReader.getMonitorList();
		if (!monitorToRegisterList.isEmpty()) {
			for (Iterator<String> it = monitorToRegisterList.iterator(); it.hasNext();) {
				String monitor = (String)it.next();
				commonDeviceAccess.registerMonitor(monitor);
			}
		}		
		ArrayList<String> receiverToRegisterList = initReader.getReceiverList();
		if (!receiverToRegisterList.isEmpty()) {
			for (Iterator<String> it = receiverToRegisterList.iterator(); it.hasNext();) {
				String receiver = (String)it.next();
				commonVideoConnector.connectToVideoSource(receiver);
			}
		}
		earlyRegistering = false;
	}
	
	/**
	 * Returns login page
	 * @param randomNumber random number for password encryption
	 * @param timeStamp timeStamp to make login file name unique
	 * @return HTML page
	 */
	private ArrayList<String> getLoginPage(String randomNumber, String timeStamp) {

		ArrayList<String> out = null;
		String enterKeyCode = "13";
		
		out = new ArrayList<String>();
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Web2cSynopticDisplay Login</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		out.add("		<link rel=\"shortcut icon\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2cLogin_" + timeStamp + ".js\"></script>");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "md5.js\"></script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<input type=\"hidden\" id=\"random\" name=\"random\" value=\"" + randomNumber + "\">");
		if (initReader != null) {
			out.add("		<div align=\"center\"><b><h1>" + initReader.getSalutation() + "</h1></b></div>");
		}
		else {
			out.add("		<div align=\"center\"><b><h1>" + web2cSalutation + "</h1></b></div>");
		}
		out.add("		<div align=\"center\"><b><h2>Please enter your username and password to log in.</h2></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td align=\"left\"><b>User name: </b></td><td class=\"control\" height=\"50\"><input type=\"text\" name=\"username\" id=\"username\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Password: </b></td><td class=\"control\" height=\"50\"><input type=\"password\" id=\"password\" name=\"password\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td> </td><td class=\"control\" height=\"25\"></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"login\" id=\"login\" value=\"Login\" onclick=\"login()\"></center></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"close\" id=\"close\" value=\"Close Login-Window\" onclick=\"closeLoginWindow()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><p align=\"center\"><b>N.B.</b> Web2c pages are best displayed with the following browser's default font settings: encoding = Western, variable-font size: 16px, fixed-font size: 13px, minimum font size: 6px or smaller</p>");
		out.add("	</body>");
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Returns Javascript code
	 * @param id session identifier
	 * @return Javascript code as text
	 */
	private ArrayList<String> getJavascript(String id) {
		
		ArrayList<String> javascript = null;
		
		javascript = new ArrayList<String>();

		javascript.add("		var xhr;");
		javascript.add("		function initPage() {");
		javascript.add("			initXhr();");
		javascript.add("			if (document.getElementById(\"username\").value == \"guest\") {");
		javascript.add("			    login();");
		javascript.add("			}");
		javascript.add("			setTimeout(closeLoginWindow," + loginTimeOut + ");");
		javascript.add("		}");
		javascript.add("		function encryptPassword() {");
		javascript.add("			var data = document.getElementById(\"password\").value;");
		javascript.add("			var random = document.getElementById(\"random\").value;");
		javascript.add("			var string = data + random;");
		javascript.add("			return hex_md5(string);");
		javascript.add("		}");
		javascript.add("		function initXhr() {"); 
		javascript.add("			try {");                                                  
		javascript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		javascript.add("			} catch (event1) {");                             
		javascript.add("				try {");
		javascript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		javascript.add("				} catch (event2) {");
		javascript.add("					xhr = false;");
		javascript.add("				}");
		javascript.add("			}");
		javascript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		javascript.add("				xhr = new XMLHttpRequest();");                 
		javascript.add("			}"); 																	
		javascript.add("		}");
		javascript.add("		function login() {");
		javascript.add("			var now = new Date();");
		javascript.add("			var data = encryptPassword();");
		javascript.add("			var username = document.getElementById(\"username\").value;");
		javascript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2c?param=<elogin><sessionId>" + id + "</sessionId><userName>\" + username + \"</userName><value>\" + data + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></elogin>\", true);"); 
		javascript.add("			xhr.onreadystatechange = function() {");
		javascript.add("				if (xhr.readyState == 4) {");
		javascript.add("					window.location.href = xhr.responseText;");
		javascript.add("           		};");   
		javascript.add("            };"); 
		javascript.add("			xhr.send(null);");  
		javascript.add("		}");
		javascript.add("		function closeLoginWindow() {");
//		javascript.add("			history.back();");  
		javascript.add("			window.close();");  
		javascript.add("		}");
		
		return javascript;
	}
	
	/**
	 * Returns error page if page generation has failed
	 * @param configFileName name of config file
	 * @return page
	 */
	private String  errorPageOut(String configFileName) {
		
		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<html>");
		strBfr.append("	<body>");
		strBfr.append("		<div>Page \"" + configFileName + "\" could not be created!</div>");
		strBfr.append("		<div>Reasons might be:</div>");
		strBfr.append("		<div> - Configuration file does not exist</div>");
		strBfr.append("		<div> - Wrong configuration file name</div>");
		strBfr.append("		<div> - Syntax error(s) in configuration file</div>");
		strBfr.append("		<div> - Invalid control system address(es) specified in configuration file</div>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Returns initial page
	 * @param web2cSession session
	 * @return page
	 */
	private String loadingPageOut(String web2cSession) {

		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		strBfr.append("<html>");
		strBfr.append("   <head>");
		strBfr.append("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		strBfr.append("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		strBfr.append("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		strBfr.append("		<title>Web2cSynopticDisplayViewer loading...</title>");
		strBfr.append("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		strBfr.append("		<link rel=\"shortcut icon\"  type=\"image/png\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		strBfr.append("     <script type=\"text/javascript\">");
		strBfr.append("			var xhr;");
		strBfr.append("			var sessionId = " + web2cSession + ";");
		strBfr.append("			function initPage() {");	
		strBfr.append("				setInterval(\"getPageLoadingStatus()\", 1000);");
		strBfr.append("			}");		
		strBfr.append("			function getPageLoadingStatus() {"); 
		strBfr.append("				try {");                                                  
		strBfr.append("					xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		strBfr.append("				} catch (event1) {");                             
		strBfr.append("					try {");
		strBfr.append("						xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		strBfr.append("					} catch (event2) {");
		strBfr.append("						xhr = false;");
		strBfr.append("					}");
		strBfr.append("				}");
		strBfr.append("				if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		strBfr.append("					xhr = new XMLHttpRequest();");                 
		strBfr.append("				}"); 	
		strBfr.append("				var now = new Date();");
		strBfr.append("				xhr.open(\"post\", \"/web2cToolkit/Web2c?param=<ldchck><sessionId>\" + sessionId + \"</sessionId><timeStamp>\" + String(now.getTime()) + \"</timeStamp></ldchck>\", true);"); 
		strBfr.append("				xhr.onreadystatechange = function() {");       
		strBfr.append("					if (xhr.readyState == 4) {"); 
		strBfr.append("						if ((xhr.responseText.indexOf(\"loading\") == -1) && (xhr.responseText != \"\")) {"); 
		strBfr.append("							window.location.href = location.href.split(\"=\")[0] + \"=\" + xhr.responseText;");
		strBfr.append("						}");      
		strBfr.append("						else {");  
		strBfr.append("							ref = document.getElementById(\"loadingText\");");
		strBfr.append("							ref.innerHTML = xhr.responseText;");      
		strBfr.append("						}");          
		strBfr.append("					}");          
		strBfr.append("				};");   
		strBfr.append("				xhr.send(null);");  
		strBfr.append("			}");  
		strBfr.append("       </script>");
		strBfr.append("	</head>");
		strBfr.append("	<body onload=\"initPage()\">");
		strBfr.append("		<h1><b><div id=\"loadingText\">Page loading: 0% ...</div></b></h1>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Updates all context relevant fields
	 */
	private void getContextRelevantFields() {

		homeDir = web2cContextHandler.getHomeDir();
		webDir = web2cContextHandler.getWebDir();
		registryReader = web2cContextHandler.getWeb2cRegistryReader();
		initReader = web2cContextHandler.getWeb2cInitReader();
		sessionLogger = web2cContextHandler.getSessionLogger();
		debugLogger = web2cContextHandler.getDebugLogger();
		debugListener = web2cContextHandler.getWeb2cKeyBoardListener();
		web2cSessionHandlerList = web2cContextHandler.getWeb2cSessionHandlerList();
		imageCache = web2cContextHandler.getWeb2cImageCache();
		commonVideoConnector = web2cContextHandler.getWeb2cCommonVideoConnector();
		commonDeviceAccess = web2cContextHandler.getWeb2cCommonDeviceAccess();
		chatHandler = web2cContextHandler.getWeb2cChatHandler();
		garbageCollector = web2cContextHandler.getWeb2cGarbageCollector();
		probe = web2cContextHandler.getWeb2cProbe();
	}
	
	/**
	 * Initializes or updates request specific fields
	 * @param request client request
	 */
	private void initializeServletMethod(HttpServletRequest request) {
			
		ipAddress = request.getRemoteAddr();
		httpSession = request.getSession();
		hostURL = request.getRequestURL().toString();	
		
		debugListener.getKeyboardInput();
	}
	
	/**
	 * Handles client request to get plot chart as byte stream
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetPlot(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": image/png");
			String parameter = (request.getParameter("param")).substring(6);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String id = parameter.split("</sessionId>")[1].split("</id>")[0].replace("<id>", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				response.setContentType("image/png");
				imageCache.getChartFromCache(associatedSessionHandler, id).writeTo(response.getOutputStream());	
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter <plot>!");
		}
	}
	
	/**
	 * Handles client request to get video image as byte stream
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetVideoImage(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": image/jpg");
			String parameter = (request.getParameter("param")).substring(6);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String id = parameter.split("</sessionId>")[1].split("</id>")[0].replace("<id>", ""); 
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				response.setContentType("image/jpg");
				imageCache.getVideoFromCache(associatedSessionHandler, id).writeTo(response.getOutputStream());
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter <vimg>!");
		}
	}
	
	/**
	 * Handles client request to get history chart as image stream
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetHplt(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": image/png");
			String parameter = (request.getParameter("param")).substring(6);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String value1 = parameter.split("</sessionId>")[1].split("</value>")[0].replace("<value>", ""); 
			String value2 = parameter.split("</sessionId>")[1].split("</value>")[1].replace("<value>", "");
			String value3 = parameter.split("</sessionId>")[1].split("</value>")[2].replace("<value>", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				response.setContentType("image/png");
				try {
					ChartUtilities.writeChartAsPNG(response.getOutputStream(), associatedSessionHandler.getWeb2cPage().getHistory().getChart(value1, value2, value3), associatedSessionHandler.getWeb2cPage().getHistory().getChartWidth(), associatedSessionHandler.getWeb2cPage().getHistory().getChartHeight());
				} catch (IOException e) {
					System.out.println("Error converting chart to PNG!");
				}
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter <hplt>!");
		}	
	}
	
	/**
	 * Handles client request to get history chart window
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetHtxt(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String value1 = parameter.split("</sessionId>")[1].split("</value>")[0].replace("<value>", ""); 
			String value2 = "";
			try {
				value2 = parameter.split("</sessionId>")[1].split("</value>")[1].replace("<value>", "");
			}
			catch (Exception e) {
				value2 = "";
			}
			String value3 = "";
			try {
				value3 = parameter.split("</sessionId>")[1].split("</value>")[2].replace("<value>", "");
			}
			catch (Exception e) {
				value3 = "";
			}
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				response.setContentType("text/html");
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<String> it1= associatedSessionHandler.getWeb2cPage().getHistory().getPage(value1, value2, value3).iterator(); it1.hasNext();) {
					strBfr.append((String)it1.next());
				}
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(strBfr.toString());
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (htxt)!");
		}
	}
	
	/**
	 * Handles client request to load finally generated Web2c page as HTML
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetLoad(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");			
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("]")[0].replace("[", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				Web2cPage web2cPage = associatedSessionHandler.getWeb2cPage();
				String error = web2cPage.getError().replace("null", "");	
				StringBuffer strBfr = new StringBuffer();
				if (!error.equals("")) {
					strBfr.append(errorPageOut((String)associatedSessionHandler.getAttribute("loadedConfigFileName")));
					strBfr.append("<div>----->>>>></div>");
					strBfr.append("<div>Reasons identified:</div>");
					for (int i = 0; i < error.split(";").length; i++ ) {
						strBfr.append("<div> \"" + error.split(";")[i] + "\" </div>");
					}
				}
				else {
					for (Iterator<String> it1 = web2cPage.getPage().iterator(); it1.hasNext();) {
						String out = (String)it1.next();
						if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
						strBfr.append(out);
					}
				}
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
				sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("loadedConfigFileName"));			
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter <load>!");
		}
	}
	
	/**
	 * Handles client request to get initial page as HTML after successful login procedure
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetPage(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");			
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("]")[0].replace("[", "");
			String configFileName = "";
			String language = "";
			Boolean dataMode = simulated;
			if (parameter.split(",").length > 1) {
				configFileName = parameter.split("]")[1].replace("/", File.separator).split(",")[0];
				language = parameter.split("]")[1].split(",")[1];
			}
			else {
				configFileName = parameter.split("]")[1].replace("/", File.separator);
				language = "0";
			}
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				if (((String)associatedSessionHandler.getAttribute("dataSource")).equals("simulation")) {
					dataMode = simulated;
				}
				else {
					dataMode = real;
				}
				String accessMode = "file";
				associatedSessionHandler.setAttribute("language", language);
				associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));			
				Web2cPageGenerator generator = new Web2cPageGenerator(sessionId, accessMode, dataMode, request.getRemoteAddr(), configFileName, Integer.valueOf((String)associatedSessionHandler.getAttribute("language")), (String)associatedSessionHandler.getAttribute("userName"));
				generator.addObserver(this);	
				associatedSessionHandler.setWeb2cPageGenerator(generator);
				generator.startGenerator();
				response.setContentType("text/html");			
				response.getWriter().print(loadingPageOut(sessionId));
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (page)!");
		}
	}
	
	/**
	 * Handles client request to initiate loading Web2c page without login procedure and to get initial page as HTML
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetOpen(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String configFileName = "";
			String language = "";
			if (parameter.split(",").length > 1) {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				language = parameter.split(",")[1];
			}
			else {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				language = "0";
			}
			Boolean dataMode = real;
			String accessMode = "open";
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxSessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("loginStatus", "in");
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("loadedConfigFileName", "");
				associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);	
				associatedSessionHandler.setAttribute("dataSource", "real");
				associatedSessionHandler.setAttribute("language", language);
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				Web2cPageGenerator generator = new Web2cPageGenerator(sessionId, accessMode, dataMode, request.getRemoteAddr(), configFileName, Integer.valueOf((String)associatedSessionHandler.getAttribute("language")), (String)associatedSessionHandler.getAttribute("userName"));
				generator.addObserver(this);
				associatedSessionHandler.setWeb2cPageGenerator(generator);
				generator.startGenerator();
				response.setContentType("text/html");			
				response.getWriter().print(loadingPageOut(sessionId));
				response.flushBuffer();
//				System.out.println("Web2c session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId() + " ActiveWeb2cSessions=" + (web2cSessionHandlerList.toArray().length));
			}
			else {
				StringBuffer strBfr = new StringBuffer();
				strBfr.append("<html>");
				strBfr.append("<body>");
				strBfr.append("<div>Page \"" + configFileName + "\" could not be created!</div>");
				strBfr.append("<div>Resources exhausted. Please try again later.</div>");
				strBfr.append("</body>");
				strBfr.append("</html>");			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (open)!");
		}
	}
	
	/**
	 * Handles client request to initiate loading Web2c page with login procedure
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetFile(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String configFileName = "";
			String language = "";
			if (parameter.split(",").length > 1) {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				language = parameter.split(",")[1];
			}
			else {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				language = "0";
			}
			Random randomGenerator = new Random();
			int randomNumber = randomGenerator.nextInt(100000);
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxSessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
				associatedSessionHandler.setAttribute("loadedConfigFileName", "");
				associatedSessionHandler.setAttribute("loginStatus", "out");
				associatedSessionHandler.setAttribute("dataSource", "real");
				associatedSessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("language", language);
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				String key = (String)(associatedSessionHandler.getAttribute("encryptionKey"));
				String id = (String)associatedSessionHandler.getAttribute("id");
				File jScriptDir = new File(webDir);
				String[] fileList = jScriptDir.list();
				for (int i = 0; i < fileList.length; i++) {
					if (fileList[i].contains("Web2cLogin_")) {
						File jScriptFile = new File(webDir + File.separator + fileList[i]);
						if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
					}
				}
				String timeStamp = String.valueOf(System.currentTimeMillis());
				Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cLogin_" + timeStamp + ".js"), debugLogger, debugListener);
				javascriptWriter.setJavascript(getJavascript(id));
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<String> it= getLoginPage(key, timeStamp).iterator(); it.hasNext();) {
					String out = (String)it.next();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
					strBfr.append(out);
				}			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
//				System.out.println("Web2c session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId() + " ActiveWeb2cSessions=" + (web2cSessionHandlerList.toArray().length));
			}
			else {
				StringBuffer strBfr = new StringBuffer();
				strBfr.append("<html>");
				strBfr.append("<body>");
				strBfr.append("<div>Page \"" + configFileName + "\" could not be created!</div>");
				strBfr.append("<div>Resources exhausted. Please try again later.</div>");
				strBfr.append("</body>");
				strBfr.append("</html>");			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (file)!");
		}
	}
	
	/**
	 * Handles client request to initiate loading Web2c page with login procedure in simulation mode
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetSimu(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String configFileName = (request.getParameter("param")).substring(6).replace("/", File.separator);
			Random randomGenerator = new Random();
			int randomNumber = randomGenerator.nextInt(100000);
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxSessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
				associatedSessionHandler.setAttribute("loadedConfigFileName", "");
				associatedSessionHandler.setAttribute("loginStatus", "out");
				associatedSessionHandler.setAttribute("dataSource", "simulation");
				associatedSessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				String key = (String)(associatedSessionHandler.getAttribute("encryptionKey"));
				String id = (String)associatedSessionHandler.getAttribute("id");
				File jScriptDir = new File(webDir);
				String[] fileList = jScriptDir.list();
				for (int i = 0; i < fileList.length; i++) {
					if (fileList[i].contains("Web2cLogin_")) {
						File jScriptFile = new File(webDir + File.separator + fileList[i]);
						if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
					}
				}
				String timeStamp = String.valueOf(System.currentTimeMillis());
				Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cLogin_" + timeStamp + ".js"), debugLogger, debugListener);
				javascriptWriter.setJavascript(getJavascript(id));
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<String> it= getLoginPage(key, timeStamp).iterator(); it.hasNext();) {
					String out = (String)it.next();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
					strBfr.append(out);
				}			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
//			System.out.println("Web2c session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId() + " ActiveWeb2cSessions=" + (web2cSessionHandlerList.toArray().length));
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (simu)!");
		}	
	}
		
	/**
	 * Handles diagnostics requests
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetDiag(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String probeRequest = (request.getParameter("param")).substring(6);
			StringBuffer strBfr = new StringBuffer();
			if (probeRequest.equalsIgnoreCase("Status")) {
				for (Iterator<String> it = probe.getStatus().iterator(); it.hasNext();) {
					strBfr.append((String)it.next());
				}
			}			
			response.setContentType("text/html");
			response.getWriter().print(strBfr.toString());
			response.flushBuffer();
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (diag)!");
		}	
	}
	
	/**
	 * Handles client request to get update
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostUpdate(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				String str = associatedSessionHandler.getWeb2cPage().getUpdate();
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + "<return>" + str + "</return>");
				response.setContentType("text/html");
				response.getWriter().print("<return>" + str + "</return>");
				response.flushBuffer();
			}
			synchronized (web2cSessionHandlerList) {
				garbageCollector.removeDeadSessions(web2cSessionHandlerList);
				garbageCollector.clearCache(imageCache);
			}
		}
		catch (Exception e) {
			synchronized (web2cSessionHandlerList) {
				garbageCollector.removeDeadSessions(web2cSessionHandlerList);
				garbageCollector.clearCache(imageCache);
			}
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <update>!");
		}	
	}
	
	/**
	 * Handles client request to perform action
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostAction(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String value = parameter.split("</sessionId>")[1].split("</value>")[0].replace("<value>", ""); 
			String id = parameter.split("</sessionId>")[1].split("</value>")[1].split("</id>")[0].replace("<id>", ""); 
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				if (value.equals("logout")) {
					synchronized (web2cSessionHandlerList) {
						garbageCollector.removeSession(web2cSessionHandlerList, associatedSessionHandler);
					}
				}
				else {	
					associatedSessionHandler.getWeb2cPage().doAction(value, id);
				}
				response.getWriter().print("");
				response.flushBuffer();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <action>!");
		}	
	}
	
	/**
	 * Handles client request to perform search
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostSearch(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (earlyRegistering == false) response.getWriter().print("alive");
			if (earlyRegistering == true) response.getWriter().print("");
			response.flushBuffer();
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <search>!");
		}	
	}
	
	/**
	 * Handles client request to perform login
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostElogin(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
    		String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String userName = parameter.split("</sessionId>")[1].split("</userName>")[0].replace("<userName>", "");
			String sentPassword = parameter.split("</sessionId>")[1].split("</userName>")[1].split("</value")[0].replace("<value>", "");
			String str = null;	
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {	
				String randomNumber = (String)(associatedSessionHandler.getAttribute("encryptionKey"));	
				String userPassword = registryReader.getPassword(userName);			    	
				Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(userPassword + randomNumber);
				if ((md5.getMD5()).equals(sentPassword)) {
					associatedSessionHandler.setAttribute("userName", userName);
					associatedSessionHandler.setAttribute("loginStatus", "in");
					str = "/web2cToolkit/Web2c?param=(page)" + "[" + sessionId + "]" + (String)(associatedSessionHandler.getAttribute("requestedConfigFileName")) + "," + associatedSessionHandler.getAttribute("language");
					response.getWriter().print(str);
				}
				else {
					associatedSessionHandler.setAttribute("userName", "guest");
					associatedSessionHandler.setAttribute("loginStatus", "out");
					if(((String)(associatedSessionHandler.getAttribute("dataSource"))).equals("simulation")) {
						str = "/web2cToolkit/Web2c?param=(simu)" + (String)(associatedSessionHandler.getAttribute("requestedConfigFileName")) + "," + associatedSessionHandler.getAttribute("language");
						response.getWriter().print(str);
						sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
					}
					else {
						str = "/web2cToolkit/Web2c?param=(file)" + (String)(associatedSessionHandler.getAttribute("requestedConfigFileName")) + "," + associatedSessionHandler.getAttribute("language");
						response.getWriter().print(str);
					}
				}
				response.flushBuffer();			    	
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <elogin>!");
		}
	}
	
	/**
	 * Checks whether page is instantiated
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostCheckPageLoadingStatus(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				String str = "";		
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				if (associatedSessionHandler.getAttribute("requestedConfigFileName").equals(associatedSessionHandler.getAttribute("loadedConfigFileName"))) {
					str = "<load>" + "[" + sessionId + "]" + (String)(associatedSessionHandler.getAttribute("loadedConfigFileName") + "," + (String)associatedSessionHandler.getAttribute("language"));
				}
				else {
					DecimalFormat formatter = new DecimalFormat("##0", new DecimalFormatSymbols(localSettings));
					str = "Page loading: " + formatter.format(constructionProgress[0]) + "% ...";
				}
				response.getWriter().print(str);
				response.flushBuffer();		
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <ldchck>!");
		}	
	}
	
	/**
	 * Gets the finally generated page from the page generator
	 * @param generator page generator
	 * @param page generated page
	 */
	public void update(Observable generator, Object page) {

		synchronized (web2cSessionHandlerList) {
			for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if ((Web2cPageGenerator)sessionHandler.getWeb2cPageGenerator() != null) {
					if (((Web2cPageGenerator)sessionHandler.getWeb2cPageGenerator()).equals(generator)) {
						sessionHandler.setWeb2cPage((Web2cPage)page);	
						sessionHandler.setAttribute("loadedConfigFileName", (String)sessionHandler.getAttribute("requestedConfigFileName"));
					}
				}
			}
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cPageGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String sessionId = "";
		private String accessMode = "";
		private boolean dataMode = simulated; 
		private String remoteAddress = "";
		private String fileName = "";
		private int pageLanguage = 0;
		private String userName = "";
		
		/**	
		 * @param id session id
		 * @param access access mode 
		 * @param data data mode 
		 * @param address remote ip address
		 * @param file config file name
		 * @param language page language
		 * @param user user name
		 */
		public Web2cPageGenerator (String id, String access, boolean data, String address, String file, int language, String user) {
			
			super();
		
			sessionId = id;
			accessMode = access;
			dataMode = data;
			remoteAddress = address;
			fileName = file;
			pageLanguage = language;
			userName = user;
		
			if (generator == null) {
				generator = new Thread(this);
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			Web2cPage web2cPage = new Web2cPage(hostURL, sessionId, accessMode, homeDir, webDir, fileName, pageLanguage, userName, remoteAddress, dataMode, commonDeviceAccess, commonVideoConnector, chatHandler, debugLogger, debugListener, constructionProgress);	
			
			setChanged();
			notifyObservers(web2cPage);
			
			generator.interrupt();
		}
		
		/**
		 * Starts generator
		 */
		public void startGenerator() {	
			
			generator.start();
		}
	}
}