package de.desy.web2cToolkit.web2cServlets;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerPage;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cArchiveViewerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cInitReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;
import de.desy.web2cToolkit.web2cUtil.web2cSecurity.Web2cMD5HashGenerator;

/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cArchiveViewer extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants, Observer {
 
	private int sessionInactivityTimeout = archiveViewerSessionTimeoutInterval;   // [s]
	
	private String ipAddress = null;
	private HttpSession httpSession = null;
	private String hostURL = null;		
	private String homeDir = null;
	private String webDir = null;
	private Web2cRegistryReader registryReader = null;
	private Web2cInitReader initReader = null;
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cGarbageCollector garbageCollector = null;
	private Web2cArchiveViewerProbe probe = null;
	
	private Web2cContextHandler web2cContextHandler = null;
	private Calendar calendar = null;	
	
	/**
	 * Web2cArchiveViewer Servlet: handles client login, requests and updates
	 */
	public Web2cArchiveViewer() {

		super();
		calendar = Calendar.getInstance();
	}     
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {		
		
		super.init(config);
		
		if (web2cContextHandler == null) {
			web2cContextHandler = new Web2cContextHandler(config.getServletContext(), "archiveViewer");
		}			
		getContextRelevantFields();
	}	
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {			
			initializeServletMethod(request);	
			if (request.getParameter("param") != null) {	
			    String servletInputParameter = (request.getParameter("param")).substring(0, 6);	    	
//				long start = System.currentTimeMillis();		    	
				if (servletInputParameter.compareToIgnoreCase("<plot>") == 0){
					doGetPlot(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("<text>") == 0) {
					doGetText(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("<load>") == 0) {
					doGetLoad(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					doGetPage(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(open)") == 0) {
					doGetOpen(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {
					doGetFile(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(diag)") == 0) {
					doGetDiag(request, response);
				}
				else {
					System.out.println("Web2cArchiveViewer-GET: Unknown servlet input parameter specified!");
				}
//				long stop = System.currentTimeMillis();
//				System.out.println("Web2cArchiveViewer server response [ms] for " + servletInputParameter + ": " + (stop - start));
			}	
			else {
		    	System.out.println("Web2cArchiveViewer-GET: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cArchiveViewer-GET: Error processing servlet!");
		}
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {				
			initializeServletMethod(request);	
			if (request.getParameter("param") != null) {
				String servletInputParameter = (request.getParameter("param")).substring(0,8);	   	
//				long start = System.currentTimeMillis();		    				
			    if (servletInputParameter.compareToIgnoreCase("<action>") == 0) {
					doPostAction(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<elogin>") == 0) {
					doPostElogin(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<ldchck>") == 0) {
					doPostCheckPageLoadingStatus(request, response);
			    }
			    else {
			    	System.out.println("Web2cArchiveViewer-POST: Unknown servlet input parameter specified!");
			    }
//				long stop = System.currentTimeMillis();
//				System.out.println("Web2cArchiveViewer-POST server response [ms] for " + servletInputParameter + ": " + (stop - start));
			}
			else {
		    	System.out.println("Web2cArchiveViewer-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cArchiveViewer-POST: Error processing servlet!");
		}
	} 
	
	/**
	 * Returns login page
	 * @param randomNumber random number for password encryption
	 * @return HTML page
	 */
	private ArrayList<String> getLoginPage(String randomNumber, String timeStamp) {

		ArrayList<String> out = null;
		String enterKeyCode = "13";
		
		out = new ArrayList<String>();

		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Web2cArchiveViewer Login</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		out.add("		<link rel=\"shortcut icon\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "md5.js\"></script>");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2cArchiveViewerLogin_" + timeStamp + ".js\"></script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<input type=\"hidden\" id=\"random\" name=\"random\" value=\"" + randomNumber + "\">");		
		if (initReader != null) {
			out.add("		<div align=\"center\"><b><h1>" + initReader.getSalutation() + "</h1></b></div>");
		}
		else {
			out.add("		<div align=\"center\"><b><h1>" + web2cSalutation + "</h1></b></div>");
		}
		out.add("		<div align=\"center\"><b><h2>Please enter your username and password to log in.</h2></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td align=\"left\"><b>User name: </b></td><td class=\"control\" height=\"50\"><input type=\"text\" name=\"username\" id=\"username\" value=\"\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Password: </b></td><td class=\"control\" height=\"50\"><input type=\"password\" id=\"password\" name=\"password\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td> </td><td class=\"control\" height=\"25\"></td></tr>");
		out.add("				<tr><td colspan=\"2\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"login\" id=\"login\" value=\"Login\" onclick=\"login()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><p align=\"center\"><b>N.B.</b> Web2cArchiveViewer pages are best displayed with the following browser's default font settings: encoding = Western, variable-font size: 16px, fixed-font size: 13px, minimum font size: 6px or smaller</p>");
		out.add("	</body>");	
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Returns Javascript code
	 * @param id session identifier
	 * @return Javascript code as text
	 */
	private ArrayList<String> getJavascript(String id) {
		
		ArrayList<String> javascript = null;
		
		javascript = new ArrayList<String>();

		javascript.add("		var xhr;");
		javascript.add("		function initPage() {");
		javascript.add("			initXhr();");
		javascript.add("			if (document.getElementById(\"username\").value == \"guest\") {");
		javascript.add("			    login();");
		javascript.add("			}");
		javascript.add("		}");
		javascript.add("		function encryptPassword() {");
		javascript.add("			var data = document.getElementById(\"password\").value;");
		javascript.add("			var random = document.getElementById(\"random\").value;");
		javascript.add("			var string = data + random;");
		javascript.add("			return hex_md5(string);");
		javascript.add("		}");
		javascript.add("		function initXhr() {"); 
		javascript.add("			try {");                                                  
		javascript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		javascript.add("			} catch (event1) {");                             
		javascript.add("				try {");
		javascript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		javascript.add("				} catch (event2) {");
		javascript.add("					xhr = false;");
		javascript.add("				}");
		javascript.add("			}");
		javascript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		javascript.add("				xhr = new XMLHttpRequest();");                 
		javascript.add("			}"); 																	
		javascript.add("		}");
		javascript.add("		function login() {");
		javascript.add("			var now = new Date();");
		javascript.add("			var data = encryptPassword();");
		javascript.add("			var username = document.getElementById(\"username\").value;");
		javascript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cArchiveViewer?param=<elogin><sessionId>" + id + "</sessionId><userName>\" + username + \"</userName><value>\" + data + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></elogin>\", true);");
		javascript.add("			xhr.onreadystatechange = function() {");
		javascript.add("				if (xhr.readyState == 4) {");
		javascript.add("					window.location.href = xhr.responseText;");
		javascript.add("           	};");   
		javascript.add("           };"); 
		javascript.add("			xhr.send(null);");  
		javascript.add("		}");
		
		return javascript;
	}	  
	
	/**
	 * Returns error page if page generation has failed
	 * @param configFileName name of config file
	 * @return page
	 */
	private String  errorPageOut(String configFileName) {
		
		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<html>");
		strBfr.append("	<body>");
		strBfr.append("		<div>Page \"" + configFileName + "\" could not be created!</div>");
		strBfr.append("		<div>Reasons might be:</div>");
		strBfr.append("		<div> - Configuration file does not exist</div>");
		strBfr.append("		<div> - Wrong configuration file name</div>");
		strBfr.append("		<div> - Syntax error(s) in configuration file</div>");
		strBfr.append("		<div> - Invalid control system address(es) specified in configuration file</div>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Returns initial page
	 * @param web2cSession session
	 * @return page
	 */
	private String loadingPageOut(String web2cSession) {

		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		strBfr.append("<html>");
		strBfr.append("   <head>");
		strBfr.append("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		strBfr.append("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		strBfr.append("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		strBfr.append("		<title>Web2cArchiveViewer loading...</title>");
		strBfr.append("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		strBfr.append("		<link rel=\"shortcut icon\"  type=\"image/png\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		strBfr.append("     <script type=\"text/javascript\">");
		strBfr.append("			var xhr;");
		strBfr.append("			var sessionId = " + web2cSession + ";");
		strBfr.append("			function initPage() {");	
		strBfr.append("				setInterval(\"getPageLoadingStatus()\", 2000);");
		strBfr.append("			}");		
		strBfr.append("			function getPageLoadingStatus() {"); 
		strBfr.append("				try {");                                                  
		strBfr.append("					xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		strBfr.append("				} catch (event1) {");                             
		strBfr.append("					try {");
		strBfr.append("						xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		strBfr.append("					} catch (event2) {");
		strBfr.append("						xhr = false;");
		strBfr.append("					}");
		strBfr.append("				}");
		strBfr.append("				if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		strBfr.append("					xhr = new XMLHttpRequest();");                 
		strBfr.append("				}"); 	
		strBfr.append("				var now = new Date();");
		strBfr.append("				xhr.open(\"post\", \"/web2cToolkit/Web2cArchiveViewer?param=<ldchck><sessionId>\" + sessionId + \"</sessionId><timeStamp>\" + String(now.getTime()) + \"</timeStamp></ldchck>\", true);"); 
		strBfr.append("				xhr.onreadystatechange = function() {");       
		strBfr.append("					if (xhr.readyState == 4) {"); 
		strBfr.append("						if ((xhr.responseText != \"loading\") && (xhr.responseText != \"\")) {"); 
		strBfr.append("							window.location.href = location.href.split(\"=\")[0] + \"=\" + xhr.responseText;");
		strBfr.append("						}");      
		strBfr.append("					}");          
		strBfr.append("				};");   
		strBfr.append("				xhr.send(null);");  
		strBfr.append("			}");  
		strBfr.append("       </script>");
		strBfr.append("	</head>");
		strBfr.append("	<body onload=\"initPage()\">");
		strBfr.append("		<h1><b><div>Page loading...</div></h1></b>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Updates all context relevant fields
	 */
	private void getContextRelevantFields() {

		homeDir = web2cContextHandler.getHomeDir();
		webDir = web2cContextHandler.getWebDir();
		registryReader = web2cContextHandler.getWeb2cRegistryReader();
		initReader = web2cContextHandler.getWeb2cInitReader();
		sessionLogger = web2cContextHandler.getSessionLogger();
		debugLogger = web2cContextHandler.getDebugLogger();
		debugListener = web2cContextHandler.getWeb2cKeyBoardListener();
		web2cSessionHandlerList = web2cContextHandler.getWeb2cSessionHandlerList();
		garbageCollector = web2cContextHandler.getWeb2cGarbageCollector();
		probe = web2cContextHandler.getWeb2cArchiveViewerProbe();
	}
	
	/**
	 * Initializes or updates request specific fields
	 * @param request client request
	 */
	private void initializeServletMethod(HttpServletRequest request) {
		
		ipAddress = request.getRemoteAddr();
		httpSession = request.getSession();
		hostURL = request.getRequestURL().toString();	
		
		debugListener.getKeyboardInput();
	}
	
	/**
	 * Handles client request to get plot chart as image stream
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetPlot(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": image/png");
			String parameter = (request.getParameter("param")).substring(6);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");	
			String parameter1 = parameter.split("</sessionId>")[1].split("</id><value>")[0].replace("<id>", "");	
			String parameter2 = parameter.split("</sessionId>")[1].split("</id><value>")[1];	
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {	
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				response.setContentType("image/png");
				associatedSessionHandler.getWeb2cArchiveViewerPage().getPlot(parameter1, parameter2).writeTo(response.getOutputStream());
				response.flushBuffer();
			}
			synchronized (web2cSessionHandlerList) {
				garbageCollector.removeDeadSessions(web2cSessionHandlerList);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter <plot>!");
		}	
	}
	
	/**
	 * Handles client request to get table as text
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetText(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/plain");
			String parameter = (request.getParameter("param")).substring(6);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");	
			String parameter1 = parameter.split("</sessionId>")[1].split("</id><value>")[0].replace("<id>", "");	
			String parameter2 = parameter.split("</sessionId>")[1].split("</id><value>")[1];	
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {	
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
					response.setContentType("text/plain");
					String[] out = associatedSessionHandler.getWeb2cArchiveViewerPage().getTable(parameter1, parameter2);
					for (int i = 0; i < out.length; i++) {	
						response.getWriter().println(out[i]);
					}
					response.flushBuffer();
			}
			synchronized (web2cSessionHandlerList) {
				garbageCollector.removeDeadSessions(web2cSessionHandlerList);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			System.out.println("Error processing parameter (text)!");
		}	
	}
	
	/**
	 * Handles client request to load finally generated Web2cArchiveViewer page as HTML
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetLoad(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");			
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("]")[0].replace("[", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				Web2cArchiveViewerPage web2cArchiveViewerPage = associatedSessionHandler.getWeb2cArchiveViewerPage();
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<String> it1 = web2cArchiveViewerPage.getPage().iterator(); it1.hasNext();) {
					String out = (String)it1.next();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
					strBfr.append(out);
				}
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
				sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("loadedConfigFileName"));			
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(6);
			String configFileName = parameter.split("]")[1].replace("/", File.separator);
			response.setContentType("text/html");
			try {
				response.getWriter().print(errorPageOut(configFileName));
				response.flushBuffer();
			} catch (IOException e1) {}
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (load)!");
			
		}
	}
	
	/**
	 * Handles client request to get initial page as HTML after successful login procedure
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetPage(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("]")[0].replace("[", "");
			String configFileName = parameter.split("]")[1].replace("/", File.separator);
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("loadedConfigFileName", configFileName);	
					associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
					associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));	
					Web2cArchiveViewerPageGenerator generator = new Web2cArchiveViewerPageGenerator(sessionId, request.getRemoteAddr(), configFileName, (String)associatedSessionHandler.getAttribute("fromTime"), (String)associatedSessionHandler.getAttribute("toTime"), (String)associatedSessionHandler.getAttribute("userName"));
					associatedSessionHandler.setWeb2cArchiveViewerPageGenerator(generator);
					generator.addObserver(this);	
					response.setContentType("text/html");			
					response.getWriter().print(loadingPageOut(sessionId));
					response.flushBuffer();
				    sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("loadedConfigFileName"));
				}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (page)!");
		}	
	}
	
	/**
	 * Handles client request to initiate loading Web2cArchiveViewer page without login procedure and to get initial page as HTML
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetOpen(HttpServletRequest request, HttpServletResponse response) {
		
		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String configFileName = "";
			String fromTime = "";
			String toTime = "";
			if (parameter.split(",").length > 1) {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				fromTime = (parameter.split(",")[1]);
				toTime = (parameter.split(",")[2]);
			}
			else {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				fromTime = "";
				toTime = "";
			}
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxSessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {
				associatedSessionHandler.setAttribute("loginStatus", "in");
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("loadedConfigFileName", "");
				associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
				associatedSessionHandler.setAttribute("fromTime", fromTime);
				associatedSessionHandler.setAttribute("toTime", toTime);
				associatedSessionHandler.setAttribute("loadedConfigFileName", configFileName);
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				Web2cArchiveViewerPageGenerator generator = new Web2cArchiveViewerPageGenerator(sessionId, request.getRemoteAddr(), configFileName, fromTime, toTime, (String)associatedSessionHandler.getAttribute("userName"));
				associatedSessionHandler.setWeb2cArchiveViewerPageGenerator(generator);
				generator.addObserver(this);	
				response.setContentType("text/html");			
				response.getWriter().print(loadingPageOut(sessionId));
				response.flushBuffer();
//				System.out.println("Web2cArchiveViewer session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId() + " ActiveWeb2cArchiveSessions=" + (web2cSessionHandlerList.toArray().length));
			    sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("loadedConfigFileName"));
			}
			else {
				StringBuffer strBfr = new StringBuffer();
				strBfr.append("<html>");
				strBfr.append("<body>");
				strBfr.append("<div>Page \"" + configFileName + "\" could not be created!</div>");
				strBfr.append("<div>Resources exhausted. Please try again later.</div>");
				strBfr.append("</body>");
				strBfr.append("</html>");			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
		    httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (open)!");
		}	
	}
	
	/**
	 * Handles client request to initiate loading Web2cArchiveViewer page with login procedure
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetFile(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String configFileName = "";
			String fromTime = "";
			String toTime = "";
			if (parameter.split(",").length > 1) {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				fromTime = (parameter.split(",")[1]);
				toTime = (parameter.split(",")[2]);
			}
			else {
				configFileName = (parameter.split(",")[0]).replace("/", File.separator);
				fromTime = "";
				toTime = "";
			}
			Random randomGenerator = new Random();
			int randomNumber = randomGenerator.nextInt(100000);
			Calendar calendar = Calendar.getInstance();
			String sessionId = String.valueOf(calendar.getTimeInMillis());
			if (web2cSessionHandlerList.toArray().length < maxArchiveViewerSessionNumber) {
				Web2cSessionHandler sessionHandler = new Web2cSessionHandler(sessionId);
				web2cSessionHandlerList.add(sessionHandler);
				sessionHandler.setAttribute("requestedConfigFileName", configFileName);
				sessionHandler.setAttribute("loadedConfigFileName", "");
				sessionHandler.setAttribute("loginStatus", "out");
				sessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				sessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				sessionHandler.setAttribute("userName", "guest");
				sessionHandler.setAttribute("fromTime", fromTime);
				sessionHandler.setAttribute("toTime", toTime);
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				sessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				sessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				String id = (String)sessionHandler.getAttribute("id");
				String key = (String)(sessionHandler.getAttribute("encryptionKey"));
				File jScriptDir = new File(webDir);
				String[] fileList = jScriptDir.list();
				for (int i = 0; i < fileList.length; i++) {
					if (fileList[i].contains("Web2cLogin_")) {
						File jScriptFile = new File(webDir + File.separator + fileList[i]);
						if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
					}
				}
				String timeStamp = String.valueOf(System.currentTimeMillis());
				Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cArchiveViewerLogin_" + timeStamp + ".js"), debugLogger, debugListener);
				javascriptWriter.setJavascript(getJavascript(id));
				StringBuffer strBfr = new StringBuffer();
				for (Iterator<String> it= getLoginPage(key, timeStamp).iterator(); it.hasNext();) {
					String out = (String)it.next();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
					strBfr.append(out);
				}			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
//				System.out.println("Web2cArchiveViewer session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId() + " ActiveWeb2cArchiveSessions=" + (web2cSessionHandlerList.toArray().length));
			}
			else {
				StringBuffer strBfr = new StringBuffer();
				strBfr.append("<html>");
				strBfr.append("<body>");
				strBfr.append("<div>Page \"" + configFileName + "\" could not be created!</div>");
				strBfr.append("<div>Resources exhausted. Please try again later.</div>");
				strBfr.append("</body>");
				strBfr.append("</html>");			
				response.setContentType("text/html");
				response.getWriter().print(strBfr.toString());
				response.flushBuffer();
			}
		    httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (file)!");
		}		
	}
		
	/**
	 * Handles diagnostics requests
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetDiag(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String probeRequest = (request.getParameter("param")).substring(6);
			StringBuffer strBfr = new StringBuffer();
			if (probeRequest.equalsIgnoreCase("Status")) {
				for (Iterator<String> it = probe.getStatus().iterator(); it.hasNext();) {
					strBfr.append((String)it.next());
				}
			}			
			response.setContentType("text/html");
			response.getWriter().print(strBfr.toString());
			response.flushBuffer();
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (diag)!");
		}	
	}
	
	/**
	 * Handles client request to perform action
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostAction(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
    		String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", ""); 
			String parameters = parameter.split("</sessionId>")[1].replace("<value>", "").replace("/value>", "");
			String parameter1 = parameters.split("<")[0];	
			String parameter2 = parameters.split("<")[1];
			String parameter3 = parameters.split("<")[2];
			String parameter4 = parameters.split("<")[3];
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {	
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				String str = associatedSessionHandler.getWeb2cArchiveViewerPage().doAction(parameter1, parameter2, parameter3, parameter4);
				response.setContentType("text/plain");
		    	response.getWriter().print(str);
				response.flushBuffer();
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <action>!");
		}	
	}
	
	/**
	 * Handles client request to perform login
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostElogin(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication())debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
    		String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String userName = parameter.split("</sessionId>")[1].split("</userName>")[0].replace("<userName>", "");
			String sentPassword = parameter.split("</sessionId>")[1].split("</userName>")[1].split("</value")[0].replace("<value>", "");				
			String str = null;
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {	
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				String userPassword = registryReader.getPassword(userName);
			    String randomNumber = (String)(associatedSessionHandler.getAttribute("encryptionKey"));
			    Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(userPassword + randomNumber);
				response.setContentType("text/plain");
				if ((md5.getMD5()).equals(sentPassword)) {
					associatedSessionHandler.setAttribute("userName", userName);
					associatedSessionHandler.setAttribute("loginStatus", "in");
					str = "/web2cToolkit/Web2cArchiveViewer?param=(page)" + "[" + sessionId + "]" + (String)(associatedSessionHandler.getAttribute("requestedConfigFileName"));
					response.getWriter().print(str);
					   sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
				}
				else {
					associatedSessionHandler.setAttribute("userName", "guest");
					associatedSessionHandler.setAttribute("loginStatus", "out");
					str = "/web2cToolkit/Web2cArchiveViewer?param=(file)" + (String)(associatedSessionHandler.getAttribute("requestedConfigFileName"));
					response.getWriter().print(str);
				}
				response.flushBuffer();
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <elogin>!");
		}	
	}
	
	/**
	 * Checks whether page is instantiated
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostCheckPageLoadingStatus(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {
				String str = "";		
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				if (associatedSessionHandler.getAttribute("requestedConfigFileName").equals(associatedSessionHandler.getAttribute("loadedConfigFileName"))) {
					str = "<load>" + "[" + sessionId + "]" + (String)(associatedSessionHandler.getAttribute("loadedConfigFileName") + "," + (String)associatedSessionHandler.getAttribute("language"));
				}
				else {
					str = "loading";
				}	
				response.getWriter().print(str);
				response.flushBuffer();		
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <ldchck>!");
		}	
	}
	
	/**
	 * Gets the finally generated page from the page generator
	 * @param generator page generator
	 * @param page generated page
	 */
	public void update(Observable generator, Object page) {

		synchronized (web2cSessionHandlerList) {
			for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if (((Web2cArchiveViewerPageGenerator)sessionHandler.getWeb2cArchiveViewerPageGenerator()).equals(generator)) {
					sessionHandler.setWeb2cArchiveViewerPage((Web2cArchiveViewerPage)page);	
					sessionHandler.setAttribute("loadedConfigFileName", (String)sessionHandler.getAttribute("requestedConfigFileName"));
				}
			}
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cArchiveViewerPageGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String sessionId = "";
		private String remoteAddress = "";
		private String fileName = "";
		private String fromTime = "";
		private String toTime = "";
		private String userName = "";
		
		/**	
		 * @param id session id
		 * @param address remote ip address
		 * @param file config file name
		 * @param from start time of requested interval
		 * @param to end time of requested interval
		 * @param user user name
		 */
		public Web2cArchiveViewerPageGenerator (String id, String address, String file, String from, String to, String user) {
			
			super();
		
			sessionId = id;
			remoteAddress = address;
			fileName = file;
			fromTime = from;
			toTime = to;
			userName = user;
		
			if (generator == null) {
				generator = new Thread(this);
				generator.start();
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			Web2cArchiveViewerPage web2cArchiveViewerPage = new Web2cArchiveViewerPage(hostURL, sessionId, homeDir, webDir, fileName, fromTime, toTime, userName, remoteAddress, debugLogger, debugListener);
			
			setChanged();
			notifyObservers(web2cArchiveViewerPage);
			
			generator.interrupt();
		}
	}
}