package de.desy.web2cToolkit.web2cServlets;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Random;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import de.desy.web2cToolkit.web2cEditor.Web2cEditorPage;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;
import de.desy.web2cToolkit.web2cUtil.web2cSecurity.Web2cMD5HashGenerator;


/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cEditor extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {

	private int sessionInactivityTimeout = editorSessionTimeoutInterval;  // [s]
	
	private String ipAddress = null;
	private HttpSession httpSession = null;
	private String hostURL = null;	
	
	private String homeDir = null;
	private String webDir = null;
	private Web2cRegistryReader registryReader = null;
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	
	private Web2cContextHandler web2cContextHandler = null;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;
 
	/**
	 * Web2cEditor Servlet: handles client login, requests and updates
	 */
	public Web2cEditor() {

		super();
	}      
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {		
		
		super.init(config);
		
		if (web2cContextHandler == null) {
			web2cContextHandler = new Web2cContextHandler(config.getServletContext(), "editor");
		}			
		getContextRelevantFields();	
	}	 	
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {				
			initializeServletMethod(request);	
			if (request.getParameter("param") != null) {
			    String servletInputParameter = (request.getParameter("param")).substring(0, 6);
				if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					doGetPage(request, response);
				}
				else if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {
					doGetFile(request, response);
				}
				else {
					System.out.println("Web2cEditor-GET : Unknown servlet input parameter specified!");
				}
			}	
			else {
		    	System.out.println("Web2cEditor-GET: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cEditor-GET: Error processing servlet!");
		}
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {			
			initializeServletMethod(request);	
			if (request.getParameter("param") != null) {
				String servletInputParameter = (request.getParameter("param")).substring(0,8);			
			    if (servletInputParameter.compareToIgnoreCase("<action>") == 0) {
					doPostAction(request, response);
			    }
			    else if (servletInputParameter.compareToIgnoreCase("<elogin>") == 0) {
					doPostElogin(request, response);
			    }
			    else {
			    	System.out.println("Web2cEditor-POST: Unknwon servlet input parameter specified!");
			    }
			}
			else {
		    	System.out.println("Web2cEditor-POST: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cEditor-POST: Error processing servlet!");
		}
	} 
	
	private ArrayList<String> getLoginPage(String randomNumber, String timeStamp) {

		ArrayList<String> out = null;
		String enterKeyCode = "13";
		
		out = new ArrayList<String>();

		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Web2cEditor Login</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		out.add("		<link rel=\"shortcut icon\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2cEditorLogin_" + timeStamp + ".js\"></script>");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "md5.js\"></script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<input type=\"hidden\" id=\"random\" name=\"random\" value=\"" + randomNumber + "\">");
		out.add("		<div align=\"center\"><b><h1>Web2cToolkit: Framework for Web-based Controls Clients</h1></b></div>");
		out.add("		<div align=\"center\"><b><h2>Welcome! Please enter your username and password to log in.</h2></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td align=\"left\"><b>User name: </b></td><td class=\"control\" height=\"50\"><input type=\"text\" name=\"username\" id=\"username\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Password: </b></td><td class=\"control\" height=\"50\"><input type=\"password\" id=\"password\" name=\"password\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td> </td><td class=\"control\" height=\"25\"></td></tr>");
		out.add("				<tr><td colspan=\"2\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"login\" id=\"login\" value=\"Login\" onclick=\"login()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><p align=\"center\"><b>N.B.</b> Web2cEditor pages are best displayed with the following browser's default font settings: encoding = Western, variable-font size: 16px, fixed-font size: 13px, minimum font size: 6px or smaller</p>");
		out.add("	</body>");	
		out.add("</html>");
		
		return out;
	}
	
	private ArrayList<String> getJavascript(String id) {
		
		ArrayList<String> javascript = null;
		
		javascript = new ArrayList<String>();

		javascript.add("		var xhr;");
		javascript.add("		function initPage() {");
		javascript.add("			initXhr();");
		javascript.add("			if (document.getElementById(\"username\").value == \"guest\") {");
		javascript.add("			    login();");
		javascript.add("			}");
		javascript.add("		}");
		javascript.add("		function encryptPassword() {");
		javascript.add("			var data = document.getElementById(\"password\").value;");
		javascript.add("			var random = document.getElementById(\"random\").value;");
		javascript.add("			var string = data + random;");
		javascript.add("			return hex_md5(string);");
		javascript.add("		}");
		javascript.add("		function initXhr() {"); 
		javascript.add("			try {");                                                  
		javascript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		javascript.add("			} catch (event1) {");                             
		javascript.add("				try {");
		javascript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		javascript.add("				} catch (event2) {");
		javascript.add("					xhr = false;");
		javascript.add("				}");
		javascript.add("			}");
		javascript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		javascript.add("				xhr = new XMLHttpRequest();");                 
		javascript.add("			}"); 																	
		javascript.add("		}");
		javascript.add("		function login() {");
		javascript.add("			var now = new Date();");
		javascript.add("			var data = encryptPassword();");
		javascript.add("			var username = document.getElementById(\"username\").value;");
		javascript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<elogin><sessionId>" + id + "</sessionId><userName>\" + username + \"</userName><value>\" + data + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></elogin>\", true);"); 
		javascript.add("			xhr.onreadystatechange = function() {");
		javascript.add("				if (xhr.readyState == 4) {");
		javascript.add("					window.location.href = xhr.responseText;");
		javascript.add("           	};");   
		javascript.add("           };"); 
		javascript.add("			xhr.send(null);");  
		javascript.add("		}");
		
		return javascript;
	}
	
	/**
	 * Updates all context relevant fields
	 */
	private void getContextRelevantFields() {

		homeDir = web2cContextHandler.getHomeDir();
		webDir = web2cContextHandler.getWebDir();
		registryReader = web2cContextHandler.getWeb2cRegistryReader();
		sessionLogger = web2cContextHandler.getSessionLogger();
		debugLogger = web2cContextHandler.getDebugLogger();
		debugListener = web2cContextHandler.getWeb2cKeyBoardListener();
		web2cSessionHandlerList = web2cContextHandler.getWeb2cSessionHandlerList();
	}
	
	/**
	 * Initializes or updates request specific fields
	 * @param request client request
	 */
	private void initializeServletMethod(HttpServletRequest request) {
		
		ipAddress = request.getRemoteAddr();
		httpSession = request.getSession();
		hostURL = request.getRequestURL().toString();	
		
		debugListener.getKeyboardInput();
	}
	
	/**
	 * Cancels dead sessions (Web2cArchiveViewer)
	 */
	private synchronized void doGarbageCollection(ArrayList<Web2cSessionHandler> sessionHandlerList) {
		
		long now = 0;
		long lastPoll = 0;
		ArrayList<Web2cSessionHandler> sessionHandlerToRemoveList = null;
		
		long inactivityTimeout = editorInactivityTimeoutInterval * 1000;
		
		synchronized(sessionHandlerList) {
			if (sessionWatcherIsEnabled == true) {
				sessionHandlerToRemoveList = new ArrayList<Web2cSessionHandler>();
				for (Iterator<Web2cSessionHandler> it = sessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (sessionHandler != null) {		
						now = Calendar.getInstance().getTimeInMillis();
						lastPoll = Long.valueOf(sessionHandler.getAttribute("lastPoll"));
						if (lastPoll != 0) {
							if ((now - lastPoll) > inactivityTimeout) {
//								System.out.println("Web2cGarbageCollector cancels sessions (Web2cArchiveViewer): " + sessionHandler.getAttribute("id") + " " + now + " " + lastPoll + " " + (now - lastPoll) + ">" + inactivityTimeout);
								sessionHandlerToRemoveList.add(sessionHandler);						
							}
							else {
//								System.out.println("Web2cGarbageCollector is continuing: " + sessionHandler.getAttribute("id") + " " + now + " " + lastPoll + " " + (now - lastPoll) + "<=" + inactivityTimeout);
							}
						}
					}
				}
				
				for (Iterator<Web2cSessionHandler> it = sessionHandlerToRemoveList.iterator(); it.hasNext();) {
					int sessionHandlerToRemoveIndex = sessionHandlerList.indexOf((Web2cSessionHandler)it.next());
//					System.out.println("Web2cEditor session canceled: ID=" + sessionHandlerList.get(sessionHandlerToRemoveIndex).getAttribute("id") + " ConfigFile=" + sessionHandlerList.get(sessionHandlerToRemoveIndex).getAttribute("loadedConfigFileName") + " CancelTime=" + String.valueOf(Calendar.getInstance().getTime().toString()) + " ActiveWeb2cArchiveViewerSessions=" + (sessionHandlerList.toArray().length - 1));
					sessionHandlerList.remove(sessionHandlerToRemoveIndex);
				}
			}
		}
	}
	
	/**
	 * Handles client request to get previously loaded Web2cEditor page as HTML
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetPage(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String parameter = (request.getParameter("param")).substring(6);
			String sessionId = parameter.split("]")[0].replace("[", "");
			String configFileName = parameter.split("]")[1].replace("/", File.separator);
			for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
				Web2cSessionHandler nextSessionHandler = (Web2cSessionHandler)it.next();
				if (((String)nextSessionHandler.getAttribute("id")).equals(sessionId)) {
					Web2cSessionHandler session = nextSessionHandler;
					Web2cEditorPage web2cEditorPage = new Web2cEditorPage(hostURL, sessionId, homeDir, webDir, webDir + File.separator + configFileName, (String)session.getAttribute("userName"), request.getRemoteAddr(), debugLogger, debugListener);
					session.setWeb2cEditorPage(web2cEditorPage);
					session.setAttribute("loadedConfigFileName", configFileName);			
					PrintWriter pageOut = new PrintWriter(response.getWriter());
					response.setContentType("text/html");
					for (Iterator<String> it1 = web2cEditorPage.getPage().iterator(); it1.hasNext();) {
						String out = (String)it1.next();
						if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
						pageOut.println(out);
					}
					pageOut.flush();
					pageOut.close();
				}
			}
//			System.out.println("Web2cEditor session created: ID=" + sessionId + " ConfigFile=" + configFileName + " CreationTime="  + String.valueOf(Calendar.getInstance().getTime().toString()) + " ClientIP=" + ipAddress + " HTTPSessionID=" + httpSession.getId());
		    httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
			doGarbageCollection(web2cSessionHandlerList);
		}
		catch (Exception e) {
			doGarbageCollection(web2cSessionHandlerList);
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (page)!");
		}
	}
	
	/**
	 * Handles client request to load Web2cEditor page
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doGetFile(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String configFileName = (request.getParameter("param")).substring(6);
			Random randomGenerator = new Random();
			int randomNumber = randomGenerator.nextInt(100000);
			Calendar calendar = Calendar.getInstance();
			String sessionId = String.valueOf(calendar.getTimeInMillis());
			if (web2cSessionHandlerList.toArray().length < maxEditorSessionNumber) {
				Web2cSessionHandler sessionHandler = new Web2cSessionHandler(sessionId);
				web2cSessionHandlerList.add(sessionHandler);
				sessionHandler.setAttribute("requestedConfigFileName", configFileName);
				sessionHandler.setAttribute("loadedConfigFileName", "");
				sessionHandler.setAttribute("loginStatus", "out");
				sessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				sessionHandler.setAttribute("userName", "guest");
				String id = (String)sessionHandler.getAttribute("id");
				String key = (String)(sessionHandler.getAttribute("encryptionKey"));
				httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
				PrintWriter pageOut = new PrintWriter(response.getWriter());
				response.setContentType("text/html");
				File jScriptDir = new File(webDir);
				String[] fileList = jScriptDir.list();
				for (int i = 0; i < fileList.length; i++) {
					if (fileList[i].contains("Web2cLogin_")) {
						File jScriptFile = new File(webDir + File.separator + fileList[i]);
						if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
					}
				}
				String timeStamp = String.valueOf(System.currentTimeMillis());
				Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cEditorLogin_" + timeStamp + ".js"), debugLogger, debugListener);
				javascriptWriter.setJavascript(getJavascript(id));
				for (Iterator<String> it= getLoginPage(key, timeStamp).iterator(); it.hasNext();) {
					String out = (String)it.next();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog(out);
					pageOut.println(out);
				}
				pageOut.flush();
				pageOut.close();
			}
			else {
				PrintWriter pageOut = new PrintWriter(response.getWriter());
				response.setContentType("text/html");
				pageOut.println("<html>");
				pageOut.println("<body>");
				pageOut.println("<div>Page \"" + configFileName + "\" could not be created!</div>");
				pageOut.println("<div>Resources exhausted. Please try again later.</div>");
				pageOut.println("</body>");
				pageOut.println("</html>");
				pageOut.flush();
				pageOut.close();
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (file)!");
		}
	}
	
	/**
	 * Handles client request to perform action
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostAction(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
    		String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", ""); 
			for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
					String action = parameter.split("</sessionId>")[1].split("</command>")[0].replace("<command>", "");
					String param1 = parameter.split("</sessionId>")[1].split("</command>")[1].split("</value>")[0].replace("<value>", "");
					String param2 = parameter.split("</sessionId>")[1].split("</command>")[1].split("</value>")[1].replace("<value>", "");
			    	String str = sessionHandler.getWeb2cEditorPage().doAction(action, param1, param2);
					response.setContentType("text/plain");
			    	PrintWriter out = new PrintWriter(response.getWriter());
		    		if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
		    		out.print(str);
					out.flush();
				}
			}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <action>!");
		}
	}
	
	/**
	 * Handles client request to perform login
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostElogin(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
    		String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.split("</sessionId>")[0].replace("<sessionId>", "");
			String userName = parameter.split("</sessionId>")[1].split("</userName>")[0].replace("<userName>", "");
			String sentPassword = parameter.split("</sessionId>")[1].split("</userName>")[1].split("</value")[0].replace("<value>", "");
			String str = null;
			for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {				    	
					String randomNumber = (String)(sessionHandler.getAttribute("encryptionKey"));	
					String userPassword = registryReader.getPassword(userName);	
					Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(userPassword + randomNumber);		
					response.setContentType("text/plain");				
					PrintWriter out = new PrintWriter(response.getWriter());
					if ((md5.getMD5()).equals(sentPassword)) {
						sessionHandler.setAttribute("userName", userName);
						sessionHandler.setAttribute("loginStatus", "in");
						str = "/web2cToolkit/Web2cEditor?param=(page)" + "[" + sessionId + "]" + (String)(sessionHandler.getAttribute("requestedConfigFileName"));
						out.print(str);
					    sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + (String)sessionHandler.getAttribute("userName") + " Page loaded: " + (String)sessionHandler.getAttribute("requestedConfigFileName"));
					}
					else {
						sessionHandler.setAttribute("userName", "guest");
						sessionHandler.setAttribute("loginStatus", "out");
						str = "/web2cToolkit/Web2cEditor?param=(file)" + (String)(sessionHandler.getAttribute("requestedConfigFileName"));
						out.print(str);
					}
				}
			}
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <elogin>!");
		}
	}
}