package de.desy.web2cToolkit.web2cServlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Random;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cGateway.Web2cGatewayPage;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cGatewayProbe;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;
import de.desy.web2cToolkit.web2cUtil.web2cSecurity.Web2cMD5HashGenerator;

/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cGateway extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {
 
	private int sessionInactivityTimeout = gatewaySessionTimeoutInterval;   // [s]
	
	private String ipAddress = null;
	private HttpSession httpSession = null;

	private String webDir = null;
	private Web2cRegistryReader registryReader = null;
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cGarbageCollector garbageCollector = null;
	private Web2cGatewayProbe probe = null;
	
	private Web2cContextHandler web2cContextHandler = null;
	private Calendar calendar = null;	
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;
	
	/**
	 * Web2cGateway Servlet: gateway to the control systems attached
	 */
	public Web2cGateway() {

		super();
		calendar = Calendar.getInstance();
	}  
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (web2cContextHandler == null) {
			web2cContextHandler = new Web2cContextHandler(config.getServletContext(), "gateway");
		}			
		getContextRelevantFields();	
		
		if (garbageCollectionIsEnabled == true) System.out.println("Java garbage collection enabled!");	
		if (garbageCollectionIsEnabled == false) System.out.println("Java garbage collection disabled!");
		
		if (sessionWatcherIsEnabled == true) System.out.println("Web2cGateway garbage collection enabled!");	
		if (sessionWatcherIsEnabled == false) System.out.println("Web2cGateway garbage collection disabled!");

	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {	
			initializeServletMethod(request);			
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);		    	
//				long start = System.currentTimeMillis();
				if (servletInputParameter.compareToIgnoreCase("(diag)") == 0) {
					doGetDiag(request, response);
				}
				else {
					System.out.println("Web2c-GET: Unknown servlet input parameter specified!");
				}
//				long stop = System.currentTimeMillis();
//				System.out.println("Web2c-GET server response [ms] for " + servletInputParameter + ": " + (stop - start));
		    }	
		    else {
		    	System.out.println("Web2c-GET: No servlet input parameter specified!");
		    }
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2c-GET: Error processing servlet!");
		}
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {	
			initializeServletMethod(request);
			if (request.getParameter("param") != null) {
				String servletInputParameter = (request.getParameter("param"));
		    	synchronized(web2cSessionHandlerList) {
		    		if (servletInputParameter.compareToIgnoreCase("checkAlive") == 0) {
		    			doPostCheckAlive(request, response);
		    		}
		    		else if (servletInputParameter.compareToIgnoreCase("loginGuest") == 0) {
		    			doPostLoginGuest(request, response);
		    		}
		    		else if (servletInputParameter.compareToIgnoreCase("loginExpert") == 0) {
		    			doPostLoginExpert(request, response);
		    		}
		    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<action>") == 0) {
						doPostAction(request, response);
				    }
				    else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<elogin>") == 0) {
						doPostElogin(request, response);
				    }
				    else {
				    	System.out.println("Web2cGateway-POST: Unknown servlet input parameter specified!");
				    }
		    	}
			}
			else {
		    	System.out.println("Web2cGateway-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cGateway-POST: Error processing servlet!");
		}
	}
	
	/**
	 * Handles diagnostics requests
	 * @param request client request
	 * @param response servlet response
	 */
	private void doGetDiag(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": text/html");
			String probeRequest = (request.getParameter("param")).substring(6);
			StringBuffer strBfr = new StringBuffer();
			if (probeRequest.equalsIgnoreCase("Status")) {
				for (Iterator<String> it = probe.getStatus().iterator(); it.hasNext();) {
					strBfr.append((String)it.next());
				}
			}			
			response.setContentType("text/html");
			response.getWriter().print(strBfr.toString());
			response.flushBuffer();
		}
		catch (Exception e) {
			debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing parameter (diag)!");
		}	
	} 
	
	/**
	 * Handles client request to perform check if server alive
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostCheckAlive (HttpServletRequest request, HttpServletResponse response) {

		try {		
			response.setContentType("text/html");
			response.getWriter().print("<return>alive</return>");
			response.flushBuffer();
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing checkAlive!");
		}	
	}
	
	/**
	 * Handles client request to perform action
	 * @param request client request
	 * @param response servlet response
	 */
	private void doPostAction(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.replace("<sessionId>", "").split("</sessionId><value>")[0];
			String value = parameter.split("</sessionId><value>")[1].split("</value>")[0]; 
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {			
					String str = "";
					if (value.equals("logout")) {
						synchronized (web2cSessionHandlerList) {
							garbageCollector.removeSession(web2cSessionHandlerList, associatedSessionHandler);
						}
						str = "<return>removed<return>";
					}
					else {
						associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
						str = associatedSessionHandler.getWeb2cGatewayPage().doAction(value);
					}			
					response.setContentType("text/html");
					response.getWriter().print(str);
					response.flushBuffer();
					if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <action>!");
		}	
	}
	
	/**
	 * Handles client request to perform expert login
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostLoginExpert(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxGatewaySessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {		
				Random randomGenerator = new Random();
				int randomNumber = randomGenerator.nextInt(100000);
				associatedSessionHandler.setAttribute("requestedConfigFileName", gatewayConfigFile);
				associatedSessionHandler.setAttribute("loadedConfigFileName", "");
				associatedSessionHandler.setAttribute("loginStatus", "out");
				associatedSessionHandler.setAttribute("dataSource", "real");
				associatedSessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("language", "0");
				String str = "<return><key>" + String.valueOf(randomNumber) + "</key><loginURL><elogin><sessionId>" + sessionId + "</sessionId><username>\"username\"</username><password>\"encryptedPassword\"</password><timeStamp>\"timeStamp\"</timeStamp></elogin></loginURL></return>";	
				response.setContentType("text/html");
				response.getWriter().print(str);
				response.flushBuffer();
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing loginExpert!");
		}
	}
	
	/**
	 * Handles client request to perform guest login
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostLoginGuest(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());			
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				if (web2cSessionHandlerList.toArray().length < maxGatewaySessionNumber) {
					associatedSessionHandler = new Web2cSessionHandler(sessionId);
					web2cSessionHandlerList.add(associatedSessionHandler);
				}
			}
			if (associatedSessionHandler != null) {	
				associatedSessionHandler.setAttribute("requestedConfigFileName", gatewayConfigFile);
				associatedSessionHandler.setAttribute("loadedConfigFileName", gatewayConfigFile);
				associatedSessionHandler.setAttribute("loginStatus", "in");
				associatedSessionHandler.setAttribute("dataSource", "real");
				associatedSessionHandler.setAttribute("encryptionKey", "0");
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("language", "0");
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				calendar.setTimeInMillis(request.getSession().getCreationTime());
				associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
				associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
				Web2cGatewayPage web2cGatewayPage = new Web2cGatewayPage(sessionId, webDir, gatewayConfigFile, "guest", ipAddress, commonDeviceAccess, debugLogger, debugListener);
				associatedSessionHandler.setWeb2cGatewayPage(web2cGatewayPage);
				String str = "<return>";
				str = str + "<updateURL><action><sessionId>" + sessionId + "</sessionId><value><do>update</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></updateURL>";
				str = str + "<registerURL><action><sessionId>" + sessionId + "</sessionId><value><do>register</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></registerURL>";
				str = str + "<executeURL><action><sessionId>" + sessionId + "</sessionId><value><do>execute</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></executeURL>";
				str = str + "<logoutURL><action><sessionId>" + sessionId + "</sessionId><value>logout</value><timeStamp>\"timeStampInMillis\"</timeStamp></action></logoutURL>";
				str = str + "</return>";
				response.setContentType("text/html");
				response.getWriter().print(str);
				response.flushBuffer();
				sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: guest Page loaded: " + gatewayConfigFile);
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);
			}
			httpSession.setMaxInactiveInterval(sessionInactivityTimeout);
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing loginGuest!");
		}
	}
	
	/**
	 * Handles client request to perform user authentication and authorization verification
	 * @param request client request
	 * @param response servlet response
	 */
	private synchronized void doPostElogin(HttpServletRequest request, HttpServletResponse response) {

		try {
			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("request from " + ipAddress + ": " + request.getParameter("param"));
			String parameter = (request.getParameter("param")).substring(8);
    		String sessionId = parameter.replace("<sessionId>", "").split("</sessionId><username>")[0];
			String userName = parameter.split("</sessionId><username>")[1].split("</username><password>")[0];
			String sentPassword = parameter.split("</username><password>")[1].split("</password>")[0];	
			Web2cSessionHandler associatedSessionHandler = null;
			synchronized (web2cSessionHandlerList) {
				for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
					Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
					if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
						associatedSessionHandler = sessionHandler;
					}
				}
			}
			if (associatedSessionHandler != null) {			    	
				String randomNumber = (String)(associatedSessionHandler.getAttribute("encryptionKey"));	
				String userPassword = registryReader.getPassword(userName);			    	
				Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(userPassword + randomNumber);
				if ((md5.getMD5()).equals(sentPassword)) {
					associatedSessionHandler.setAttribute("userName", userName);
					associatedSessionHandler.setAttribute("loginStatus", "in");	
					associatedSessionHandler.setAttribute("loadedConfigFileName", gatewayConfigFile);	
					associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
					calendar.setTimeInMillis(request.getSession().getCreationTime());
					associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + request.getRemoteAddr());
					associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + request.getSession().getId() + ", HTTPSession creation time: " + String.valueOf(calendar.getTime().toString()));
					Web2cGatewayPage web2cGatewayPage = new Web2cGatewayPage(sessionId, webDir, gatewayConfigFile, userName, ipAddress, commonDeviceAccess, debugLogger, debugListener);
					associatedSessionHandler.setWeb2cGatewayPage(web2cGatewayPage);
					String str = "<return>";
    				str = str + "<updateURL><action><sessionId>" + sessionId + "</sessionId><value><do>update</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></updateURL>";
    				str = str + "<registerURL><action><sessionId>" + sessionId + "</sessionId><value><do>register</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></registerURL>";
    				str = str + "<executeURL><action><sessionId>" + sessionId + "</sessionId><value><do>execute</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></executeURL>";
    				str = str + "<logoutURL><action><sessionId>" + sessionId + "</sessionId><value>logout</value><timeStamp>\"timeStampInMillis\"</timeStamp></action></logoutURL>";
    				str = str + "</return>";
    				response.setContentType("text/html");
    				response.getWriter().print(str);
    				response.flushBuffer();
	    			if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("response to " + ipAddress + ": " + str);	
					sessionLogger.appendLog("IP Address: " + request.getRemoteAddr() + " User Name: " + userName + " Page loaded: " + gatewayConfigFile);
				}
    		}
		}
		catch (Exception e) {
			debugLogger.appendLog("request " + ipAddress + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error processing <elogin>!");
		}
	}
	
	/**
	 * Initializes or updates request specific fields
	 * @param request client request
	 */
	private synchronized void initializeServletMethod(HttpServletRequest request) {
			
		ipAddress = request.getRemoteAddr();
		httpSession = request.getSession();
		
		debugListener.getKeyboardInput();
	}
	
	/**
	 * Updates all context relevant fields
	 */
	private synchronized void getContextRelevantFields() {

		webDir = web2cContextHandler.getWebDir();
		registryReader = web2cContextHandler.getWeb2cRegistryReader();
		sessionLogger = web2cContextHandler.getSessionLogger();
		debugLogger = web2cContextHandler.getDebugLogger();
		debugListener = web2cContextHandler.getWeb2cKeyBoardListener();
		web2cSessionHandlerList = web2cContextHandler.getWeb2cSessionHandlerList();
		commonDeviceAccess = web2cContextHandler.getWeb2cCommonDeviceAccess();
		garbageCollector = web2cContextHandler.getWeb2cGarbageCollector();
		probe = web2cContextHandler.getWeb2cGatewayProbe();
	}
}