package de.desy.web2cToolkit.web2cServlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitter;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cMessengerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;

/**
 * @author R.Bacher, DESY
 * @version see de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cMessenger extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {
 
	private int inactivityTimeout = messengerInactivityTimeoutInterval;   // [s]
	
	private String ipAddress = null;
	private HttpSession httpSession = null;
	private String hostURL = null;	

	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cTwitter twitterInterface = null;
	private Web2cMessengerProbe probe = null;

	private Web2cContextHandler web2cContextHandler = null;
	
	/**
	 * Web2cMessager Servlet: handles messaging updates
	 * Implemented: Twitter
	 */
	public Web2cMessenger() {

		super();
	}     
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (web2cContextHandler == null) {
			web2cContextHandler = new Web2cContextHandler(config.getServletContext(), "messenger");
		}			
		getContextRelevantFields();
	}	
	
	/**
	 * Destroys the servlet
	 */
	public void destroy() {
		
		ArrayList<String> monitors = twitterInterface.getMonitors();		
		if (monitors != null && !monitors.isEmpty()) {
			for (Iterator<String> it = monitors.iterator(); it.hasNext();) {
				String monitorId = (String)it.next();
				commonDeviceAccess.removeRequester(monitorId);
			}
		}
		
		twitterInterface.closeMessenger();
	} 
	
	/**
	 * Updates all context relevant fields
	 */
	private void getContextRelevantFields() {
		
		debugLogger = web2cContextHandler.getDebugLogger();
		debugListener = web2cContextHandler.getWeb2cKeyBoardListener();
		probe = web2cContextHandler.getWeb2cMessengerProbe();
		twitterInterface = web2cContextHandler.getWeb2cTwitterInterface();
		commonDeviceAccess = web2cContextHandler.getWeb2cCommonDeviceAccess();
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		ipAddress = request.getRemoteAddr();
		httpSession = request.getSession();
		hostURL = request.getRequestURL().toString();	

		try {	
		    if (request.getParameter("param") != null) {	
				String parameter = request.getParameter("param");
				String str = "";
				if (parameter.equalsIgnoreCase("TwitterStatus")) {
					str = "<div>Last Twitter message from " + Calendar.getInstance().getTime().toString() + ": " + twitterInterface.getMessage() + "</div>";
				}
				else if (parameter.equalsIgnoreCase("TwitterPermit")) {
					str = "<div>Enable twitter connection: " + twitterInterface.getAuthorizationURL() + "</div>";
					
					StringBuffer strBfr = new StringBuffer();
					strBfr.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
					strBfr.append("<html>");
					strBfr.append("   <head>");
					strBfr.append("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
					strBfr.append("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
					strBfr.append("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
					strBfr.append("		<title>Web2cMessenger: requesting permit from twitter profile owner...</title>");
					strBfr.append("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//					strBfr.append("		<link rel=\"shortcut icon\"  type=\"image/png\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
					strBfr.append("     <script type=\"text/javascript\">");
					strBfr.append("			function getTwitterPermit() {");
					strBfr.append("				location.replace(\"" + twitterInterface.getAuthorizationURL() + "\");");
					strBfr.append("			}");  
					strBfr.append("       </script>");
					strBfr.append("	</head>");
					strBfr.append("	<body onload=\"getTwitterPermit()\">");
					strBfr.append("	</body>");
					strBfr.append("</html>");
					str = strBfr.toString();
				}
				else if (parameter.equalsIgnoreCase("TwitterOpen")) {
					twitterInterface.openMessenger();
					str = "<div>Try to start updating twitter status ...   Return status (0 = ok): " + twitterInterface.getError() + "</div>";
				}
				else if (parameter.equalsIgnoreCase("TwitterClose")) {
					twitterInterface.closeMessenger();
					str = "<div>Try to stop updating twitter status ...</div>";
				}
				else if (parameter.equalsIgnoreCase("(diag)Status")) {
					StringBuffer strBfr = new StringBuffer();
					ArrayList<String> list = probe.getStatus();
					for (Iterator<String> it1 = list.iterator(); it1.hasNext();) {
						strBfr.append((String)it1.next());
					}
					str = strBfr.toString();
				}			
				response.setContentType("text/html");
				response.getWriter().print(str);
				response.flushBuffer();
				
				httpSession.setMaxInactiveInterval(inactivityTimeout);		
				debugListener.getKeyboardInput();
				if (debugListener.getDebugWebCommunication()) debugLogger.appendLog("Web2cMessenger: Get-Request from " + ipAddress + " (" + hostURL + ")");	
			}	
			else {
				System.out.println("Web2cMessenger-GET: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Web2cMessenger-GET: Error processing servlet!");
		}
	}
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	}
}