package de.desy.web2cToolkit.web2cUtil.web2cDiagnostics;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitter;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;

/**
 * @author R.Bacher, DESY
 */
public class Web2cMessengerProbe implements IWeb2cConstants {

	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cTwitter twitterInterface = null;
	private Calendar calendar = null;
	private String serviceStarted = "";
	private String versionNumber = "";

	/**
	 * Administration interface
	 * @param twitter twitter interface
	 * @param cda common device access
	 * @param gc garbage collector
	 */
	public Web2cMessengerProbe(Web2cTwitter twitter, Web2cCommonDeviceAccess cda, Web2cGarbageCollector gc) {
		
		super();
		twitterInterface = twitter;
		commonDeviceAccess = cda;
		calendar = Calendar.getInstance();
		serviceStarted = String.valueOf(calendar.getTime().toString());
		versionNumber = web2cVersion;
	}
	
	/**
	 * Returns the Web2cMessenger status page
	 * return status page
	 */
	public ArrayList<String> getStatus() {
		
		ArrayList<String> statusList = new ArrayList<String>();
		
		statusList.add("<div><b><h1>Web2cMessenger Status Page (" + String.valueOf(Calendar.getInstance().getTime().toString()) + ")</h1></b></div>");
		
		statusList.add("<div><b><h2>General:</h2></b></div>");
		statusList.add("<div>	Web2cMessenger service version number: " + versionNumber + "</div>");		
		statusList.add("<div>	Web2cMessenger service started at: " + serviceStarted + "</div>");	
		statusList.add("<br>");

		statusList.add("<div><b><h2>Twitter:</h2></b></div>");
		if (twitterInterface.serviceEnabled() == true) {
			statusList.add("<div>	Connection: enabled</div>");
			if (twitterInterface.serviceStarted() == true) {
				statusList.add("<div>	Publishing: started</div>");
				statusList.add("<div>	Last message published: " + twitterInterface.getMessage() + "</div>");
			}
			if (twitterInterface.serviceStarted() == false) statusList.add("<div>	Publishing: stopped</div>");
		}
		if (twitterInterface.serviceEnabled() == false) statusList.add("<div>	Connection: disabled</div>");
		statusList.add("<br>");
		
		statusList.add("<div><b><h2>Device Access Details:</h2></b></div>");
		statusList.add("<div>	Total Number of monitors: " + commonDeviceAccess.getPropertyList().size() + "</div>");
		int numberOfRequesters = 0;
		for (Iterator<Integer> it1 = commonDeviceAccess.getNumberOfRequesterList().iterator(); it1.hasNext();) {
			numberOfRequesters = numberOfRequesters + (int)it1.next();
		}
		statusList.add("<div>	Total Number of requesters: " + numberOfRequesters + "</div>");
		statusList.add("<br>");
		Iterator<Integer> it2 = commonDeviceAccess.getNumberOfRequesterList().iterator();
		for (Iterator<String> it1 = commonDeviceAccess.getPropertyList().iterator(); it1.hasNext();) {
			String property = (String)it1.next();
			int number = (int)it2.next();
			statusList.add("<div>	Monitor established for: " + property + ", Number of requesters: " + number + "</div>");
		}
		
		return statusList;
	}
}
