package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cConfigReader implements IWeb2cConstants {
	
	private ArrayList<String> pageAttributeNames = null;
	private ArrayList<String> pageAttributeValues = null;
	
	private ArrayList<ArrayList<String>> embeddedPagesAttributeNames = null;
	private ArrayList<ArrayList<String>> embeddedPagesAttributeValues = null;
	
	private ArrayList<ArrayList<String>> sectionsAttributeNames = null;
	private ArrayList<ArrayList<String>> sectionsAttributeValues = null;
	
	private ArrayList<ArrayList<String>> sectionsElements = null;
	private ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeNames = null;
	private ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeValues = null;
	
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	
	private String error = "";

	/**
	 * Reader for Page Configuration File
	 * @param file name of page configuration file
	 * @param logger debug logger
	 * @param listener debug listener
	 */
	public Web2cConfigReader(File file, Web2cFileLogger logger, Web2cKeyBoardListener listener) {

		debugLogger = logger;
		debugListener = listener;
		parseConfigFile(file);
	}
	
	/**
	 * Returns the names of the valid page attributes
	 * @return list
	 */
	public ArrayList<String> getPageAttributeNames() {
		return pageAttributeNames;
	}
	
	/**
	 * Returns the values of the valid page attributes
	 * @return list
	 */
	public ArrayList<String> getPageAttributeValues() {
		return pageAttributeValues;
	}
	
	/**
	 * Returns the names of the valid attributes of all embedded pages
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getEmbeddedPagesAttributeNames() {
		return embeddedPagesAttributeNames;
	}
	
	/**
	 * Returns the values of the valid attributes of all embedded pages
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getEmbeddedPagesAttributeValues() {
		return embeddedPagesAttributeValues;
	}
	
	/**
	 * Returns the names of the valid attributes of all sections
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getSectionsAttributeNames() {
		return sectionsAttributeNames;
	}
	
	/**
	 * Returns the values of the valid attributes of all sections
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getSectionsAttributeValues() {
		return sectionsAttributeValues;
	}
	
	/**
	 * Returns the elements of all sections
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getSectionsElements() {
		return sectionsElements;
	}
	
	/**
	 * Returns the names of the valid attributes of the elements of all sections
	 * @return list
	 */
	public ArrayList<ArrayList<ArrayList<String>>> getSectionsElementsAttributeNames() {
		return sectionsElementsAttributeNames;
	}
	
	/**
	 * Returns the values of the valid attributes of the elements of all sections
	 * @return list
	 */
	public ArrayList<ArrayList<ArrayList<String>>> getSectionsElementsAttributeValues() {
		return sectionsElementsAttributeValues;
	}
	
	/**
	 * Returns the error code of the class
	 * @return error
	 */
	public String getError() {
		return error;
	}
	
    @SuppressWarnings("unchecked")
	private void parseConfigFile(File file) {
 
    	Iterator<Element> it1 = null;
    	Iterator<Element> it2 = null;
    	Iterator<Element> it3 = null;
    	Iterator<Element> it4 = null;
    	Iterator<Element> it5 = null;
    	Iterator<Element> it6 = null;
    	Iterator<Element> it7 = null;
    	Iterator<Element> it8 = null;
    	
    	SAXBuilder builder = null;
    	Document doc = null;
    	
    	Element page = null;
    	List<Element> pageTagList = null;
    	Element item = null;
    	Element pageAttributes = null;
    	List<Element> pageAttributesList = null;
    	Element pageAttribute = null;
    	Element pageElements = null;
    	Element embeddedPageAttribute = null;
    	List<Element> embeddedPageAttributeList = null;
    	List<Element> sectionsList = null;
    	Element section  = null;
    	List<Element> sectionTagList = null;
    	Element sectionAttributes = null;
    	List<Element> sectionAttributesList = null;
    	Element sectionAttribute = null;
    	Element sectionElements = null;
    	List<Element> sectionElementsTagList = null;
    	Element sectionElement = null;
    	List<Element> sectionElementAttributesList = null;
    	Element sectionElementAttribute = null;
		
    	ArrayList<String> names = null;
    	ArrayList<String> names1 = null;
		ArrayList<String> values1 = null;
		ArrayList<ArrayList<String>> sectionsNames = null;
		ArrayList<ArrayList<String>> sectionsValues = null;
		ArrayList<String> names2 = null;
		ArrayList<String> values2 = null;
		ArrayList<String> embeddedPagesNames = null;
		ArrayList<String> embeddedPagesValues = null;
		ArrayList<ArrayList<String>> elements = null;
		ArrayList<ArrayList<String>> elementsNames = null;
		ArrayList<ArrayList<String>> elementsValues = null;
		ArrayList<ArrayList<ArrayList<String>>> sectionsElementsNames = null;
		ArrayList<ArrayList<ArrayList<String>>> sectionsElementsValues = null;
		
    	try {
    		pageAttributeNames = new ArrayList<String>();
    		pageAttributeValues = new ArrayList<String>();
    		embeddedPagesAttributeNames = new ArrayList<ArrayList<String>>();
    		embeddedPagesAttributeValues= new ArrayList<ArrayList<String>>();
    		sectionsAttributeNames = new ArrayList<ArrayList<String>>();
    		sectionsAttributeValues = new ArrayList<ArrayList<String>>();
    		sectionsElementsAttributeNames = new ArrayList<ArrayList<ArrayList<String>>>();
    		sectionsElementsAttributeValues = new ArrayList<ArrayList<ArrayList<String>>>();
    		builder = new SAXBuilder();
			doc = builder.build(file);
			page = doc.getRootElement();
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(page.getName());
			pageTagList = (List<Element>)page.getChildren();  
			for (it1 = pageTagList.iterator(); it1.hasNext();) {
				item = (Element)it1.next();
				if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("	" + item.getName());
				if (item.getName().equals("PageAttributes")) {
					pageAttributes = item;
					pageAttributesList = (List<Element>)pageAttributes.getChildren();
					for (it2 = pageAttributesList.iterator(); it2.hasNext();) {
						pageAttribute = (Element)it2.next();
						pageAttributeNames.add(pageAttribute.getName());
						pageAttributeValues.add(pageAttribute.getValue());
						if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("		" + pageAttribute.getName() + ": " +  pageAttribute.getValue());
					}
				}
				else if (item.getName().equals("PageElements")) {
					pageElements = item;
					sectionsList = (List<Element>)pageElements.getChildren();  
					sectionsElementsNames = new ArrayList<ArrayList<ArrayList<String>>>();
					sectionsElementsValues = new ArrayList<ArrayList<ArrayList<String>>>();
					sectionsNames = new ArrayList<ArrayList<String>>();
					sectionsValues = new ArrayList<ArrayList<String>>();
					elements = new ArrayList<ArrayList<String>>();
					for (it3 = sectionsList.iterator(); it3.hasNext();) { 
						section = (Element)it3.next();
						if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("		" + section.getName());
						if (section.getName().equals("EmbeddedPage")) {
							embeddedPageAttributeList = (List<Element>)section.getChildren();
							embeddedPagesNames =  new ArrayList<String>();
							embeddedPagesValues =  new ArrayList<String>();
							for (it8 = embeddedPageAttributeList.iterator(); it8.hasNext();) {
								embeddedPageAttribute = (Element)it8.next();
								embeddedPagesNames.add(embeddedPageAttribute.getName());
								embeddedPagesValues.add(embeddedPageAttribute.getValue());
								if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("			" + embeddedPageAttribute.getName() + ": " +  embeddedPageAttribute.getValue());
							}
							embeddedPagesAttributeNames.add(embeddedPagesNames);
							embeddedPagesAttributeValues.add(embeddedPagesValues);
						}
						else {
						sectionTagList = (List<Element>)section.getChildren();
						for (it4 = sectionTagList.iterator(); it4.hasNext();) {
							item = (Element)it4.next();
							if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("			" + item.getName());
							if (item.getName().equals("SectionAttributes")) {
								sectionAttributes = item;
								sectionAttributesList = (List<Element>)sectionAttributes.getChildren();
								names1 = new ArrayList<String>();
								values1 = new ArrayList<String>();
								for (it5 = sectionAttributesList.iterator(); it5.hasNext();) {
									sectionAttribute = (Element)it5.next();
									names1.add(sectionAttribute.getName());
									values1.add(sectionAttribute.getValue());
									if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("				" + sectionAttribute.getName()  + ": " +  sectionAttribute.getValue());
								}
								sectionsNames.add(names1);
								sectionsValues.add(values1);
							}
							else if (item.getName().equals("SectionElements")) {
								sectionElements = item;
								sectionElementsTagList = (List<Element>)sectionElements.getChildren();
								elementsNames = new ArrayList<ArrayList<String>>();
								elementsValues = new ArrayList<ArrayList<String>>();
								names = new ArrayList<String>();
								for (it6 = sectionElementsTagList.iterator(); it6.hasNext();) {
									sectionElement = (Element)it6.next();
									if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("				" + sectionElement.getName());
									sectionElementAttributesList = (List<Element>)sectionElement.getChildren();
									names2 = new ArrayList<String>();
									values2 = new ArrayList<String>();
									for (it7 = sectionElementAttributesList.iterator(); it7.hasNext();) {
										sectionElementAttribute = (Element)it7.next();
										names2.add(sectionElementAttribute.getName());
										values2.add(sectionElementAttribute.getValue());
										if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog("					" + sectionElementAttribute.getName() + ": " +  sectionElementAttribute.getValue());
									}
									elementsNames.add(names2);
									elementsValues.add(values2);
									names.add(sectionElement.getName());
								}
								sectionsElementsNames.add(elementsNames);
								sectionsElementsValues.add(elementsValues);
								elements.add(names);
								}
							}
						}
					}
				}
			}
			sectionsElementsAttributeNames = sectionsElementsNames;
			sectionsElementsAttributeValues = sectionsElementsValues;
			sectionsAttributeNames = sectionsNames;
			sectionsAttributeValues = sectionsValues;
			sectionsElements = elements;
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(embeddedPagesAttributeNames.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(embeddedPagesAttributeValues.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(sectionsElements.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(sectionsAttributeNames.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(sectionsAttributeValues.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(sectionsElementsAttributeNames.toString());
			if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(sectionsElementsAttributeValues.toString());
		} catch (JDOMException e) {
			e.printStackTrace();
			error = error + "[Configuration file parser] Error parsing configuration file;";
		} catch (IOException e) {
			e.printStackTrace();
			error = error + "[Configuration file reader] Error reading configuration file;";
		}
    }
}
