package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cConfigWriter implements IWeb2cConstants {
	
	private File configFile = null;
	private ArrayList<String> configData = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	
	/**
	 * Writer for Page Configuration File
	 * @param file name of page configuration file
	 * @param logger debug logger
	 * @param listener debug listener
	 */
	public Web2cConfigWriter(File file, Web2cFileLogger logger, Web2cKeyBoardListener listener) {
		
		configFile = file;
		debugLogger = logger;
		debugListener = listener;
	}
	
	/**
	 * Returns true if configuration file exists, else false
	 * @return status
	 */
	public boolean checkFileExists() {
		
		boolean fileExists = true;
		
		File file = new File(configFile.getAbsolutePath());
		fileExists = file.exists();
		
		return fileExists;
	}
	
	/**
	 * Writes page configuration file (xml-type)
	 */
	public void setXML(ArrayList<String> xml) {
		
		configData = new ArrayList<String>();
		configData = xml;
		
	    for (int i = 0; i < configData.size(); i++) {
	    	if (debugListener.getDebugConfigurationFileParsing()) debugLogger.appendLog(configData.get(i));
	    }
	    
	    try {
	    	BufferedWriter out = new BufferedWriter(new FileWriter(configFile.getCanonicalFile()));
			for (Iterator<String> it = configData.iterator(); it.hasNext();) {
				out.write((String)(it.next()));
				out.newLine();
			}
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error accessing config file");
		}
	}
}
