package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cExtensionManifestReader implements IWeb2cConstants {
	
	private File manifestFile = null;
	private String className = "";
	private int componentSize = 1;
	private String componentType = "";
	private ArrayList<String> commandList = new ArrayList<String>();
	private ArrayList<String> actionList = new ArrayList<String>();
	private ArrayList<String> monitorList = new ArrayList<String>();
	private ArrayList<String> receiverList = new ArrayList<String>();

	/**
	 * Reader for manifest file 
	 * @param name of manifest
	 */
	public Web2cExtensionManifestReader(File file) {
		
		manifestFile = file;
		parseManifest();
	}
	
	/**
	 * Returns the extension class name 
	 * @return class name
	 */
	public String getClassName() {
		
		return className;
	}
	
	/**
	 * Returns the Web2c-type of the component the extension belongs to
	 * @return type
	 */
	public String getComponentType() {
		
		return componentType;
	}
	
	/**
	 * Returns the number of rows of the component's GUI 
	 * @return number of rows
	 */
	public int getComponentSize() {
		
		return componentSize;
	}
	
	/**
	 * Returns the remote command addresses belonging to the extension 
	 * @return remote command addresses
	 */
	public ArrayList<String> getRemoteCommands() {
		
		return commandList;
	}
	
	/**
	 * Returns the action addresses belonging to the extension 
	 * @return action addresses
	 */
	public ArrayList<String> getActions() {
		
		return actionList;
	}
	
	/**
	 * Returns the monitor link addresses belonging to the extension 
	 * @return monitor addresses
	 */
	public ArrayList<String> getMonitors() {
		
		return monitorList;
	}
	
	/**
	 * Returns the receiver link addresses belonging to the extension
	 * @return receiver addresses
	 */
	public ArrayList<String> getReceivers() {
		
		return receiverList;
	}
	
	@SuppressWarnings("unchecked")
	private void parseManifest() {
		
	   	SAXBuilder builder = null;
    	Document manifest = null;
    	Element extension = null;
    	List<Element> extensionAttributesList = null;
    	Element extensionAttribute = null;
    	
		builder = new SAXBuilder();
		try {
			manifest = builder.build(manifestFile);
		} catch (JDOMException e) {
			System.out.println("Error (" + e.toString() + ") parsing manifest file.");
		} catch (IOException e) {
			System.out.println("Error (" + e.toString() + ") reading manifest file.");
		}
		
		extension = manifest.getRootElement();
		extensionAttributesList = (List<Element>)extension.getChildren();   
		for (Iterator<Element> it1 = extensionAttributesList.iterator(); it1.hasNext();) {
			extensionAttribute = (Element)it1.next();  
			String value = extensionAttribute.getValue();
			if ((extensionAttribute.getName()).equals("extensionClass")) {
				if (!value.equals("") && !value.equals(" ")) className = value;
				else className = "";
			} 
			if ((extensionAttribute.getName()).equals("guiComponentType")) {
				if (!value.equals("") && !value.equals(" ")) componentType = value;
				else componentType = "";
			} 
			if ((extensionAttribute.getName()).equals("guiDataSize")) {
				if (!value.equals("") && !value.equals(" ")) componentSize = Integer.valueOf(value);
				else componentSize = 0;
			} 
			if ((extensionAttribute.getName()).equals("remoteCommandURI")) {
				if (!value.equals("") && !value.equals(" ")) commandList.add(value);
			} 
			if ((extensionAttribute.getName()).equals("actionURI")) {
				if (!value.equals("") && !value.equals(" ")) actionList.add(value);
			} 
			if ((extensionAttribute.getName()).equals("monitorURI")) {
				if (!value.equals("") && !value.equals(" ")) monitorList.add(value);
			} 
			if ((extensionAttribute.getName()).equals("receiverURI")) {
				if (!value.equals("") && !value.equals(" ")) receiverList.add(value);
			} 
		}
	}
}
