package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cInitReader implements IWeb2cConstants {
	
	private File initFile = null;
	private String salutation = web2cSalutation;
	private String homeDir = "";
	private ArrayList<String> monitorList = new ArrayList<String>();
	private ArrayList<String> receiverList = new ArrayList<String>();

	/**
	 * Reader for init File 
	 * @param file name of init file
	 */
	public Web2cInitReader(File file, String dir) {
		
		initFile = file;
		homeDir = dir;
		parseInitFile();
	}
	
	/**
	 * Returns the login salutation
	 * @return salutation
	 */
	public String getSalutation() {
		
		return salutation;
	}
	
	/**
	 * Returns the list of monitor links for early link registering
	 * @return list
	 */
	public ArrayList<String> getMonitorList() {
		
		return monitorList;
	}
	
	/**
	 * Returns the list of receiver links for early link registering
	 * @return list
	 */
	public ArrayList<String> getReceiverList() {
		
		return receiverList;
	}
	
	@SuppressWarnings("unchecked")
	private void parseInitFile() {
		
	   	SAXBuilder builder = null;
    	Document initConfig = null;
    	Element config = null;
    	List<Element> configList = null;
    	Element configElement = null;
    	
    	String fileName = "";
    	
		builder = new SAXBuilder();
		try {
			initConfig = builder.build(initFile);
		} catch (JDOMException e) {
			System.out.println("Error parsing init file.");
		} catch (IOException e) {
			System.out.println("Error reading init file.");
		}
		
		config = initConfig.getRootElement();
		configList = (List<Element>)config.getChildren();   
		for (Iterator<Element> it1 = configList.iterator(); it1.hasNext();) {
			configElement = (Element)it1.next();  
			if ((configElement.getName()).equals("salutation")) {
				salutation = configElement.getValue();
				if (salutation.equals("") || salutation.equals(" ")) salutation = web2cSalutation;
			} 
			if ((configElement.getName()).equals("earlyRegistering")) {
				fileName = configElement.getValue();
				if (!fileName.equals("") && !fileName.equals(" ")) {
					try {
						FileReader fileReader = new FileReader(homeDir + File.separator + fileName);
						BufferedReader bufferedReader = new BufferedReader(fileReader);
						boolean enfOfFile = false;
						boolean isVideoComponent = false;
						while (!enfOfFile) {
							String line = bufferedReader.readLine();
							if (line == null) {
								enfOfFile = true;
							}
							else {
								if (line.contains("<Video>")) isVideoComponent = true;
								if (line.contains("</Video>")) isVideoComponent = false;
								if (line.contains("<uri>") && (line.contains("tine") || line.contains("doocs") || line.contains("tango") || line.contains("epics"))) {
									String uri = line.substring(line.indexOf("<uri>") + 5, line.indexOf("</uri>"));
									if (isVideoComponent == false) monitorList.add(uri);
									if (isVideoComponent == true) receiverList.add(uri);
								}
								if (line.contains("<condition>") && (line.contains("tine") || line.contains("doocs") || line.contains("tango") || line.contains("epics"))) {
									String uri = line.substring(line.indexOf("<condition>") + 11, line.indexOf("="));
									monitorList.add(uri);
								}
							}
						}
					} catch (FileNotFoundException e) {
						System.out.println("Error generating monitor/receiver list for early link registering.");
					} catch (IOException e) {
						System.out.println("Error generating monitor/receiver list for early link registering.");
					}
				}
			} 
		}
	}
}
