package de.desy.web2cToolkit.web2cUtil.web2cInterfaces;

import java.io.ByteArrayOutputStream;

/**
 * @author R.Bacher, DESY
 */
public interface IWeb2cExtension {
	
	/**
	 * Executes Web2c extension code
	 * Returns the execution status of the extension code (0, if successfully executed).
	 * 
	 * Returns data to be visualized by the component's GUI (e.g. Web2cValue, if any) or data to be used to determine the data a to be visualized by the component's GUI (e.g. Web2cValue, if any)
	 * Returns commands issued by user action at the components's GUI to be executed by the control systems attached (e.g. Web2cButton, if any)
	 *
	 * Accepts monitor and/or data (optional) recently read from the the control systems attached.
	 * Returns set values (optional) to be written to and executed by the control systems attached after the execution of the component's extension code
	 * 
	 * Data formats (in/out):	 
	 * 	GUI data (out: String): 
	 * 	- Live: &#060;value&#062;data[startIndex],..,data[stopIndex]&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp&#060;&#047;timeStamp&#062;
	 * 	- History: &#060;value&#062;data1&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp1&#060;&#047;timeStamp&#062;&#060;value&#062;data2&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp2&#060;&#047;timeStamp&#062;.....)
	 * 	GUI data (out: ByteArrayOutputStream): jpeg-encoded image
	 * 
	 * 	Monitor data (in: String)
	 * 	- Live: &#060;value&#062;data[startIndex],..,data[stopIndex]&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp&#060;&#047;timeStamp&#062;
	 * 	- History: &#060;value&#062;data1&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp1&#060;&#047;timeStamp&#062;&#060;value&#062;data2&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp2&#060;&#047;timeStamp&#062;.....)
	 *
	 * 	Receiver data (in: ByteArrayOutputStream): jpeg-encoded image
	 * 
	 * 	Remote command data (out: String): remoteCommandURI(setValue)
	 * 
	 * 	Action data (out: String): actionURI(setValue)
	 * 
	 * @param web2cComponentType Web2c-type of component associated to this extension
	 * 
	 * @param web2cComponentRemoteCommandURI array of remote command URI
	 * @param web2cComponentData array of data (GUI data or remote commands)
	 * 
	 * @param monitorURI array of monitor URI to be read prior to executing the extension code
	 * @param monitorData array of monitor data (in same order as monitorURI array)
	 * 
	 * @param receiverURI array of video receiver URI to be read prior to executing the extension code
	 * @param receiverData array of video receiver data (in same order as receiverURI array)
	 * 
	 * @param actionURI list of remote actions to be performed after executing the extension code
	 * @param actionData list of remote actions including set values (in same order as actionURI array)
	 * 
	 * @return execution status
	 */
	public int executeExtensionCode(String web2cComponentType,
									String[] web2cComponentRemoteCommandURI, 
									Object[] web2cComponentData, 
									String[] monitorURI, 
									String[] monitorData, 
									String[] receiverURI, 
									ByteArrayOutputStream[] receiverData,
									String[] actionURI, 
									String[] actionData); 
}
