package de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl;

import java.io.File;
import java.util.ArrayList;

import javax.servlet.ServletContext;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatHandler;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitter;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cArchiveViewerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cGatewayProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cMessengerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cProbe;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cInitReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author  R.Bacher, DESY
 */
public class Web2cContextHandler implements IWeb2cConstants {
	
	private String homeDir = null;
	private String webDir = null;
	private Web2cRegistryReader registryReader = null;
	private Web2cInitReader initReader = null;
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private Web2cFileLogger actionLogger = null;
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cImageCache imageCache = null;
	private Web2cCommonVideoConnector commonVideoConnector = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cChatHandler chatHandler = null;
	private Web2cGarbageCollector garbageCollector = null;
	private ServletContext servletContext = null;
	private String servletRuntimeEngine = null;
	private Web2cProbe probe = null;
	private Web2cGatewayProbe gatewayProbe = null;
	private Web2cArchiveViewerProbe archiveViewerProbe = null;
	private Web2cMessengerProbe messengerProbe = null;
	private Web2cTwitter twitterInterface = null;

	/**
	 * Constructor; instantiates all context relevant fields
	 * @param context servlet context
	 * @param runtimeEngine servlet
	 */
	public Web2cContextHandler(ServletContext context, String runtimeEngine) {
		
		servletContext = context;
		servletRuntimeEngine = runtimeEngine;
		
		System.out.println("Initializing servlet: " + servletRuntimeEngine);

		if (web2cSessionHandlerList == null) {
			web2cSessionHandlerList = new ArrayList<Web2cSessionHandler>();
		}
		if (servletRuntimeEngine.equals("synopticDisplay")) {
			if (commonVideoConnector == null) {
				commonVideoConnector = new Web2cCommonVideoConnector();
			}
			if (chatHandler == null) {
				chatHandler = new Web2cChatHandler();
			}
		}
		if (garbageCollector == null) {
			garbageCollector = new Web2cGarbageCollector(servletRuntimeEngine);
		}
		if (debugListener == null) {
			debugListener = new Web2cKeyBoardListener();
		}
		if (homeDir == null) {
			homeDir = servletContext.getRealPath(userDir);
		}
		if (webDir == null) {
			webDir = servletContext.getRealPath(web2cDir);
		}
		if (registryReader == null) {
			registryReader = new Web2cRegistryReader(new File(registryDir + File.separator + registryFile));
		}
		if (servletRuntimeEngine.equals("archiveViewer") || servletRuntimeEngine.equals("synopticDisplay")) {
			if (initReader == null) {
				try {
					initReader = new Web2cInitReader(new File(registryDir + File.separator + initFile), homeDir);
				}
				catch (Exception e) {}
			}	
		}
		if (servletRuntimeEngine.equals("synopticDisplay")) {
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + synopticDisplaySessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + synopticDisplayLogFile), false);
			}
			if (actionLogger == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + cdaActionLogFile), true);
			}
		}
		else if (servletRuntimeEngine.equals("archiveViewer")) {
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + archiveViewerSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + archiveViewerLogFile), false);
			}
		}		
		else if (servletRuntimeEngine.equals("editor")) {
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + editorSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + editorLogFile), false);
			}
		}
		else if (servletRuntimeEngine.equals("gateway")) {
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + gatewaySessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + gatewayLogFile), false);
			}
			if (actionLogger == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + cdaActionLogFile), true);
			}
		}
		else if (servletRuntimeEngine.equals("messenger")) {
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + messengerLogFile), false);
			}
			if (actionLogger == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + cdaActionLogFile), true);
			}
		}
		if (commonDeviceAccess == null) {
			commonDeviceAccess = new Web2cCommonDeviceAccess(debugListener, debugLogger, actionLogger);
		}
		
		if (servletRuntimeEngine.equals("synopticDisplay")) {
			if (imageCache == null) {
				imageCache = new Web2cImageCache();
			}
		}

		if (servletRuntimeEngine.equals("messenger")) {
			if (twitterInterface == null) {
				twitterInterface = new Web2cTwitter(commonDeviceAccess, homeDir, debugLogger);
			}
		}
		
		if (probe == null) {
			if (servletRuntimeEngine.equals("synopticDisplay")) {
				probe = new Web2cProbe(web2cSessionHandlerList, imageCache, commonDeviceAccess, garbageCollector);
			}
			if (servletRuntimeEngine.equals("archiveViewer")) {
				archiveViewerProbe = new Web2cArchiveViewerProbe(web2cSessionHandlerList, garbageCollector);
			}
			if (servletRuntimeEngine.equals("gateway")) {
				gatewayProbe = new Web2cGatewayProbe(web2cSessionHandlerList, commonDeviceAccess, garbageCollector);
			}
			if (servletRuntimeEngine.equals("messenger")) {
				messengerProbe = new Web2cMessengerProbe(twitterInterface, commonDeviceAccess, garbageCollector);
			}
		}
	}
	
	/**
	 * Gets session handler list
	 * @return  session handler list
	 */
	public ArrayList<Web2cSessionHandler>getWeb2cSessionHandlerList() {
		
		return web2cSessionHandlerList;
	}
	
	/**
	 * Gets probe
	 * @return probe
	 */
	public Web2cProbe getWeb2cProbe() {
		
		return probe;
	}
	
	/**
	 * Gets archive viewer probe
	 * @return probe
	 */
	public Web2cArchiveViewerProbe getWeb2cArchiveViewerProbe() {
		
		return archiveViewerProbe;
	}
	
	/**
	 * Gets gateway probe
	 * @return probe
	 */
	public Web2cGatewayProbe getWeb2cGatewayProbe() {
		
		return gatewayProbe;
	}
	
	/**
	 * Gets messenger probe
	 * @return probe
	 */
	public Web2cMessengerProbe getWeb2cMessengerProbe() {
		
		return messengerProbe;
	}
	
	/**
	 * Gets image cache
	 * @return image cache
	 */
	public Web2cImageCache getWeb2cImageCache() {
		
		return imageCache;
	}
	
	/**
	 * Gets video connector
	 * @return video connector
	 */
	public Web2cCommonVideoConnector getWeb2cCommonVideoConnector() {
		
		return commonVideoConnector;
	}	
	
	/**
	 * Gets chat handler
	 * @return chat handler
	 */
	public Web2cChatHandler getWeb2cChatHandler() {
		
		return chatHandler;
	}	
	
	/**
	 * Gets common device access interface
	 * @return common device access interface
	 */
	public Web2cCommonDeviceAccess getWeb2cCommonDeviceAccess() {
		
		return commonDeviceAccess;
	}
	
	/**
	 * Gets twitter interface
	 * @return twitter interface
	 */
	public Web2cTwitter getWeb2cTwitterInterface() {
		
		return twitterInterface;
	}
	
	/**
	 * Gets garbage collector
	 * @return garbage collector
	 */
	public Web2cGarbageCollector getWeb2cGarbageCollector() {
		
		return garbageCollector;
	}
	
	public Web2cKeyBoardListener getWeb2cKeyBoardListener() {
		
		return debugListener;
	}
	
	/**
	 * Gets registry reader
	 * @return registry reader
	 */
	public Web2cRegistryReader getWeb2cRegistryReader() {
		
		return registryReader;
	}
	
	/**
	 * Gets init reader
	 * @return init reader
	 */
	public Web2cInitReader getWeb2cInitReader() {
		
		return initReader;
	}
	
	/**
	 * Gets home dir
	 * @return  home dir
	 */
	public String getHomeDir() {
		
		return homeDir;
	}
	
	/**
	 * Gets web dir
	 * @return  web dir
	 */
	public String getWebDir() {
		
		return webDir;
	}
	
	/**
	 * Gets session logger
	 * @return  session logger
	 */
	public Web2cFileLogger getSessionLogger() {
		
		return sessionLogger;
	}
	
	/**
	 * Gets debug logger
	 * @return  debug logger
	 */
	public Web2cFileLogger getDebugLogger() {
		
		return debugLogger;
	}
	
	/**
	 * Gets action logger
	 * @return  action logger
	 */
	public Web2cFileLogger getActionLogger() {
		
		return actionLogger;
	}
}
