package de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl;

import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerPage;
import de.desy.web2cToolkit.web2cEditor.Web2cEditorPage;
import de.desy.web2cToolkit.web2cGateway.Web2cGatewayPage;
import de.desy.web2cToolkit.web2cServlets.Web2c.Web2cPageGenerator;
import de.desy.web2cToolkit.web2cServlets.Web2cArchiveViewer.Web2cArchiveViewerPageGenerator;

/**
 * @author  R.Bacher, DESY
 */
public class Web2cSessionHandler {
	
	private String id = ""; 
	private Web2cPage web2cPage = null;
	private Web2cEditorPage web2cEditorPage = null;
	private Web2cArchiveViewerPage web2cArchiveViewerPage = null;
	private Web2cGatewayPage web2cGatewayPage = null;
	private String loginStatus = "";
	private String dataSource = "";
	private String userName = "";
	private String loadedConfigFileName = "";
	private String requestedConfigFileName = "";
	private String encryptionKey = "";
	private long lastPoll = 0;
	private String fromTime = "";
	private String toTime = "";
	private String language = "";
	private String clientInfo = "";
	private String httpSessionInfo = "";
	private Web2cPageGenerator web2cPageGenerator = null;
	private Web2cArchiveViewerPageGenerator web2cArchiveViewerPageGenerator = null;

	/**
	 * Constructor
	 * @param identifier unique Web2cSession identifier
	 */
	public Web2cSessionHandler(String identifier) {
		super();
		id = identifier;
	}
	
	/**
	 * Returns the Web2c page
	 * @return page
	 */
	public Web2cPage getWeb2cPage() {
		
		return web2cPage;
	}
	
	/**
	 * Sets the Web2c page
	 * @param page Web2c page
	 */
	public void setWeb2cPage(Web2cPage page) {
		
		web2cPage = page;
	}
	
	/**
	 * Returns the Web2cEditor page
	 * @return page
	 */
	public Web2cEditorPage getWeb2cEditorPage() {
		
		return web2cEditorPage;
	}
	
	/**
	 * Sets the Web2cEditor page
	 * @param page Web2cEditor page
	 */
	public void setWeb2cEditorPage(Web2cEditorPage page) {
		
		web2cEditorPage = page;
	}
	
	/**
	 * Returns the Web2cArchiveViewer page
	 * @return page
	 */
	public Web2cArchiveViewerPage getWeb2cArchiveViewerPage() {
		
		return web2cArchiveViewerPage;
	}
	
	/**
	 * Sets the Web2cArchiveViewer page
	 * @param page Web2cArchiveViewer page
	 */
	public void setWeb2cArchiveViewerPage(Web2cArchiveViewerPage page) {
		
		web2cArchiveViewerPage = page;
	}
	
	/**
	 * Returns the Web2cGateway page
	 * @return page
	 */
	public Web2cGatewayPage getWeb2cGatewayPage() {
		
		return web2cGatewayPage;
	}
	
	/**
	 * Sets the Web2cGateway page
	 * @param page Web2cGateway page
	 */
	public void setWeb2cGatewayPage(Web2cGatewayPage page) {
		
		web2cGatewayPage = page;
	}
	
	/**
	 * Returns requested attribute value
	 * @param requestedAttributeName name of requested attribute
	 * @return attribute value
	 */
	public String getAttribute(String requestedAttributeName) {
		
		String attribute = "";
		
		if (requestedAttributeName.equals("id")) attribute = id;		
		if (requestedAttributeName.equals("loginStatus")) attribute = loginStatus;		
		if (requestedAttributeName.equals("encryptionKey")) attribute = encryptionKey;
		if (requestedAttributeName.equals("dataSource")) attribute = dataSource;		
		if (requestedAttributeName.equals("userName")) attribute = userName;		
		if (requestedAttributeName.equals("loadedConfigFileName")) attribute = loadedConfigFileName;		
		if (requestedAttributeName.equals("requestedConfigFileName")) attribute = requestedConfigFileName;			
		if (requestedAttributeName.equals("lastPoll")) attribute = String.valueOf(lastPoll);	
		if (requestedAttributeName.equals("fromTime")) attribute = fromTime;	
		if (requestedAttributeName.equals("toTime")) attribute = toTime;			
		if (requestedAttributeName.equals("language")) attribute = language;				
		if (requestedAttributeName.equals("clientInfo")) attribute = clientInfo;			
		if (requestedAttributeName.equals("httpSessionInfo")) attribute = httpSessionInfo;
		
		return attribute;
	}
	
	/**
	 * Sets requested attribute value
	 * @param requestedAttributeName name of attribute
	 * @param attributeValueToSet value of attribute to set
	 */
	public void setAttribute(String requestedAttributeName, String attributeValueToSet) {
		
		if (requestedAttributeName.equals("id")) id = (String)attributeValueToSet;		
		if (requestedAttributeName.equals("loginStatus")) loginStatus = (String)attributeValueToSet;		
		if (requestedAttributeName.equals("encryptionKey")) encryptionKey = (String)attributeValueToSet;
		if (requestedAttributeName.equals("dataSource")) dataSource = (String)attributeValueToSet;;	
		if (requestedAttributeName.equals("userName")) userName = (String)attributeValueToSet;;	
		if (requestedAttributeName.equals("loadedConfigFileName")) loadedConfigFileName = (String)attributeValueToSet;;	
		if (requestedAttributeName.equals("requestedConfigFileName")) requestedConfigFileName = (String)attributeValueToSet;		
		if (requestedAttributeName.equals("lastPoll")) lastPoll = Long.valueOf(attributeValueToSet);	
		if (requestedAttributeName.equals("fromTime")) fromTime = (String)attributeValueToSet;		
		if (requestedAttributeName.equals("toTime")) toTime = (String)attributeValueToSet;			
		if (requestedAttributeName.equals("language")) language = (String)attributeValueToSet;		
		if (requestedAttributeName.equals("clientInfo")) clientInfo = (String)attributeValueToSet;			
		if (requestedAttributeName.equals("httpSessionInfo")) httpSessionInfo = (String)attributeValueToSet;		
	}
	
	/**
	 * Returns the Web2c page generator
	 * @return page generator
	 */
	public Web2cPageGenerator getWeb2cPageGenerator() {
		
		return web2cPageGenerator;
	}
	
	/**
	 * Sets the Web2c page generator
	 * @param generator page generator
	 */
	public void setWeb2cPageGenerator(Web2cPageGenerator generator) {
		
		 web2cPageGenerator = generator;
	}
	
	/**
	 * Returns the Web2c archive viewer page generator
	 * @return page generator
	 */
	public Web2cArchiveViewerPageGenerator getWeb2cArchiveViewerPageGenerator() {
		
		return web2cArchiveViewerPageGenerator;
	}
	
	/**
	 * Sets the Web2c page generator
	 * @param generator page generator
	 */
	public void setWeb2cArchiveViewerPageGenerator(Web2cArchiveViewerPageGenerator generator) {
		
		 web2cArchiveViewerPageGenerator = generator;
	}
}
